@@ -28,7 +28,7 @@ require_once dirname(__FILE__).'/include/html.inc.php';
 
 if (hasRequest('csv_export')) {
 	$csvExport = true;
-	$csvRows = array();
+	$csvRows = [];
 
 	$page['type'] = detect_page_type(PAGE_TYPE_CSV);
 	$page['file'] = 'zbx_events_export.csv';
@@ -40,8 +40,7 @@ else {
 
 	$page['title'] = _('Latest events');
 	$page['file'] = 'events.php';
-	$page['hist_arg'] = array('groupid', 'hostid');
-	$page['scripts'] = array('class.calendar.js', 'gtlc.js');
+	$page['scripts'] = ['class.calendar.js', 'gtlc.js'];
 	$page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
 	if (PAGE_TYPE_HTML == $page['type']) {
@@ -58,44 +57,40 @@ if ($allow_discovery) {
 }
 
 //		VAR			TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
-$fields = array(
-	'source'=>			array(T_ZBX_INT, O_OPT, P_SYS,	IN($allowed_sources), null),
-	'groupid'=>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'hostid'=>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'triggerid'=>		array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'period'=>			array(T_ZBX_INT, O_OPT, null,	null,		null),
-	'stime'=>			array(T_ZBX_STR, O_OPT, null,	null,		null),
-	'load'=>			array(T_ZBX_STR, O_OPT, P_SYS,	NULL,		null),
-	'fullscreen'=>		array(T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	null),
-	'csv_export'=>		array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'filter_rst'=>		array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'filter_set'=>		array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
+$fields = [
+	'source'=>			[T_ZBX_INT, O_OPT, P_SYS,	IN($allowed_sources), null],
+	'groupid'=>			[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
+	'hostid'=>			[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
+	'triggerid'=>		[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
+	'period'=>			[T_ZBX_INT, O_OPT, null,	null,		null],
+	'stime'=>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'load'=>			[T_ZBX_STR, O_OPT, P_SYS,	NULL,		null],
+	'fullscreen'=>		[T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	null],
+	'csv_export'=>		[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_rst'=>		[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_set'=>		[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	// ajax
-	'filterState' =>	array(T_ZBX_INT, O_OPT, P_ACT,	null,		null),
-	'favobj'=>			array(T_ZBX_STR, O_OPT, P_ACT,	null,		null),
-	'favid'=>			array(T_ZBX_INT, O_OPT, P_ACT,	null,		null)
-);
+	'favobj'=>			[T_ZBX_STR, O_OPT, P_ACT,	null,		null],
+	'favid'=>			[T_ZBX_INT, O_OPT, P_ACT,	null,		null]
+];
 check_fields($fields);
 
 /*
  * Permissions
  */
-if (getRequest('groupid') && !API::HostGroup()->isReadable(array(getRequest('groupid')))) {
+if (getRequest('groupid') && !API::HostGroup()->isReadable([getRequest('groupid')])) {
 	access_deny();
 }
-if (getRequest('hostid') && !API::Host()->isReadable(array(getRequest('hostid')))) {
+if (getRequest('hostid') && !API::Host()->isReadable([getRequest('hostid')])) {
 	access_deny();
 }
-if (getRequest('triggerid') && !API::Trigger()->isReadable(array(getRequest('triggerid')))) {
+if (getRequest('triggerid') && !API::Trigger()->isReadable([getRequest('triggerid')])) {
 	access_deny();
 }
 
 /*
  * Ajax
  */
-if (hasRequest('filterState')) {
-	CProfile::update('web.events.filter.state', getRequest('filterState'), PROFILE_TYPE_INT);
-}
 if (hasRequest('favobj')) {
 	// saving fixed/dynamic setting to profile
 	if ('timelinefixedperiod' == getRequest('favobj')) {
@@ -165,18 +160,18 @@ $till = $from + $period;
  * Display
  */
 if ($source == EVENT_SOURCE_TRIGGERS) {
-	$pageFilter = new CPageFilter(array(
-		'groups' => array(
+	$pageFilter = new CPageFilter([
+		'groups' => [
 			'monitored_hosts' => true,
 			'with_monitored_triggers' => true
-		),
-		'hosts' => array(
+		],
+		'hosts' => [
 			'monitored_hosts' => true,
 			'with_monitored_triggers' => true
-		),
+		],
 		'hostid' => getRequest('hostid'),
 		'groupid' => getRequest('groupid')
-	));
+	]);
 }
 
 if ($csvExport) {
@@ -193,100 +188,63 @@ else {
 		// use the host ID from the page filter since it may not be present in the request
 		// if all hosts are selected, preserve the selected trigger
 		if ($triggerId != 0 && $pageFilter->hostid != 0) {
-			$hostId = $pageFilter->hostid;
+			$hostid = $pageFilter->hostid;
 
-			$oldTriggers = API::Trigger()->get(array(
-				'output' => array('triggerid', 'description', 'expression'),
-				'selectHosts' => array('hostid', 'host'),
-				'selectItems' => array('itemid', 'hostid', 'key_', 'type', 'flags', 'status'),
-				'selectFunctions' => API_OUTPUT_EXTEND,
-				'triggerids' => $triggerId
-			));
-			$oldTrigger = reset($oldTriggers);
+			$old_triggers = API::Trigger()->get([
+				'output' => ['description', 'expression'],
+				'selectHosts' => ['hostid', 'host'],
+				'triggerids' => [$triggerId]
+			]);
+			$old_trigger = reset($old_triggers);
 
-			$oldTrigger['hosts'] = zbx_toHash($oldTrigger['hosts'], 'hostid');
+			$old_trigger['hosts'] = zbx_toHash($old_trigger['hosts'], 'hostid');
 
 			// if the trigger doesn't belong to the selected host - find a new one on that host
-			if (!isset($oldTrigger['hosts'][$hostId])) {
+			if (!array_key_exists($hostid, $old_trigger['hosts'])) {
 				$triggerId = 0;
 
-				$oldTrigger['items'] = zbx_toHash($oldTrigger['items'], 'itemid');
-				$oldTrigger['functions'] = zbx_toHash($oldTrigger['functions'], 'functionid');
-				$oldExpression = triggerExpression($oldTrigger);
+				$old_expression = CMacrosResolverHelper::resolveTriggerExpression($old_trigger['expression']);
 
-				$newTriggers = API::Trigger()->get(array(
-					'output' => array('triggerid', 'description', 'expression'),
-					'selectHosts' => array('hostid', 'host'),
-					'selectItems' => array('itemid', 'key_'),
-					'selectFunctions' => API_OUTPUT_EXTEND,
-					'filter' => array('description' => $oldTrigger['description']),
-					'hostids' => $hostId
-				));
+				$new_triggers = API::Trigger()->get([
+					'output' => ['triggerid', 'description', 'expression'],
+					'selectHosts' => ['hostid', 'host'],
+					'filter' => ['description' => $old_trigger['description']],
+					'hostids' => [$hostid]
+				]);
 
-				foreach ($newTriggers as $newTrigger) {
-					if (count($oldTrigger['items']) != count($newTrigger['items'])) {
-						continue;
-					}
+				$new_triggers = CMacrosResolverHelper::resolveTriggerExpressions($new_triggers);
 
-					$newTrigger['items'] = zbx_toHash($newTrigger['items'], 'itemid');
-					$newTrigger['hosts'] = zbx_toHash($newTrigger['hosts'], 'hostid');
-					$newTrigger['functions'] = zbx_toHash($newTrigger['functions'], 'functionid');
+				foreach ($new_triggers as $new_trigger) {
+					$new_trigger['hosts'] = zbx_toHash($new_trigger['hosts'], 'hostid');
 
-					$found = false;
-					foreach ($newTrigger['functions'] as $fnum => $function) {
-						foreach ($oldTrigger['functions'] as $ofnum => $oldFunction) {
-							// compare functions
-							if (($function['function'] != $oldFunction['function']) || ($function['parameter'] != $oldFunction['parameter'])) {
-								continue;
-							}
-							// compare that functions uses same item keys
-							if ($newTrigger['items'][$function['itemid']]['key_'] != $oldTrigger['items'][$oldFunction['itemid']]['key_']) {
-								continue;
-							}
-							// rewrite itemid so we could compare expressions
-							// of two triggers form different hosts
-							$newTrigger['functions'][$fnum]['itemid'] = $oldFunction['itemid'];
-							$found = true;
+					foreach ($old_trigger['hosts'] as $old_host) {
+						$new_expression = triggerExpressionReplaceHost($new_trigger['expression'],
+							$new_trigger['hosts'][$hostid]['host'], $old_host['host']
+						);
 
-							unset($oldTrigger['functions'][$ofnum]);
-							break;
+						if ($old_expression === $new_expression) {
+							CProfile::update('web.events.filter.triggerid', $new_trigger['triggerid'], PROFILE_TYPE_ID);
+							$triggerId = $new_trigger['triggerid'];
+							break 2;
 						}
-						if (!$found) {
-							break;
-						}
-					}
-					if (!$found) {
-						continue;
-					}
-
-					// if we found same trigger we overwriting it's hosts and items for expression compare
-					$newTrigger['hosts'] = $oldTrigger['hosts'];
-					$newTrigger['items'] = $oldTrigger['items'];
-
-					$newExpression = triggerExpression($newTrigger);
-
-					if (strcmp($oldExpression, $newExpression) == 0) {
-						CProfile::update('web.events.filter.triggerid', $newTrigger['triggerid'], PROFILE_TYPE_ID);
-						$triggerId = $newTrigger['triggerid'];
-						break;
 					}
 				}
 			}
 		}
 	}
 
-	$eventsWidget = new CWidget();
+	$eventsWidget = (new CWidget())->setTitle(_('Events'));
 
 	$csvDisabled = true;
 
 	// header
-	$frmForm = new CForm();
+	$frmForm = (new CForm('get'))
+		->addVar('stime', $stime, 'stime_csv')
+		->addVar('period', $period, 'period_csv')
+		->addVar('page', getPageNumber(), 'page_csv');
 	if (hasRequest('source')) {
 		$frmForm->addVar('source', getRequest('source'), 'source_csv');
 	}
-	$frmForm->addVar('stime', $stime, 'stime_csv');
-	$frmForm->addVar('period', $period, 'period_csv');
-	$frmForm->addVar('page', getPageNumber(), 'page_csv');
 
 	if ($source == EVENT_SOURCE_TRIGGERS) {
 		$frmForm->addVar('groupid', $pageFilter->groupid, 'groupid_csv');
@@ -296,69 +254,54 @@ else {
 			$frmForm->addVar('triggerid', $triggerId, 'triggerid_csv');
 		}
 	}
-	$frmForm->addItem(new CSubmit('csv_export', _('Export to CSV')));
 
-	$eventsWidget->addPageHeader(
-		_('HISTORY OF EVENTS').SPACE.'['.zbx_date2str(DATE_TIME_FORMAT_SECONDS).']',
-		array(
-			$frmForm,
-			SPACE,
-			get_icon('fullscreen', array('fullscreen' => getRequest('fullscreen')))
-		)
-	);
+	$frmForm->addVar('fullscreen', getRequest('fullscreen'));
+	$frmForm->addVar('stime', $stime);
+	$frmForm->addVar('period', $period);
 
-	$headerForm = new CForm('get');
-	$headerForm->addVar('fullscreen', getRequest('fullscreen'));
-	$headerForm->addVar('stime', $stime);
-	$headerForm->addVar('period', $period);
+	$controls = new CList();
 
 	// add host and group filters to the form
 	if ($source == EVENT_SOURCE_TRIGGERS) {
 		if (getRequest('triggerid') != 0) {
-			$headerForm->addVar('triggerid', getRequest('triggerid'), 'triggerid_filter');
+			$frmForm->addVar('triggerid', getRequest('triggerid'), 'triggerid_filter');
 		}
 
-		$headerForm->addItem(array(
-			_('Group').SPACE,
-			$pageFilter->getGroupsCB()
-		));
-		$headerForm->addItem(array(
-			SPACE._('Host').SPACE,
-			$pageFilter->getHostsCB()
-		));
+		$controls->addItem([_('Group'), SPACE, $pageFilter->getGroupsCB()]);
+		$controls->addItem([_('Host'), SPACE, $pageFilter->getHostsCB()]);
 	}
 
 	if ($allow_discovery) {
-		$cmbSource = new CComboBox('source', $source, 'submit()');
-		$cmbSource->addItem(EVENT_SOURCE_TRIGGERS, _('Trigger'));
-		$cmbSource->addItem(EVENT_SOURCE_DISCOVERY, _('Discovery'));
-		$headerForm->addItem(array(SPACE._('Source').SPACE, $cmbSource));
+		$controls->addItem([_('Source'), SPACE, new CComboBox('source', $source, 'submit()', [
+			EVENT_SOURCE_TRIGGERS => _('Trigger'),
+			EVENT_SOURCE_DISCOVERY => _('Discovery')
+		])]);
 	}
 
-	$eventsWidget->addHeader(_('Events'), $headerForm);
-	$eventsWidget->addHeaderRowNumber();
+	$controls->addItem(new CSubmit('csv_export', _('Export to CSV')));
+	$controls->addItem(get_icon('fullscreen', ['fullscreen' => getRequest('fullscreen')]));
 
-	$filterForm = null;
+	$frmForm->addItem($controls);
+	$eventsWidget->setControls($frmForm);
+
+	$filterForm = (new CFilter('web.events.filter.state'))
+		->addVar('fullscreen', getRequest('fullscreen'));
 
 	if ($source == EVENT_SOURCE_TRIGGERS) {
-		$filterForm = new CFormTable(null, null, 'get');
-		$filterForm->setTableClass('formtable old-filter');
-		$filterForm->setAttribute('name', 'zbx_filter');
-		$filterForm->setAttribute('id', 'zbx_filter');
-		$filterForm->addVar('triggerid', $triggerId);
-		$filterForm->addVar('stime', $stime);
-		$filterForm->addVar('period', $period);
+		$filterForm->addVar('triggerid', $triggerId)
+			->addVar('stime', $stime)
+			->addVar('period', $period);
 		$filterForm->addVar('groupid', $pageFilter->groupid);
 		$filterForm->addVar('hostid', $pageFilter->hostid);
 
 		if ($triggerId > 0) {
-			$dbTrigger = API::Trigger()->get(array(
+			$dbTrigger = API::Trigger()->get([
 				'triggerids' => $triggerId,
-				'output' => array('description', 'expression'),
-				'selectHosts' => array('name'),
+				'output' => ['description', 'expression'],
+				'selectHosts' => ['name'],
 				'preservekeys' => true,
 				'expandDescription' => true
-			));
+			]);
 			if ($dbTrigger) {
 				$dbTrigger = reset($dbTrigger);
 				$host = reset($dbTrigger['hosts']);
@@ -373,13 +316,17 @@ else {
 			$trigger = '';
 		}
 
-		$filterForm->addRow(new CRow(array(
-			new CCol(_('Trigger'), 'form_row_l'),
-			new CCol(array(
-				new CTextBox('trigger', $trigger, 96, true),
-				new CButton('btn1', _('Select'),
-					'return PopUp("popup.php?'.
-						'dstfrm='.$filterForm->getName().
+		$filterColumn = new CFormList();
+
+		$filterColumn->addRow(
+			_('Trigger'),
+			[
+				(new CTextBox('trigger', $trigger, true))->setWidth(ZBX_TEXTAREA_FILTER_BIG_WIDTH),
+				(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+				(new CButton('btn1', _('Select')))
+					->addClass(ZBX_STYLE_BTN_GREY)
+					->onClick('return PopUp("popup.php?'.
+						'dstfrm=zbx_filter'.
 						'&dstfld1=triggerid'.
 						'&dstfld2=trigger'.
 						'&srctbl=triggers'.
@@ -389,59 +336,55 @@ else {
 						'&monitored_hosts=1'.
 						'&with_monitored_triggers=1'.
 						($pageFilter->hostid ? '&only_hostid='.$pageFilter->hostid : '').
-						'");',
-					'T'
-				)
-			), 'form_row_r')
-		)));
+						'");'
+					)
+			]
+		);
 
-		$filterForm->addItemToBottomRow(new CSubmit('filter_set', _('Filter')));
-		$filterForm->addItemToBottomRow(new CSubmit('filter_rst', _('Reset')));
+		$filterForm->addColumn($filterColumn);
 	}
 
-	$eventsWidget->addFlicker($filterForm, CProfile::get('web.events.filter.state', 0));
+	$filterForm->addNavigator();
 
-	$scroll = new CDiv();
-	$scroll->setAttribute('id', 'scrollbar_cntr');
-	$eventsWidget->addFlicker($scroll, CProfile::get('web.events.filter.state', 0));
+	$eventsWidget->addItem($filterForm);
 
-	$table = new CTableInfo(_('No events found.'));
+	$table = new CTableInfo();
 }
 
 // trigger events
 if ($source == EVENT_OBJECT_TRIGGER) {
-	$firstEvent = API::Event()->get(array(
+	$firstEvent = API::Event()->get([
 		'source' => EVENT_SOURCE_TRIGGERS,
 		'object' => EVENT_OBJECT_TRIGGER,
 		'output' => API_OUTPUT_EXTEND,
 		'objectids' => $triggerId ? $triggerId : null,
-		'sortfield' => array('clock'),
+		'sortfield' => ['clock'],
 		'sortorder' => ZBX_SORT_UP,
 		'limit' => 1
-	));
+	]);
 	$firstEvent = reset($firstEvent);
 }
 
 // discovery events
 else {
-	$firstEvent = API::Event()->get(array(
+	$firstEvent = API::Event()->get([
 		'output' => API_OUTPUT_EXTEND,
 		'source' => EVENT_SOURCE_DISCOVERY,
 		'object' => EVENT_OBJECT_DHOST,
-		'sortfield' => array('clock'),
+		'sortfield' => ['clock'],
 		'sortorder' => ZBX_SORT_UP,
 		'limit' => 1
-	));
+	]);
 	$firstEvent = reset($firstEvent);
 
-	$firstDServiceEvent = API::Event()->get(array(
+	$firstDServiceEvent = API::Event()->get([
 		'output' => API_OUTPUT_EXTEND,
 		'source' => EVENT_SOURCE_DISCOVERY,
 		'object' => EVENT_OBJECT_DSERVICE,
-		'sortfield' => array('clock'),
+		'sortfield' => ['clock'],
 		'sortorder' => ZBX_SORT_UP,
 		'limit' => 1
-	));
+	]);
 	$firstDServiceEvent = reset($firstDServiceEvent);
 
 	if ($firstDServiceEvent && (!$firstEvent || $firstDServiceEvent['eventid'] < $firstEvent['eventid'])) {
@@ -453,13 +396,13 @@ $config = select_config();
 
 // headers
 if ($source == EVENT_SOURCE_DISCOVERY) {
-	$header = array(
+	$header = [
 		_('Time'),
 		_('IP'),
 		_('DNS'),
 		_('Description'),
 		_('Status')
-	);
+	];
 
 	if ($csvExport) {
 		$csvRows[] = $header;
@@ -469,7 +412,7 @@ if ($source == EVENT_SOURCE_DISCOVERY) {
 	}
 }
 else {
-	$header = array(
+	$header = [
 		_('Time'),
 		($pageFilter->hostid == 0) ? _('Host') : null,
 		_('Description'),
@@ -478,7 +421,7 @@ else {
 		_('Duration'),
 		$config['event_ack_enable'] ? _('Ack') : null,
 		_('Actions')
-	);
+	];
 
 	if ($csvExport) {
 		$csvRows[] = $header;
@@ -492,7 +435,7 @@ if (!$firstEvent) {
 	$starttime = null;
 
 	if (!$csvExport) {
-		$events = array();
+		$events = [];
 		$paging = getPagingLine($events, ZBX_SORT_UP);
 	}
 }
@@ -501,31 +444,31 @@ else {
 
 	if ($source == EVENT_SOURCE_DISCOVERY) {
 		// fetch discovered service and discovered host events separately
-		$dHostEvents = API::Event()->get(array(
+		$dHostEvents = API::Event()->get([
 			'source' => EVENT_SOURCE_DISCOVERY,
 			'object' => EVENT_OBJECT_DHOST,
 			'time_from' => $from,
 			'time_till' => $till,
-			'output' => array('eventid', 'object', 'objectid', 'clock', 'value'),
-			'sortfield' => array('clock', 'eventid'),
+			'output' => ['eventid', 'object', 'objectid', 'clock', 'value'],
+			'sortfield' => ['clock', 'eventid'],
 			'sortorder' => ZBX_SORT_DOWN,
 			'limit' => $config['search_limit'] + 1
-		));
-		$dServiceEvents = API::Event()->get(array(
+		]);
+		$dServiceEvents = API::Event()->get([
 			'source' => EVENT_SOURCE_DISCOVERY,
 			'object' => EVENT_OBJECT_DSERVICE,
 			'time_from' => $from,
 			'time_till' => $till,
-			'output' => array('eventid', 'object', 'objectid', 'clock', 'value'),
-			'sortfield' => array('clock', 'eventid'),
+			'output' => ['eventid', 'object', 'objectid', 'clock', 'value'],
+			'sortfield' => ['clock', 'eventid'],
 			'sortorder' => ZBX_SORT_DOWN,
 			'limit' => $config['search_limit'] + 1
-		));
+		]);
 		$dsc_events = array_merge($dHostEvents, $dServiceEvents);
-		CArrayHelper::sort($dsc_events, array(
-			array('field' => 'clock', 'order' => ZBX_SORT_DOWN),
-			array('field' => 'eventid', 'order' => ZBX_SORT_DOWN)
-		));
+		CArrayHelper::sort($dsc_events, [
+			['field' => 'clock', 'order' => ZBX_SORT_DOWN],
+			['field' => 'eventid', 'order' => ZBX_SORT_DOWN]
+		]);
 		$dsc_events = array_slice($dsc_events, 0, $config['search_limit'] + 1);
 
 		$paging = getPagingLine($dsc_events, ZBX_SORT_DOWN);
@@ -534,13 +477,13 @@ else {
 			$csvDisabled = zbx_empty($dsc_events);
 		}
 
-		$objectids = array();
+		$objectids = [];
 		foreach ($dsc_events as $event_data) {
 			$objectids[$event_data['objectid']] = $event_data['objectid'];
 		}
 
 		// object dhost
-		$dhosts = array();
+		$dhosts = [];
 		$res = DBselect(
 			'SELECT s.dserviceid,s.dhostid,s.ip,s.dns'.
 			' FROM dservices s'.
@@ -551,7 +494,7 @@ else {
 		}
 
 		// object dservice
-		$dservices = array();
+		$dservices = [];
 		$res = DBselect(
 			'SELECT s.dserviceid,s.ip,s.dns,s.type,s.port'.
 			' FROM dservices s'.
@@ -599,22 +542,22 @@ else {
 			}
 
 			if ($csvExport) {
-				$csvRows[] = array(
+				$csvRows[] = [
 					zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event_data['clock']),
 					$event_data['object_data']['ip'],
 					$event_data['object_data']['dns'],
 					$event_data['description'],
 					discovery_value($event_data['value'])
-				);
+				];
 			}
 			else {
-				$table->addRow(array(
+				$table->addRow([
 					zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event_data['clock']),
 					$event_data['object_data']['ip'],
 					zbx_empty($event_data['object_data']['dns']) ? SPACE : $event_data['object_data']['dns'],
 					$event_data['description'],
-					new CCol(discovery_value($event_data['value']), discovery_value_style($event_data['value']))
-				));
+					(new CCol(discovery_value($event_data['value'])))->addClass(discovery_value_style($event_data['value']))
+				]);
 			}
 		}
 	}
@@ -622,41 +565,41 @@ else {
 	// source not discovery i.e. trigger
 	else {
 		if ($csvExport || $pageFilter->hostsSelected) {
-			$knownTriggerIds = array();
-			$validTriggerIds = array();
+			$knownTriggerIds = [];
+			$validTriggerIds = [];
 
-			$triggerOptions = array(
-				'output' => array('triggerid'),
+			$triggerOptions = [
+				'output' => ['triggerid'],
 				'preservekeys' => true,
 				'monitored' => true
-			);
+			];
 
 			$allEventsSliceLimit = $config['search_limit'];
 
-			$eventOptions = array(
+			$eventOptions = [
 				'source' => EVENT_SOURCE_TRIGGERS,
 				'object' => EVENT_OBJECT_TRIGGER,
 				'time_from' => $from,
 				'time_till' => $till,
-				'output' => array('eventid', 'objectid'),
-				'sortfield' => array('clock', 'eventid'),
+				'output' => ['eventid', 'objectid'],
+				'sortfield' => ['clock', 'eventid'],
 				'sortorder' => ZBX_SORT_DOWN,
 				'limit' => $allEventsSliceLimit + 1
-			);
+			];
 
 			if ($triggerId) {
-				$knownTriggerIds = array($triggerId => $triggerId);
+				$knownTriggerIds = [$triggerId => $triggerId];
 				$validTriggerIds = $knownTriggerIds;
 
-				$eventOptions['objectids'] = array($triggerId);;
+				$eventOptions['objectids'] = [$triggerId];;
 			}
 			elseif ($pageFilter->hostid > 0) {
-				$hostTriggers = API::Trigger()->get(array(
-					'output' => array('triggerid'),
+				$hostTriggers = API::Trigger()->get([
+					'output' => ['triggerid'],
 					'hostids' => $pageFilter->hostid,
 					'monitored' => true,
 					'preservekeys' => true
-				));
+				]);
 				$filterTriggerIds = array_map('strval', array_keys($hostTriggers));
 				$knownTriggerIds = array_combine($filterTriggerIds, $filterTriggerIds);
 				$validTriggerIds = $knownTriggerIds;
@@ -670,7 +613,7 @@ else {
 				$triggerOptions['groupids'] = $pageFilter->groupid;
 			}
 
-			$events = array();
+			$events = [];
 
 			while (true) {
 				$allEventsSlice = API::Event()->get($eventOptions);
@@ -695,7 +638,7 @@ else {
 
 				foreach ($allEventsSlice as $event) {
 					if (isset($validTriggerIds[$event['objectid']])) {
-						$events[] = array('eventid' => $event['eventid']);
+						$events[] = ['eventid' => $event['eventid']];
 					}
 				}
 
@@ -722,31 +665,33 @@ else {
 			$paging = getPagingLine($events, ZBX_SORT_DOWN);
 
 			// query event with extend data
-			$events = API::Event()->get(array(
+			$events = API::Event()->get([
 				'source' => EVENT_SOURCE_TRIGGERS,
 				'object' => EVENT_OBJECT_TRIGGER,
 				'eventids' => zbx_objectValues($events, 'eventid'),
 				'output' => API_OUTPUT_EXTEND,
 				'select_acknowledges' => API_OUTPUT_COUNT,
-				'sortfield' => array('clock', 'eventid'),
+				'sortfield' => ['clock', 'eventid'],
 				'sortorder' => ZBX_SORT_DOWN,
 				'nopermissions' => true
-			));
+			]);
 
 			if (!$csvExport) {
 				$csvDisabled = zbx_empty($events);
 			}
 
-			$triggers = API::Trigger()->get(array(
-				'output' => array('triggerid', 'description', 'expression', 'priority', 'flags', 'url'),
-				'selectHosts' => array('hostid', 'name', 'status'),
-				'selectItems' => array('itemid', 'hostid', 'name', 'key_', 'value_type'),
-				'triggerids' => zbx_objectValues($events, 'objectid')
-			));
-			$triggers = zbx_toHash($triggers, 'triggerid');
+			$triggers = API::Trigger()->get([
+				'output' => ['triggerid', 'description', 'expression', 'priority', 'flags', 'url'],
+				'selectHosts' => ['hostid', 'name', 'status'],
+				'selectItems' => ['itemid', 'hostid', 'name', 'key_', 'value_type'],
+				'triggerids' => zbx_objectValues($events, 'objectid'),
+				'preservekeys' => true
+			]);
+
+			$triggers = CMacrosResolverHelper::resolveTriggerUrls($triggers);
 
 			// fetch hosts
-			$hosts = array();
+			$hosts = [];
 			foreach ($triggers as &$trigger) {
 				$hosts[] = reset($trigger['hosts']);
 
@@ -758,13 +703,13 @@ else {
 
 			$hostids = zbx_objectValues($hosts, 'hostid');
 
-			$hosts = API::Host()->get(array(
-				'output' => array('name', 'hostid', 'status'),
+			$hosts = API::Host()->get([
+				'output' => ['name', 'hostid', 'status'],
 				'hostids' => $hostids,
 				'selectGraphs' => API_OUTPUT_COUNT,
 				'selectScreens' => API_OUTPUT_COUNT,
 				'preservekeys' => true
-			));
+			]);
 
 			// fetch scripts for the host JS menu
 			if (!$csvExport && $pageFilter->hostid == 0) {
@@ -772,7 +717,7 @@ else {
 			}
 
 			// actions
-			$actions = getEventActionsStatus(zbx_objectValues($events, 'eventid'));
+			$actions = makeEventsActions(zbx_objectValues($events, 'eventid'));
 
 			// events
 			foreach ($events as $event) {
@@ -781,10 +726,10 @@ else {
 				$host = reset($trigger['hosts']);
 				$host = $hosts[$host['hostid']];
 
-				$description = CMacrosResolverHelper::resolveEventDescription(zbx_array_merge($trigger, array(
+				$description = CMacrosResolverHelper::resolveEventDescription(zbx_array_merge($trigger, [
 					'clock' => $event['clock'],
 					'ns' => $event['ns']
-				)));
+				]));
 
 				// duration
 				$event['duration'] = ($nextEvent = get_next_event($event, $events))
@@ -792,28 +737,26 @@ else {
 					: zbx_date2age($event['clock']);
 
 				// action
-				$action = isset($actions[$event['eventid']]) ? $actions[$event['eventid']] : ' - ';
+				$action = isset($actions[$event['eventid']]) ? $actions[$event['eventid']] : '';
 
 				if ($csvExport) {
-					$csvRows[] = array(
+					$csvRows[] = [
 						zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
 						($pageFilter->hostid == 0) ? $host['name'] : null,
 						$description,
 						trigger_value2str($event['value']),
-						getSeverityCaption($trigger['priority']),
+						getSeverityName($trigger['priority'], $config),
 						$event['duration'],
 						$config['event_ack_enable'] ? ($event['acknowledges'] ? _('Yes') : _('No')) : null,
 						strip_tags((string) $action)
-					);
+					];
 				}
 				else {
-					$triggerDescription = new CSpan($description, 'pointer link_menu');
-					$triggerDescription->setMenuPopup(
-						CMenuPopupHelper::getTrigger($trigger, null, $event['clock'])
-					);
-
-					// acknowledge
-					$ack = getEventAckState($event, true);
+					$triggerDescription = (new CSpan($description))
+						->addClass(ZBX_STYLE_LINK_ACTION)
+						->setMenuPopup(
+							CMenuPopupHelper::getTrigger($trigger, null, $event['clock'])
+						);
 
 					// add colors and blinking to span depending on configuration and trigger parameters
 					$statusSpan = new CSpan(trigger_value2str($event['value']));
@@ -829,36 +772,36 @@ else {
 					$hostName = null;
 
 					if ($pageFilter->hostid == 0) {
-						$hostName = new CSpan($host['name'], 'link_menu');
-						$hostName->setMenuPopup(CMenuPopupHelper::getHost($host, $scripts[$host['hostid']]));
+						$hostName = (new CSpan($host['name']))
+							->addClass(ZBX_STYLE_LINK_ACTION)
+							->setMenuPopup(CMenuPopupHelper::getHost($host, $scripts[$host['hostid']]));
 					}
 
-					$table->addRow(array(
-						new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
-								'tr_events.php?triggerid='.$event['objectid'].'&eventid='.$event['eventid'],
-							'action'
-						),
+					$table->addRow([
+						(new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
+								'tr_events.php?triggerid='.$event['objectid'].'&eventid='.$event['eventid']))
+							->addClass('action'),
 						$hostName,
 						$triggerDescription,
 						$statusSpan,
-						getSeverityCell($trigger['priority'], null, !$event['value']),
+						getSeverityCell($trigger['priority'], $config, null, !$event['value']),
 						$event['duration'],
-						$config['event_ack_enable'] ? $ack : null,
-						$action
-					));
+						$config['event_ack_enable'] ? getEventAckState($event, $page['file']) : null,
+						(new CCol($action))->addClass(ZBX_STYLE_NOWRAP)
+					]);
 				}
 			}
 		}
 		else {
 			if (!$csvExport) {
-				$events = array();
+				$events = [];
 				$paging = getPagingLine($events, ZBX_SORT_UP);
 			}
 		}
 	}
 
 	if (!$csvExport) {
-		$table = array($paging, $table, $paging);
+		$table = [$table, $paging];
 	}
 }
 
@@ -868,13 +811,13 @@ if ($csvExport) {
 else {
 	$eventsWidget->addItem($table);
 
-	$timeline = array(
+	$timeline = [
 		'period' => $period,
 		'starttime' => date(TIMESTAMP_FORMAT, $starttime),
 		'usertime' => date(TIMESTAMP_FORMAT, $till)
-	);
+	];
 
-	$objData = array(
+	$objData = [
 		'id' => 'timeline_1',
 		'loadSBox' => 0,
 		'loadImage' => 0,
@@ -883,7 +826,7 @@ else {
 		'mainObject' => 1,
 		'periodFixed' => CProfile::get('web.events.timelinefixed', 1),
 		'sliderMaximumTimePeriod' => ZBX_MAX_PERIOD
-	);
+	];
 
 	zbx_add_post_js('jqBlink.blink();');
 	zbx_add_post_js('timeControl.addObject("scroll_events_id", '.zbx_jsvalue($timeline).', '.zbx_jsvalue($objData).');');
