@@ -26,73 +26,71 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of host prototypes');
 $page['file'] = 'host_prototypes.php';
-$page['scripts'] = array('effects.js', 'class.cviewswitcher.js', 'multiselect.js');
-
-//???
-$page['hist_arg'] = array('parent_discoveryid');
+$page['scripts'] = ['effects.js', 'class.cviewswitcher.js', 'multiselect.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
-$fields = array(
-	'hostid' =>					array(T_ZBX_INT, O_NO,	P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'),
-	'parent_discoveryid' =>		array(T_ZBX_INT, O_MAND, P_SYS,	DB_ID, null),
-	'host' =>		        	array(T_ZBX_STR, O_OPT, null,		NOT_EMPTY,	'isset({add}) || isset({update})', _('Host name')),
-	'name' =>	            	array(T_ZBX_STR, O_OPT, null,		null,		'isset({add}) || isset({update})'),
-	'status' =>		        	array(T_ZBX_INT, O_OPT, null,		IN(array(HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED)), null),
-	'inventory_mode' =>			array(T_ZBX_INT, O_OPT, null, IN(array(HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC)), null),
-	'templates' =>		    	array(T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null),
-	'add_template' =>			array(T_ZBX_STR, O_OPT, null,		null,	null),
-	'add_templates' =>		    array(T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null),
-	'group_links' =>			array(T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null),
-	'group_prototypes' =>		array(T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null),
-	'unlink' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,		null),
-	'group_hostid' =>			array(T_ZBX_INT, O_OPT, null,	DB_ID,		null),
+$fields = [
+	'hostid' =>					[T_ZBX_INT, O_NO,	P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'],
+	'parent_discoveryid' =>		[T_ZBX_INT, O_MAND, P_SYS,	DB_ID, null],
+	'host' =>		        	[T_ZBX_STR, O_OPT, null,		NOT_EMPTY,	'isset({add}) || isset({update})', _('Host name')],
+	'name' =>	            	[T_ZBX_STR, O_OPT, null,		null,		'isset({add}) || isset({update})'],
+	'status' =>		        	[T_ZBX_INT, O_OPT, null,		IN([HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED]), null],
+	'inventory_mode' =>			[T_ZBX_INT, O_OPT, null, IN([HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC]), null],
+	'templates' =>		    	[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'add_template' =>			[T_ZBX_STR, O_OPT, null,		null,	null],
+	'add_templates' =>		    [T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'group_links' =>			[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'group_prototypes' =>		[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'unlink' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,		null],
+	'group_hostid' =>			[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
+	'show_inherited_macros' =>	[T_ZBX_INT, O_OPT, null, IN([0,1]), null],
 	// actions
-	'action' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,
+	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 									IN('"hostprototype.massdelete","hostprototype.massdisable",'.
 										'"hostprototype.massenable"'
 									),
 									null
-								),
-	'add' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'update' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'clone' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'update' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'delete' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'cancel' =>					array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'form' =>					array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'form_refresh' =>			array(T_ZBX_INT, O_OPT, null,	null,		null),
+								],
+	'add' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'update' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'clone' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'update' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'delete' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'form' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'form_refresh' =>			[T_ZBX_INT, O_OPT, null,	null,		null],
 	// sort and sortorder
-	'sort' =>					array(T_ZBX_STR, O_OPT, P_SYS, IN('"name","status"'),						null),
-	'sortorder' =>				array(T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null)
-);
+	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS, IN('"name","status"'),						null],
+	'sortorder' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
+];
 check_fields($fields);
 
 // permissions
 if (getRequest('parent_discoveryid')) {
-	$discoveryRule = API::DiscoveryRule()->get(array(
+	$discoveryRule = API::DiscoveryRule()->get([
 		'itemids' => $_REQUEST['parent_discoveryid'],
 		'output' => API_OUTPUT_EXTEND,
-		'selectHosts' => array('flags'),
+		'selectHosts' => ['flags'],
 		'editable' => true
-	));
+	]);
 	$discoveryRule = reset($discoveryRule);
 	if (!$discoveryRule || $discoveryRule['hosts'][0]['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
 		access_deny();
 	}
 
 	if (getRequest('hostid')) {
-		$hostPrototype = API::HostPrototype()->get(array(
+		$hostPrototype = API::HostPrototype()->get([
 			'hostids' => getRequest('hostid'),
 			'output' => API_OUTPUT_EXTEND,
 			'selectGroupLinks' => API_OUTPUT_EXTEND,
 			'selectGroupPrototypes' => API_OUTPUT_EXTEND,
-			'selectTemplates' => array('templateid', 'name'),
-			'selectParentHost' => array('hostid'),
+			'selectTemplates' => ['templateid', 'name'],
+			'selectParentHost' => ['hostid'],
 			'selectInventory' => API_OUTPUT_EXTEND,
 			'editable' => true
-		));
+		]);
 		$hostPrototype = reset($hostPrototype);
 		if (!$hostPrototype) {
 			access_deny();
@@ -108,7 +106,7 @@ else {
  */
 // add templates to the list
 if (getRequest('add_template')) {
-	foreach (getRequest('add_templates', array()) as $templateId) {
+	foreach (getRequest('add_templates', []) as $templateId) {
 		$_REQUEST['templates'][$templateId] = $templateId;
 	}
 }
@@ -120,7 +118,7 @@ elseif (getRequest('unlink')) {
 }
 elseif (isset($_REQUEST['delete']) && isset($_REQUEST['hostid'])) {
 	DBstart();
-	$result = API::HostPrototype()->delete(array(getRequest('hostid')));
+	$result = API::HostPrototype()->delete([getRequest('hostid')]);
 	$result = DBend($result);
 
 	if ($result) {
@@ -141,20 +139,20 @@ elseif (isset($_REQUEST['clone']) && isset($_REQUEST['hostid'])) {
 elseif (hasRequest('add') || hasRequest('update')) {
 	DBstart();
 
-	$newHostPrototype = array(
+	$newHostPrototype = [
 		'host' => getRequest('host'),
 		'name' => getRequest('name'),
 		'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
-		'groupLinks' => array(),
-		'groupPrototypes' => array(),
-		'templates' => getRequest('templates', array()),
-		'inventory' => array(
+		'groupLinks' => [],
+		'groupPrototypes' => [],
+		'templates' => getRequest('templates', []),
+		'inventory' => [
 			'inventory_mode' => getRequest('inventory_mode')
-		)
-	);
+		]
+	];
 
 	// add custom group prototypes
-	foreach (getRequest('group_prototypes', array()) as $groupPrototype) {
+	foreach (getRequest('group_prototypes', []) as $groupPrototype) {
 		if (!$groupPrototype['group_prototypeid']) {
 			unset($groupPrototype['group_prototypeid']);
 		}
@@ -171,17 +169,17 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			// add group prototypes based on existing host groups
 			$groupPrototypesByGroupId = zbx_toHash($hostPrototype['groupLinks'], 'groupid');
 			unset($groupPrototypesByGroupId[0]);
-			foreach (getRequest('group_links', array()) as $groupId) {
+			foreach (getRequest('group_links', []) as $groupId) {
 				if (isset($groupPrototypesByGroupId[$groupId])) {
-					$newHostPrototype['groupLinks'][] = array(
+					$newHostPrototype['groupLinks'][] = [
 						'groupid' => $groupPrototypesByGroupId[$groupId]['groupid'],
 						'group_prototypeid' => $groupPrototypesByGroupId[$groupId]['group_prototypeid']
-					);
+					];
 				}
 				else {
-					$newHostPrototype['groupLinks'][] = array(
+					$newHostPrototype['groupLinks'][] = [
 						'groupid' => $groupId
-					);
+					];
 				}
 			}
 		}
@@ -189,7 +187,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			unset($newHostPrototype['groupPrototypes'], $newHostPrototype['groupLinks']);
 		}
 
-		$newHostPrototype = CArrayHelper::unsetEqualValues($newHostPrototype, $hostPrototype, array('hostid'));
+		$newHostPrototype = CArrayHelper::unsetEqualValues($newHostPrototype, $hostPrototype, ['hostid']);
 		$result = API::HostPrototype()->update($newHostPrototype);
 
 		show_messages($result, _('Host prototype updated'), _('Cannot update host prototype'));
@@ -198,10 +196,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		$newHostPrototype['ruleid'] = getRequest('parent_discoveryid');
 
 		// add group prototypes based on existing host groups
-		foreach (getRequest('group_links', array()) as $groupId) {
-			$newHostPrototype['groupLinks'][] = array(
+		foreach (getRequest('group_links', []) as $groupId) {
+			$newHostPrototype['groupLinks'][] = [
 				'groupid' => $groupId
-			);
+			];
 		}
 
 		$result = API::HostPrototype()->create($newHostPrototype);
@@ -217,17 +215,17 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	}
 }
 // GO
-elseif (hasRequest('action') && str_in_array(getRequest('action'), array('hostprototype.massenable', 'hostprototype.massdisable')) && hasRequest('group_hostid')) {
+elseif (hasRequest('action') && str_in_array(getRequest('action'), ['hostprototype.massenable', 'hostprototype.massdisable']) && hasRequest('group_hostid')) {
 	$enable = (getRequest('action') == 'hostprototype.massenable');
 	$status = $enable ? HOST_STATUS_MONITORED : HOST_STATUS_NOT_MONITORED;
-	$update = array();
+	$update = [];
 
 	DBstart();
 	foreach ((array) getRequest('group_hostid') as $hostPrototypeId) {
-		$update[] = array(
+		$update[] = [
 			'hostid' => $hostPrototypeId,
 			'status' => $status
-		);
+		];
 	}
 
 	$result = API::HostPrototype()->update($update);
@@ -258,60 +256,63 @@ elseif (hasRequest('action') && getRequest('action') == 'hostprototype.massdelet
 	show_messages($result, _('Host prototypes deleted'), _('Cannot delete host prototypes'));
 }
 
+$config = select_config();
+
 /*
  * Display
  */
 if (isset($_REQUEST['form'])) {
-	$data = array(
+	$data = [
 		'discovery_rule' => $discoveryRule,
-		'host_prototype' => array(
+		'host_prototype' => [
 			'hostid' => getRequest('hostid'),
 			'templateid' => getRequest('templateid'),
 			'host' => getRequest('host'),
 			'name' => getRequest('name'),
 			'status' => getRequest('status', HOST_STATUS_MONITORED),
-			'templates' => array(),
-			'inventory' => array(
-				'inventory_mode' => getRequest('inventory_mode', HOST_INVENTORY_DISABLED)
-			),
-			'groupPrototypes' => getRequest('group_prototypes', array())
-		),
-		'groups' => array()
-	);
+			'templates' => [],
+			'inventory' => [
+				'inventory_mode' => getRequest('inventory_mode', $config['default_inventory_mode'])
+			],
+			'groupPrototypes' => getRequest('group_prototypes', [])
+		],
+		'groups' => [],
+		'show_inherited_macros' => getRequest('show_inherited_macros', 0)
+	];
 
 	// add already linked and new templates
-	$data['host_prototype']['templates'] = API::Template()->get(array(
-		'output' => array('templateid', 'name'),
-		'templateids' => getRequest('templates', array())
-	));
+	$data['host_prototype']['templates'] = API::Template()->get([
+		'output' => ['templateid', 'name'],
+		'templateids' => getRequest('templates', [])
+	]);
 
 	// add parent host
-	$parentHost = API::Host()->get(array(
+	$parentHost = API::Host()->get([
 		'output' => API_OUTPUT_EXTEND,
-		'selectGroups' => array('groupid', 'name'),
+		'selectGroups' => ['groupid', 'name'],
 		'selectInterfaces' => API_OUTPUT_EXTEND,
-		'selectMacros' => API_OUTPUT_EXTEND,
+		'selectMacros' => ['macro', 'value'],
 		'hostids' => $discoveryRule['hostid'],
 		'templated_hosts' => true
-	));
+	]);
 	$parentHost = reset($parentHost);
 	$data['parent_host'] = $parentHost;
 
 	if (getRequest('group_links')) {
-		$data['groups'] = API::HostGroup()->get(array(
+		$data['groups'] = API::HostGroup()->get([
 			'output' => API_OUTPUT_EXTEND,
 			'groupids' => getRequest('group_links'),
 			'editable' => true,
 			'preservekeys' => true
-		));
+		]);
 	}
 
 	if ($parentHost['proxy_hostid']) {
-		$proxy = API::Proxy()->get(array(
-			'output' => array('host', 'proxyid'),
+		$proxy = API::Proxy()->get([
+			'output' => ['host', 'proxyid'],
 			'proxyids' => $parentHost['proxy_hostid'],
 			'limit' => 1
-		));
+		]);
 		$data['proxy'] = reset($proxy);
 	}
 
@@ -319,24 +320,28 @@ if (isset($_REQUEST['form'])) {
 	if (getRequest('hostid') && !getRequest('form_refresh')) {
 		$data['host_prototype'] = array_merge($data['host_prototype'], $hostPrototype);
 
-		$data['groups'] = API::HostGroup()->get(array(
+		if (!array_key_exists('inventory_mode', $data['host_prototype']['inventory'])) {
+			$data['host_prototype']['inventory']['inventory_mode'] = HOST_INVENTORY_DISABLED;
+		}
+
+		$data['groups'] = API::HostGroup()->get([
 			'output' => API_OUTPUT_EXTEND,
 			'groupids' => zbx_objectValues($data['host_prototype']['groupLinks'], 'groupid'),
 			'editable' => true,
 			'preservekeys' => true
-		));
+		]);
 
 		// add parent templates
 		if ($hostPrototype['templateid']) {
-			$data['parents'] = array();
+			$data['parents'] = [];
 			$hostPrototypeId = $hostPrototype['templateid'];
 			while ($hostPrototypeId) {
-				$parentHostPrototype = API::HostPrototype()->get(array(
-					'output' => array('itemid', 'templateid'),
-					'selectParentHost' => array('hostid', 'name'),
-					'selectDiscoveryRule' => array('itemid'),
+				$parentHostPrototype = API::HostPrototype()->get([
+					'output' => ['itemid', 'templateid'],
+					'selectParentHost' => ['hostid', 'name'],
+					'selectDiscoveryRule' => ['itemid'],
 					'hostids' => $hostPrototypeId
-				));
+				]);
 				$parentHostPrototype = reset($parentHostPrototype);
 				$hostPrototypeId = null;
 
@@ -349,7 +354,7 @@ if (isset($_REQUEST['form'])) {
 	}
 
 	// order linked templates
-	CArrayHelper::sort($data['host_prototype']['templates'], array('name'));
+	CArrayHelper::sort($data['host_prototype']['templates'], ['name']);
 
 	// render view
 	$itemView = new CView('configuration.host.prototype.edit', $data);
@@ -363,40 +368,40 @@ else {
 	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
 	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
-	$data = array(
+	$data = [
 		'form' => getRequest('form'),
 		'parent_discoveryid' => getRequest('parent_discoveryid'),
 		'discovery_rule' => $discoveryRule,
 		'sort' => $sortField,
 		'sortorder' => $sortOrder
-	);
+	];
 
 	// get items
-	$data['hostPrototypes'] = API::HostPrototype()->get(array(
+	$data['hostPrototypes'] = API::HostPrototype()->get([
 		'discoveryids' => $data['parent_discoveryid'],
 		'output' => API_OUTPUT_EXTEND,
-		'selectTemplates' => array('templateid', 'name'),
+		'selectTemplates' => ['templateid', 'name'],
 		'editable' => true,
 		'sortfield' => $sortField,
 		'limit' => $config['search_limit'] + 1
-	));
+	]);
 
 	order_result($data['hostPrototypes'], $sortField, $sortOrder);
 
 	$data['paging'] = getPagingLine($data['hostPrototypes'], $sortOrder);
 
 	// fetch templates linked to the prototypes
-	$templateIds = array();
+	$templateIds = [];
 	foreach ($data['hostPrototypes'] as $hostPrototype) {
 		$templateIds = array_merge($templateIds, zbx_objectValues($hostPrototype['templates'], 'templateid'));
 	}
 	$templateIds = array_unique($templateIds);
 
-	$linkedTemplates = API::Template()->get(array(
-		'output' => array('templateid', 'name'),
+	$linkedTemplates = API::Template()->get([
+		'output' => ['templateid', 'name'],
 		'templateids' => $templateIds,
-		'selectParentTemplates' => array('hostid', 'name')
-	));
+		'selectParentTemplates' => ['hostid', 'name']
+	]);
 	$data['linkedTemplates'] = zbx_toHash($linkedTemplates, 'templateid');
 
 	// fetch source templates and LLD rules
@@ -413,10 +418,10 @@ else {
 		foreach ($data['hostPrototypes'] as &$hostPrototype) {
 			if ($hostPrototype['templateid']) {
 				$sourceTemplate = $hostPrototypeSourceTemplates[$hostPrototypeSourceIds[$hostPrototype['hostid']]];
-				$hostPrototype['sourceTemplate'] = array(
+				$hostPrototype['sourceTemplate'] = [
 					'hostid' => $sourceTemplate['parent_hostid'],
 					'name' => $sourceTemplate['name']
-				);
+				];
 				$sourceDiscoveryRuleId = get_realrule_by_itemid_and_hostid($discoveryRule['itemid'], $sourceTemplate['hostid']);
 				$hostPrototype['sourceDiscoveryRuleId'] = $sourceDiscoveryRuleId;
 			}
