@@ -40,7 +40,7 @@ abstract class CItemGeneral extends CApiService {
 	 *
 	 * @return array
 	 */
-	abstract public function get($options = array());
+	abstract public function get($options = []);
 
 	public function __construct() {
 		parent::__construct();
@@ -48,57 +48,57 @@ abstract class CItemGeneral extends CApiService {
 		// template - if templated item, value is taken from template item, cannot be changed on host
 		// system - values should not be updated
 		// host - value should be null for template items
-		$this->fieldRules = array(
-			'type'					=> array('template' => 1),
-			'snmp_community'		=> array(),
-			'snmp_oid'				=> array('template' => 1),
-			'hostid'				=> array(),
-			'name'					=> array('template' => 1),
-			'description'			=> array(),
-			'key_'					=> array('template' => 1),
-			'delay'					=> array(),
-			'history'				=> array(),
-			'trends'				=> array(),
-			'status'				=> array(),
-			'value_type'			=> array('template' => 1),
-			'trapper_hosts'			=> array(),
-			'units'					=> array('template' => 1),
-			'multiplier'			=> array('template' => 1),
-			'delta'					=> array(),
-			'snmpv3_contextname'	=> array(),
-			'snmpv3_securityname'	=> array(),
-			'snmpv3_securitylevel'	=> array(),
-			'snmpv3_authprotocol'	=> array(),
-			'snmpv3_authpassphrase'	=> array(),
-			'snmpv3_privprotocol'	=> array(),
-			'snmpv3_privpassphrase'	=> array(),
-			'formula'				=> array('template' => 1),
-			'error'					=> array('system' => 1),
-			'lastlogsize'			=> array('system' => 1),
-			'logtimefmt'			=> array(),
-			'templateid'			=> array('system' => 1),
-			'valuemapid'			=> array('template' => 1),
-			'delay_flex'			=> array(),
-			'params'				=> array(),
-			'ipmi_sensor'			=> array('template' => 1),
-			'data_type'				=> array('template' => 1),
-			'authtype'				=> array(),
-			'username'				=> array(),
-			'password'				=> array(),
-			'publickey'				=> array(),
-			'privatekey'			=> array(),
-			'mtime'					=> array('system' => 1),
-			'flags'					=> array(),
-			'filter'				=> array(),
-			'interfaceid'			=> array('host' => 1),
-			'port'					=> array(),
-			'inventory_link'		=> array(),
-			'lifetime'				=> array()
-		);
+		$this->fieldRules = [
+			'type'					=> ['template' => 1],
+			'snmp_community'		=> [],
+			'snmp_oid'				=> ['template' => 1],
+			'hostid'				=> [],
+			'name'					=> ['template' => 1],
+			'description'			=> [],
+			'key_'					=> ['template' => 1],
+			'delay'					=> [],
+			'history'				=> [],
+			'trends'				=> [],
+			'status'				=> [],
+			'value_type'			=> ['template' => 1],
+			'trapper_hosts'			=> [],
+			'units'					=> ['template' => 1],
+			'multiplier'			=> ['template' => 1],
+			'delta'					=> [],
+			'snmpv3_contextname'	=> [],
+			'snmpv3_securityname'	=> [],
+			'snmpv3_securitylevel'	=> [],
+			'snmpv3_authprotocol'	=> [],
+			'snmpv3_authpassphrase'	=> [],
+			'snmpv3_privprotocol'	=> [],
+			'snmpv3_privpassphrase'	=> [],
+			'formula'				=> ['template' => 1],
+			'error'					=> ['system' => 1],
+			'lastlogsize'			=> ['system' => 1],
+			'logtimefmt'			=> [],
+			'templateid'			=> ['system' => 1],
+			'valuemapid'			=> ['template' => 1],
+			'delay_flex'			=> [],
+			'params'				=> [],
+			'ipmi_sensor'			=> ['template' => 1],
+			'data_type'				=> ['template' => 1],
+			'authtype'				=> [],
+			'username'				=> [],
+			'password'				=> [],
+			'publickey'				=> [],
+			'privatekey'			=> [],
+			'mtime'					=> ['system' => 1],
+			'flags'					=> [],
+			'filter'				=> [],
+			'interfaceid'			=> ['host' => 1],
+			'port'					=> [],
+			'inventory_link'		=> [],
+			'lifetime'				=> []
+		];
 
-		$this->errorMessages = array_merge($this->errorMessages, array(
+		$this->errorMessages = array_merge($this->errorMessages, [
 			self::ERROR_NO_INTERFACE => _('Cannot find host interface on "%1$s" for item key "%2$s".')
-		));
+		]);
 	}
 
 	/**
@@ -115,33 +115,33 @@ abstract class CItemGeneral extends CApiService {
 	 */
 	protected function checkInput(array &$items, $update = false) {
 		if ($update) {
-			$itemDbFields = array('itemid' => null);
+			$itemDbFields = ['itemid' => null];
 
-			$dbItemsFields = array('itemid', 'templateid');
+			$dbItemsFields = ['itemid', 'templateid'];
 			foreach ($this->fieldRules as $field => $rule) {
 				if (!isset($rule['system'])) {
 					$dbItemsFields[] = $field;
 				}
 			}
 
-			$dbItems = $this->get(array(
+			$dbItems = $this->get([
 				'output' => $dbItemsFields,
 				'itemids' => zbx_objectValues($items, 'itemid'),
 				'editable' => true,
 				'preservekeys' => true
-			));
+			]);
 
-			$dbHosts = API::Host()->get(array(
-				'output' => array('hostid', 'status', 'name'),
+			$dbHosts = API::Host()->get([
+				'output' => ['hostid', 'status', 'name'],
 				'hostids' => zbx_objectValues($dbItems, 'hostid'),
 				'templated_hosts' => true,
 				'editable' => true,
-				'selectApplications' => array('applicationid'),
+				'selectApplications' => ['applicationid', 'flags'],
 				'preservekeys' => true
-			));
+			]);
 		}
 		else {
-			$itemDbFields = array(
+			$itemDbFields = [
 				'name' => null,
 				'key_' => null,
 				'hostid' => null,
@@ -149,31 +149,31 @@ abstract class CItemGeneral extends CApiService {
 				'value_type' => null,
 				'delay' => '0',
 				'delay_flex' => ''
-			);
+			];
 
-			$dbHosts = API::Host()->get(array(
-				'output' => array('hostid', 'status', 'name'),
+			$dbHosts = API::Host()->get([
+				'output' => ['hostid', 'status', 'name'],
 				'hostids' => zbx_objectValues($items, 'hostid'),
 				'templated_hosts' => true,
 				'editable' => true,
-				'selectApplications' => array('applicationid'),
+				'selectApplications' => ['applicationid', 'flags'],
 				'preservekeys' => true
-			));
+			]);
 		}
 
 		// interfaces
-		$interfaces = API::HostInterface()->get(array(
-			'output' => array('interfaceid', 'hostid', 'type'),
+		$interfaces = API::HostInterface()->get([
+			'output' => ['interfaceid', 'hostid', 'type'],
 			'hostids' => zbx_objectValues($dbHosts, 'hostid'),
 			'nopermissions' => true,
 			'preservekeys' => true
-		));
+		]);
 
 		if ($update) {
-			$updateDiscoveredValidator = new CUpdateDiscoveredValidator(array(
-				'allowed' => array('itemid', 'status'),
-				'messageAllowedField' => _('Cannot update "%1$s" for a discovered item.')
-			));
+			$updateDiscoveredValidator = new CUpdateDiscoveredValidator([
+				'allowed' => ['itemid', 'status'],
+				'messageAllowedField' => _('Cannot update "%2$s" for a discovered item "%1$s".')
+			]);
 			foreach ($items as $item) {
 				// check permissions
 				if (!isset($dbItems[$item['itemid']])) {
@@ -181,13 +181,20 @@ abstract class CItemGeneral extends CApiService {
 						_('No permissions to referred object or it does not exist!'));
 				}
 
+				$dbItem = $dbItems[$item['itemid']];
+
+				$itemName = isset($item['name']) ? $item['name'] : $dbItem['name'];
+
 				// discovered fields, except status, cannot be updated
-				$this->checkPartialValidator($item, $updateDiscoveredValidator, $dbItems[$item['itemid']]);
+				$updateDiscoveredValidator->setObjectName($itemName);
+				$this->checkPartialValidator($item, $updateDiscoveredValidator, $dbItem);
 			}
 
-			$items = $this->extendObjects($this->tableName(), $items, array('name'));
+			$items = $this->extendObjects($this->tableName(), $items, ['name', 'flags']);
 		}
 
+		$item_key_parser = new CItemKey();
+
 		foreach ($items as $inum => &$item) {
 			$item = $this->clearValues($item);
 
@@ -202,7 +209,7 @@ abstract class CItemGeneral extends CApiService {
 
 				$this->checkNoParameters(
 					$item,
-					array('templateid', 'state'),
+					['templateid', 'state'],
 					_('Cannot update "%1$s" for item "%2$s".'),
 					$item['name']
 				);
@@ -244,7 +251,7 @@ abstract class CItemGeneral extends CApiService {
 
 				$this->checkNoParameters(
 					$item,
-					array('templateid', 'state'),
+					['templateid', 'state'],
 					_('Cannot set "%1$s" for item "%2$s".'),
 					$item['name']
 				);
@@ -262,6 +269,12 @@ abstract class CItemGeneral extends CApiService {
 				$item['data_type'] = 0;
 			}
 
+			// For non-numeric types, whichever value was entered in trends field, is overwritten to zero.
+			if ($fullItem['value_type'] == ITEM_VALUE_TYPE_STR || $fullItem['value_type'] == ITEM_VALUE_TYPE_LOG
+					|| $fullItem['value_type'] == ITEM_VALUE_TYPE_TEXT) {
+				$item['trends'] = 0;
+			}
+
 			// check if the item requires an interface
 			$itemInterfaceType = itemTypeInterface($fullItem['type']);
 			if ($itemInterfaceType !== false && $host['status'] != HOST_STATUS_TEMPLATE) {
@@ -281,63 +294,107 @@ abstract class CItemGeneral extends CApiService {
 			}
 
 			// item key
-			if (($fullItem['type'] == ITEM_TYPE_DB_MONITOR && strcmp($fullItem['key_'], ZBX_DEFAULT_KEY_DB_MONITOR) == 0)
-					|| ($fullItem['type'] == ITEM_TYPE_SSH && strcmp($fullItem['key_'], ZBX_DEFAULT_KEY_SSH) == 0)
+			if ($fullItem['type'] == ITEM_TYPE_DB_MONITOR) {
+				if (!isset($fullItem['flags']) || $fullItem['flags'] != ZBX_FLAG_DISCOVERY_RULE) {
+					if (strcmp($fullItem['key_'], ZBX_DEFAULT_KEY_DB_MONITOR) == 0) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_('Check the key, please. Default example was passed.')
+						);
+					}
+				}
+				elseif ($fullItem['flags'] == ZBX_FLAG_DISCOVERY_RULE) {
+					if (strcmp($fullItem['key_'], ZBX_DEFAULT_KEY_DB_MONITOR_DISCOVERY) == 0) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_('Check the key, please. Default example was passed.')
+						);
+					}
+				}
+			}
+			elseif (($fullItem['type'] == ITEM_TYPE_SSH && strcmp($fullItem['key_'], ZBX_DEFAULT_KEY_SSH) == 0)
 					|| ($fullItem['type'] == ITEM_TYPE_TELNET && strcmp($fullItem['key_'], ZBX_DEFAULT_KEY_TELNET) == 0)
 					|| ($fullItem['type'] == ITEM_TYPE_JMX && strcmp($fullItem['key_'], ZBX_DEFAULT_KEY_JMX) == 0)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Check the key, please. Default example was passed.'));
 			}
 
 			// key
-			$itemKey = new CItemKey($fullItem['key_']);
-			if (!$itemKey->isValid()) {
+			if ($item_key_parser->parse($fullItem['key_']) != CParser::PARSE_SUCCESS) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_params($this->getErrorMsg(self::ERROR_INVALID_KEY), array(
-						$fullItem['key_'],
-						$fullItem['name'],
-						$host['name'],
-						$itemKey->getError()
-					))
+					_params($this->getErrorMsg(self::ERROR_INVALID_KEY), [
+						$fullItem['key_'], $fullItem['name'], $host['name'], $item_key_parser->getError()
+					])
 				);
 			}
 
 			// parameters
 			if ($fullItem['type'] == ITEM_TYPE_AGGREGATE) {
-				$params = $itemKey->getParameters();
+				$params_num = $item_key_parser->getParamsNum();
 
-				if (!str_in_array($itemKey->getKeyId(), array('grpmax', 'grpmin', 'grpsum', 'grpavg'))
-						|| count($params) != 4
-						|| !str_in_array($params[2], array('last', 'min', 'max', 'avg', 'sum', 'count'))) {
+				if (!str_in_array($item_key_parser->getKey(), ['grpmax', 'grpmin', 'grpsum', 'grpavg'])
+						|| $params_num > 4 || $params_num < 3
+						|| ($params_num == 3 && $item_key_parser->getParam(2) !== 'last')
+						|| !str_in_array($item_key_parser->getParam(2), ['last', 'min', 'max', 'avg', 'sum', 'count'])) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
 						_s('Key "%1$s" does not match <grpmax|grpmin|grpsum|grpavg>["Host group(s)", "Item key",'.
-							' "<last|min|max|avg|sum|count>", "parameter"].', $itemKey->getKeyId()));
+							' "<last|min|max|avg|sum|count>", "parameter"].', $item_key_parser->getKey()));
 				}
 			}
 
 			// type of information
-			if ($fullItem['type'] == ITEM_TYPE_AGGREGATE && $fullItem['value_type'] != ITEM_VALUE_TYPE_FLOAT
-					&& $fullItem['value_type'] != ITEM_VALUE_TYPE_UINT64) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Type of information must be "Numeric (float)" for aggregate items.'));
-			}
-
-			// log
-			if ($fullItem['value_type'] != ITEM_VALUE_TYPE_LOG && str_in_array($itemKey->getKeyId(), array('log', 'logrt', 'eventlog'))) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Type of information must be "Log" for log key.'));
+			if ($fullItem['type'] == ITEM_TYPE_AGGREGATE && $fullItem['value_type'] != ITEM_VALUE_TYPE_UINT64
+					&& $fullItem['value_type'] != ITEM_VALUE_TYPE_FLOAT) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_('Type of information must be "Numeric (unsigned)" or "Numeric (float)" for aggregate items.'));
 			}
 
 			// update interval
 			if ($fullItem['type'] != ITEM_TYPE_TRAPPER && $fullItem['type'] != ITEM_TYPE_SNMPTRAP) {
-				if ($fullItem['delay'] < 0 || $fullItem['delay'] > SEC_PER_DAY) {
+				// delay must be between 0 and 86400, if delay is 0, delay_flex interval must be set.
+				if ($fullItem['delay'] < 0 || $fullItem['delay'] > SEC_PER_DAY
+					|| ($fullItem['delay'] == 0 && $fullItem['delay_flex'] === '')) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
 						_('Item will not be refreshed. Please enter a correct update interval.')
 					);
 				}
 
-				$nextCheck = calculateItemNextCheck(0, $fullItem['delay'], $fullItem['delay_flex'], time());
+				// Don't parse empty strings, they will not be valid.
+				if ($fullItem['delay_flex'] === '') {
+					continue;
+				}
 
-				if ($nextCheck == ZBX_JAN_2038) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_('Item will not be refreshed. Please enter a correct update interval.')
+				// Validate item delay_flex string. First check syntax with parser, then validate time ranges.
+				$item_delay_flex_parser = new CItemDelayFlexParser($fullItem['delay_flex']);
+
+				if ($item_delay_flex_parser->isValid()) {
+					$delay_flex_validator = new CItemDelayFlexValidator();
+
+					if ($delay_flex_validator->validate($item_delay_flex_parser->getIntervals())) {
+						// Some valid intervals exist at this point.
+						$flexible_intervals = $item_delay_flex_parser->getFlexibleIntervals();
+
+						// If there are no flexible intervals, skip the next check calculation.
+						if (!$flexible_intervals) {
+							continue;
+						}
+
+						$nextCheck = calculateItemNextCheck(0, $fullItem['delay'],
+							$item_delay_flex_parser->getFlexibleIntervals($flexible_intervals),
+							time()
+						);
+
+						if ($nextCheck == ZBX_JAN_2038) {
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_('Item will not be refreshed. Please enter a correct update interval.')
+							);
+						}
+					}
+					else {
+						self::exception(ZBX_API_ERROR_PARAMETERS, $delay_flex_validator->getError());
+					}
+				}
+				else {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid interval "%1$s": %2$s.',
+						$fullItem['delay_flex'],
+						$item_delay_flex_parser->getError())
 					);
 				}
 			}
@@ -360,19 +417,18 @@ abstract class CItemGeneral extends CApiService {
 
 			// snmp trap
 			if ($fullItem['type'] == ITEM_TYPE_SNMPTRAP
-					&& strcmp($fullItem['key_'], 'snmptrap.fallback') != 0
-					&& strcmp($itemKey->getKeyId(), 'snmptrap') != 0) {
+					&& $fullItem['key_'] !== 'snmptrap.fallback' && $item_key_parser->getKey() !== 'snmptrap') {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('SNMP trap key is invalid.'));
 			}
 
 			// snmp oid
-			if ((in_array($fullItem['type'], array(ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C, ITEM_TYPE_SNMPV3)))
+			if ((in_array($fullItem['type'], [ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C, ITEM_TYPE_SNMPV3]))
 					&& zbx_empty($fullItem['snmp_oid'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No SNMP OID specified.'));
 			}
 
 			// snmp community
-			if (in_array($fullItem['type'], array(ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C))
+			if (in_array($fullItem['type'], [ITEM_TYPE_SNMPV1, ITEM_TYPE_SNMPV2C])
 					&& zbx_empty($fullItem['snmp_community'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No SNMP community specified.'));
 			}
@@ -385,10 +441,10 @@ abstract class CItemGeneral extends CApiService {
 
 			if (isset($fullItem['snmpv3_securitylevel']) && $fullItem['snmpv3_securitylevel'] != ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV) {
 				// snmpv3 authprotocol
-				if (str_in_array($fullItem['snmpv3_securitylevel'], array(ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV, ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV))) {
+				if (str_in_array($fullItem['snmpv3_securitylevel'], [ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV, ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV])) {
 					if (isset($fullItem['snmpv3_authprotocol']) && (zbx_empty($fullItem['snmpv3_authprotocol'])
 							|| !str_in_array($fullItem['snmpv3_authprotocol'],
-								array(ITEM_AUTHPROTOCOL_MD5, ITEM_AUTHPROTOCOL_SHA)))) {
+								[ITEM_AUTHPROTOCOL_MD5, ITEM_AUTHPROTOCOL_SHA]))) {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect authentication protocol for item "%1$s".', $fullItem['name']));
 					}
 				}
@@ -397,16 +453,28 @@ abstract class CItemGeneral extends CApiService {
 				if ($fullItem['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
 					if (isset($fullItem['snmpv3_privprotocol']) && (zbx_empty($fullItem['snmpv3_privprotocol'])
 							|| !str_in_array($fullItem['snmpv3_privprotocol'],
-								array(ITEM_PRIVPROTOCOL_DES, ITEM_PRIVPROTOCOL_AES)))) {
+								[ITEM_PRIVPROTOCOL_DES, ITEM_PRIVPROTOCOL_AES]))) {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect privacy protocol for item "%1$s".', $fullItem['name']));
 					}
 				}
 			}
 
-			// check that the given applications belong to the item's host
 			if (isset($item['applications']) && $item['applications']) {
+				/*
+				 * 'flags' is available for update and item prototypes.
+				 * Don't allow discovered or any other application types for item prototypes in 'applications' option.
+				 */
+				if (array_key_exists('flags', $fullItem) && $fullItem['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+					foreach ($host['applications'] as $num => $application) {
+						if ($application['flags'] != ZBX_FLAG_DISCOVERY_NORMAL) {
+							unset($host['applications'][$num]);
+						}
+					}
+				}
+
+				// check that the given applications belong to the item's host
 				$dbApplicationIds = zbx_objectValues($host['applications'], 'applicationid');
-				foreach($item['applications'] as $appId) {
+				foreach ($item['applications'] as $appId) {
 					if (!in_array($appId, $dbApplicationIds)) {
 						$error = _s('Application with ID "%1$s" is not available on "%2$s".', $appId, $host['name']);
 						self::exception(ZBX_API_ERROR_PARAMETERS, $error);
@@ -414,8 +482,6 @@ abstract class CItemGeneral extends CApiService {
 				}
 			}
 
-			$this->checkDelayFlex($fullItem);
-
 			$this->checkSpecificFields($fullItem);
 		}
 		unset($item);
@@ -475,7 +541,7 @@ abstract class CItemGeneral extends CApiService {
 	 *							false, if the item does not need an interface
 	 */
 	public static function findInterfaceForItem(array $item, array $interfaces) {
-		$typeInterface = array();
+		$typeInterface = [];
 		foreach ($interfaces as $interface) {
 			if ($interface['main'] == 1) {
 				$typeInterface[$interface['type']] = $interface;
@@ -485,16 +551,16 @@ abstract class CItemGeneral extends CApiService {
 		// find item interface type
 		$type = itemTypeInterface($item['type']);
 
-		$matchingInterface = array();
+		$matchingInterface = [];
 
 		// the item can use any interface
 		if ($type == INTERFACE_TYPE_ANY) {
-			$interfaceTypes = array(
+			$interfaceTypes = [
 				INTERFACE_TYPE_AGENT,
 				INTERFACE_TYPE_SNMP,
 				INTERFACE_TYPE_JMX,
 				INTERFACE_TYPE_IPMI
-			);
+			];
 			foreach ($interfaceTypes as $itype) {
 				if (isset($typeInterface[$itype])) {
 					$matchingInterface = $typeInterface[$itype];
@@ -504,7 +570,7 @@ abstract class CItemGeneral extends CApiService {
 		}
 		// the item uses a specific type of interface
 		elseif ($type !== false) {
-			$matchingInterface = (isset($typeInterface[$type])) ? $typeInterface[$type] : array();
+			$matchingInterface = (isset($typeInterface[$type])) ? $typeInterface[$type] : [];
 		}
 		// the item does not need an interface
 		else {
@@ -524,10 +590,10 @@ abstract class CItemGeneral extends CApiService {
 
 		$ids = array_unique($ids);
 
-		$count = $this->get(array(
+		$count = $this->get([
 			'itemids' => $ids,
 			'countOutput' => true
-		));
+		]);
 
 		return (count($ids) == $count);
 	}
@@ -542,11 +608,11 @@ abstract class CItemGeneral extends CApiService {
 
 		$ids = array_unique($ids);
 
-		$count = $this->get(array(
+		$count = $this->get([
 			'itemids' => $ids,
 			'editable' => true,
 			'countOutput' => true
-		));
+		]);
 
 		return (count($ids) == $count);
 	}
@@ -581,30 +647,30 @@ abstract class CItemGeneral extends CApiService {
 	 */
 	protected function checkUseInGraphAxis(array $itemids, $checkMax = false) {
 		if ($checkMax) {
-			$filter = array('ymax_itemid' => $itemids);
+			$filter = ['ymax_itemid' => $itemids];
 			$itemIdColumn = 'ymax_itemid';
 			$typeColumn = 'ymax_type';
 		}
 		else {
-			$filter = array('ymin_itemid' => $itemids);
+			$filter = ['ymin_itemid' => $itemids];
 			$itemIdColumn = 'ymin_itemid';
 			$typeColumn = 'ymin_type';
 		}
 
 		// make it work for both graphs and graph prototypes
-		$filter['flags'] = array(
+		$filter['flags'] = [
 			ZBX_FLAG_DISCOVERY_PROTOTYPE,
 			ZBX_FLAG_DISCOVERY_NORMAL,
 			ZBX_FLAG_DISCOVERY_CREATED
-		);
+		];
 
 		// check if the items are used in Y axis min/max values in any graphs
-		$graphs = API::Graph()->get(array(
-			'output' => array($itemIdColumn, $typeColumn, 'graphtype'),
+		$graphs = API::Graph()->get([
+			'output' => [$itemIdColumn, $typeColumn, 'graphtype'],
 			'filter' => $filter
-		));
+		]);
 
-		$updateGraphs = array();
+		$updateGraphs = [];
 		foreach ($graphs as &$graph) {
 			// check if Y type is actually set to GRAPH_YAXIS_TYPE_ITEM_VALUE
 			if ($graph[$typeColumn] == GRAPH_YAXIS_TYPE_ITEM_VALUE) {
@@ -620,6 +686,7 @@ abstract class CItemGeneral extends CApiService {
 				$updateGraphs[] = $graph;
 			}
 		}
+		unset($graph);
 
 		// if there are graphs, that have an y*_itemid column set, but the
 		// y*_type column is not set to GRAPH_YAXIS_TYPE_ITEM_VALUE, set y*_itemid to NULL.
@@ -652,26 +719,26 @@ abstract class CItemGeneral extends CApiService {
 	 */
 	protected function prepareInheritedItems(array $itemsToInherit, array $hostIds = null) {
 		// fetch all child hosts
-		$chdHosts = API::Host()->get(array(
-			'output' => array('hostid', 'host', 'status'),
-			'selectParentTemplates' => array('templateid'),
+		$chdHosts = API::Host()->get([
+			'output' => ['hostid', 'host', 'status'],
+			'selectParentTemplates' => ['templateid'],
 			'selectInterfaces' => API_OUTPUT_EXTEND,
 			'templateids' => zbx_objectValues($itemsToInherit, 'hostid'),
 			'hostids' => $hostIds,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'templated_hosts' => true
-		));
+		]);
 		if (empty($chdHosts)) {
-			return array();
+			return [];
 		}
 
-		$newItems = array();
+		$newItems = [];
 		foreach ($chdHosts as $hostId => $host) {
 			$templateids = zbx_toHash($host['parentTemplates'], 'templateid');
 
 			// skip items not from parent templates of current host
-			$parentItems = array();
+			$parentItems = [];
 			foreach ($itemsToInherit as $inum => $parentItem) {
 				if (isset($templateids[$parentItem['hostid']])) {
 					$parentItems[$inum] = $parentItem;
@@ -679,17 +746,35 @@ abstract class CItemGeneral extends CApiService {
 			}
 
 			// check existing items to decide insert or update
-			$exItems = API::Item()->get(array(
-				'output' => array('itemid', 'type', 'key_', 'flags', 'templateid'),
+			$exItems = API::Item()->get([
+				'output' => ['itemid', 'type', 'key_', 'flags', 'templateid'],
 				'hostids' => $hostId,
 				'preservekeys' => true,
 				'nopermissions' => true,
-				'filter' => array('flags' => null)
-			));
+				'filter' => ['flags' => null]
+			]);
 
 			$exItemsKeys = zbx_toHash($exItems, 'key_');
 			$exItemsTpl = zbx_toHash($exItems, 'templateid');
 
+			$itemids_with_application_prototypes = [];
+
+			foreach ($parentItems as $parentItem) {
+				if (isset($parentItem['applicationPrototypes']) && is_array($parentItem['applicationPrototypes'])
+						&& !array_key_exists('ruleid', $parentItem)) {
+					$itemids_with_application_prototypes[$parentItem['itemid']] = true;
+				}
+			}
+
+			if ($itemids_with_application_prototypes) {
+				$discovery_rules = DBfetchArray(DBselect(
+					'SELECT id.itemid,id.parent_itemid'.
+					' FROM item_discovery id'.
+					' WHERE '.dbConditionInt('id.itemid', array_keys($itemids_with_application_prototypes))
+				));
+				$discovery_rules = zbx_toHash($discovery_rules, 'itemid');
+			}
+
 			foreach ($parentItems as $parentItem) {
 				$exItem = null;
 
@@ -709,7 +794,7 @@ abstract class CItemGeneral extends CApiService {
 						&& !idcmp($exItemsKeys[$parentItem['key_']]['templateid'], $parentItem['itemid'])) {
 						self::exception(
 							ZBX_API_ERROR_PARAMETERS,
-							_params($this->getErrorMsg(self::ERROR_EXISTS), array($parentItem['key_'], $host['host']))
+							_params($this->getErrorMsg(self::ERROR_EXISTS), [$parentItem['key_'], $host['host']])
 						);
 					}
 				}
@@ -722,10 +807,10 @@ abstract class CItemGeneral extends CApiService {
 
 						self::exception(
 							ZBX_API_ERROR_PARAMETERS,
-							_params($this->getErrorMsg(self::ERROR_EXISTS_TEMPLATE), array(
+							_params($this->getErrorMsg(self::ERROR_EXISTS_TEMPLATE), [
 								$parentItem['key_'],
 								$host['host']
-							))
+							])
 						);
 					}
 				}
@@ -742,10 +827,10 @@ abstract class CItemGeneral extends CApiService {
 					elseif ($interface !== false) {
 						self::exception(
 							ZBX_API_ERROR_PARAMETERS,
-							_params($this->getErrorMsg(self::ERROR_NO_INTERFACE), array(
+							_params($this->getErrorMsg(self::ERROR_NO_INTERFACE), [
 								$host['host'],
 								$parentItem['key_']
-							))
+							])
 						);
 					}
 				}
@@ -763,6 +848,40 @@ abstract class CItemGeneral extends CApiService {
 					$newItem['applications'] = get_same_applications_for_host($parentItem['applications'], $host['hostid']);
 				}
 
+				if ($parentItem['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE
+						&& array_key_exists('applicationPrototypes', $parentItem)) {
+
+					// Get discovery rule ID for current item prototype, if it is not yet set.
+					if (array_key_exists('ruleid', $parentItem)) {
+						$discovery_ruleid = $parentItem['ruleid'];
+					}
+					else {
+						$discovery_ruleid = $discovery_rules[$parentItem['itemid']]['parent_itemid'];
+					}
+
+					$newItem['applicationPrototypes'] = [];
+
+					$db_application_prototypes = DBfetchArray(DBselect(
+						'SELECT ap.application_prototypeid,ap.name'.
+						' FROM application_prototype ap'.
+						' WHERE ap.itemid='.zbx_dbstr($discovery_ruleid).
+							' AND '.dbConditionString('ap.name',
+								zbx_objectValues($parentItem['applicationPrototypes'], 'name')
+							)
+					));
+
+					$db_application_prototypes = zbx_toHash($db_application_prototypes, 'name');
+
+					foreach ($parentItem['applicationPrototypes'] as $application_prototype) {
+						$db_application_prototype = $db_application_prototypes[$application_prototype['name']];
+
+						$newItem['applicationPrototypes'][] = [
+							'name' => $application_prototype['name'],
+							'templateid' => $db_application_prototype['application_prototypeid']
+						];
+					}
+				}
+
 				if ($exItem) {
 					$newItem['itemid'] = $exItem['itemid'];
 				}
@@ -785,11 +904,11 @@ abstract class CItemGeneral extends CApiService {
 	 * @param array $items
 	 */
 	protected function checkExistingItems(array $items) {
-		$itemKeysByHostId = array();
-		$itemIds = array();
+		$itemKeysByHostId = [];
+		$itemIds = [];
 		foreach ($items as $item) {
 			if (!isset($itemKeysByHostId[$item['hostid']])) {
-				$itemKeysByHostId[$item['hostid']] = array();
+				$itemKeysByHostId[$item['hostid']] = [];
 			}
 			$itemKeysByHostId[$item['hostid']][] = $item['key_'];
 
@@ -798,7 +917,7 @@ abstract class CItemGeneral extends CApiService {
 			}
 		}
 
-		$sqlWhere = array();
+		$sqlWhere = [];
 		foreach ($itemKeysByHostId as $hostId => $keys) {
 			$sqlWhere[] = '(i.hostid='.zbx_dbstr($hostId).' AND '.dbConditionString('i.key_', $keys).')';
 		}
@@ -820,58 +939,19 @@ abstract class CItemGeneral extends CApiService {
 		}
 	}
 
-	/**
-	 * Validate flexible intervals.
-	 * Flexible intervals is string with format:
-	 *   'delay/day1-day2,time1-time2;interval2;interval3;...' (day2 is optional)
-	 * Examples:
-	 *   600/5-7,00:00-09:00;600/1-2,00:00-09:00
-	 *   600/5,0:0-9:0;600/1-2,0:0-9:0
-	 *
-	 * @param array $item
-	 *
-	 * @return bool
-	 */
-	protected function checkDelayFlex(array $item) {
-		if (array_key_exists('delay_flex', $item)) {
-			$delayFlex = $item['delay_flex'];
-
-			if (!is_string($delayFlex)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Incorrect flexible interval in item "%1$s". Flexible interval must be a string.', $item['name']));
-			}
-
-			if ($delayFlex === '') {
-				return true;
-			}
-
-			$validator = new CTimePeriodValidator();
-			$intervals = explode(';', rtrim($delayFlex, ';'));
-			foreach ($intervals as $interval) {
-				if (!preg_match('#^\d+/(.+)$#', $interval, $matches)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect flexible interval "%1$s".', $interval));
-				}
-
-				if (!$validator->validate($matches[1])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, $validator->getError());
-				}
-			}
-		}
-	}
-
 	protected function addRelatedObjects(array $options, array $result) {
 		$result = parent::addRelatedObjects($options, $result);
 
 		// adding hosts
 		if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'itemid', 'hostid');
-			$hosts = API::Host()->get(array(
+			$hosts = API::Host()->get([
 				'hostids' => $relationMap->getRelatedIds(),
 				'templated_hosts' => true,
 				'output' => $options['selectHosts'],
 				'nopermissions' => true,
 				'preservekeys' => true
-			));
+			]);
 			$result = $relationMap->mapMany($result, $hosts, 'hosts');
 		}
 
