@@ -28,7 +28,7 @@ class CMaintenance extends CApiService {
 
 	protected $tableName = 'maintenances';
 	protected $tableAlias = 'm';
-	protected $sortColumns = array('maintenanceid', 'name', 'maintenance_type', 'active_till', 'active_since');
+	protected $sortColumns = ['maintenanceid', 'name', 'maintenance_type', 'active_till', 'active_since'];
 
 	/**
 	 * Get maintenances data.
@@ -48,21 +48,21 @@ class CMaintenance extends CApiService {
 	 *
 	 * @return array
 	 */
-	public function get(array $options = array()) {
-		$result = array();
+	public function get(array $options = []) {
+		$result = [];
 		$userType = self::$userData['type'];
 		$userid = self::$userData['userid'];
 
-		$sqlParts = array(
-			'select'	=> array('maintenance' => 'm.maintenanceid'),
-			'from'		=> array('maintenances' => 'maintenances m'),
-			'where'		=> array(),
-			'group'		=> array(),
-			'order'		=> array(),
+		$sqlParts = [
+			'select'	=> ['maintenance' => 'm.maintenanceid'],
+			'from'		=> ['maintenances' => 'maintenances m'],
+			'where'		=> [],
+			'group'		=> [],
+			'order'		=> [],
 			'limit'		=> null
-		);
+		];
 
-		$defOptions = array(
+		$defOptions = [
 			'groupids'					=> null,
 			'hostids'					=> null,
 			'maintenanceids'			=> null,
@@ -87,11 +87,11 @@ class CMaintenance extends CApiService {
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null
-		);
+		];
 		$options = zbx_array_merge($defOptions, $options);
 
 		// editable + PERMISSION CHECK
-		$maintenanceids = array();
+		$maintenanceids = [];
 		if ($userType == USER_TYPE_SUPER_ADMIN || $options['nopermissions']) {
 			if (!is_null($options['groupids']) || !is_null($options['hostids'])) {
 				if (!is_null($options['groupids'])) {
@@ -253,30 +253,6 @@ class CMaintenance extends CApiService {
 		return $result;
 	}
 
-
-	/**
-	 * Check if maintenance exists.
-	 *
-	 * @deprecated	As of version 2.4, use get method instead.
-	 *
-	 * @param array	$object
-	 *
-	 * @return bool
-	 */
-	public function exists(array $object) {
-		$this->deprecated('maintenance.exists method is deprecated.');
-
-		$keyFields = array(array('maintenanceid', 'name'));
-
-		$maintenance = $this->get(array(
-			'output' => array('maintenanceid'),
-			'filter' => zbx_array_mintersect($keyFields, $object),
-			'limit' => 1
-		));
-
-		return (bool) $maintenance;
-	}
-
 	/**
 	 * Add maintenances.
 	 *
@@ -290,8 +266,8 @@ class CMaintenance extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
-		$hostids = array();
-		$groupids = array();
+		$hostids = [];
+		$groupids = [];
 		foreach ($maintenances as $maintenance) {
 			$hostids = array_merge($hostids, $maintenance['hostids']);
 			$groupids = array_merge($groupids, $maintenance['groupids']);
@@ -303,12 +279,12 @@ class CMaintenance extends CApiService {
 		}
 
 		// hosts permissions
-		$options = array(
+		$options = [
 			'hostids' => $hostids,
 			'editable' => true,
-			'output' => array('hostid'),
+			'output' => ['hostid'],
 			'preservekeys' => true
-		);
+		];
 		$updHosts = API::Host()->get($options);
 		foreach ($hostids as $hostid) {
 			if (!isset($updHosts[$hostid])) {
@@ -316,12 +292,12 @@ class CMaintenance extends CApiService {
 			}
 		}
 		// groups permissions
-		$options = array(
+		$options = [
 			'groupids' => $groupids,
 			'editable' => true,
-			'output' => array('groupid'),
+			'output' => ['groupid'],
 			'preservekeys' => true
-		);
+		];
 		$updGroups = API::HostGroup()->get($options);
 		foreach ($groupids as $groupid) {
 			if (!isset($updGroups[$groupid])) {
@@ -332,38 +308,38 @@ class CMaintenance extends CApiService {
 		$this->removeSecondsFromTimes($maintenances);
 
 		$tid = 0;
-		$insert = array();
-		$timeperiods = array();
-		$insertTimeperiods = array();
+		$insert = [];
+		$timeperiods = [];
+		$insertTimeperiods = [];
 		$now = time();
 		$now -= $now % SEC_PER_MIN;
 
 		// check fields
 		foreach ($maintenances as $maintenance) {
-			$dbFields = array(
+			$dbFields = [
 				'name' => null,
 				'active_since' => $now,
 				'active_till' => $now + SEC_PER_DAY
-			);
+			];
 
 			if (!check_db_fields($dbFields, $maintenance)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect parameters for maintenance.'));
 			}
 		}
 
-		$collectionValidator = new CCollectionValidator(array(
+		$collectionValidator = new CCollectionValidator([
 			'uniqueField' => 'name',
 			'messageDuplicate' => _('Maintenance "%1$s" already exists.')
-		));
+		]);
 		$this->checkValidator($maintenances, $collectionValidator);
 
 		// validate if maintenance name already exists
-		$dbMaintenances = $this->get(array(
-			'output' => array('name'),
-			'filter' => array('name' => zbx_objectValues($maintenances, 'name')),
+		$dbMaintenances = $this->get([
+			'output' => ['name'],
+			'filter' => ['name' => zbx_objectValues($maintenances, 'name')],
 			'nopermissions' => true,
 			'limit' => 1
-		));
+		]);
 
 		if ($dbMaintenances) {
 			$dbMaintenance = reset($dbMaintenances);
@@ -398,11 +374,11 @@ class CMaintenance extends CApiService {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one maintenance period must be created.'));
 				}
 
-				$dbFields = array(
+				$dbFields = [
 					'timeperiod_type' => TIMEPERIOD_TYPE_ONETIME,
 					'period' => SEC_PER_HOUR,
 					'start_date' =>	$now
-				);
+				];
 				check_db_fields($dbFields, $timeperiod);
 
 				$tid++;
@@ -413,35 +389,35 @@ class CMaintenance extends CApiService {
 		$maintenanceids = DB::insert('maintenances', $insert);
 		$timeperiodids = DB::insert('timeperiods', $insertTimeperiods);
 
-		$insertWindows = array();
+		$insertWindows = [];
 		foreach ($timeperiods as $tid => $mnum) {
-			$insertWindows[] = array(
+			$insertWindows[] = [
 				'timeperiodid' => $timeperiodids[$tid],
 				'maintenanceid' => $maintenanceids[$mnum]
-			);
+			];
 		}
 		DB::insert('maintenances_windows', $insertWindows);
 
-		$insertHosts = array();
-		$insertGroups = array();
+		$insertHosts = [];
+		$insertGroups = [];
 		foreach ($maintenances as $mnum => $maintenance) {
 			foreach ($maintenance['hostids'] as $hostid) {
-				$insertHosts[] = array(
+				$insertHosts[] = [
 					'hostid' => $hostid,
 					'maintenanceid' => $maintenanceids[$mnum]
-				);
+				];
 			}
 			foreach ($maintenance['groupids'] as $groupid) {
-				$insertGroups[] = array(
+				$insertGroups[] = [
 					'groupid' => $groupid,
 					'maintenanceid' => $maintenanceids[$mnum]
-				);
+				];
 			}
 		}
 		DB::insert('maintenances_hosts', $insertHosts);
 		DB::insert('maintenances_groups', $insertGroups);
 
-		return array('maintenanceids' => $maintenanceids);
+		return ['maintenanceids' => $maintenanceids];
 	}
 
 	/**
@@ -460,17 +436,17 @@ class CMaintenance extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
-		$updMaintenances = $this->get(array(
+		$updMaintenances = $this->get([
 			'maintenanceids' => zbx_objectValues($maintenances, 'maintenanceid'),
 			'editable' => true,
 			'output' => API_OUTPUT_EXTEND,
-			'selectGroups' => array('groupid'),
-			'selectHosts' => array('hostid'),
+			'selectGroups' => ['groupid'],
+			'selectHosts' => ['hostid'],
 			'selectTimeperiods' => API_OUTPUT_EXTEND,
 			'preservekeys' => true
-		));
+		]);
 
-		$maintenanceNamesChanged = array();
+		$maintenanceNamesChanged = [];
 		foreach ($maintenances as $maintenance) {
 			if (!isset($updMaintenances[$maintenance['maintenanceid']])) {
 				self::exception(ZBX_API_ERROR_PERMISSIONS, _(
@@ -493,12 +469,12 @@ class CMaintenance extends CApiService {
 
 		// check if maintenance already exists
 		if ($maintenanceNamesChanged) {
-			$dbMaintenances = $this->get(array(
-				'output' => array('name'),
-				'filter' => array('name' => $maintenanceNamesChanged),
+			$dbMaintenances = $this->get([
+				'output' => ['name'],
+				'filter' => ['name' => $maintenanceNamesChanged],
 				'nopermissions' => true,
 				'limit' => 1
-			));
+			]);
 
 			if ($dbMaintenances) {
 				$dbMaintenance = reset($dbMaintenances);
@@ -508,8 +484,8 @@ class CMaintenance extends CApiService {
 			}
 		}
 
-		$hostids = array();
-		$groupids = array();
+		$hostids = [];
+		$groupids = [];
 
 		foreach ($maintenances as $maintenance) {
 			// validate maintenance active since
@@ -548,12 +524,12 @@ class CMaintenance extends CApiService {
 		}
 
 		// validate hosts permissions
-		$options = array(
+		$options = [
 			'hostids' => $hostids,
 			'editable' => true,
-			'output' => array('hostid'),
+			'output' => ['hostid'],
 			'preservekeys' => true
-		);
+		];
 		$updHosts = API::Host()->get($options);
 		foreach ($hostids as $hostid) {
 			if (!isset($updHosts[$hostid])) {
@@ -561,12 +537,12 @@ class CMaintenance extends CApiService {
 			}
 		}
 		// validate groups permissions
-		$options = array(
+		$options = [
 			'groupids' => $groupids,
 			'editable' => true,
-			'output' => array('groupid'),
+			'output' => ['groupid'],
 			'preservekeys' => true
-		);
+		];
 		$updGroups = API::HostGroup()->get($options);
 		foreach ($groupids as $groupid) {
 			if (!isset($updGroups[$groupid])) {
@@ -576,21 +552,21 @@ class CMaintenance extends CApiService {
 
 		$this->removeSecondsFromTimes($maintenances);
 
-		$update = array();
+		$update = [];
 		foreach ($maintenances as $mnum => $maintenance) {
-			$dbFields = array(
+			$dbFields = [
 				'maintenanceid' => null
-			);
+			];
 
 			// validate fields
 			if (!check_db_fields($dbFields, $maintenance)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect parameters for maintenance.'));
 			}
 
-			$update[$mnum] = array(
+			$update[$mnum] = [
 				'values' => $maintenance,
-				'where' => array('maintenanceid' => $maintenance['maintenanceid'])
-			);
+				'where' => ['maintenanceid' => $maintenance['maintenanceid']]
+			];
 
 			// update time periods
 			$this->replaceTimePeriods($updMaintenances[$maintenance['maintenanceid']], $maintenance);
@@ -598,8 +574,8 @@ class CMaintenance extends CApiService {
 		DB::update('maintenances', $update);
 
 		// some of the hosts and groups bound to maintenance must be deleted, other inserted and others left alone
-		$insertHosts = array();
-		$insertGroups = array();
+		$insertHosts = [];
+		$insertGroups = [];
 
 		foreach ($maintenances as $maintenance) {
 			// putting apart those host<->maintenance connections that should be inserted, deleted and not changed
@@ -613,16 +589,16 @@ class CMaintenance extends CApiService {
 			);
 
 			foreach ($hostDiff['first'] as $host) {
-				$insertHosts[] = array(
+				$insertHosts[] = [
 					'hostid' => $host['hostid'],
 					'maintenanceid' => $maintenance['maintenanceid']
-				);
+				];
 			}
 			foreach ($hostDiff['second'] as $host) {
-				$deleteHosts = array(
+				$deleteHosts = [
 					'hostid' => $host['hostid'],
 					'maintenanceid' => $maintenance['maintenanceid']
-				);
+				];
 				DB::delete('maintenances_hosts', $deleteHosts);
 			}
 
@@ -634,16 +610,16 @@ class CMaintenance extends CApiService {
 			);
 
 			foreach ($groupDiff['first'] as $group) {
-				$insertGroups[] = array(
+				$insertGroups[] = [
 					'groupid' => $group['groupid'],
 					'maintenanceid' => $maintenance['maintenanceid']
-				);
+				];
 			}
 			foreach ($groupDiff['second'] as $group) {
-				$deleteGroups = array(
+				$deleteGroups = [
 					'groupid' => $group['groupid'],
 					'maintenanceid' => $maintenance['maintenanceid']
-				);
+				];
 				DB::delete('maintenances_groups', $deleteGroups);
 			}
 		}
@@ -651,7 +627,7 @@ class CMaintenance extends CApiService {
 		DB::insert('maintenances_hosts', $insertHosts);
 		DB::insert('maintenances_groups', $insertGroups);
 
-		return array('maintenanceids' => $maintenanceids);
+		return ['maintenanceids' => $maintenanceids];
 	}
 
 	/**
@@ -666,12 +642,12 @@ class CMaintenance extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
 		}
 
-		$options = array(
+		$options = [
 			'maintenanceids' => $maintenanceids,
 			'editable' => true,
-			'output' => array('maintenanceid'),
+			'output' => ['maintenanceid'],
 			'preservekeys' => true
-		);
+		];
 		$maintenances = $this->get($options);
 		foreach ($maintenanceids as $maintenanceid) {
 			if (!isset($maintenances[$maintenanceid])) {
@@ -679,7 +655,7 @@ class CMaintenance extends CApiService {
 			}
 		}
 
-		$timeperiodids = array();
+		$timeperiodids = [];
 		$dbTimeperiods = DBselect(
 			'SELECT DISTINCT tp.timeperiodid'.
 			' FROM timeperiods tp,maintenances_windows mw'.
@@ -690,29 +666,29 @@ class CMaintenance extends CApiService {
 			$timeperiodids[] = $timeperiod['timeperiodid'];
 		}
 
-		$midCond = array('maintenanceid' => $maintenanceids);
+		$midCond = ['maintenanceid' => $maintenanceids];
 
 		// remove maintenanceid from hosts table
-		$options = array(
+		$options = [
 			'real_hosts' => true,
-			'output' => array('hostid'),
-			'filter' => array('maintenanceid' => $maintenanceids)
-		);
+			'output' => ['hostid'],
+			'filter' => ['maintenanceid' => $maintenanceids]
+		];
 		$hosts = API::Host()->get($options);
 		if (!empty($hosts)) {
-			DB::update('hosts', array(
-				'values' => array('maintenanceid' => 0),
-				'where' => array('hostid' => zbx_objectValues($hosts, 'hostid'))
-			));
+			DB::update('hosts', [
+				'values' => ['maintenanceid' => 0],
+				'where' => ['hostid' => zbx_objectValues($hosts, 'hostid')]
+			]);
 		}
 
-		DB::delete('timeperiods', array('timeperiodid' => $timeperiodids));
+		DB::delete('timeperiods', ['timeperiodid' => $timeperiodids]);
 		DB::delete('maintenances_windows', $midCond);
 		DB::delete('maintenances_hosts', $midCond);
 		DB::delete('maintenances_groups', $midCond);
 		DB::delete('maintenances', $midCond);
 
-		return array('maintenanceids' => $maintenanceids);
+		return ['maintenanceids' => $maintenanceids];
 	}
 
 	/**
@@ -755,13 +731,13 @@ class CMaintenance extends CApiService {
 
 		// link new time periods to maintenance
 		$oldTimePeriods = zbx_toHash($oldMaintenance['timeperiods'], 'timeperiodid');
-		$newMaintenanceWindows = array();
+		$newMaintenanceWindows = [];
 		foreach ($timePeriods as $tp) {
 			if (!isset($oldTimePeriods[$tp['timeperiodid']])) {
-				$newMaintenanceWindows[] = array(
+				$newMaintenanceWindows[] = [
 					'maintenanceid' => $maintenance['maintenanceid'],
 					'timeperiodid' => $tp['timeperiodid']
-				);
+				];
 			}
 		}
 		DB::insert('maintenances_windows', $newMaintenanceWindows);
@@ -773,33 +749,33 @@ class CMaintenance extends CApiService {
 		// selectGroups
 		if ($options['selectGroups'] !== null && $options['selectGroups'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'maintenanceid', 'groupid', 'maintenances_groups');
-			$groups = API::HostGroup()->get(array(
+			$groups = API::HostGroup()->get([
 				'output' => $options['selectGroups'],
 				'hostgroupids' => $relationMap->getRelatedIds(),
 				'preservekeys' => true
-			));
+			]);
 			$result = $relationMap->mapMany($result, $groups, 'groups');
 		}
 
 		// selectHosts
 		if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'maintenanceid', 'hostid', 'maintenances_hosts');
-			$groups = API::Host()->get(array(
+			$groups = API::Host()->get([
 				'output' => $options['selectHosts'],
 				'hostids' => $relationMap->getRelatedIds(),
 				'preservekeys' => true
-			));
+			]);
 			$result = $relationMap->mapMany($result, $groups, 'hosts');
 		}
 
 		// selectTimeperiods
 		if ($options['selectTimeperiods'] !== null && $options['selectTimeperiods'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'maintenanceid', 'timeperiodid', 'maintenances_windows');
-			$timeperiods = API::getApiService()->select('timeperiods', array(
+			$timeperiods = API::getApiService()->select('timeperiods', [
 				'output' => $options['selectTimeperiods'],
-				'filter' => array('timeperiodid' => $relationMap->getRelatedIds()),
+				'filter' => ['timeperiodid' => $relationMap->getRelatedIds()],
 				'preservekeys' => true
-			));
+			]);
 			$result = $relationMap->mapMany($result, $timeperiods, 'timeperiods');
 		}
 
