@@ -28,7 +28,7 @@ class CProxy extends CApiService {
 
 	protected $tableName = 'hosts';
 	protected $tableAlias = 'h';
-	protected $sortColumns = array('hostid', 'host', 'status');
+	protected $sortColumns = ['hostid', 'host', 'status'];
 
 	/**
 	 * Get proxy data.
@@ -44,20 +44,20 @@ class CProxy extends CApiService {
 	 *
 	 * @return array
 	 */
-	public function get($options = array()) {
-		$result = array();
+	public function get($options = []) {
+		$result = [];
 
 		$userType = self::$userData['type'];
 
-		$sqlParts = array(
-			'select'	=> array('hostid' => 'h.hostid'),
-			'from'		=> array('hosts' => 'hosts h'),
-			'where'		=> array('h.status IN ('.HOST_STATUS_PROXY_ACTIVE.','.HOST_STATUS_PROXY_PASSIVE.')'),
-			'order'		=> array(),
+		$sqlParts = [
+			'select'	=> ['hostid' => 'h.hostid'],
+			'from'		=> ['hosts' => 'hosts h'],
+			'where'		=> ['h.status IN ('.HOST_STATUS_PROXY_ACTIVE.','.HOST_STATUS_PROXY_PASSIVE.')'],
+			'order'		=> [],
 			'limit'		=> null
-		);
+		];
 
-		$defOptions = array(
+		$defOptions = [
 			'proxyids'					=> null,
 			'editable'					=> null,
 			'nopermissions'				=> null,
@@ -77,14 +77,14 @@ class CProxy extends CApiService {
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null
-		);
+		];
 		$options = zbx_array_merge($defOptions, $options);
 
 		// editable + PERMISSION CHECK
 		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
 			if ($permission == PERM_READ_WRITE) {
-				return array();
+				return [];
 			}
 		}
 
@@ -115,7 +115,7 @@ class CProxy extends CApiService {
 		// countOutput
 		if (!is_null($options['countOutput'])) {
 			$options['sortfield'] = '';
-			$sqlParts['select'] = array('COUNT(DISTINCT h.hostid) AS rowscount');
+			$sqlParts['select'] = ['COUNT(DISTINCT h.hostid) AS rowscount'];
 		}
 
 		// limit
@@ -144,7 +144,7 @@ class CProxy extends CApiService {
 
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
-			$result = $this->unsetExtraFields($result, array('hostid'), $options['output']);
+			$result = $this->unsetExtraFields($result, ['hostid'], $options['output']);
 		}
 
 		// removing keys (hash -> array)
@@ -155,141 +155,6 @@ class CProxy extends CApiService {
 		return $result;
 	}
 
-	protected function checkInput(&$proxies, $method) {
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS,
-				_('No permissions to referred object or it does not exist!'));
-		}
-
-		$create = ($method == 'create');
-		$update = ($method == 'update');
-
-		$proxyIds = zbx_objectValues($proxies, 'proxyid');
-
-		foreach ($proxies as &$proxy) {
-			if (isset($proxy['proxyid'])) {
-				$proxy['hostid'] = $proxy['proxyid'];
-			}
-			elseif (isset($proxy['hostid'])) {
-				$proxy['proxyid'] = $proxy['hostid'];
-			}
-		}
-		unset($proxy);
-
-		// permissions
-		if ($update) {
-			$proxyDBfields = array('proxyid' => null);
-
-			$dbProxies = $this->get(array(
-				'output' => array('proxyid', 'hostid', 'host', 'status'),
-				'proxyids' => $proxyIds,
-				'editable' => true,
-				'preservekeys' => true
-			));
-		}
-		else {
-			$proxyDBfields = array('host' => null);
-		}
-
-		foreach ($proxies as &$proxy) {
-			if (!check_db_fields($proxyDBfields, $proxy)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Wrong fields for proxy "%1$s".', $proxy['host']));
-			}
-
-			if ($update) {
-				if (!isset($dbProxies[$proxy['proxyid']])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_('No permissions to referred object or it does not exist!'));
-				}
-
-				if (isset($proxy['status'])
-						&& ($proxy['status'] != HOST_STATUS_PROXY_ACTIVE
-						&& $proxy['status'] != HOST_STATUS_PROXY_PASSIVE)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value used for proxy status "%1$s".', $proxy['status']));
-				}
-
-				$status = isset($proxy['status']) ? $proxy['status'] : $dbProxies[$proxy['proxyid']]['status'];
-			}
-			else {
-				if (!isset($proxy['status'])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('No status for proxy.'));
-				}
-				elseif ($proxy['status'] != HOST_STATUS_PROXY_ACTIVE && $proxy['status'] != HOST_STATUS_PROXY_PASSIVE) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value used for proxy status "%1$s".', $proxy['status']));
-				}
-
-				$status = $proxy['status'];
-			}
-
-			// host
-			if (isset($proxy['host'])) {
-				if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $proxy['host'])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect characters used for proxy name "%1$s".', $proxy['host']));
-				}
-
-				$proxiesExists = $this->get(array(
-					'output' => array('proxyid'),
-					'filter' => array('host' => $proxy['host'])
-				));
-				foreach ($proxiesExists as $proxyExists) {
-					if ($create || bccomp($proxyExists['proxyid'], $proxy['proxyid']) != 0) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Proxy "%s" already exists.', $proxy['host']));
-					}
-				}
-			}
-
-			// interface
-			if ($status == HOST_STATUS_PROXY_PASSIVE) {
-				if ($create && empty($proxy['interface'])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('No interface provided for proxy "%s".', $proxy['host']));
-				}
-
-				if (isset($proxy['interface'])) {
-					if (!is_array($proxy['interface']) || empty($proxy['interface'])) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_s('No interface provided for proxy "%s".', $proxy['host']));
-					}
-
-					// mark the interface as main to pass host interface validation
-					$proxy['interface']['main'] = INTERFACE_PRIMARY;
-				}
-			}
-
-			// check if hosts exist
-			if (!empty($proxy['hosts'])) {
-				$hostIds = zbx_objectValues($proxy['hosts'], 'hostid');
-
-				$hosts = API::Host()->get(array(
-					'hostids' => $hostIds,
-					'editable' => true,
-					'output' => array('hostid', 'proxy_hostid', 'name'),
-					'preservekeys' => true
-				));
-
-				if (empty($hosts)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_('No permissions to referred object or it does not exist!'));
-				}
-			}
-		}
-		unset($proxy);
-
-		// check if any of the affected hosts are discovered
-		$hostIds = array();
-		foreach ($proxies as $proxy) {
-			if (isset($proxy['hosts'])) {
-				$hostIds = array_merge($hostIds, zbx_objectValues($proxy['hosts'], 'hostid'));
-			}
-		}
-		$this->checkValidator($hostIds, new CHostNormalValidator(array(
-			'message' => _('Cannot update proxy for discovered host "%1$s".')
-		)));
-	}
-
 	/**
 	 * Create proxy.
 	 *
@@ -298,26 +163,78 @@ class CProxy extends CApiService {
 	 * @return array
 	 */
 	public function create(array $proxies) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
+		}
+
 		$proxies = zbx_toArray($proxies);
 
 		$proxies = $this->convertDeprecatedValues($proxies);
 
-		$this->checkInput($proxies, __FUNCTION__);
+		$this->validateCreate($proxies);
 
-		$proxyIds = DB::insert('hosts', $proxies);
+		foreach ($proxies as &$proxy) {
+			// Clean encryption fields.
+			if ($proxy['status'] == HOST_STATUS_PROXY_PASSIVE) {
+				if (!array_key_exists('tls_connect', $proxy)) {
+					$proxy['tls_psk_identity'] = '';
+					$proxy['tls_psk'] = '';
+					$proxy['tls_issuer'] = '';
+					$proxy['tls_subject'] = '';
+				}
+				else {
+					if ($proxy['tls_connect'] != HOST_ENCRYPTION_PSK) {
+						$proxy['tls_psk_identity'] = '';
+						$proxy['tls_psk'] = '';
+					}
 
-		$hostUpdate = array();
+					if ($proxy['tls_connect'] != HOST_ENCRYPTION_CERTIFICATE) {
+						$proxy['tls_issuer'] = '';
+						$proxy['tls_subject'] = '';
+					}
+				}
+			}
+			elseif ($proxy['status'] == HOST_STATUS_PROXY_ACTIVE) {
+				if (!array_key_exists('tls_accept', $proxy)) {
+					$proxy['tls_psk_identity'] = '';
+					$proxy['tls_psk'] = '';
+					$proxy['tls_issuer'] = '';
+					$proxy['tls_subject'] = '';
+				}
+				else {
+					if (($proxy['tls_accept'] & HOST_ENCRYPTION_PSK) != HOST_ENCRYPTION_PSK) {
+						$proxy['tls_psk_identity'] = '';
+						$proxy['tls_psk'] = '';
+					}
+
+					if (($proxy['tls_accept'] & HOST_ENCRYPTION_CERTIFICATE) != HOST_ENCRYPTION_CERTIFICATE) {
+						$proxy['tls_issuer'] = '';
+						$proxy['tls_subject'] = '';
+					}
+				}
+			}
+
+			// Mark the interface as main to pass host interface validation.
+			if ($proxy['status'] == HOST_STATUS_PROXY_PASSIVE && array_key_exists('interface', $proxy)) {
+				$proxy['interface']['main'] = INTERFACE_PRIMARY;
+			}
+		}
+		unset($proxy);
+
+		$proxyids = DB::insert('hosts', $proxies);
+
+		$hostUpdate = [];
 		foreach ($proxies as $key => $proxy) {
 			if (!empty($proxy['hosts'])) {
-				$hostUpdate[] = array(
-					'values' => array('proxy_hostid' => $proxyIds[$key]),
-					'where' => array('hostid' => zbx_objectValues($proxy['hosts'], 'hostid'))
-				);
+				$hostUpdate[] = [
+					'values' => ['proxy_hostid' => $proxyids[$key]],
+					'where' => ['hostid' => zbx_objectValues($proxy['hosts'], 'hostid')]
+				];
 			}
 
 			// create interface
 			if ($proxy['status'] == HOST_STATUS_PROXY_PASSIVE) {
-				$proxy['interface']['hostid'] = $proxyIds[$key];
+				$proxy['interface']['hostid'] = $proxyids[$key];
 
 				if (!API::HostInterface()->create($proxy['interface'])) {
 					self::exception(ZBX_API_ERROR_INTERNAL, _('Proxy interface creation failed.'));
@@ -327,7 +244,7 @@ class CProxy extends CApiService {
 
 		DB::update('hosts', $hostUpdate);
 
-		return array('proxyids' => $proxyIds);
+		return ['proxyids' => $proxyids];
 	}
 
 	/**
@@ -338,53 +255,110 @@ class CProxy extends CApiService {
 	 * @return array
 	 */
 	public function update(array $proxies) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
+		}
+
 		$proxies = zbx_toArray($proxies);
 
 		$proxies = $this->convertDeprecatedValues($proxies);
 
-		$this->checkInput($proxies, __FUNCTION__);
+		$proxyids = zbx_objectValues($proxies, 'proxyid');
 
-		$proxyIds = $proxyUpdate = $hostUpdate = array();
+		foreach ($proxies as &$proxy) {
+			if (array_key_exists('proxyid', $proxy)) {
+				$proxy['hostid'] = $proxy['proxyid'];
+			}
+			elseif (array_key_exists('hostid', $proxy)) {
+				$proxy['proxyid'] = $proxy['hostid'];
+			}
+		}
+		unset($proxy);
+
+		$db_proxies = $this->get([
+			'output' => ['proxyid', 'hostid', 'host', 'status', 'tls_connect', 'tls_accept', 'tls_issuer',
+				'tls_subject', 'tls_psk_identity', 'tls_psk'
+			],
+			'proxyids' => $proxyids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		$this->validateUpdate($proxies, $db_proxies);
+
+		foreach ($proxies as &$proxy) {
+			$status = array_key_exists('status', $proxy) ? $proxy['status'] : $db_proxies[$proxy['proxyid']]['status'];
+
+			// Clean encryption fields.
+			$tls_connect = array_key_exists('tls_connect', $proxy)
+				? $proxy['tls_connect']
+				: $db_proxies[$proxy['proxyid']]['tls_connect'];
+
+			$tls_accept = array_key_exists('tls_accept', $proxy)
+				? $proxy['tls_accept']
+				: $db_proxies[$proxy['proxyid']]['tls_accept'];
+
+			// Clean PSK fields.
+			if ($tls_connect != HOST_ENCRYPTION_PSK && ($tls_accept & HOST_ENCRYPTION_PSK) != HOST_ENCRYPTION_PSK) {
+				$proxy['tls_psk_identity'] = '';
+				$proxy['tls_psk'] = '';
+			}
+
+			// Clean certificate fields.
+			if ($tls_connect != HOST_ENCRYPTION_CERTIFICATE
+					&& ($tls_accept & HOST_ENCRYPTION_CERTIFICATE) != HOST_ENCRYPTION_CERTIFICATE) {
+				$proxy['tls_issuer'] = '';
+				$proxy['tls_subject'] = '';
+			}
+
+			// Mark the interface as main to pass host interface validation.
+			if ($status == HOST_STATUS_PROXY_PASSIVE && array_key_exists('interface', $proxy)) {
+				$proxy['interface']['main'] = INTERFACE_PRIMARY;
+			}
+		}
+		unset($proxy);
+
+		$proxyUpdate = [];
+		$hostUpdate = [];
 
 		foreach ($proxies as $proxy) {
-			$proxyIds[] = $proxy['proxyid'];
-
-			$proxyUpdate[] = array(
+			$proxyUpdate[] = [
 				'values' => $proxy,
-				'where' => array('hostid' => $proxy['proxyid'])
-			);
+				'where' => ['hostid' => $proxy['proxyid']]
+			];
 
 			if (isset($proxy['hosts'])) {
 				// unset proxy for all hosts except for discovered hosts
-				$hostUpdate[] = array(
-					'values' => array('proxy_hostid' => 0),
-					'where' => array(
+				$hostUpdate[] = [
+					'values' => ['proxy_hostid' => 0],
+					'where' => [
 						'proxy_hostid' => $proxy['proxyid'],
 						'flags' => ZBX_FLAG_DISCOVERY_NORMAL
-					)
-				);
+					]
+				];
 
-				$hostUpdate[] = array(
-					'values' => array('proxy_hostid' => $proxy['proxyid']),
-					'where' => array('hostid' => zbx_objectValues($proxy['hosts'], 'hostid'))
-				);
+				$hostUpdate[] = [
+					'values' => ['proxy_hostid' => $proxy['proxyid']],
+					'where' => ['hostid' => zbx_objectValues($proxy['hosts'], 'hostid')]
+				];
 			}
 
-			// if this is an active proxy - delete it's interface;
-			if (isset($proxy['status']) && $proxy['status'] == HOST_STATUS_PROXY_ACTIVE) {
-				$interfaces = API::HostInterface()->get(array(
+			if (array_key_exists('status', $proxy) && $proxy['status'] == HOST_STATUS_PROXY_ACTIVE) {
+				// If this is an active proxy, delete it's interface.
+
+				$interfaces = API::HostInterface()->get([
 					'hostids' => $proxy['hostid'],
-					'output' => array('interfaceid')
-				));
+					'output' => ['interfaceid']
+				]);
 				$interfaceIds = zbx_objectValues($interfaces, 'interfaceid');
 
 				if ($interfaceIds) {
 					API::HostInterface()->delete($interfaceIds);
 				}
 			}
+			elseif (array_key_exists('interface', $proxy) && is_array($proxy['interface'])) {
+				// Update the interface of a passive proxy.
 
-			// update the interface of a passive proxy
-			elseif (isset($proxy['interface']) && is_array($proxy['interface'])) {
 				$proxy['interface']['hostid'] = $proxy['hostid'];
 
 				$result = isset($proxy['interface']['interfaceid'])
@@ -400,7 +374,7 @@ class CProxy extends CApiService {
 		DB::update('hosts', $proxyUpdate);
 		DB::update('hosts', $hostUpdate);
 
-		return array('proxyids' => $proxyIds);
+		return ['proxyids' => $proxyids];
 	}
 
 	/**
@@ -420,7 +394,7 @@ class CProxy extends CApiService {
 		);
 		$dbProxies = DBfetchArrayAssoc($dbProxies, 'hostid');
 
-		$actionIds = array();
+		$actionIds = [];
 
 		// get conditions
 		$dbActions = DBselect(
@@ -434,23 +408,23 @@ class CProxy extends CApiService {
 		}
 
 		if ($actionIds) {
-			DB::update('actions', array(
-				'values' => array('status' => ACTION_STATUS_DISABLED),
-				'where' => array('actionid' => $actionIds)
-			));
+			DB::update('actions', [
+				'values' => ['status' => ACTION_STATUS_DISABLED],
+				'where' => ['actionid' => $actionIds]
+			]);
 		}
 
 		// delete action conditions
-		DB::delete('conditions', array(
+		DB::delete('conditions', [
 			'conditiontype' => CONDITION_TYPE_PROXY,
 			'value' => $proxyIds
-		));
+		]);
 
 		// delete interface
-		DB::delete('interface', array('hostid' => $proxyIds));
+		DB::delete('interface', ['hostid' => $proxyIds]);
 
 		// delete host
-		DB::delete('hosts', array('hostid' => $proxyIds));
+		DB::delete('hosts', ['hostid' => $proxyIds]);
 
 		// TODO: remove info from API
 		foreach ($dbProxies as $proxy) {
@@ -458,7 +432,7 @@ class CProxy extends CApiService {
 			add_audit(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_PROXY, '['.$proxy['host'].'] ['.$proxy['hostid'].']');
 		}
 
-		return array('proxyids' => $proxyIds);
+		return ['proxyids' => $proxyIds];
 	}
 
 	/**
@@ -493,10 +467,10 @@ class CProxy extends CApiService {
 
 		$proxyIds = array_unique($proxyIds);
 
-		$count = $this->get(array(
+		$count = $this->get([
 			'proxyids' => $proxyIds,
 			'countOutput' => true
-		));
+		]);
 
 		return (count($proxyIds) == $count);
 	}
@@ -515,11 +489,11 @@ class CProxy extends CApiService {
 
 		$proxyIds = array_unique($proxyIds);
 
-		$count = $this->get(array(
+		$count = $this->get([
 			'proxyids' => $proxyIds,
 			'editable' => true,
 			'countOutput' => true
-		));
+		]);
 
 		return (count($proxyIds) == $count);
 	}
@@ -594,28 +568,28 @@ class CProxy extends CApiService {
 
 		// selectHosts
 		if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
-			$hosts = API::Host()->get(array(
-				'output' => $this->outputExtend($options['selectHosts'], array('hostid', 'proxy_hostid')),
+			$hosts = API::Host()->get([
+				'output' => $this->outputExtend($options['selectHosts'], ['hostid', 'proxy_hostid']),
 				'proxyids' => $proxyIds,
 				'preservekeys' => true
-			));
+			]);
 
 			$relationMap = $this->createRelationMap($hosts, 'proxy_hostid', 'hostid');
-			$hosts = $this->unsetExtraFields($hosts, array('proxy_hostid', 'hostid'), $options['selectHosts']);
+			$hosts = $this->unsetExtraFields($hosts, ['proxy_hostid', 'hostid'], $options['selectHosts']);
 			$result = $relationMap->mapMany($result, $hosts, 'hosts');
 		}
 
 		// adding host interface
 		if ($options['selectInterface'] !== null && $options['selectInterface'] != API_OUTPUT_COUNT) {
-			$interfaces = API::HostInterface()->get(array(
-				'output' => $this->outputExtend($options['selectInterface'], array('interfaceid', 'hostid')),
+			$interfaces = API::HostInterface()->get([
+				'output' => $this->outputExtend($options['selectInterface'], ['interfaceid', 'hostid']),
 				'hostids' => $proxyIds,
 				'nopermissions' => true,
 				'preservekeys' => true
-			));
+			]);
 
 			$relationMap = $this->createRelationMap($interfaces, 'hostid', 'interfaceid');
-			$interfaces = $this->unsetExtraFields($interfaces, array('hostid', 'interfaceid'), $options['selectInterface']);
+			$interfaces = $this->unsetExtraFields($interfaces, ['hostid', 'interfaceid'], $options['selectInterface']);
 			$result = $relationMap->mapOne($result, $interfaces, 'interface');
 
 			foreach ($result as $key => $proxy) {
@@ -649,4 +623,247 @@ class CProxy extends CApiService {
 
 		return $proxies;
 	}
+
+	/**
+	 * Validate connections from/to proxy and PSK fields.
+	 *
+	 * @param array $proxies	proxies data array
+	 *
+	 * @throws APIException	if incorrect encryption options.
+	 */
+	protected function validateEncryption(array $proxies) {
+		foreach ($proxies as $proxy) {
+			$available_connect_types = [HOST_ENCRYPTION_NONE, HOST_ENCRYPTION_PSK, HOST_ENCRYPTION_CERTIFICATE];
+			$available_accept_types = [
+				HOST_ENCRYPTION_NONE, HOST_ENCRYPTION_PSK, (HOST_ENCRYPTION_NONE | HOST_ENCRYPTION_PSK),
+				HOST_ENCRYPTION_CERTIFICATE, (HOST_ENCRYPTION_NONE | HOST_ENCRYPTION_CERTIFICATE),
+				(HOST_ENCRYPTION_PSK | HOST_ENCRYPTION_CERTIFICATE),
+				(HOST_ENCRYPTION_NONE | HOST_ENCRYPTION_PSK | HOST_ENCRYPTION_CERTIFICATE)
+			];
+
+			if ($proxy['status'] == HOST_STATUS_PROXY_PASSIVE && array_key_exists('tls_connect', $proxy)
+					&& !in_array($proxy['tls_connect'], $available_connect_types)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect value used for connections to proxy field.'));
+			}
+
+			if ($proxy['status'] == HOST_STATUS_PROXY_ACTIVE && array_key_exists('tls_accept', $proxy)
+					&& !in_array($proxy['tls_accept'], $available_accept_types)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect value used for connections from proxy field.'));
+			}
+
+			// PSK validation.
+			if ((array_key_exists('tls_connect', $proxy) && $proxy['tls_connect'] == HOST_ENCRYPTION_PSK
+					&& $proxy['status'] == HOST_STATUS_PROXY_PASSIVE)
+						|| (array_key_exists('tls_accept', $proxy)
+							&& ($proxy['tls_accept'] & HOST_ENCRYPTION_PSK) == HOST_ENCRYPTION_PSK
+							&& $proxy['status'] == HOST_STATUS_PROXY_ACTIVE)) {
+				if (!array_key_exists('tls_psk_identity', $proxy) || zbx_empty($proxy['tls_psk_identity'])) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _('PSK identity cannot be empty.'));
+				}
+
+				if (!array_key_exists('tls_psk', $proxy) || zbx_empty($proxy['tls_psk'])) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _('PSK cannot be empty.'));
+				}
+
+				if (!preg_match('/^([0-9a-f]{2})+$/i', $proxy['tls_psk'])) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _(
+						'Incorrect value used for PSK field. It should consist of an even number of hexadecimal characters.'
+					));
+				}
+
+				if (strlen($proxy['tls_psk']) < PSK_MIN_LEN) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('PSK is too short. Minimum is %1$s hex-digits.', PSK_MIN_LEN)
+					);
+				}
+			}
+		}
+	}
+
+	/**
+	 * Validates the input parameters for the create() method.
+	 *
+	 * @param array $proxies	proxies data array
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	protected function validateCreate(array $proxies) {
+		$proxy_db_fields = ['host' => null];
+		$names = [];
+
+		foreach ($proxies as $proxy) {
+			if (!check_db_fields($proxy_db_fields, $proxy)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Wrong fields for proxy "%1$s".', $proxy['host']));
+			}
+
+			if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $proxy['host'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect characters used for proxy name "%1$s".', $proxy['host'])
+				);
+			}
+
+			$names[$proxy['host']] = true;
+		}
+
+		$proxy_exists = $this->get([
+			'output' => ['proxyid', 'host'],
+			'filter' => ['host' => array_keys($names)],
+			'limit' => 1
+		]);
+
+		if ($proxy_exists) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Proxy "%s" already exists.', $proxy_exists[0]['host']));
+		}
+
+		$hostids = [];
+
+		foreach ($proxies as $proxy) {
+			if (!array_key_exists('status', $proxy)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('No status for proxy.'));
+			}
+			elseif ($proxy['status'] != HOST_STATUS_PROXY_ACTIVE && $proxy['status'] != HOST_STATUS_PROXY_PASSIVE) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect value used for proxy status "%1$s".', $proxy['status'])
+				);
+			}
+
+			// interface
+			if ($proxy['status'] == HOST_STATUS_PROXY_PASSIVE
+					&& (!array_key_exists('interface', $proxy)
+						|| !is_array($proxy['interface']) || !$proxy['interface'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('No interface provided for proxy "%s".', $proxy['host']));
+			}
+
+			if (array_key_exists('hosts', $proxy) && $proxy['hosts']) {
+				$hostids = array_merge($hostids, zbx_objectValues($proxy['hosts'], 'hostid'));
+			}
+		}
+
+		if ($hostids) {
+			// Check if host exists.
+			$hosts = API::Host()->get([
+				'output' => ['hostid'],
+				'hostids' => $hostids,
+				'editable' => true
+			]);
+
+			if (!$hosts) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
+
+			// Check if any of the affected hosts are discovered.
+			$this->checkValidator($hostids, new CHostNormalValidator([
+				'message' => _('Cannot update proxy for discovered host "%1$s".')
+			]));
+		}
+
+		$this->validateEncryption($proxies);
+	}
+
+	/**
+	 * Validates the input parameters for the update() method.
+	 *
+	 * @param array $proxies		proxies data array
+	 * @param array $db_proxies		db proxies data array
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	protected function validateUpdate(array $proxies, array $db_proxies) {
+		$proxy_db_fields = ['proxyid' => null];
+		$names = [];
+
+		foreach ($proxies as $proxy) {
+			if (!check_db_fields($proxy_db_fields, $proxy)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Wrong fields for proxy "%1$s".', $proxy['host']));
+			}
+
+			if (!array_key_exists($proxy['proxyid'], $db_proxies)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('No permissions to referred object or it does not exist!'));
+			}
+
+			// host
+			if (array_key_exists('host', $proxy)) {
+				if (!preg_match('/^'.ZBX_PREG_HOST_FORMAT.'$/', $proxy['host'])) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Incorrect characters used for proxy name "%1$s".', $proxy['host'])
+					);
+				}
+
+				if ($proxy['host'] !== $db_proxies[$proxy['proxyid']]['host']) {
+					$names[$proxy['host']] = true;
+				}
+			}
+		}
+
+		// Check names that have been changed.
+		if ($names) {
+			$proxies_exists = $this->get([
+				'output' => ['proxyid'],
+				'filter' => ['host' => array_keys($names)]
+			]);
+
+			foreach ($proxies as $proxy) {
+				if (array_key_exists('host', $proxy) && $proxy['host'] !== $db_proxies[$proxy['proxyid']]['host']) {
+					foreach ($proxies_exists as $proxy_exists) {
+						if (bccomp($proxy_exists['proxyid'], $proxy['proxyid']) != 0) {
+							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Proxy "%s" already exists.', $proxy['host']));
+						}
+					}
+				}
+			}
+		}
+
+		$hostids = [];
+
+		foreach ($proxies as $proxy) {
+			if (array_key_exists('status', $proxy) && ($proxy['status'] != HOST_STATUS_PROXY_ACTIVE
+					&& $proxy['status'] != HOST_STATUS_PROXY_PASSIVE)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect value used for proxy status "%1$s".', $proxy['status'])
+				);
+			}
+
+			if (array_key_exists('hosts', $proxy) && $proxy['hosts']) {
+				$hostids = array_merge($hostids, zbx_objectValues($proxy['hosts'], 'hostid'));
+			}
+		}
+
+		if ($hostids) {
+			// Check if host exists.
+			$hosts = API::Host()->get([
+				'output' => ['hostid'],
+				'hostids' => $hostids,
+				'editable' => true
+			]);
+
+			if (!$hosts) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
+
+			// Check if any of the affected hosts are discovered.
+			$this->checkValidator($hostids, new CHostNormalValidator([
+				'message' => _('Cannot update proxy for discovered host "%1$s".')
+			]));
+		}
+
+		$status = array_key_exists('status', $proxy) ? $proxy['status'] : $db_proxies[$proxy['proxyid']]['status'];
+
+		// interface
+		if ($status == HOST_STATUS_PROXY_PASSIVE && array_key_exists('interface', $proxy)
+				&& (!is_array($proxy['interface']) || !$proxy['interface'])) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('No interface provided for proxy "%s".', $proxy['host'])
+			);
+		}
+
+		$proxies = $this->extendFromObjects(zbx_toHash($proxies, 'proxyid'), $db_proxies, [
+			'status', 'tls_connect', 'tls_accept', 'tls_psk_identity', 'tls_psk'
+		]);
+
+		$this->validateEncryption($proxies);
+	}
 }
