@@ -21,16 +21,17 @@
 
 class CConfigurationExportBuilder {
 
-	const EXPORT_VERSION = '2.0';
-
 	/**
 	 * @var array
 	 */
-	protected $data = array();
+	protected $data = [];
 
+	/**
+	 * @param $version  current export version
+	 */
 	public function __construct() {
-		$this->data['version'] = self::EXPORT_VERSION;
-		$this->data['date'] = date('Y-m-d\TH:i:s\Z', time() - date('Z'));
+		$this->data['version'] = ZABBIX_EXPORT_VERSION;
+		$this->data['date'] = date(DATE_TIME_FORMAT_SECONDS_XML, time() - date('Z'));
 	}
 
 	/**
@@ -39,7 +40,7 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	public function getExport() {
-		return array('zabbix_export' => $this->data);
+		return ['zabbix_export' => $this->data];
 	}
 
 	/**
@@ -57,12 +58,10 @@ class CConfigurationExportBuilder {
 	 * @param array $templates
 	 */
 	public function buildTemplates(array $templates) {
-		order_result($templates, 'host');
-
-		$this->data['templates'] = array();
+		$this->data['templates'] = [];
 
 		foreach ($templates as $template) {
-			$this->data['templates'][] = array(
+			$this->data['templates'][] = [
 				'template' => $template['host'],
 				'name' => $template['name'],
 				'description' => $template['description'],
@@ -73,8 +72,10 @@ class CConfigurationExportBuilder {
 				'macros' => $this->formatMacros($template['macros']),
 				'templates' => $this->formatTemplateLinkage($template['parentTemplates']),
 				'screens' => $this->formatScreens($template['screens'])
-			);
+			];
 		}
+
+		order_result($this->data['templates'], 'template');
 	}
 
 	/**
@@ -83,14 +84,12 @@ class CConfigurationExportBuilder {
 	 * @param $hosts
 	 */
 	public function buildHosts(array $hosts) {
-		order_result($hosts, 'host');
-
-		$this->data['hosts'] = array();
+		$this->data['hosts'] = [];
 
 		foreach ($hosts as $host) {
 			$host = $this->createInterfaceReferences($host);
 
-			$this->data['hosts'][] = array(
+			$this->data['hosts'][] = [
 				'host' => $host['host'],
 				'name' => $host['name'],
 				'description' => $host['description'],
@@ -100,6 +99,12 @@ class CConfigurationExportBuilder {
 				'ipmi_privilege' => $host['ipmi_privilege'],
 				'ipmi_username' => $host['ipmi_username'],
 				'ipmi_password' => $host['ipmi_password'],
+				'tls_connect' => $host['tls_connect'],
+				'tls_accept' => $host['tls_accept'],
+				'tls_issuer' => $host['tls_issuer'],
+				'tls_subject' => $host['tls_subject'],
+				'tls_psk_identity' => $host['tls_psk_identity'],
+				'tls_psk' => $host['tls_psk'],
 				'templates' => $this->formatTemplateLinkage($host['parentTemplates']),
 				'groups' => $this->formatGroups($host['groups']),
 				'interfaces' => $this->formatHostInterfaces($host['interfaces']),
@@ -108,8 +113,10 @@ class CConfigurationExportBuilder {
 				'discovery_rules' => $this->formatDiscoveryRules($host['discoveryRules']),
 				'macros' => $this->formatMacros($host['macros']),
 				'inventory' => $this->formatHostInventory($host['inventory'])
-			);
+			];
 		}
+
+		order_result($this->data['hosts'], 'host');
 	}
 
 	/**
@@ -145,14 +152,14 @@ class CConfigurationExportBuilder {
 	 * @param array $images
 	 */
 	public function buildImages(array $images) {
-		$this->data['images'] = array();
+		$this->data['images'] = [];
 
 		foreach ($images as $image) {
-			$this->data['images'][] = array(
+			$this->data['images'][] = [
 				'name' => $image['name'],
 				'imagetype' => $image['imagetype'],
 				'encodedImage' => $image['encodedImage']
-			);
+			];
 		}
 	}
 
@@ -162,12 +169,11 @@ class CConfigurationExportBuilder {
 	 * @param array $maps
 	 */
 	public function buildMaps(array $maps) {
-		order_result($maps, 'name');
-
-		$this->data['maps'] = array();
+		$this->data['maps'] = [];
 
 		foreach ($maps as $map) {
-			$this->data['maps'][] = array(
+			$tmpSelements = $this->formatMapElements($map['selements']);
+			$this->data['maps'][] = [
 				'name' => $map['name'],
 				'width' => $map['width'],
 				'height' => $map['height'],
@@ -196,10 +202,52 @@ class CConfigurationExportBuilder {
 				'background' => $map['backgroundid'],
 				'iconmap' => $map['iconmap'],
 				'urls' => $this->formatMapUrls($map['urls']),
-				'selements' => $this->formatMapElements($map['selements']),
-				'links' => $this->formatMapLinks($map['links'])
-			);
+				'selements' => $tmpSelements,
+				'links' => $this->formatMapLinks($map['links'], $tmpSelements)
+			];
 		}
+
+		order_result($this->data['maps'], 'name');
+	}
+
+	/**
+	 * Format mappings.
+	 *
+	 * @param array $mappings
+	 *
+	 * @return array
+	 */
+	protected function formatMappings(array $mappings) {
+		$result = [];
+
+		foreach ($mappings as $mapping) {
+			$result[] = [
+				'value' => $mapping['value'],
+				'newvalue' => $mapping['newvalue']
+			];
+		}
+
+		CArrayHelper::sort($result, ['value']);
+
+		return $result;
+	}
+
+	/**
+	 * Format value maps.
+	 *
+	 * @param array $valuemaps
+	 */
+	public function buildValueMaps(array $valuemaps) {
+		$this->data['value_maps'] = [];
+
+		foreach ($valuemaps as $valuemap) {
+			$this->data['value_maps'][] = [
+				'name' => $valuemap['name'],
+				'mappings' => $this->formatMappings($valuemap['mappings'])
+			];
+		}
+
+		CArrayHelper::sort($this->data['value_maps'], ['name']);
 	}
 
 	/**
@@ -211,10 +259,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function createInterfaceReferences(array $host) {
-		$references = array(
+		$references = [
 			'num' => 1,
-			'refs' => array()
-		);
+			'refs' => []
+		];
 
 		// create interface references
 		foreach ($host['interfaces'] as &$interface) {
@@ -257,12 +305,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatDiscoveryRules(array $discoveryRules) {
-		$result = array();
-
-		order_result($discoveryRules, 'name');
+		$result = [];
 
 		foreach ($discoveryRules as $discoveryRule) {
-			$data = array(
+			$data = [
 				'name' => $discoveryRule['name'],
 				'type' => $discoveryRule['type'],
 				'snmp_community' => $discoveryRule['snmp_community'],
@@ -294,7 +340,7 @@ class CConfigurationExportBuilder {
 				'trigger_prototypes' => $this->formatTriggers($discoveryRule['triggerPrototypes']),
 				'graph_prototypes' => $this->formatGraphs($discoveryRule['graphPrototypes']),
 				'host_prototypes' => $this->formatHostPrototypes($discoveryRule['hostPrototypes'])
-			);
+			];
 
 			if (isset($discoveryRule['interface_ref'])) {
 				$data['interface_ref'] = $discoveryRule['interface_ref'];
@@ -303,6 +349,8 @@ class CConfigurationExportBuilder {
 			$result[] = $data;
 		}
 
+		order_result($result, 'key');
+
 		return $result;
 	}
 
@@ -327,12 +375,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatGraphs(array $graphs) {
-		$result = array();
-
-		order_result($graphs, 'name');
+		$result = [];
 
 		foreach ($graphs as $graph) {
-			$result[] = array(
+			$result[] = [
 				'name' => $graph['name'],
 				'width' => $graph['width'],
 				'height' => $graph['height'],
@@ -350,9 +396,11 @@ class CConfigurationExportBuilder {
 				'ymin_item_1' => $graph['ymin_itemid'],
 				'ymax_item_1' => $graph['ymax_itemid'],
 				'graph_items' => $this->formatGraphItems($graph['gitems'])
-			);
+			];
 		}
 
+		order_result($result, 'name');
+
 		return $result;
 	}
 
@@ -364,21 +412,21 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatHostPrototypes(array $hostPrototypes) {
-		$result = array();
-
-		order_result($hostPrototypes, 'host');
+		$result = [];
 
 		foreach ($hostPrototypes as $hostPrototype) {
-			$result[] = array(
+			$result[] = [
 				'host' => $hostPrototype['host'],
 				'name' => $hostPrototype['name'],
 				'status' => $hostPrototype['status'],
 				'group_links' => $this->formatGroupLinks($hostPrototype['groupLinks']),
 				'group_prototypes' => $this->formatGroupPrototypes($hostPrototype['groupPrototypes']),
 				'templates' => $this->formatTemplateLinkage($hostPrototype['templates'])
-			);
+			];
 		}
 
+		order_result($result, 'host');
+
 		return $result;
 	}
 
@@ -390,12 +438,14 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatGroupLinks(array $groupLinks) {
-		$result = array();
+		$result = [];
+
+		order_result($groupLinks, 'name');
 
 		foreach ($groupLinks as $groupLink) {
-			$result[] = array(
+			$result[] = [
 				'group' => $groupLink['groupid'],
-			);
+			];
 		}
 
 		return $result;
@@ -409,14 +459,16 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatGroupPrototypes(array $groupPrototypes) {
-		$result = array();
+		$result = [];
 
 		foreach ($groupPrototypes as $groupPrototype) {
-			$result[] = array(
+			$result[] = [
 				'name' => $groupPrototype['name']
-			);
+			];
 		}
 
+		order_result($result, 'name');
+
 		return $result;
 	}
 
@@ -428,16 +480,16 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatTemplateLinkage(array $templates) {
-		$result = array();
-
-		order_result($templates, 'host');
+		$result = [];
 
 		foreach ($templates as $template) {
-			$result[] = array(
+			$result[] = [
 				'name' => $template['host']
-			);
+			];
 		}
 
+		order_result($result, 'name');
+
 		return $result;
 	}
 
@@ -449,28 +501,23 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatTriggers(array $triggers) {
-		$result = array();
-
-		order_result($triggers, 'description');
+		$result = [];
 
 		foreach ($triggers as $trigger) {
-			$tr = array(
+			$result[] = [
 				'expression' => $trigger['expression'],
 				'name' => $trigger['description'],
 				'url' => $trigger['url'],
 				'status' => $trigger['status'],
 				'priority' => $trigger['priority'],
 				'description' => $trigger['comments'],
-				'type' => $trigger['type']
-			);
-
-			if (isset($trigger['dependencies'])) {
-				$tr['dependencies'] = $this->formatDependencies($trigger['dependencies']);
-			}
-
-			$result[] = $tr;
+				'type' => $trigger['type'],
+				'dependencies' => $this->formatDependencies($trigger['dependencies'])
+			];
 		}
 
+		CArrayHelper::sort($result, ['name', 'expression']);
+
 		return $result;
 	}
 
@@ -482,12 +529,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatHostInterfaces(array $interfaces) {
-		$result = array();
-
-		order_result($interfaces, 'ip');
+		$result = [];
 
 		foreach ($interfaces as $interface) {
-			$result[] = array(
+			$result[] = [
 				'default' => $interface['main'],
 				'type' => $interface['type'],
 				'useip' => $interface['useip'],
@@ -496,9 +541,11 @@ class CConfigurationExportBuilder {
 				'port' => $interface['port'],
 				'bulk' => $interface['bulk'],
 				'interface_ref' => $interface['interface_ref']
-			);
+			];
 		}
 
+		CArrayHelper::sort($result, ['type', 'ip', 'dns', 'port']);
+
 		return $result;
 	}
 
@@ -510,16 +557,16 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatGroups(array $groups) {
-		$result = array();
-
-		order_result($groups, 'name');
+		$result = [];
 
 		foreach ($groups as $group) {
-			$result[] = array(
+			$result[] = [
 				'name' => $group['name']
-			);
+			];
 		}
 
+		order_result($result, 'name');
+
 		return $result;
 	}
 
@@ -531,12 +578,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatItems(array $items) {
-		$result = array();
-
-		order_result($items, 'name');
+		$result = [];
 
 		foreach ($items as $item) {
-			$data = array(
+			$data = [
 				'name' => $item['name'],
 				'type' => $item['type'],
 				'snmp_community' => $item['snmp_community'],
@@ -574,7 +619,11 @@ class CConfigurationExportBuilder {
 				'applications' => $this->formatApplications($item['applications']),
 				'valuemap' => $item['valuemap'],
 				'logtimefmt' => $item['logtimefmt']
-			);
+			];
+
+			if ($item['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+				$data['application_prototypes'] = $this->formatApplications($item['applicationPrototypes']);
+			}
 
 			if (isset($item['interface_ref'])) {
 				$data['interface_ref'] = $item['interface_ref'];
@@ -583,6 +632,8 @@ class CConfigurationExportBuilder {
 			$result[] = $data;
 		}
 
+		order_result($result, 'key');
+
 		return $result;
 	}
 
@@ -594,16 +645,16 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatApplications(array $applications) {
-		$result = array();
-
-		order_result($applications, 'name');
+		$result = [];
 
 		foreach ($applications as $application) {
-			$result[] = array(
+			$result[] = [
 				'name' => $application['name']
-			);
+			];
 		}
 
+		order_result($result, 'name');
+
 		return $result;
 	}
 
@@ -615,17 +666,17 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatMacros(array $macros) {
-		$result = array();
-
-		$macros = order_macros($macros, 'macro');
+		$result = [];
 
 		foreach ($macros as $macro) {
-			$result[] = array(
+			$result[] = [
 				'macro' => $macro['macro'],
 				'value' => $macro['value']
-			);
+			];
 		}
 
+		$macros = order_macros($result, 'macro');
+
 		return $result;
 	}
 
@@ -637,19 +688,19 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatScreens(array $screens) {
-		$result = array();
-
-		order_result($screens, 'name');
+		$result = [];
 
 		foreach ($screens as $screen) {
-			$result[] = array(
+			$result[] = [
 				'name' => $screen['name'],
 				'hsize' => $screen['hsize'],
 				'vsize' => $screen['vsize'],
 				'screen_items' => $this->formatScreenItems($screen['screenitems'])
-			);
+			];
 		}
 
+		order_result($result, 'name');
+
 		return $result;
 	}
 
@@ -661,15 +712,17 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatDependencies(array $dependencies) {
-		$result = array();
+		$result = [];
 
 		foreach ($dependencies as $dependency) {
-			$result[] = array(
+			$result[] = [
 				'name' => $dependency['description'],
 				'expression' => $dependency['expression']
-			);
+			];
 		}
 
+		CArrayHelper::sort($result, ['name', 'expression']);
+
 		return $result;
 	}
 
@@ -681,10 +734,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatScreenItems(array $screenItems) {
-		$result = array();
+		$result = [];
 
 		foreach ($screenItems as $screenItem) {
-			$result[] = array(
+			$result[] = [
 				'resourcetype' => $screenItem['resourcetype'],
 				'width' => $screenItem['width'],
 				'height' => $screenItem['height'],
@@ -702,9 +755,11 @@ class CConfigurationExportBuilder {
 				'resource' => $screenItem['resourceid'],
 				'max_columns' => $screenItem['max_columns'],
 				'application' => $screenItem['application']
-			);
+			];
 		}
 
+		CArrayHelper::sort($result, ['y', 'x']);
+
 		return $result;
 	}
 
@@ -716,10 +771,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatGraphItems(array $graphItems) {
-		$result = array();
+		$result = [];
 
 		foreach ($graphItems as $graphItem) {
-			$result[] = array(
+			$result[] = [
 				'sortorder'=> $graphItem['sortorder'],
 				'drawtype'=> $graphItem['drawtype'],
 				'color'=> $graphItem['color'],
@@ -727,9 +782,11 @@ class CConfigurationExportBuilder {
 				'calc_fnc'=> $graphItem['calc_fnc'],
 				'type'=> $graphItem['type'],
 				'item'=> $graphItem['itemid']
-			);
+			];
 		}
 
+		CArrayHelper::sort($result, ['sortorder']);
+
 		return $result;
 	}
 
@@ -741,16 +798,18 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatMapUrls(array $urls) {
-		$result = array();
+		$result = [];
 
 		foreach ($urls as $url) {
-			$result[] = array(
+			$result[] = [
 				'name' => $url['name'],
 				'url' => $url['url'],
 				'elementtype' => $url['elementtype']
-			);
+			];
 		}
 
+		CArrayHelper::sort($result, ['name', 'url']);
+
 		return $result;
 	}
 
@@ -762,37 +821,63 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatMapElementUrls(array $urls) {
-		$result = array();
+		$result = [];
 
 		foreach ($urls as $url) {
-			$result[] = array(
+			$result[] = [
 				'name' => $url['name'],
 				'url' => $url['url']
-			);
+			];
 		}
 
+		CArrayHelper::sort($result, ['name', 'url']);
+
 		return $result;
 	}
 
 	/**
 	 * Format map links.
 	 *
-	 * @param array $links
+	 * @param array $links			Map links
+	 * @param array $selements		Map elements
 	 *
 	 * @return array
 	 */
-	protected function formatMapLinks(array $links) {
-		$result = array();
+	protected function formatMapLinks(array $links, array $selements) {
+		$result = [];
+
+		// Get array where key is selementid and value is sort position.
+		$flipped_selements = [];
+		$selements = array_values($selements);
+
+		foreach ($selements as $key => $item) {
+			if (array_key_exists('selementid', $item)) {
+				$flipped_selements[$item['selementid']] = $key;
+			}
+		}
+
+		foreach ($links as &$link) {
+			$link['selementpos1'] = $flipped_selements[$link['selementid1']];
+			$link['selementpos2'] = $flipped_selements[$link['selementid2']];
+
+			// Sort selements positons asc.
+			if ($link['selementpos2'] < $link['selementpos1']) {
+				zbx_swap($link['selementpos1'], $link['selementpos2']);
+			}
+		}
+		unset($link);
+
+		CArrayHelper::sort($links, ['selementpos1', 'selementpos2']);
 
 		foreach ($links as $link) {
-			$result[] = array(
+			$result[] = [
 				'drawtype' => $link['drawtype'],
 				'color' => $link['color'],
 				'label' => $link['label'],
 				'selementid1' => $link['selementid1'],
 				'selementid2' => $link['selementid2'],
 				'linktriggers' => $this->formatMapLinkTriggers($link['linktriggers'])
-			);
+			];
 		}
 
 		return $result;
@@ -806,14 +891,22 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatMapLinkTriggers(array $linktriggers) {
-		$result = array();
+		$result = [];
+
+		foreach ($linktriggers as &$linktrigger) {
+			$linktrigger['description'] = $linktrigger['triggerid']['description'];
+			$linktrigger['expression'] = $linktrigger['triggerid']['expression'];
+		}
+		unset($linktrigger);
+
+		CArrayHelper::sort($linktriggers, ['description', 'expression']);
 
 		foreach ($linktriggers as $linktrigger) {
-			$result[] = array(
+			$result[] = [
 				'drawtype' => $linktrigger['drawtype'],
 				'color' => $linktrigger['color'],
 				'trigger' => $linktrigger['triggerid']
-			);
+			];
 		}
 
 		return $result;
@@ -827,10 +920,10 @@ class CConfigurationExportBuilder {
 	 * @return array
 	 */
 	protected function formatMapElements(array $elements) {
-		$result = array();
+		$result = [];
 
 		foreach ($elements as $element) {
-			$result[] = array(
+			$result[] = [
 				'elementtype' => $element['elementtype'],
 				'label' => $element['label'],
 				'label_location' => $element['label_location'],
@@ -850,9 +943,11 @@ class CConfigurationExportBuilder {
 				'icon_maintenance' => $element['iconid_maintenance'],
 				'application' => $element['application'],
 				'urls' => $this->formatMapElementUrls($element['urls'])
-			);
+			];
 		}
 
+		CArrayHelper::sort($result, ['y', 'x']);
+
 		return $result;
 	}
 }
