@@ -25,14 +25,9 @@
 class CConfigurationImport {
 
 	/**
-	 * @var CImportReader
+	 * @var CImportDataAdapter
 	 */
-	protected $reader;
-
-	/**
-	 * @var CImportFormatter
-	 */
-	protected $formatter;
+	protected $adapter;
 
 	/**
 	 * @var CImportReferencer
@@ -54,137 +49,86 @@ class CConfigurationImport {
 	 */
 	protected $options;
 
-	/**
-	 * @var string with import data in one of supported formats
-	 */
-	protected $source;
-
 	/**
 	 * @var array with data read from source string
 	 */
 	protected $data;
 
 	/**
-	 * @var array with formatted data received from formatter
+	 * @var array  cached data from the adapter
 	 */
-	protected $formattedData = array();
+	protected $formattedData = [];
 
 	/**
 	 * Constructor.
 	 * Source string must be suitable for reader class,
 	 * i.e. if string contains json then reader should be able to read json.
 	 *
-	 * @param string					$source						configuration data in specified format
 	 * @param array						$options					import options "createMissing", "updateExisting" and "deleteMissing"
 	 * @param CImportReferencer			$referencer					class containing all importable objects
 	 * @param CImportedObjectContainer	$importedObjectContainer	class containing processed host and template IDs
 	 * @param CTriggerExpression		$triggerExpression			class to parse trigger expression
 	 */
-	public function __construct($source, array $options = array(), CImportReferencer $referencer,
+	public function __construct(array $options = [], CImportReferencer $referencer,
 			CImportedObjectContainer $importedObjectContainer, CTriggerExpression $triggerExpression) {
-		$this->options = array(
-			'groups' => array('createMissing' => false),
-			'hosts' => array('updateExisting' => false, 'createMissing' => false),
-			'templates' => array('updateExisting' => false, 'createMissing' => false),
-			'templateScreens' => array('updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false),
-			'applications' => array('updateExisting' => false, 'createMissing' => false),
-			'templateLinkage' => array('createMissing' => false),
-			'items' => array('updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false),
-			'discoveryRules' => array('updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false),
-			'triggers' => array('updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false),
-			'graphs' => array('updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false),
-			'screens' => array('updateExisting' => false, 'createMissing' => false),
-			'maps' => array('updateExisting' => false, 'createMissing' => false),
-			'images' => array('updateExisting' => false, 'createMissing' => false)
-		);
+		$this->options = [
+			'groups' => ['createMissing' => false],
+			'hosts' => ['updateExisting' => false, 'createMissing' => false],
+			'templates' => ['updateExisting' => false, 'createMissing' => false],
+			'templateScreens' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
+			'applications' => ['updateExisting' => false, 'createMissing' => false],
+			'templateLinkage' => ['createMissing' => false],
+			'items' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
+			'discoveryRules' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
+			'triggers' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
+			'graphs' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
+			'screens' => ['updateExisting' => false, 'createMissing' => false],
+			'maps' => ['updateExisting' => false, 'createMissing' => false],
+			'images' => ['updateExisting' => false, 'createMissing' => false],
+			'valueMaps' => ['updateExisting' => false, 'createMissing' => false]
+		];
 
 		$this->options = array_merge($this->options, $options);
-		$this->source = $source;
 		$this->referencer = $referencer;
 		$this->importedObjectContainer = $importedObjectContainer;
 		$this->triggerExpression = $triggerExpression;
 	}
 
-	/**
-	 * Set reader that is used to read data from source string that is passed to constructor.
-	 *
-	 * @param CImportReader $reader
-	 */
-	public function setReader(CImportReader $reader) {
-		$this->reader = $reader;
-	}
-
 	/**
 	 * Import configuration data.
 	 *
-	 * @todo   for 1.8 version import old class CXmlImport18 is used
-	 *
-	 * @throws Exception
-	 * @throws UnexpectedValueException
+	 * @param CImportDataAdapter $adapter   an object to provide access to the imported data
 	 *
 	 * @return bool
 	 */
-	public function import() {
-		if (empty($this->reader)) {
-			throw new UnexpectedValueException('Reader is not set.');
-		}
+	public function import(CImportDataAdapter $adapter) {
+		$this->adapter = $adapter;
 
-		$this->data = $this->reader->read($this->source);
+		// parse all import for references to resolve them all together with less sql count
+		$this->gatherReferences();
 
-		$version = $this->getImportVersion();
+		$this->processGroups();
+		$this->processTemplates();
+		$this->processHosts();
 
-		// if import version is 1.8 we use old class that support it.
-		// old import class process hosts, maps and screens separately.
-		if ($version == '1.8') {
-			CXmlImport18::import($this->source);
+		// delete missing objects from processed hosts and templates
+		$this->deleteMissingDiscoveryRules();
+		$this->deleteMissingTriggers();
+		$this->deleteMissingGraphs();
+		$this->deleteMissingItems();
+		$this->deleteMissingApplications();
 
-			if ($this->options['maps']['updateExisting'] || $this->options['maps']['createMissing']) {
-				CXmlImport18::parseMap($this->options);
-			}
-
-			if ($this->options['screens']['updateExisting'] || $this->options['screens']['createMissing']) {
-				CXmlImport18::parseScreen($this->options);
-			}
-
-			if ($this->options['hosts']['updateExisting']
-					|| $this->options['hosts']['createMissing']
-					|| $this->options['templates']['updateExisting']
-					|| $this->options['templates']['createMissing']) {
-				CXmlImport18::parseMain($this->options);
-			}
-		}
-		else {
-			$this->formatter = $this->getFormatter($version);
-
-			// pass data to formatter
-			// export has root key "zabbix_export" which is not passed
-			$this->formatter->setData($this->data['zabbix_export']);
-
-			// parse all import for references to resolve them all together with less sql count
-			$this->gatherReferences();
-
-			$this->processGroups();
-			$this->processTemplates();
-			$this->processHosts();
-
-			// delete missing objects from processed hosts and templates
-			$this->deleteMissingDiscoveryRules();
-			$this->deleteMissingTriggers();
-			$this->deleteMissingGraphs();
-			$this->deleteMissingItems();
-			$this->deleteMissingApplications();
-
-			// import objects
-			$this->processApplications();
-			$this->processItems();
-			$this->processDiscoveryRules();
-			$this->processTriggers();
-			$this->processGraphs();
-			$this->processImages();
-			$this->processMaps();
-			$this->processTemplateScreens();
-			$this->processScreens();
-		}
+		// import objects
+		$this->processApplications();
+		$this->processValueMaps();
+		$this->processItems();
+		$this->processTriggers();
+		$this->processDiscoveryRules();
+		$this->processGraphs();
+		$this->processImages();
+		$this->processMaps();
+		$this->processTemplateScreens();
+		$this->processScreens();
 
 		return true;
 	}
@@ -197,21 +141,21 @@ class CConfigurationImport {
 	 * @see CImportReferencer
 	 */
 	protected function gatherReferences() {
-		$groupsRefs = array();
-		$templatesRefs = array();
-		$hostsRefs = array();
-		$applicationsRefs = array();
-		$itemsRefs = array();
-		$valueMapsRefs = array();
-		$triggersRefs = array();
-		$graphsRefs = array();
-		$iconMapsRefs = array();
-		$mapsRefs = array();
-		$screensRefs = array();
-		$templateScreensRefs = array();
-		$macrosRefs = array();
-		$proxyRefs = array();
-		$hostPrototypesRefs = array();
+		$groupsRefs = [];
+		$templatesRefs = [];
+		$hostsRefs = [];
+		$applicationsRefs = [];
+		$itemsRefs = [];
+		$valueMapsRefs = [];
+		$triggersRefs = [];
+		$graphsRefs = [];
+		$iconMapsRefs = [];
+		$mapsRefs = [];
+		$screensRefs = [];
+		$templateScreensRefs = [];
+		$macrosRefs = [];
+		$proxyRefs = [];
+		$hostPrototypesRefs = [];
 
 		foreach ($this->getFormattedGroups() as $group) {
 			$groupsRefs[$group['name']] = $group['name'];
@@ -224,8 +168,10 @@ class CConfigurationImport {
 				$groupsRefs[$group['name']] = $group['name'];
 			}
 
-			foreach ($template['macros'] as $macro) {
-				$macrosRefs[$template['host']][$macro['macro']] = $macro['macro'];
+			if (array_key_exists('macros', $template)) {
+				foreach ($template['macros'] as $macro) {
+					$macrosRefs[$template['host']][$macro['macro']] = $macro['macro'];
+				}
 			}
 
 			if (!empty($template['templates'])) {
@@ -242,7 +188,7 @@ class CConfigurationImport {
 				$groupsRefs[$group['name']] = $group['name'];
 			}
 
-			if (!empty($host['macros'])) {
+			if (array_key_exists('macros', $host)) {
 				foreach ($host['macros'] as $macro) {
 					$macrosRefs[$host['host']][$macro['macro']] = $macro['macro'];
 				}
@@ -265,6 +211,10 @@ class CConfigurationImport {
 			}
 		}
 
+		foreach ($this->getFormattedValueMaps() as $valuemap) {
+			$valueMapsRefs[$valuemap['name']] = $valuemap['name'];
+		}
+
 		foreach ($this->getFormattedItems() as $host => $items) {
 			foreach ($items as $item) {
 				$itemsRefs[$host][$item['key_']] = $item['key_'];
@@ -376,8 +326,10 @@ class CConfigurationImport {
 				$itemsRefs[$expression['host']][$expression['item']] = $expression['item'];
 			}
 
-			foreach ($trigger['dependencies'] as $dependency) {
-				$triggersRefs[$dependency['name']][$dependency['expression']] = $dependency['expression'];
+			if (array_key_exists('dependencies', $trigger)) {
+				foreach ($trigger['dependencies'] as $dependency) {
+					$triggersRefs[$dependency['name']][$dependency['expression']] = $dependency['expression'];
+				}
 			}
 		}
 
@@ -453,6 +405,12 @@ class CConfigurationImport {
 							$graphsRefs[$resource['host']][$resource['name']] = $resource['name'];
 							break;
 
+						case SCREEN_RESOURCE_CLOCK:
+							if ($screenItem['style'] != TIME_TYPE_HOST) {
+								break;
+							}
+							// break; is not missing here
+
 						case SCREEN_RESOURCE_SIMPLE_GRAPH:
 						case SCREEN_RESOURCE_LLD_SIMPLE_GRAPH:
 						case SCREEN_RESOURCE_PLAIN_TEXT:
@@ -487,6 +445,12 @@ class CConfigurationImport {
 								$graphsRefs[$resource['host']][$resource['name']] = $resource['name'];
 								break;
 
+							case SCREEN_RESOURCE_CLOCK:
+								if ($screenItem['style'] != TIME_TYPE_HOST) {
+									break;
+								}
+								// break; is not missing here
+
 							case SCREEN_RESOURCE_SIMPLE_GRAPH:
 							case SCREEN_RESOURCE_LLD_SIMPLE_GRAPH:
 							case SCREEN_RESOURCE_PLAIN_TEXT:
@@ -604,7 +568,7 @@ class CConfigurationImport {
 			return;
 		}
 
-		$applicationsToCreate = array();
+		$applicationsToCreate = [];
 
 		foreach ($allApplications as $host => $applications) {
 			$hostId = $this->referencer->resolveHostOrTemplate($host);
@@ -632,6 +596,48 @@ class CConfigurationImport {
 		$this->referencer->refreshApplications();
 	}
 
+	/**
+	 * Import value maps.
+	 */
+	protected function processValueMaps() {
+		if (!$this->options['valueMaps']['createMissing'] && !$this->options['valueMaps']['updateExisting']) {
+			return;
+		}
+
+		$all_valuemaps = $this->getFormattedValueMaps();
+
+		if (!$all_valuemaps) {
+			return;
+		}
+
+		$valuemaps_to_create = [];
+		$valuemaps_to_update = [];
+
+		foreach ($all_valuemaps as $valuemap) {
+			$valuemapid = $this->referencer->resolveValueMap($valuemap['name']);
+
+			if ($valuemapid) {
+				$valuemap['valuemapid'] = $valuemapid;
+				$valuemaps_to_update[] = $valuemap;
+			}
+			else {
+				$valuemaps_to_create[] = $valuemap;
+			}
+		}
+
+		if ($this->options['valueMaps']['createMissing'] && $valuemaps_to_create) {
+			$valuemapids = API::ValueMap()->create($valuemaps_to_create);
+
+			foreach ($valuemaps_to_create as $key => $valuemap) {
+				$this->referencer->addValueMapRef($valuemap['name'], $valuemapids['valuemapids'][$key]);
+			}
+		}
+
+		if ($this->options['valueMaps']['updateExisting'] && $valuemaps_to_update) {
+			API::ValueMap()->update($valuemaps_to_update);
+		}
+	}
+
 	/**
 	 * Import items.
 	 */
@@ -646,8 +652,8 @@ class CConfigurationImport {
 			return;
 		}
 
-		$itemsToCreate = array();
-		$itemsToUpdate = array();
+		$itemsToCreate = [];
+		$itemsToUpdate = [];
 
 		foreach ($allItems as $host => $items) {
 			$hostId = $this->referencer->resolveHostOrTemplate($host);
@@ -661,7 +667,7 @@ class CConfigurationImport {
 				$item['hostid'] = $hostId;
 
 				if (isset($item['applications']) && $item['applications']) {
-					$applicationsIds = array();
+					$applicationsIds = [];
 
 					foreach ($item['applications'] as $application) {
 						if ($applicationId = $this->referencer->resolveApplication($hostId, $application['name'])) {
@@ -676,7 +682,7 @@ class CConfigurationImport {
 					$item['applications'] = $applicationsIds;
 				}
 
-				if (isset($item['interface_ref']) && $item['interface_ref']) {
+				if (array_key_exists('interface_ref', $item) && $item['interface_ref']) {
 					$item['interfaceid'] = $this->referencer->interfacesCache[$hostId][$item['interface_ref']];
 				}
 
@@ -746,8 +752,8 @@ class CConfigurationImport {
 			}
 		}
 
-		$itemsToCreate = array();
-		$itemsToUpdate = array();
+		$itemsToCreate = [];
+		$itemsToUpdate = [];
 
 		foreach ($allDiscoveryRules as $host => $discoveryRules) {
 			$hostId = $this->referencer->resolveHostOrTemplate($host);
@@ -755,7 +761,7 @@ class CConfigurationImport {
 			foreach ($discoveryRules as $item) {
 				$item['hostid'] = $hostId;
 
-				if (isset($item['interface_ref'])) {
+				if (array_key_exists('interface_ref', $item) && $item['interface_ref']) {
 					$item['interfaceid'] = $this->referencer->interfacesCache[$hostId][$item['interface_ref']];
 				}
 
@@ -777,7 +783,7 @@ class CConfigurationImport {
 		}
 
 		// create/update discovery rules and add processed rules to array $processedRules
-		$processedRules = array();
+		$processedRules = [];
 
 		if ($this->options['discoveryRules']['createMissing'] && $itemsToCreate) {
 			$newItemsIds = API::DiscoveryRule()->create($itemsToCreate);
@@ -805,10 +811,10 @@ class CConfigurationImport {
 		$this->referencer->refreshItems();
 
 		// process prototypes
-		$prototypesToUpdate = array();
-		$prototypesToCreate = array();
-		$hostPrototypesToUpdate = array();
-		$hostPrototypesToCreate = array();
+		$prototypesToUpdate = [];
+		$prototypesToCreate = [];
+		$hostPrototypesToUpdate = [];
+		$hostPrototypesToCreate = [];
 
 		foreach ($allDiscoveryRules as $host => $discoveryRules) {
 			$hostId = $this->referencer->resolveHostOrTemplate($host);
@@ -826,7 +832,7 @@ class CConfigurationImport {
 				foreach ($item['item_prototypes'] as $prototype) {
 					$prototype['hostid'] = $hostId;
 
-					$applicationsIds = array();
+					$applicationsIds = [];
 
 					foreach ($prototype['applications'] as $application) {
 						$applicationsIds[] = $this->referencer->resolveApplication($hostId, $application['name']);
@@ -834,7 +840,11 @@ class CConfigurationImport {
 
 					$prototype['applications'] = $applicationsIds;
 
-					if (isset($prototype['interface_ref'])) {
+					if (array_key_exists('application_prototypes', $prototype)) {
+						$prototype['applicationPrototypes'] = $prototype['application_prototypes'];
+					}
+
+					if (array_key_exists('interface_ref', $prototype) && $prototype['interface_ref']) {
 						$prototype['interfaceid'] = $this->referencer->interfacesCache[$hostId][$prototype['interface_ref']];
 					}
 
@@ -855,7 +865,7 @@ class CConfigurationImport {
 					}
 
 					$prototypeId = $this->referencer->resolveItem($hostId, $prototype['key_']);
-					$prototype['rule'] = array('hostid' => $hostId, 'key' => $item['key_']);
+					$prototype['rule'] = ['hostid' => $hostId, 'key' => $item['key_']];
 
 					if ($prototypeId) {
 						$prototype['itemid'] = $prototypeId;
@@ -869,7 +879,7 @@ class CConfigurationImport {
 				// host prototype
 				foreach ($item['host_prototypes'] as $hostPrototype) {
 					// resolve group prototypes
-					$groupLinks = array();
+					$groupLinks = [];
 
 					foreach ($hostPrototype['group_links'] as $groupLink) {
 						$groupId = $this->referencer->resolveGroup($groupLink['group']['name']);
@@ -884,7 +894,7 @@ class CConfigurationImport {
 							));
 						}
 
-						$groupLinks[] = array('groupid' => $groupId);
+						$groupLinks[] = ['groupid' => $groupId];
 					}
 
 					$hostPrototype['groupLinks'] = $groupLinks;
@@ -892,7 +902,7 @@ class CConfigurationImport {
 					unset($hostPrototype['group_links'], $hostPrototype['group_prototypes']);
 
 					// resolve templates
-					$templates = array();
+					$templates = [];
 
 					foreach ($hostPrototype['templates'] as $template) {
 						$templateId = $this->referencer->resolveTemplate($template['name']);
@@ -907,7 +917,7 @@ class CConfigurationImport {
 							));
 						}
 
-						$templates[] = array('templateid' => $templateId);
+						$templates[] = ['templateid' => $templateId];
 					}
 
 					$hostPrototype['templates'] = $templates;
@@ -924,7 +934,7 @@ class CConfigurationImport {
 					}
 				}
 
-				if (isset($item['interface_ref'])) {
+				if (array_key_exists('interface_ref', $item) && $item['interface_ref']) {
 					$item['interfaceid'] = $this->referencer->interfacesCache[$hostId][$item['interface_ref']];
 				}
 				unset($item['item_prototypes']);
@@ -978,10 +988,12 @@ class CConfigurationImport {
 		$this->referencer->refreshItems();
 
 		// first we need to create item prototypes and only then graph prototypes
-		$triggersToCreate = array();
-		$triggersToUpdate = array();
-		$graphsToCreate = array();
-		$graphsToUpdate = array();
+		$triggersToCreate = [];
+		$triggersToUpdate = [];
+		$graphsToCreate = [];
+		$graphsToUpdate = [];
+		// the list of triggers to process dependencies
+		$triggers = [];
 
 		foreach ($allDiscoveryRules as $host => $discoveryRules) {
 			$hostId = $this->referencer->resolveHostOrTemplate($host);
@@ -1013,6 +1025,9 @@ class CConfigurationImport {
 
 					$triggerId = $this->referencer->resolveTrigger($trigger['description'], $trigger['expression']);
 
+					$triggers[] = $trigger;
+					unset($trigger['dependencies']);
+
 					if ($triggerId) {
 						$trigger['triggerid'] = $triggerId;
 						$triggersToUpdate[] = $trigger;
@@ -1096,7 +1111,12 @@ class CConfigurationImport {
 		}
 
 		if ($triggersToCreate) {
-			API::TriggerPrototype()->create($triggersToCreate);
+			$result = API::TriggerPrototype()->create($triggersToCreate);
+
+			foreach ($result['triggerids'] as $tnum => $triggerid) {
+				$trigger = $triggersToCreate[$tnum];
+				$this->referencer->addTriggerRef($trigger['description'], $trigger['expression'], $triggerid);
+			}
 		}
 		if ($triggersToUpdate) {
 			API::TriggerPrototype()->update($triggersToUpdate);
@@ -1108,6 +1128,50 @@ class CConfigurationImport {
 		if ($graphsToUpdate) {
 			API::GraphPrototype()->update($graphsToUpdate);
 		}
+
+		$this->processTriggerPrototypeDependencies($triggers);
+	}
+
+	/**
+	 * Update trigger dependencies
+	 *
+	 * @param array $triggers
+	 *
+	 * @throws Exception
+	 */
+	protected function processTriggerPrototypeDependencies(array $triggers) {
+		$dependencies = [];
+
+		foreach ($triggers as $trigger) {
+			if (!array_key_exists('dependencies', $trigger)) {
+				continue;
+			}
+
+			$deps = [];
+			$triggerid = $this->referencer->resolveTrigger($trigger['description'], $trigger['expression']);
+
+			foreach ($trigger['dependencies'] as $dependency) {
+				$dep_triggerid = $this->referencer->resolveTrigger($dependency['name'], $dependency['expression']);
+
+				if (!$dep_triggerid) {
+					throw new Exception(_s('Trigger prototype "%1$s" depends on trigger "%2$s", which does not exist.',
+						$trigger['description'],
+						$dependency['name']
+					));
+				}
+
+				$deps[] = ['triggerid' => $dep_triggerid];
+			}
+
+			$dependencies[] = [
+				'triggerid' => $triggerid,
+				'dependencies' => $deps
+			];
+		}
+
+		if ($dependencies) {
+			API::TriggerPrototype()->update($dependencies);
+		}
 	}
 
 	/**
@@ -1126,8 +1190,8 @@ class CConfigurationImport {
 			return;
 		}
 
-		$graphsToCreate = array();
-		$graphsToUpdate = array();
+		$graphsToCreate = [];
+		$graphsToUpdate = [];
 
 		foreach ($allGraphs as $graph) {
 			if ($graph['ymin_item_1']) {
@@ -1216,9 +1280,10 @@ class CConfigurationImport {
 			return;
 		}
 
-		$triggersToCreate = array();
-		$triggersToUpdate = array();
-		$triggersToCreateDependencies = array();
+		$triggersToCreate = [];
+		$triggersToUpdate = [];
+		// the list of triggers to process dependencies
+		$triggers = [];
 
 		foreach ($allTriggers as $trigger) {
 			// search for existing items in trigger expressions
@@ -1238,80 +1303,83 @@ class CConfigurationImport {
 			$triggerId = $this->referencer->resolveTrigger($trigger['description'], $trigger['expression']);
 
 			if ($triggerId) {
-				$deps = array();
+				if ($this->options['triggers']['updateExisting']) {
+					$triggers[] = $trigger;
 
-				foreach ($trigger['dependencies'] as $dependency) {
-					$depTriggerId = $this->referencer->resolveTrigger($dependency['name'], $dependency['expression']);
-					if (!$depTriggerId) {
-						throw new Exception(_s('Trigger "%1$s" depends on trigger "%2$s", which does not exist.',
-							$trigger['description'],
-							$dependency['name']
-						));
-					}
-
-					$deps[] = array('triggerid' => $depTriggerId);
+					$trigger['triggerid'] = $triggerId;
+					unset($trigger['dependencies']);
+					$triggersToUpdate[] = $trigger;
 				}
-
-				$trigger['dependencies'] = $deps;
-				$trigger['triggerid'] = $triggerId;
-				$triggersToUpdate[] = $trigger;
 			}
 			else {
-				$triggersToCreateDependencies[] = $trigger['dependencies'];
-				unset($trigger['dependencies']);
-				$triggersToCreate[] = $trigger;
+				if ($this->options['triggers']['createMissing']) {
+					$triggers[] = $trigger;
+
+					unset($trigger['dependencies']);
+					$triggersToCreate[] = $trigger;
+				}
 			}
 		}
 
-		$triggerDependencies = array();
-		$newTriggers = array();
+		if ($triggersToCreate) {
+			$result = API::Trigger()->create($triggersToCreate);
 
-		if ($this->options['triggers']['createMissing'] && $triggersToCreate) {
-			$newTriggerIds = API::Trigger()->create($triggersToCreate);
-
-			foreach ($newTriggerIds['triggerids'] as $tnum => $triggerId) {
+			foreach ($result['triggerids'] as $tnum => $triggerid) {
 				$trigger = $triggersToCreate[$tnum];
-				$this->referencer->addTriggerRef($trigger['description'], $trigger['expression'], $triggerId);
-
-				$newTriggers[$triggerId] = $trigger;
+				$this->referencer->addTriggerRef($trigger['description'], $trigger['expression'], $triggerid);
 			}
 		}
 
-		// if we have new triggers with dependencies and they were created, create their dependencies
-		if ($triggersToCreateDependencies && isset($newTriggerIds)) {
-			foreach ($newTriggerIds['triggerids'] as $tnum => $triggerId) {
-				$deps = array();
-
-				foreach ($triggersToCreateDependencies[$tnum] as $dependency) {
-					$depTriggerId = $this->referencer->resolveTrigger($dependency['name'], $dependency['expression']);
-
-					if (!$depTriggerId) {
-						$trigger = $newTriggers[$triggerId];
-						throw new Exception(_s('Trigger "%1$s" depends on trigger "%2$s", which does not exist.', $trigger['description'], $dependency['name']));
-					}
-
-					$deps[] = array('triggerid' => $depTriggerId);
-				}
-
-				if ($deps) {
-					$triggerDependencies[] = array(
-						'triggerid' => $triggerId,
-						'dependencies' => $deps
-					);
-				}
-			}
-		}
-
-		if ($this->options['triggers']['updateExisting'] && $triggersToUpdate) {
+		if ($triggersToUpdate) {
 			API::Trigger()->update($triggersToUpdate);
 		}
 
-		if ($triggerDependencies) {
-			API::Trigger()->update($triggerDependencies);
-		}
-
 		// refresh triggers because template triggers can be inherited to host and used in maps
 		$this->referencer->refreshTriggers();
+
+		$this->processTriggerDependencies($triggers);
+	}
+
+	/**
+	 * Update trigger dependencies
+	 *
+	 * @param array $triggers
+	 *
+	 * @throws Exception
+	 */
+	protected function processTriggerDependencies(array $triggers) {
+		$dependencies = [];
+
+		foreach ($triggers as $trigger) {
+			if (!array_key_exists('dependencies', $trigger)) {
+				continue;
+			}
+
+			$deps = [];
+			$triggerid = $this->referencer->resolveTrigger($trigger['description'], $trigger['expression']);
+
+			foreach ($trigger['dependencies'] as $dependency) {
+				$dep_triggerid = $this->referencer->resolveTrigger($dependency['name'], $dependency['expression']);
+
+				if (!$dep_triggerid) {
+					throw new Exception(_s('Trigger "%1$s" depends on trigger "%2$s", which does not exist.',
+						$trigger['description'],
+						$dependency['name']
+					));
+				}
+
+				$deps[] = ['triggerid' => $dep_triggerid];
+			}
+
+			$dependencies[] = [
+				'triggerid' => $triggerid,
+				'dependencies' => $deps
+			];
+		}
+
+		if ($dependencies) {
+			API::Trigger()->update($dependencies);
+		}
 	}
 
 	/**
@@ -1335,14 +1403,14 @@ class CConfigurationImport {
 
 		$allImages = zbx_toHash($allImages, 'name');
 
-		$dbImages = API::Image()->get(array(
-			'output' => array('imageid', 'name'),
-			'filter' => array('name' => array_keys($allImages))
-		));
+		$dbImages = API::Image()->get([
+			'output' => ['imageid', 'name'],
+			'filter' => ['name' => array_keys($allImages)]
+		]);
 		$dbImages = zbx_toHash($dbImages, 'name');
 
-		$imagesToUpdate = array();
-		$imagesToCreate = array();
+		$imagesToUpdate = [];
+		$imagesToCreate = [];
 
 		foreach ($allImages as $imageName => $image) {
 			if (isset($dbImages[$imageName])) {
@@ -1403,8 +1471,8 @@ class CConfigurationImport {
 			$screenImporter = new CTemplateScreenImporter($this->options, $this->referencer,
 				$this->importedObjectContainer
 			);
-			$screenImporter->import($screens);
 			$screenImporter->delete($screens);
+			$screenImporter->import($screens);
 		}
 	}
 
@@ -1428,7 +1496,7 @@ class CConfigurationImport {
 			return;
 		}
 
-		$itemIdsXML = array();
+		$itemIdsXML = [];
 
 		$allItems = $this->getFormattedItems();
 
@@ -1446,14 +1514,14 @@ class CConfigurationImport {
 			}
 		}
 
-		$dbItemIds = API::Item()->get(array(
-			'output' => array('itemid'),
+		$dbItemIds = API::Item()->get([
+			'output' => ['itemid'],
 			'hostids' => $processedHostIds,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false,
-			'filter' => array('flags' => ZBX_FLAG_DISCOVERY_NORMAL)
-		));
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
+		]);
 
 		$itemsToDelete = array_diff_key($dbItemIds, $itemIdsXML);
 
@@ -1484,7 +1552,7 @@ class CConfigurationImport {
 			return;
 		}
 
-		$applicationIdsXML = array();
+		$applicationIdsXML = [];
 
 		$allApplications = $this->getFormattedApplications();
 
@@ -1502,13 +1570,14 @@ class CConfigurationImport {
 			}
 		}
 
-		$dbApplicationIds = API::Application()->get(array(
-			'output' => array('applicationid'),
+		$dbApplicationIds = API::Application()->get([
+			'output' => ['applicationid'],
 			'hostids' => $processedHostIds,
 			'preservekeys' => true,
 			'nopermissions' => true,
-			'inherited' => false
-		));
+			'inherited' => false,
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
+		]);
 
 		$applicationsToDelete = array_diff_key($dbApplicationIds, $applicationIdsXML);
 		if ($applicationsToDelete) {
@@ -1539,7 +1608,7 @@ class CConfigurationImport {
 			return;
 		}
 
-		$triggersXML = array();
+		$triggersXML = [];
 
 		$allTriggers = $this->getFormattedTriggers();
 
@@ -1553,20 +1622,20 @@ class CConfigurationImport {
 			}
 		}
 
-		$dbTriggerIds = API::Trigger()->get(array(
-			'output' => array('triggerid'),
+		$dbTriggerIds = API::Trigger()->get([
+			'output' => ['triggerid'],
 			'hostids' => $processedHostIds,
-			'selectHosts' => array('hostid'),
+			'selectHosts' => ['hostid'],
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false,
-			'filter' => array('flags' => ZBX_FLAG_DISCOVERY_NORMAL)
-		));
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
+		]);
 
 		// check that potentially deletable trigger belongs to same hosts that are in XML
 		// if some triggers belong to more hosts than current XML contains, don't delete them
 		$triggersToDelete = array_diff_key($dbTriggerIds, $triggersXML);
-		$triggerIdsToDelete = array();
+		$triggerIdsToDelete = [];
 		$processedHostIds = array_flip($processedHostIds);
 
 		foreach ($triggersToDelete as $triggerId => $trigger) {
@@ -1604,7 +1673,7 @@ class CConfigurationImport {
 			return;
 		}
 
-		$graphsIdsXML = array();
+		$graphsIdsXML = [];
 
 		// gather host IDs for graphs that exist in XML
 		$allGraphs = $this->getFormattedGraphs();
@@ -1624,20 +1693,20 @@ class CConfigurationImport {
 			}
 		}
 
-		$dbGraphIds = API::Graph()->get(array(
-			'output' => array('graphid'),
+		$dbGraphIds = API::Graph()->get([
+			'output' => ['graphid'],
 			'hostids' => $processedHostIds,
-			'selectHosts' => array('hostid'),
+			'selectHosts' => ['hostid'],
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false,
-			'filter' => array('flags' => ZBX_FLAG_DISCOVERY_NORMAL)
-		));
+			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
+		]);
 
 		// check that potentially deletable graph belongs to same hosts that are in XML
 		// if some graphs belong to more hosts than current XML contains, don't delete them
 		$graphsToDelete = array_diff_key($dbGraphIds, $graphsIdsXML);
-		$graphIdsToDelete = array();
+		$graphIdsToDelete = [];
 		$processedHostIds = array_flip($processedHostIds);
 
 		foreach ($graphsToDelete as $graphId => $graph) {
@@ -1675,7 +1744,7 @@ class CConfigurationImport {
 			return;
 		}
 
-		$discoveryRuleIdsXML = array();
+		$discoveryRuleIdsXML = [];
 
 		$allDiscoveryRules = $this->getFormattedDiscoveryRules();
 
@@ -1693,13 +1762,13 @@ class CConfigurationImport {
 			}
 		}
 
-		$dbDiscoveryRuleIds = API::DiscoveryRule()->get(array(
-			'output' => array('itemid'),
+		$dbDiscoveryRuleIds = API::DiscoveryRule()->get([
+			'output' => ['itemid'],
 			'hostids' => $processedHostIds,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
-		));
+		]);
 
 		$discoveryRulesToDelete = array_diff_key($dbDiscoveryRuleIds, $discoveryRuleIdsXML);
 
@@ -1710,10 +1779,10 @@ class CConfigurationImport {
 		// refresh discovery rules because templated ones can be inherited to host and used for prototypes
 		$this->referencer->refreshItems();
 
-		$hostPrototypeIdsXML = array();
-		$triggerPrototypeIdsXML = array();
-		$itemPrototypeIdsXML = array();
-		$graphPrototypeIdsXML = array();
+		$hostPrototypeIdsXML = [];
+		$triggerPrototypeIdsXML = [];
+		$itemPrototypeIdsXML = [];
+		$graphPrototypeIdsXML = [];
 
 		foreach ($allDiscoveryRules as $host => $discoveryRules) {
 			$hostId = $this->referencer->resolveHostOrTemplate($host);
@@ -1766,13 +1835,13 @@ class CConfigurationImport {
 		}
 
 		// delete missing host prototypes
-		$dbHostPrototypeIds = API::HostPrototype()->get(array(
-			'output' => array('hostid'),
+		$dbHostPrototypeIds = API::HostPrototype()->get([
+			'output' => ['hostid'],
 			'discoveryids' => $discoveryRuleIdsXML,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
-		));
+		]);
 
 		$hostPrototypesToDelete = array_diff_key($dbHostPrototypeIds, $hostPrototypeIdsXML);
 
@@ -1781,13 +1850,13 @@ class CConfigurationImport {
 		}
 
 		// delete missing trigger prototypes
-		$dbTriggerPrototypeIds = API::TriggerPrototype()->get(array(
-			'output' => array('triggerid'),
+		$dbTriggerPrototypeIds = API::TriggerPrototype()->get([
+			'output' => ['triggerid'],
 			'hostids' => $processedHostIds,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
-		));
+		]);
 
 		$triggerPrototypesToDelete = array_diff_key($dbTriggerPrototypeIds, $triggerPrototypeIdsXML);
 
@@ -1797,13 +1866,13 @@ class CConfigurationImport {
 		}
 
 		// delete missing graph prototypes
-		$dbGraphPrototypeIds = API::GraphPrototype()->get(array(
-			'output' => array('graphid'),
+		$dbGraphPrototypeIds = API::GraphPrototype()->get([
+			'output' => ['graphid'],
 			'hostids' => $processedHostIds,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
-		));
+		]);
 
 		$graphPrototypesToDelete = array_diff_key($dbGraphPrototypeIds, $graphPrototypeIdsXML);
 
@@ -1813,13 +1882,13 @@ class CConfigurationImport {
 		}
 
 		// delete missing item prototypes
-		$dbItemPrototypeIds = API::ItemPrototype()->get(array(
-			'output' => array('itemid'),
+		$dbItemPrototypeIds = API::ItemPrototype()->get([
+			'output' => ['itemid'],
 			'hostids' => $processedHostIds,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
-		));
+		]);
 
 		$itemPrototypesToDelete = array_diff_key($dbItemPrototypeIds, $itemPrototypeIdsXML);
 
@@ -1828,39 +1897,6 @@ class CConfigurationImport {
 		}
 	}
 
-	/**
-	 * Method for creating an import formatter for the specified import version.
-	 *
-	 * @throws InvalidArgumentException
-	 *
-	 * @param string $version
-	 *
-	 * @return CImportFormatter
-	 */
-	protected function getFormatter($version) {
-		switch ($version) {
-			case '2.0':
-				$converter = new C24TriggerConverter(new CFunctionMacroParser(), new CMacroParser('#'));
-
-				return new C20ImportFormatter($converter);
-			default:
-				throw new InvalidArgumentException('Unknown import version.');
-		}
-	}
-
-	/**
-	 * Get configuration import version.
-	 *
-	 * @return string
-	 */
-	protected function getImportVersion() {
-		if (isset($this->data['zabbix_export']['version'])) {
-			return $this->data['zabbix_export']['version'];
-		}
-
-		return '1.8';
-	}
-
 	/**
 	 * Get formatted groups.
 	 *
@@ -1868,7 +1904,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedGroups() {
 		if (!isset($this->formattedData['groups'])) {
-			$this->formattedData['groups'] = $this->formatter->getGroups();
+			$this->formattedData['groups'] = $this->adapter->getGroups();
 		}
 
 		return $this->formattedData['groups'];
@@ -1881,7 +1917,7 @@ class CConfigurationImport {
 	 */
 	public function getFormattedTemplates() {
 		if (!isset($this->formattedData['templates'])) {
-			$this->formattedData['templates'] = $this->formatter->getTemplates();
+			$this->formattedData['templates'] = $this->adapter->getTemplates();
 		}
 
 		return $this->formattedData['templates'];
@@ -1894,7 +1930,7 @@ class CConfigurationImport {
 	 */
 	public function getFormattedHosts() {
 		if (!isset($this->formattedData['hosts'])) {
-			$this->formattedData['hosts'] = $this->formatter->getHosts();
+			$this->formattedData['hosts'] = $this->adapter->getHosts();
 		}
 
 		return $this->formattedData['hosts'];
@@ -1907,12 +1943,25 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedApplications() {
 		if (!isset($this->formattedData['applications'])) {
-			$this->formattedData['applications'] = $this->formatter->getApplications();
+			$this->formattedData['applications'] = $this->adapter->getApplications();
 		}
 
 		return $this->formattedData['applications'];
 	}
 
+	/**
+	 * Get formatted value maps.
+	 *
+	 * @return array
+	 */
+	protected function getFormattedValueMaps() {
+		if (!isset($this->formattedData['valueMaps'])) {
+			$this->formattedData['valueMaps'] = $this->adapter->getValueMaps();
+		}
+
+		return $this->formattedData['valueMaps'];
+	}
+
 	/**
 	 * Get formatted items.
 	 *
@@ -1920,7 +1969,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedItems() {
 		if (!isset($this->formattedData['items'])) {
-			$this->formattedData['items'] = $this->formatter->getItems();
+			$this->formattedData['items'] = $this->adapter->getItems();
 		}
 
 		return $this->formattedData['items'];
@@ -1933,7 +1982,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedDiscoveryRules() {
 		if (!isset($this->formattedData['discoveryRules'])) {
-			$this->formattedData['discoveryRules'] = $this->formatter->getDiscoveryRules();
+			$this->formattedData['discoveryRules'] = $this->adapter->getDiscoveryRules();
 
 			foreach ($this->formattedData['discoveryRules'] as &$discoveryRules) {
 				foreach ($discoveryRules as &$discoveryRule) {
@@ -1957,7 +2006,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedTriggers() {
 		if (!isset($this->formattedData['triggers'])) {
-			$this->formattedData['triggers'] = $this->formatter->getTriggers();
+			$this->formattedData['triggers'] = $this->adapter->getTriggers();
 
 			foreach ($this->formattedData['triggers'] as &$trigger) {
 				$trigger['parsedExpressions'] = $this->parseTriggerExpression($trigger['expression']);
@@ -1978,7 +2027,7 @@ class CConfigurationImport {
 	 * @throws Exception
 	 */
 	protected function parseTriggerExpression($expression) {
-		$expressions = array();
+		$expressions = [];
 
 		$result = $this->triggerExpression->parse($expression);
 		if (!$result) {
@@ -1986,10 +2035,10 @@ class CConfigurationImport {
 		}
 
 		foreach ($result->getTokensByType(CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO) as $token) {
-			$expressions[] = array(
+			$expressions[] = [
 				'host' => $token['data']['host'],
 				'item' => $token['data']['item'],
-			);
+			];
 		}
 
 		return $expressions;
@@ -2002,7 +2051,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedGraphs() {
 		if (!isset($this->formattedData['graphs'])) {
-			$this->formattedData['graphs'] = $this->formatter->getGraphs();
+			$this->formattedData['graphs'] = $this->adapter->getGraphs();
 		}
 
 		return $this->formattedData['graphs'];
@@ -2015,7 +2064,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedImages() {
 		if (!isset($this->formattedData['images'])) {
-			$this->formattedData['images'] = $this->formatter->getImages();
+			$this->formattedData['images'] = $this->adapter->getImages();
 		}
 
 		return $this->formattedData['images'];
@@ -2028,7 +2077,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedMaps() {
 		if (!isset($this->formattedData['maps'])) {
-			$this->formattedData['maps'] = $this->formatter->getMaps();
+			$this->formattedData['maps'] = $this->adapter->getMaps();
 		}
 
 		return $this->formattedData['maps'];
@@ -2041,7 +2090,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedScreens() {
 		if (!isset($this->formattedData['screens'])) {
-			$this->formattedData['screens'] = $this->formatter->getScreens();
+			$this->formattedData['screens'] = $this->adapter->getScreens();
 		}
 
 		return $this->formattedData['screens'];
@@ -2054,7 +2103,7 @@ class CConfigurationImport {
 	 */
 	protected function getFormattedTemplateScreens() {
 		if (!isset($this->formattedData['templateScreens'])) {
-				$this->formattedData['templateScreens'] = $this->formatter->getTemplateScreens();
+				$this->formattedData['templateScreens'] = $this->adapter->getTemplateScreens();
 		}
 
 		return $this->formattedData['templateScreens'];
