@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2016 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -32,7 +32,8 @@ class CXmlImportReader extends CImportReader {
 	 */
 	public function read($string) {
 		libxml_use_internal_errors(true);
-		$result = simplexml_load_string($string);
+		libxml_disable_entity_loader(true);
+		$result = simplexml_load_string($string, null, LIBXML_IMPORT_FLAGS);
 		if (!$result) {
 			$errors = libxml_get_errors();
 			libxml_clear_errors();
@@ -76,15 +77,31 @@ class CXmlImportReader extends CImportReader {
 		while ($xml->read()) {
 			switch ($xml->nodeType) {
 				case XMLReader::ELEMENT:
-					if (!is_array($array)) {
-						$array = array();
-					}
-
 					$nodeName = $xml->name;
 					if (isset($array[$nodeName])) {
 						$nodeName .= count($array);
 					}
-					$array[$nodeName] = $xml->isEmptyElement ? '' : $this->xmlToArray($xml);
+
+					// a special case for 1.8 import where attributes are still used
+					// attributes must be added to the array as if they where child elements
+					if ($xml->hasAttributes) {
+						while ($xml->moveToNextAttribute()) {
+							$array[$nodeName][$xml->name] = $xml->value;
+						}
+
+						// we assume that an element with attributes always contains child elements, not a text node
+						// works for 1.8 XML
+						$xmlToArray = $this->xmlToArray($xml);
+						if (is_array($xmlToArray)) {
+							foreach ($xmlToArray as $name => $value) {
+								$array[$nodeName][$name] = $value;
+							}
+						}
+					}
+					else {
+						$array[$nodeName] = $xml->isEmptyElement ? '' : $this->xmlToArray($xml);
+					}
+
 					break;
 
 				case XMLReader::TEXT:
