@@ -24,7 +24,7 @@ class CUrl {
 	private $url;
 	protected $reference;
 	protected $query;
-	protected $arguments = array();
+	protected $arguments = [];
 
 	/**
 	 * WARNING: the class doesn't support parsing query strings with multi-dimentional arrays.
@@ -56,10 +56,6 @@ class CUrl {
 
 			$this->formatArguments();
 		}
-
-		if (isset($_COOKIE['zbx_sessionid'])) {
-			$this->setArgument('sid', substr($_COOKIE['zbx_sessionid'], 16, 16));
-		}
 	}
 
 	/**
@@ -72,9 +68,6 @@ class CUrl {
 	public function formatGetArguments() {
 		$this->arguments = $_GET;
 
-		if (isset($_COOKIE['zbx_sessionid'])) {
-			$this->setArgument('sid', substr($_COOKIE['zbx_sessionid'], 16, 16));
-		}
 		$this->formatQuery();
 	}
 
@@ -126,17 +119,6 @@ class CUrl {
 		return isset($this->arguments[$key]) ? $this->arguments[$key] : null;
 	}
 
-	public function setQuery($query) {
-		$this->query = $query;
-		$this->formatArguments();
-		$this->formatQuery();
-	}
-
-	public function getQuery() {
-		$this->formatQuery();
-		return $this->query;
-	}
-
 	public function setReference($reference) {
 		$this->reference = $reference;
 	}
