@@ -25,7 +25,7 @@
  */
 class CFrontendSetup {
 
-	const MIN_PHP_VERSION = '5.3.0';
+	const MIN_PHP_VERSION = '5.4.0';
 	const MIN_PHP_MEMORY_LIMIT = 134217728; // 128*1024*1024
 	const MIN_PHP_POST_MAX_SIZE = 16777216; // 16*1024*1024
 	const MIN_PHP_UPLOAD_MAX_FILESIZE = 2097152; // 2*1024*1024
@@ -33,6 +33,7 @@ class CFrontendSetup {
 	const MIN_PHP_MAX_INPUT_TIME = 300;
 	const MIN_PHP_GD_VERSION = '2.0';
 	const MIN_PHP_LIBXML_VERSION = '2.6.15';
+	const REQUIRED_PHP_ARG_SEPARATOR_OUTPUT = '&';
 
 	/**
 	 * Check OK, setup can continue.
@@ -55,7 +56,7 @@ class CFrontendSetup {
 	 * @return array
 	 */
 	public function checkRequirements() {
-		$result = array();
+		$result = [];
 
 		$result[] = $this->checkPhpVersion();
 		$result[] = $this->checkPhpMemoryLimit();
@@ -87,6 +88,7 @@ class CFrontendSetup {
 		$result[] = $this->checkPhpSession();
 		$result[] = $this->checkPhpSessionAutoStart();
 		$result[] = $this->checkPhpGettext();
+		$result[] = $this->checkPhpArgSeparatorOutput();
 
 		return $result;
 	}
@@ -99,13 +101,13 @@ class CFrontendSetup {
 	public function checkPhpVersion() {
 		$check = version_compare(phpversion(), self::MIN_PHP_VERSION, '>=');
 
-		return array(
+		return [
 			'name' => _('PHP version'),
 			'current' => phpversion(),
 			'required' => self::MIN_PHP_VERSION,
 			'result' => $check ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _s('Minimum required PHP version is %s.', self::MIN_PHP_VERSION)
-		);
+		];
 	}
 
 	/**
@@ -117,13 +119,13 @@ class CFrontendSetup {
 		$current = ini_get('memory_limit');
 		$check = ($current == '-1' || str2mem($current) >= self::MIN_PHP_MEMORY_LIMIT);
 
-		return array(
-			'name' => _('PHP option memory_limit'),
+		return [
+			'name' => _s('PHP option "%1$s"', 'memory_limit'),
 			'current' => $current,
 			'required' => mem2str(self::MIN_PHP_MEMORY_LIMIT),
 			'result' => $check ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _s('Minimum required PHP memory limit is %s (configuration option "memory_limit").', mem2str(self::MIN_PHP_MEMORY_LIMIT))
-		);
+		];
 	}
 
 	/**
@@ -134,13 +136,13 @@ class CFrontendSetup {
 	public function checkPhpPostMaxSize() {
 		$current = ini_get('post_max_size');
 
-		return array(
-			'name' => _('PHP option post_max_size'),
+		return [
+			'name' => _s('PHP option "%1$s"', 'post_max_size'),
 			'current' => $current,
 			'required' => mem2str(self::MIN_PHP_POST_MAX_SIZE),
 			'result' => (str2mem($current) >= self::MIN_PHP_POST_MAX_SIZE) ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _s('Minimum required size of PHP post is %s (configuration option "post_max_size").', mem2str(self::MIN_PHP_POST_MAX_SIZE))
-		);
+		];
 	}
 
 	/**
@@ -151,47 +153,63 @@ class CFrontendSetup {
 	public function checkPhpUploadMaxFilesize() {
 		$current = ini_get('upload_max_filesize');
 
-		return array(
-			'name' => _('PHP option upload_max_filesize'),
+		return [
+			'name' => _s('PHP option "%1$s"', 'upload_max_filesize'),
 			'current' => $current,
 			'required' => mem2str(self::MIN_PHP_UPLOAD_MAX_FILESIZE),
 			'result' => (str2mem($current) >= self::MIN_PHP_UPLOAD_MAX_FILESIZE) ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _s('Minimum required PHP upload filesize is %s (configuration option "upload_max_filesize").', mem2str(self::MIN_PHP_UPLOAD_MAX_FILESIZE))
-		);
+		];
 	}
 
 	/**
 	 * Checks for minimum PHP max execution time.
 	 *
+	 * Value of "max_execution_time" is used to set up OS level timer that fires signal after specified number of
+	 * seconds has passed. Handler of this signal interrupts execution. On *nix platforms this is done with call
+	 * to "setitimer()" (http://linux.die.net/man/2/setitimer) and in this case integer value of "max_input_time"
+	 * is used to specify how many seconds timer has to wait before sending signal:
+	 * - Value "0" is valid because in this case timer is removed and will not fire and stop execution.
+	 * - Value "-1" is valid because this signed integer and is used as value for "it_value.tv_sec" for "new_value"
+	 *   argument in call to "setitimer()". As negative values for "tv_sec" are not allowed the call will fail. Errors
+	 *   are not checked and timer is not set.
+	 * - Any value bigger than "0" is valid and sets up timer to fire after specified number of seconds.
+	 *
 	 * @return array
 	 */
 	public function checkPhpMaxExecutionTime() {
 		$current = ini_get('max_execution_time');
 
-		return array(
-			'name' => _('PHP option max_execution_time'),
+		$currentIsValid = ($current === '0' || $current === '-1' || $current >= self::MIN_PHP_MAX_EXECUTION_TIME);
+
+		return [
+			'name' => _s('PHP option "%1$s"', 'max_execution_time'),
 			'current' => $current,
 			'required' => self::MIN_PHP_MAX_EXECUTION_TIME,
-			'result' => ($current >= self::MIN_PHP_MAX_EXECUTION_TIME) ? self::CHECK_OK : self::CHECK_FATAL,
+			'result' => $currentIsValid ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _s('Minimum required limit on execution time of PHP scripts is %s (configuration option "max_execution_time").', self::MIN_PHP_MAX_EXECUTION_TIME)
-		);
+		];
 	}
 
 	/**
 	 * Checks for minimum PHP max input time.
 	 *
+	 * @see checkPhpMaxExecutionTime()
+	 *
 	 * @return array
 	 */
 	public function checkPhpMaxInputTime() {
 		$current = ini_get('max_input_time');
 
-		return array(
-			'name' => _('PHP option max_input_time'),
+		$currentIsValid = ($current === '0' || $current === '-1' || $current >= self::MIN_PHP_MAX_INPUT_TIME);
+
+		return [
+			'name' => _s('PHP option "%1$s"', 'max_input_time'),
 			'current' => $current,
 			'required' => self::MIN_PHP_MAX_INPUT_TIME,
-			'result' => ($current >= self::MIN_PHP_MAX_INPUT_TIME) ? self::CHECK_OK : self::CHECK_FATAL,
+			'result' => $currentIsValid ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _s('Minimum required limit on input parse time for PHP scripts is %s (configuration option "max_input_time").', self::MIN_PHP_MAX_INPUT_TIME)
-		);
+		];
 	}
 
 	/**
@@ -202,13 +220,13 @@ class CFrontendSetup {
 	public function checkPhpTimeZone() {
 		$current = ini_get('date.timezone');
 
-		return array(
-			'name' => _('PHP time zone'),
+		return [
+			'name' => _s('PHP option "%1$s"', 'date.timezone'),
 			'current' => $current ? $current : _('unknown'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('Time zone for PHP is not set (configuration parameter "date.timezone").')
-		);
+		];
 	}
 
 	/**
@@ -217,21 +235,21 @@ class CFrontendSetup {
 	 * @return array
 	 */
 	public function checkPhpDatabases() {
-		$current = array();
+		$current = [];
 
-		$databases = $this->getSupportedDatabases();
+		$databases = self::getSupportedDatabases();
 		foreach ($databases as $database => $name) {
 			$current[] = $name;
 			$current[] = BR();
 		}
 
-		return array(
+		return [
 			'name' => _('PHP databases support'),
 			'current' => empty($current) ? _('off') : new CSpan($current),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('At least one of MySQL, PostgreSQL, Oracle, SQLite3 or IBM DB2 should be supported.')
-		);
+		];
 	}
 
 	/**
@@ -239,35 +257,37 @@ class CFrontendSetup {
 	 *
 	 * @return array
 	 */
-	public function getSupportedDatabases() {
-		$allowedDb = array();
-		if (zbx_is_callable(array('mysqli_connect', 'mysqli_connect_error', 'mysqli_error', 'mysqli_query',
-				'mysqli_fetch_assoc', 'mysqli_free_result', 'mysqli_real_escape_string', 'mysqli_close'))) {
-			$allowedDb[ZBX_DB_MYSQL] = 'MySQL';
+	public static function getSupportedDatabases() {
+		$allowed_db = [];
+
+		if (zbx_is_callable(['mysqli_close', 'mysqli_connect', 'mysqli_connect_error', 'mysqli_error',
+				'mysqli_fetch_assoc', 'mysqli_free_result', 'mysqli_query', 'mysqli_real_escape_string'])) {
+			$allowed_db[ZBX_DB_MYSQL] = 'MySQL';
 		}
 
-		if (zbx_is_callable(array('pg_pconnect', 'pg_fetch_array', 'pg_fetch_row', 'pg_exec', 'pg_getlastoid'))) {
-			$allowedDb[ZBX_DB_POSTGRESQL] = 'PostgreSQL';
+		if (zbx_is_callable(['pg_close', 'pg_connect', 'pg_escape_bytea', 'pg_escape_string', 'pg_fetch_assoc',
+				'pg_free_result', 'pg_last_error', 'pg_parameter_status', 'pg_query', 'pg_unescape_bytea'])) {
+			$allowed_db[ZBX_DB_POSTGRESQL] = 'PostgreSQL';
 		}
 
-		if (zbx_is_callable(array('oci_connect', 'oci_error', 'oci_parse', 'oci_execute', 'oci_fetch_assoc',
-				'oci_commit', 'oci_close', 'oci_rollback', 'oci_field_type', 'oci_new_descriptor',
-				'oci_bind_by_name', 'oci_free_statement'))) {
-			$allowedDb[ZBX_DB_ORACLE] = 'Oracle';
+		if (zbx_is_callable(['oci_bind_by_name', 'oci_close', 'oci_commit', 'oci_connect', 'oci_error', 'oci_execute',
+				'oci_fetch_assoc', 'oci_field_type', 'oci_free_statement', 'oci_new_descriptor', 'oci_parse',
+				'oci_rollback'])) {
+			$allowed_db[ZBX_DB_ORACLE] = 'Oracle';
 		}
 
-		if (zbx_is_callable(array('db2_connect', 'db2_set_option', 'db2_commit', 'db2_rollback', 'db2_autocommit',
-				'db2_prepare', 'db2_execute', 'db2_stmt_errormsg', 'db2_fetch_assoc', 'db2_free_result',
-				'db2_escape_string', 'db2_close'))) {
-			$allowedDb[ZBX_DB_DB2] = 'IBM DB2';
+		if (zbx_is_callable(['db2_autocommit', 'db2_bind_param', 'db2_close', 'db2_commit', 'db2_conn_errormsg',
+				'db2_connect', 'db2_escape_string', 'db2_execute', 'db2_fetch_assoc', 'db2_free_result', 'db2_prepare',
+				'db2_rollback', 'db2_set_option', 'db2_stmt_errormsg'])) {
+			$allowed_db[ZBX_DB_DB2] = 'IBM DB2';
 		}
 
 		// Semaphore related functions are checked elsewhere. The 'false' is to prevent autoloading of the SQLite3 class.
-		if (class_exists('SQLite3', false) && zbx_is_callable(array('ftok', 'sem_acquire', 'sem_release', 'sem_get'))) {
-			$allowedDb[ZBX_DB_SQLITE3] = 'SQLite3';
+		if (class_exists('SQLite3', false) && zbx_is_callable(['ftok', 'sem_acquire', 'sem_get', 'sem_release'])) {
+			$allowed_db[ZBX_DB_SQLITE3] = 'SQLite3';
 		}
 
-		return $allowedDb;
+		return $allowed_db;
 	}
 
 	/**
@@ -287,13 +307,13 @@ class CFrontendSetup {
 				function_exists('bcsqrt') &&
 				function_exists('bcsub');
 
-		return array(
+		return [
 			'name' => _('PHP bcmath'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP bcmath extension missing (PHP configuration parameter --enable-bcmath).')
-		);
+		];
 	}
 
 	/**
@@ -304,13 +324,13 @@ class CFrontendSetup {
 	public function checkPhpMbstring() {
 		$current = extension_loaded('mbstring');
 
-		return array(
+		return [
 			'name' => _('PHP mbstring'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP mbstring extension missing (PHP configuration parameter --enable-mbstring).')
-		);
+		];
 	}
 
 	/**
@@ -325,13 +345,13 @@ class CFrontendSetup {
 	public function checkPhpMbstringFuncOverload() {
 		$current = ini_get('mbstring.func_overload');
 
-		return array(
-			'name' => _('PHP mbstring.func_overload'),
+		return [
+			'name' => _s('PHP option "%1$s"', 'mbstring.func_overload'),
 			'current' => ($current & 2) ? _('on') : _('off'),
 			'required' => _('off'),
 			'result' => ($current & 2) ? self::CHECK_FATAL : self::CHECK_OK,
 			'error' => _('PHP string function overloading must be disabled.')
-		);
+		];
 	}
 
 	/**
@@ -346,13 +366,13 @@ class CFrontendSetup {
 	public function checkPhpAlwaysPopulateRawPostData() {
 		$current = ini_get('always_populate_raw_post_data');
 
-		return array(
-			'name' => _('PHP always_populate_raw_post_data'),
+		return [
+			'name' => _s('PHP option "%1$s"', 'always_populate_raw_post_data'),
 			'current' => ($current != -1) ? _('on') : _('off'),
 			'required' => _('off'),
 			'result' => ($current != -1) ? self::CHECK_FATAL : self::CHECK_OK,
-			'error' => _('PHP always_populate_raw_post_data must be set to -1.')
-		);
+			'error' => _s('PHP option "%1$s" must be set to "%2$s"', 'always_populate_raw_post_data', -1)
+		];
 	}
 
 	/**
@@ -363,13 +383,13 @@ class CFrontendSetup {
 	public function checkPhpSockets() {
 		$current = function_exists('socket_create');
 
-		return array(
+		return [
 			'name' => _('PHP sockets'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP sockets extension missing (PHP configuration parameter --enable-sockets).')
-		);
+		];
 	}
 
 	/**
@@ -388,13 +408,13 @@ class CFrontendSetup {
 		}
 		$check = version_compare($current, self::MIN_PHP_GD_VERSION, '>=');
 
-		return array(
+		return [
 			'name' => _('PHP gd'),
 			'current' => $current,
 			'required' => self::MIN_PHP_GD_VERSION,
 			'result' => $check ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP gd extension missing (PHP configuration parameter --with-gd).')
-		);
+		];
 	}
 
 	/**
@@ -411,13 +431,13 @@ class CFrontendSetup {
 			$current = false;
 		}
 
-		return array(
+		return [
 			'name' => _('PHP gd PNG support'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP gd PNG image support missing.')
-		);
+		];
 	}
 
 	/**
@@ -436,13 +456,13 @@ class CFrontendSetup {
 			$current = false;
 		}
 
-		return array(
+		return [
 			'name' => _('PHP gd JPEG support'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP gd JPEG image support missing.')
-		);
+		];
 	}
 
 	/**
@@ -459,13 +479,13 @@ class CFrontendSetup {
 			$current = false;
 		}
 
-		return array(
+		return [
 			'name' => _('PHP gd FreeType support'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP gd FreeType support missing.')
-		);
+		];
 	}
 
 	/**
@@ -482,13 +502,13 @@ class CFrontendSetup {
 		}
 		$check = version_compare($current, self::MIN_PHP_LIBXML_VERSION, '>=');
 
-		return array(
+		return [
 			'name' => _('PHP libxml'),
 			'current' => $current,
 			'required' => self::MIN_PHP_LIBXML_VERSION,
 			'result' => $check ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP libxml extension missing.')
-		);
+		];
 	}
 
 	/**
@@ -499,13 +519,13 @@ class CFrontendSetup {
 	public function checkPhpXmlWriter() {
 		$current = extension_loaded('xmlwriter');
 
-		return array(
+		return [
 			'name' => _('PHP xmlwriter'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP xmlwriter extension missing.')
-		);
+		];
 	}
 
 	/**
@@ -516,13 +536,13 @@ class CFrontendSetup {
 	public function checkPhpXmlReader() {
 		$current = extension_loaded('xmlreader');
 
-		return array(
+		return [
 			'name' => _('PHP xmlreader'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP xmlreader extension missing.')
-		);
+		];
 	}
 
 	/**
@@ -543,13 +563,13 @@ class CFrontendSetup {
 				function_exists('ctype_xdigit') &&
 				function_exists('ctype_upper');
 
-		return array(
+		return [
 			'name' => _('PHP ctype'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP ctype extension missing (PHP configuration parameter --enable-ctype).')
-		);
+		];
 	}
 
 	/**
@@ -560,13 +580,13 @@ class CFrontendSetup {
 	public function checkPhpSession() {
 		$current = (function_exists('session_start') && function_exists('session_write_close'));
 
-		return array(
+		return [
 			'name' => _('PHP session'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP session extension missing (PHP configuration parameter --enable-session).')
-		);
+		];
 	}
 
 	/**
@@ -577,13 +597,13 @@ class CFrontendSetup {
 	public function checkPhpSessionAutoStart() {
 		$current = !ini_get('session.auto_start');
 
-		return array(
-			'name' => _('PHP session auto start'),
+		return [
+			'name' => _s('PHP option "%1$s"', 'session.auto_start'),
 			'current' => $current ? _('off') : _('on'),
 			'required' => _('off'),
 			'result' => $current ? self::CHECK_OK : self::CHECK_FATAL,
 			'error' => _('PHP session auto start must be disabled (PHP directive "session.auto_start").')
-		);
+		];
 	}
 
 	/**
@@ -594,12 +614,31 @@ class CFrontendSetup {
 	public function checkPhpGettext() {
 		$current = function_exists('bindtextdomain');
 
-		return array(
+		return [
 			'name' => _('PHP gettext'),
 			'current' => $current ? _('on') : _('off'),
 			'required' => null,
 			'result' => $current ? self::CHECK_OK : self::CHECK_WARNING,
 			'error' => _('PHP gettext extension missing (PHP configuration parameter --with-gettext). Translations will not be available.')
-		);
+		];
+	}
+
+	/**
+	 * Checks for arg_separator.output
+	 *
+	 * @return array
+	 */
+	public function checkPhpArgSeparatorOutput() {
+		$current = ini_get('arg_separator.output');
+
+		return [
+			'name' => _s('PHP option "%1$s"', 'arg_separator.output'),
+			'current' => htmlspecialchars($current),
+			'required' => htmlspecialchars(self::REQUIRED_PHP_ARG_SEPARATOR_OUTPUT),
+			'result' => ($current === self::REQUIRED_PHP_ARG_SEPARATOR_OUTPUT) ? self::CHECK_OK : self::CHECK_FATAL,
+			'error' => _s('PHP option "%1$s" must be set to "%2$s"', 'arg_separator.output',
+				self::REQUIRED_PHP_ARG_SEPARATOR_OUTPUT
+			)
+		];
 	}
 }
