@@ -37,11 +37,11 @@ class CWebUser {
 		try {
 			self::setDefault();
 
-			self::$data = API::User()->login(array(
+			self::$data = API::User()->login([
 				'user' => $login,
 				'password' => $password,
 				'userData' => true
-			));
+			]);
 
 			if (!self::$data) {
 				throw new Exception();
@@ -52,10 +52,6 @@ class CWebUser {
 				throw new Exception();
 			}
 
-			if (empty(self::$data['url'])) {
-				self::$data['url'] = CProfile::get('web.menu.view.last', 'index.php');
-			}
-
 			$result = (bool) self::$data;
 
 			if (isset(self::$data['attempt_failed']) && self::$data['attempt_failed']) {
@@ -86,22 +82,23 @@ class CWebUser {
 	public static function logout() {
 		self::$data['sessionid'] = self::getSessionCookie();
 		self::$data = API::User()->logout();
+		CSession::destroy();
 		zbx_unsetcookie('zbx_sessionid');
 	}
 
 	public static function checkAuthentication($sessionId) {
 		try {
 			if ($sessionId !== null) {
-				self::$data = API::User()->checkAuthentication(array($sessionId));
+				self::$data = API::User()->checkAuthentication([$sessionId]);
 			}
 
 			if ($sessionId === null || empty(self::$data)) {
 				self::setDefault();
-				self::$data = API::User()->login(array(
+				self::$data = API::User()->login([
 					'user' => ZBX_GUEST_USER,
 					'password' => '',
 					'userData' => true
-				));
+				]);
 
 				if (empty(self::$data)) {
 					clear_messages(1);
@@ -145,13 +142,13 @@ class CWebUser {
 	}
 
 	public static function setDefault() {
-		self::$data = array(
+		self::$data = [
 			'alias' => ZBX_GUEST_USER,
 			'userid' => 0,
 			'lang' => 'en_gb',
 			'type' => '0',
 			'debug_mode' => false
-		);
+		];
 	}
 
 	/**
