@@ -19,9 +19,10 @@
 **/
 
 
-define('ZABBIX_VERSION',		'2.4.8');
-define('ZABBIX_API_VERSION',	'2.4.8');
-define('ZABBIX_DB_VERSION',		2040000);
+define('ZABBIX_VERSION',		'3.0.1');
+define('ZABBIX_API_VERSION',	'3.0.1');
+define('ZABBIX_EXPORT_VERSION',	'3.0');
+define('ZABBIX_DB_VERSION',		3000000);
 
 define('ZABBIX_COPYRIGHT_FROM',	'2001');
 define('ZABBIX_COPYRIGHT_TO',	'2016');
@@ -29,7 +30,7 @@ define('ZABBIX_COPYRIGHT_TO',	'2016');
 define('ZBX_LOGIN_ATTEMPTS',	5);
 define('ZBX_LOGIN_BLOCK',		30); // sec
 
-define('ZBX_MIN_PERIOD',		3600); // 1 hour
+define('ZBX_MIN_PERIOD',		60); // 1 minute
 define('ZBX_MAX_PERIOD',		63072000); // the maximum period for the time bar control, ~2 years (2 * 365 * 86400)
 define('ZBX_MAX_DATE',			2147483647); // 19 Jan 2038 05:14:07
 define('ZBX_PERIOD_DEFAULT',	3600); // 1 hour
@@ -103,6 +104,9 @@ define('ZBX_DB_SQLITE3',	'SQLITE3');
 
 define('ZBX_DB_MAX_ID', '9223372036854775807');
 
+// maximum number of records for create() or update() API calls
+define('ZBX_DB_MAX_INSERTS', 10000);
+
 define('PAGE_TYPE_HTML',				0);
 define('PAGE_TYPE_IMAGE',				1);
 define('PAGE_TYPE_XML',					2);
@@ -128,6 +132,7 @@ define('T_ZBX_DBL',			2);
 define('T_ZBX_CLR',			5);
 define('T_ZBX_DBL_BIG',		9);
 define('T_ZBX_DBL_STR',		10);
+define('T_ZBX_TP',			11);
 
 define('O_MAND',	0);
 define('O_OPT',		1);
@@ -238,6 +243,12 @@ define('HOST_STATUS_TEMPLATE',		3);
 define('HOST_STATUS_PROXY_ACTIVE',	5);
 define('HOST_STATUS_PROXY_PASSIVE',	6);
 
+define('HOST_ENCRYPTION_NONE',			1);
+define('HOST_ENCRYPTION_PSK',			2);
+define('HOST_ENCRYPTION_CERTIFICATE',	4);
+
+define('PSK_MIN_LEN',	32);
+
 define('HOST_MAINTENANCE_STATUS_OFF',	0);
 define('HOST_MAINTENANCE_STATUS_ON',	1);
 
@@ -275,6 +286,16 @@ define('TIMEPERIOD_TYPE_WEEKLY',	3);
 define('TIMEPERIOD_TYPE_MONTHLY',	4);
 define('TIMEPERIOD_TYPE_YEARLY',	5);
 
+// report periods
+define('REPORT_PERIOD_TODAY',			0);
+define('REPORT_PERIOD_YESTERDAY',		1);
+define('REPORT_PERIOD_CURRENT_WEEK',	2);
+define('REPORT_PERIOD_CURRENT_MONTH',	3);
+define('REPORT_PERIOD_CURRENT_YEAR',	4);
+define('REPORT_PERIOD_LAST_WEEK',		5);
+define('REPORT_PERIOD_LAST_MONTH',		6);
+define('REPORT_PERIOD_LAST_YEAR',		7);
+
 define('SYSMAP_LABEL_ADVANCED_OFF',	0);
 define('SYSMAP_LABEL_ADVANCED_ON',	1);
 
@@ -322,6 +343,9 @@ define('SYSMAP_EXPAND_MACROS_ON',	1);
 define('SYSMAP_GRID_ALIGN_ON',	1);
 define('SYSMAP_GRID_ALIGN_OFF',	0);
 
+define('PUBLIC_SHARING',	0);
+define('PRIVATE_SHARING',	1);
+
 define('ZBX_ITEM_DELAY_DEFAULT', 30);
 
 define('ITEM_TYPE_ZABBIX',			0);
@@ -341,6 +365,7 @@ define('ITEM_TYPE_SSH',				13);
 define('ITEM_TYPE_TELNET',			14);
 define('ITEM_TYPE_CALCULATED',		15);
 define('ITEM_TYPE_JMX',				16);
+define('ITEM_TYPE_SNMPTRAP',		17);
 
 define('ITEM_VALUE_TYPE_FLOAT',		0);
 define('ITEM_VALUE_TYPE_STR',		1); // aka Character
@@ -353,10 +378,11 @@ define('ITEM_DATA_TYPE_OCTAL',			1);
 define('ITEM_DATA_TYPE_HEXADECIMAL',	2);
 define('ITEM_DATA_TYPE_BOOLEAN',		3);
 
-define('ZBX_DEFAULT_KEY_DB_MONITOR',	'db.odbc.select[<unique short description>,<dsn>]');
-define('ZBX_DEFAULT_KEY_SSH',			'ssh.run[<unique short description>,<ip>,<port>,<encoding>]');
-define('ZBX_DEFAULT_KEY_TELNET',		'telnet.run[<unique short description>,<ip>,<port>,<encoding>]');
-define('ZBX_DEFAULT_KEY_JMX',			'jmx[<object name>,<attribute name>]');
+define('ZBX_DEFAULT_KEY_DB_MONITOR',			'db.odbc.select[<unique short description>,<dsn>]');
+define('ZBX_DEFAULT_KEY_DB_MONITOR_DISCOVERY',	'db.odbc.discovery[<unique short description>,<dsn>]');
+define('ZBX_DEFAULT_KEY_SSH',					'ssh.run[<unique short description>,<ip>,<port>,<encoding>]');
+define('ZBX_DEFAULT_KEY_TELNET',				'telnet.run[<unique short description>,<ip>,<port>,<encoding>]');
+define('ZBX_DEFAULT_KEY_JMX',					'jmx[<object name>,<attribute name>]');
 
 define('SYSMAP_ELEMENT_USE_ICONMAP_ON',		1);
 define('SYSMAP_ELEMENT_USE_ICONMAP_OFF',	0);
@@ -371,8 +397,6 @@ define('ITEM_STATUS_NOTSUPPORTED',	3);
 define('ITEM_STATE_NORMAL',			0);
 define('ITEM_STATE_NOTSUPPORTED',	1);
 
-define('ITEM_TYPE_SNMPTRAP', 17);
-
 define('ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV',	0);
 define('ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV',		1);
 define('ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV',		2);
@@ -394,6 +418,9 @@ define('ITEM_LOGTYPE_SUCCESS_AUDIT',	8);
 define('ITEM_LOGTYPE_CRITICAL',			9);
 define('ITEM_LOGTYPE_VERBOSE',			10);
 
+define('ITEM_DELAY_FLEX_TYPE_FLEXIBLE',		0);
+define('ITEM_DELAY_FLEX_TYPE_SCHEDULING',	1);
+
 define('GRAPH_ITEM_DRAWTYPE_LINE',			0);
 define('GRAPH_ITEM_DRAWTYPE_FILLED_REGION',	1);
 define('GRAPH_ITEM_DRAWTYPE_BOLD_LINE',		2);
@@ -447,6 +474,9 @@ define('ALERT_STATUS_FAILED',	2);
 define('ALERT_TYPE_MESSAGE',	0);
 define('ALERT_TYPE_COMMAND',	1);
 
+define('MEDIA_STATUS_ACTIVE',	0);
+define('MEDIA_STATUS_DISABLED',	1);
+
 define('MEDIA_TYPE_STATUS_ACTIVE',		0);
 define('MEDIA_TYPE_STATUS_DISABLED',	1);
 
@@ -456,6 +486,13 @@ define('MEDIA_TYPE_SMS',		2);
 define('MEDIA_TYPE_JABBER',		3);
 define('MEDIA_TYPE_EZ_TEXTING',	100);
 
+define('SMTP_CONNECTION_SECURITY_NONE',		0);
+define('SMTP_CONNECTION_SECURITY_STARTTLS',	1);
+define('SMTP_CONNECTION_SECURITY_SSL_TLS',	2);
+
+define('SMTP_AUTHENTICATION_NONE',		0);
+define('SMTP_AUTHENTICATION_NORMAL',	1);
+
 define('EZ_TEXTING_LIMIT_USA',		0);
 define('EZ_TEXTING_LIMIT_CANADA',	1);
 
@@ -491,6 +528,7 @@ define('OPERATION_TYPE_TEMPLATE_ADD',	6);
 define('OPERATION_TYPE_TEMPLATE_REMOVE',7);
 define('OPERATION_TYPE_HOST_ENABLE',	8);
 define('OPERATION_TYPE_HOST_DISABLE',	9);
+define('OPERATION_TYPE_HOST_INVENTORY',	10);
 
 define('CONDITION_EVAL_TYPE_AND_OR',		0);
 define('CONDITION_EVAL_TYPE_AND',			1);
@@ -520,6 +558,8 @@ define('SCREEN_RESOURCE_HISTORY',			17);
 define('SCREEN_RESOURCE_CHART',				18);
 define('SCREEN_RESOURCE_LLD_SIMPLE_GRAPH',	19);
 define('SCREEN_RESOURCE_LLD_GRAPH',			20);
+// used in Monitoring > Web > Details (httpdetails.php)
+define('SCREEN_RESOURCE_HTTPTEST_DETAILS',		21);
 
 define('SCREEN_SORT_TRIGGERS_DATE_DESC',			0);
 define('SCREEN_SORT_TRIGGERS_SEVERITY_DESC',		1);
@@ -631,6 +671,9 @@ define('PERM_DENY',			0);
 define('PARAM_TYPE_TIME',		0);
 define('PARAM_TYPE_COUNTS',		1);
 
+define('ZBX_DEFAULT_AGENT', 'Zabbix');
+define('ZBX_AGENT_OTHER', -1);
+
 define('HTTPTEST_AUTH_NONE',	0);
 define('HTTPTEST_AUTH_BASIC',	1);
 define('HTTPTEST_AUTH_NTLM',	2);
@@ -662,6 +705,10 @@ define('EVENT_ACK_ENABLED',		'1');
 define('EVENT_NOT_ACKNOWLEDGED',	'0');
 define('EVENT_ACKNOWLEDGED',		'1');
 
+define('ZBX_ACKNOWLEDGE_SELECTED',	0);
+define('ZBX_ACKNOWLEDGE_PROBLEM',	1);
+define('ZBX_ACKNOWLEDGE_ALL',		2);
+
 define('EVENT_SOURCE_TRIGGERS',				0);
 define('EVENT_SOURCE_DISCOVERY',			1);
 define('EVENT_SOURCE_AUTO_REGISTRATION',	2);
@@ -706,7 +753,7 @@ define('GRAPH_STACKED_ALFA', 15); // 0..100 transparency
 define('GRAPH_ZERO_LINE_COLOR_LEFT',	'AAAAAA');
 define('GRAPH_ZERO_LINE_COLOR_RIGHT',	'888888');
 
-define('GRAPH_TRIGGER_LINE_OPPOSITE_COLOR', '000');
+define('GRAPH_TRIGGER_LINE_OPPOSITE_COLOR', '000000');
 
 define('ZBX_MAX_TREND_DIFF', 3600);
 
@@ -775,7 +822,7 @@ define('SPACE',	'&nbsp;');
  */
 define('NAME_DELIMITER', ': ');
 
-define('UNKNOWN_VALUE', '-');
+define('UNKNOWN_VALUE', '');
 
 // suffixes
 define('ZBX_BYTE_SUFFIXES', 'KMGT');
@@ -792,7 +839,6 @@ define('ZBX_PREG_DEF_FONT_STRING', '/^[0-9\.:% ]+$/');
 define('ZBX_PREG_DNS_FORMAT', '([0-9a-zA-Z_\.\-$]|\{\$?'.ZBX_PREG_MACRO_NAME.'\})*');
 define('ZBX_PREG_HOST_FORMAT', ZBX_PREG_INTERNAL_NAMES);
 define('ZBX_PREG_MACRO_NAME_FORMAT', '(\{[A-Z\.]+\})');
-define('ZBX_PREG_EXPRESSION_USER_MACROS', '(\{\$'.ZBX_PREG_MACRO_NAME.'\})');
 define('ZBX_PREG_EXPRESSION_LLD_MACROS', '(\{\#'.ZBX_PREG_MACRO_NAME_LLD.'\})');
 
 // !!! should be used with "x" modifier
@@ -853,6 +899,8 @@ define('ZBX_SOCKET_BYTES_LIMIT',    1048576);   // socket response size limit, 1
 // value is also used in servercheck.js file
 define('SERVER_CHECK_INTERVAL', 10);
 
+define('DATE_TIME_FORMAT_SECONDS_XML', 'Y-m-d\TH:i:s\Z');
+
 // XML export|import tags
 define('XML_TAG_MACRO',				'macro');
 define('XML_TAG_HOST',				'host');
@@ -873,6 +921,12 @@ define('ZBX_DEFAULT_IMPORT_HOST_GROUP', 'Imported hosts');
 // so this flag is not needed here.
 define('LIBXML_IMPORT_FLAGS', LIBXML_NONET);
 
+// XML validation
+define('XML_STRING',		0x01);
+define('XML_ARRAY',			0x02);
+define('XML_INDEXED_ARRAY',	0x04);
+define('XML_REQUIRED',		0x08);
+
 // API errors
 define('ZBX_API_ERROR_INTERNAL',	111);
 define('ZBX_API_ERROR_PARAMETERS',	100);
@@ -883,12 +937,12 @@ define('ZBX_API_ERROR_NO_METHOD',	300);
 define('API_OUTPUT_EXTEND',		'extend');
 define('API_OUTPUT_COUNT',		'count');
 
-define('SEC_PER_MIN',	60);
-define('SEC_PER_HOUR',	3600);
-define('SEC_PER_DAY',	86400);
-define('SEC_PER_WEEK',	604800); // 7 * SEC_PER_DAY
-define('SEC_PER_MONTH',	2592000); // 30 * SEC_PER_DAY
-define('SEC_PER_YEAR',	31536000); // 365 * SEC_PER_DAY
+define('SEC_PER_MIN',			60);
+define('SEC_PER_HOUR',			3600);
+define('SEC_PER_DAY',			86400);
+define('SEC_PER_WEEK',			604800);
+define('SEC_PER_MONTH',			2592000);
+define('SEC_PER_YEAR',			31536000);
 
 define('ZBX_JAN_2038', 2145916800);
 
@@ -898,24 +952,55 @@ define('ZBX_MIN_PORT_NUMBER', 0);
 define('ZBX_MAX_PORT_NUMBER', 65535);
 
 // input fields
-define('ZBX_TEXTBOX_STANDARD_SIZE',		50);
-define('ZBX_TEXTBOX_SMALL_SIZE',		25);
-define('ZBX_TEXTBOX_FILTER_SIZE',		20);
-define('ZBX_TEXTAREA_STANDARD_WIDTH',	312);
-define('ZBX_TEXTAREA_BIG_WIDTH',		524);
-define('ZBX_TEXTAREA_STANDARD_ROWS',	7);
+define('ZBX_TEXTAREA_MACRO_WIDTH',				200);
+define('ZBX_TEXTAREA_MACRO_VALUE_WIDTH',		250);
+define('ZBX_TEXTAREA_COLOR_WIDTH',				96);
+define('ZBX_TEXTAREA_FILTER_SMALL_WIDTH',		150);
+define('ZBX_TEXTAREA_FILTER_STANDARD_WIDTH',	300);
+define('ZBX_TEXTAREA_FILTER_BIG_WIDTH',			524);
+define('ZBX_TEXTAREA_TINY_WIDTH',				75);
+define('ZBX_TEXTAREA_SMALL_WIDTH',				150);
+define('ZBX_TEXTAREA_STANDARD_WIDTH',			300);
+define('ZBX_TEXTAREA_BIG_WIDTH',				524);
+define('ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH',	75);
+define('ZBX_TEXTAREA_NUMERIC_BIG_WIDTH',		150);
+define('ZBX_TEXTAREA_2DIGITS_WIDTH',			35);	// please use for date selector only
+define('ZBX_TEXTAREA_4DIGITS_WIDTH',			50);	// please use for date selector only
+define('ZBX_TEXTAREA_INTERFACE_IP_WIDTH',		225);
+define('ZBX_TEXTAREA_INTERFACE_DNS_WIDTH',		175);
+define('ZBX_TEXTAREA_INTERFACE_USEIP_WIDTH',	100);
+define('ZBX_TEXTAREA_INTERFACE_PORT_WIDTH',		100);
+define('ZBX_TEXTAREA_STANDARD_ROWS',			7);
 
-// widgets
-define('WIDGET_DISCOVERY_STATUS',	'dscvry');
-define('WIDGET_FAVOURITE_GRAPHS',	'favgrph');
-define('WIDGET_FAVOURITE_MAPS',		'favmap');
-define('WIDGET_FAVOURITE_SCREENS',	'favscr');
-define('WIDGET_HOST_STATUS',		'hoststat');
-define('WIDGET_LAST_ISSUES',		'lastiss');
-define('WIDGET_SLIDESHOW',			'hat_slides');
-define('WIDGET_SYSTEM_STATUS',		'syssum');
-define('WIDGET_WEB_OVERVIEW',		'webovr');
-define('WIDGET_ZABBIX_STATUS',		'stszbx');
+// decoration borders
+define('ZBX_HOST_INTERFACE_WIDTH',				750);
+
+// overviews help
+define('ZBX_OVERVIEW_HELP_MIN_WIDTH',			125);
+
+// dashboard widgets
+define('WIDGET_DISCOVERY_STATUS',		'dscvry');
+define('WIDGET_FAVOURITE_GRAPHS',		'favgrph');
+define('WIDGET_FAVOURITE_MAPS',			'favmap');
+define('WIDGET_FAVOURITE_SCREENS',		'favscr');
+define('WIDGET_HOST_STATUS',			'hoststat');
+define('WIDGET_LAST_ISSUES',			'lastiss');
+define('WIDGET_SYSTEM_STATUS',			'syssum');
+define('WIDGET_WEB_OVERVIEW',			'webovr');
+define('WIDGET_ZABBIX_STATUS',			'stszbx');
+// event details widgets
+define('WIDGET_HAT_TRIGGERDETAILS',		'hat_triggerdetails');
+define('WIDGET_HAT_EVENTDETAILS',		'hat_eventdetails');
+define('WIDGET_HAT_EVENTACK',			'hat_eventack');
+define('WIDGET_HAT_EVENTACTIONMSGS',	'hat_eventactionmsgs');
+define('WIDGET_HAT_EVENTACTIONMCMDS',	'hat_eventactionmcmds');
+define('WIDGET_HAT_EVENTLIST',			'hat_eventlist');
+// search widget
+define('WIDGET_SEARCH_HOSTS',			'search_hosts');
+define('WIDGET_SEARCH_HOSTGROUP',		'search_hostgroup');
+define('WIDGET_SEARCH_TEMPLATES',		'search_templates');
+// slideshow
+define('WIDGET_SLIDESHOW',				'hat_slides');
 
 // validation
 define('DB_ID',		"({}>=0&&bccomp({},\"9223372036854775807\")<=0)&&");
@@ -930,9 +1015,10 @@ define('ZBX_VALID_WARNING',	2);
 define('THEME_DEFAULT', 'default');
 
 // the default theme
-define('ZBX_DEFAULT_THEME', 'originalblue');
+define('ZBX_DEFAULT_THEME', 'blue-theme');
 
 define('ZABBIX_HOMEPAGE', 'http://www.zabbix.com');
+define('ZBX_DEFAULT_URL', 'zabbix.php?action=dashboard.view');
 
 // non translatable date formats
 define('TIMESTAMP_FORMAT', 'YmdHis');
@@ -941,10 +1027,6 @@ define('TIMESTAMP_FORMAT_ZERO_TIME', 'Ymd0000');
 // date format context, usable for translators
 define('DATE_FORMAT_CONTEXT', 'Date format (see http://php.net/date)');
 
-// actions
-define('LONG_DESCRIPTION',	0);
-define('SHORT_DESCRIPTION',	1);
-
 // availability report modes
 define('AVAILABILITY_REPORT_BY_HOST', 0);
 define('AVAILABILITY_REPORT_BY_TEMPLATE', 1);
@@ -970,9 +1052,189 @@ define('HISTORY_LATEST', 'showlatest');
 // configuration -> maps default add icon name
 define('MAP_DEFAULT_ICON', 'Server_(96)');
 
+// CSS styles
+define('ZBX_STYLE_ACTION_BUTTONS', 'action-buttons');
+define('ZBX_STYLE_ACTIVE_INDIC', 'active-indic');
+define('ZBX_STYLE_ACTIVE_BG', 'active-bg');
+define('ZBX_STYLE_ADM_IMG', 'adm-img');
+define('ZBX_STYLE_ARTICLE', 'article');
+define('ZBX_STYLE_AVERAGE_BG', 'average-bg');
+define('ZBX_STYLE_ARROW_DOWN', 'arrow-down');
+define('ZBX_STYLE_ARROW_LEFT', 'arrow-left');
+define('ZBX_STYLE_ARROW_RIGHT', 'arrow-right');
+define('ZBX_STYLE_ARROW_UP', 'arrow-up');
+define('ZBX_STYLE_BLUE', 'blue');
+define('ZBX_STYLE_BTN_ADD_FAV', 'btn-add-fav');
+define('ZBX_STYLE_BTN_ALT', 'btn-alt');
+define('ZBX_STYLE_BTN_CONF', 'btn-conf');
+define('ZBX_STYLE_BTN_DEBUG', 'btn-debug');
+define('ZBX_STYLE_BTN_GREY', 'btn-grey');
+define('ZBX_STYLE_BTN_INFO', 'btn-info');
+define('ZBX_STYLE_BTN_LINK', 'btn-link');
+define('ZBX_STYLE_BTN_MAX', 'btn-max');
+define('ZBX_STYLE_BTN_MIN', 'btn-min');
+define('ZBX_STYLE_BTN_REMOVE_FAV', 'btn-remove-fav');
+define('ZBX_STYLE_BTN_RESET', 'btn-reset');
+define('ZBX_STYLE_BTN_SEARCH', 'btn-search');
+define('ZBX_STYLE_BTN_WIDGET_ACTION', 'btn-widget-action');
+define('ZBX_STYLE_BTN_WIDGET_COLLAPSE', 'btn-widget-collapse');
+define('ZBX_STYLE_BTN_WIDGET_EXPAND', 'btn-widget-expand');
+define('ZBX_STYLE_BOTTOM', 'bottom');
+define('ZBX_STYLE_BROWSER_LOGO_CHROME', 'browser-logo-chrome');
+define('ZBX_STYLE_BROWSER_LOGO_FF', 'browser-logo-ff');
+define('ZBX_STYLE_BROWSER_LOGO_IE', 'browser-logo-ie');
+define('ZBX_STYLE_BROWSER_LOGO_OPERA', 'browser-logo-opera');
+define('ZBX_STYLE_BROWSER_LOGO_SAFARI', 'browser-logo-safari');
+define('ZBX_STYLE_BROWSER_WARNING_CONTAINER', 'browser-warning-container');
+define('ZBX_STYLE_BROWSER_WARNING_FOOTER', 'browser-warning-footer');
+define('ZBX_STYLE_CELL', 'cell');
+define('ZBX_STYLE_CELL_WIDTH', 'cell-width');
+define('ZBX_STYLE_CENTER', 'center');
+define('ZBX_STYLE_CLOCK', 'clock');
+define('ZBX_STYLE_CLOCK_FACE', 'clock-face');
+define('ZBX_STYLE_CLOCK_HAND', 'clock-hand');
+define('ZBX_STYLE_CLOCK_HAND_SEC', 'clock-hand-sec');
+define('ZBX_STYLE_CLOCK_LINES', 'clock-lines');
+define('ZBX_STYLE_COLOR_PICKER', 'color-picker');
+define('ZBX_STYLE_CURSOR_MOVE', 'cursor-move');
+define('ZBX_STYLE_CURSOR_POINTER', 'cursor-pointer');
+define('ZBX_STYLE_DASHBRD_WIDGET_HEAD', 'dashbrd-widget-head');
+define('ZBX_STYLE_DASHBRD_WIDGET_FOOT', 'dashbrd-widget-foot');
+define('ZBX_STYLE_DASHED_BORDER', 'dashed-border');
+define('ZBX_STYLE_DEBUG_OUTPUT', 'debug-output');
+define('ZBX_STYLE_DISABLED', 'disabled');
+define('ZBX_STYLE_DISASTER_BG', 'disaster-bg');
+define('ZBX_STYLE_DRAG_ICON', 'drag-icon');
+define('ZBX_STYLE_DRAG_DROP_AREA', 'drag-drop-area');
+define('ZBX_STYLE_TABLE_FORMS_SEPARATOR', 'table-forms-separator');
+define('ZBX_STYLE_FILTER_CONTAINER', 'filter-container');
+define('ZBX_STYLE_FILTER_BTN_CONTAINER', 'filter-btn-container');
+define('ZBX_STYLE_FILTER_FORMS', 'filter-forms');
+define('ZBX_STYLE_FILTER_TRIGGER', 'filter-trigger');
+define('ZBX_STYLE_FILTER_ACTIVE', 'filter-active');
+define('ZBX_STYLE_FLOAT_LEFT', 'float-left');
+define('ZBX_STYLE_FORM_INPUT_MARGIN', 'form-input-margin');
+define('ZBX_STYLE_FORM_NEW_GROUP', 'form-new-group');
+define('ZBX_STYLE_FOOTER', 'footer');
+define('ZBX_STYLE_GREEN', 'green');
+define('ZBX_STYLE_GREEN_BG', 'green-bg');
+define('ZBX_STYLE_GREY', 'grey');
+define('ZBX_STYLE_HEADER_TITLE', 'header-title');
+define('ZBX_STYLE_HIDDEN', 'hidden');
+define('ZBX_STYLE_HIGH_BG', 'high-bg');
+define('ZBX_STYLE_HOR_LIST', 'hor-list');
+define('ZBX_STYLE_ICON_ACKN', 'icon-ackn');
+define('ZBX_STYLE_ICON_CAL', 'icon-cal');
+define('ZBX_STYLE_ICON_DEPEND_DOWN', 'icon-depend-down');
+define('ZBX_STYLE_ICON_DEPEND_UP', 'icon-depend-up');
+define('ZBX_STYLE_ICON_MAINT', 'icon-maint');
+define('ZBX_STYLE_ICON_WZRD_ACTION', 'icon-wzrd-action');
+define('ZBX_STYLE_INACTIVE_BG', 'inactive-bg');
+define('ZBX_STYLE_INFO_BG', 'info-bg');
+define('ZBX_STYLE_INPUT_COLOR_PICKER', 'input-color-picker');
+define('ZBX_STYLE_LEFT', 'left');
+define('ZBX_STYLE_LINK_ACTION', 'link-action');
+define('ZBX_STYLE_LINK_ALT', 'link-alt');
+define('ZBX_STYLE_LIST_HOR_CHECK_RADIO', 'list-hor-check-radio');
+define('ZBX_STYLE_LIST_HOR_MIN_WIDTH', 'list-hor-min-width');
+define('ZBX_STYLE_LIST_CHECK_RADIO', 'list-check-radio');
+define('ZBX_STYLE_LIST_TABLE', 'list-table');
+define('ZBX_STYLE_LOCAL_CLOCK', 'local-clock');
+define('ZBX_STYLE_LOG_NA_BG', 'log-na-bg');
+define('ZBX_STYLE_LOG_INFO_BG', 'log-info-bg');
+define('ZBX_STYLE_LOG_WARNING_BG', 'log-warning-bg');
+define('ZBX_STYLE_LOG_HIGH_BG', 'log-high-bg');
+define('ZBX_STYLE_LOG_DISASTER_BG', 'log-disaster-bg');
+define('ZBX_STYLE_LOGO', 'logo');
+define('ZBX_STYLE_MAP_AREA', 'map-area');
+define('ZBX_STYLE_MIDDLE', 'middle');
+define('ZBX_STYLE_MSG_GOOD', 'msg-good');
+define('ZBX_STYLE_MSG_BAD', 'msg-bad');
+define('ZBX_STYLE_MSG_BAD_GLOBAL', 'msg-bad-global');
+define('ZBX_STYLE_MSG_DETAILS', 'msg-details');
+define('ZBX_STYLE_MSG_DETAILS_BORDER', 'msg-details-border');
+define('ZBX_STYLE_NA_BG', 'na-bg');
+define('ZBX_STYLE_NAV', 'nav');
+define('ZBX_STYLE_NORMAL_BG', 'normal-bg');
+define('ZBX_STYLE_NOTIF_BODY', 'notif-body');
+define('ZBX_STYLE_NOTIF_INDIC', 'notif-indic');
+define('ZBX_STYLE_NOTIF_INDIC_CONTAINER', 'notif-indic-container');
+define('ZBX_STYLE_NOTHING_TO_SHOW', 'nothing-to-show');
+define('ZBX_STYLE_NOWRAP', 'nowrap');
+define('ZBX_STYLE_ORANGE', 'orange');
+define('ZBX_STYLE_OVERLAY_CLOSE_BTN', 'overlay-close-btn');
+define('ZBX_STYLE_OVERLAY_DESCR', 'overlay-descr');
+define('ZBX_STYLE_OVERLAY_DESCR_URL', 'overlay-descr-url');
+define('ZBX_STYLE_OVERFLOW_ELLIPSIS', 'overflow-ellipsis');
+define('ZBX_STYLE_OBJECT_GROUP', 'object-group');
+define('ZBX_STYLE_PAGING_BTN_CONTAINER', 'paging-btn-container');
+define('ZBX_STYLE_PAGING_SELECTED', 'paging-selected');
+define('ZBX_STYLE_PROGRESS_BAR_BG', 'progress-bar-bg');
+define('ZBX_STYLE_PROGRESS_BAR_CONTAINER', 'progress-bar-container');
+define('ZBX_STYLE_PROGRESS_BAR_LABEL', 'progress-bar-label');
+define('ZBX_STYLE_RADIO_SEGMENTED', 'radio-segmented');
+define('ZBX_STYLE_RED', 'red');
+define('ZBX_STYLE_RED_BG', 'red-bg');
+define('ZBX_STYLE_REL_CONTAINER', 'rel-container');
+define('ZBX_STYLE_RIGHT', 'right');
+define('ZBX_STYLE_ROW', 'row');
+define('ZBX_STYLE_SCREEN_TABLE', 'screen-table');
+define('ZBX_STYLE_SEARCH', 'search');
+define('ZBX_STYLE_SELECTED', 'selected');
+define('ZBX_STYLE_SELECTED_ITEM_COUNT', 'selected-item-count');
+define('ZBX_STYLE_SERVER_NAME', 'server-name');
+define('ZBX_STYLE_SETUP_CONTAINER', 'setup-container');
+define('ZBX_STYLE_SETUP_FOOTER', 'setup-footer');
+define('ZBX_STYLE_SETUP_LEFT', 'setup-left');
+define('ZBX_STYLE_SETUP_LEFT_CURRENT', 'setup-left-current');
+define('ZBX_STYLE_SETUP_RIGHT', 'setup-right');
+define('ZBX_STYLE_SETUP_RIGHT_BODY', 'setup-right-body');
+define('ZBX_STYLE_SETUP_TITLE', 'setup-title');
+define('ZBX_STYLE_SIGNIN_CONTAINER', 'signin-container');
+define('ZBX_STYLE_SIGNIN_LINKS', 'signin-links');
+define('ZBX_STYLE_SIGNIN_LOGO', 'signin-logo');
+define('ZBX_STYLE_SIGN_IN_TXT', 'sign-in-txt');
+define('ZBX_STYLE_STATUS_CONTAINER', 'status-container');
+define('ZBX_STYLE_STATUS_GREEN', 'status-green');
+define('ZBX_STYLE_STATUS_GREY', 'status-grey');
+define('ZBX_STYLE_STATUS_RED', 'status-red');
+define('ZBX_STYLE_STATUS_YELLOW', 'status-yellow');
+define('ZBX_STYLE_SUBFILTER_ENABLED', 'subfilter-enabled');
+define('ZBX_STYLE_TABLE', 'table');
+define('ZBX_STYLE_TABLE_FORMS', 'table-forms');
+define('ZBX_STYLE_TABLE_FORMS_CONTAINER', 'table-forms-container');
+define('ZBX_STYLE_TABLE_FORMS_TD_LEFT', 'table-forms-td-left');
+define('ZBX_STYLE_TABLE_FORMS_TD_RIGHT', 'table-forms-td-right');
+define('ZBX_STYLE_TABLE_PAGING', 'table-paging');
+define('ZBX_STYLE_TABLE_STATS', 'table-stats');
+define('ZBX_STYLE_TABS_NAV', 'tabs-nav');
+define('ZBX_STYLE_TFOOT_BUTTONS', 'tfoot-buttons');
+define('ZBX_STYLE_TD_DRAG_ICON', 'td-drag-icon');
+define('ZBX_STYLE_TIME_ZONE', 'time-zone');
+define('ZBX_STYLE_TOP', 'top');
+define('ZBX_STYLE_TOP_NAV', 'top-nav');
+define('ZBX_STYLE_TOP_NAV_CONTAINER', 'top-nav-container');
+define('ZBX_STYLE_TOP_NAV_HELP', 'top-nav-help');
+define('ZBX_STYLE_TOP_NAV_ICONS', 'top-nav-icons');
+define('ZBX_STYLE_TOP_NAV_PROFILE', 'top-nav-profile');
+define('ZBX_STYLE_TOP_NAV_SIGNOUT', 'top-nav-signout');
+define('ZBX_STYLE_TOP_NAV_ZBBSHARE', 'top-nav-zbbshare');
+define('ZBX_STYLE_TOP_SUBNAV', 'top-subnav');
+define('ZBX_STYLE_TOP_SUBNAV_CONTAINER', 'top-subnav-container');
+define('ZBX_STYLE_TREEVIEW', 'treeview');
+define('ZBX_STYLE_TREEVIEW_PLUS', 'treeview-plus');
+define('ZBX_STYLE_UPPERCASE', 'uppercase');
+define('ZBX_STYLE_WARNING_BG', 'warning-bg');
+define('ZBX_STYLE_YELLOW', 'yellow');
+
 // server variables
 define('HTTPS', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] && $_SERVER['HTTPS'] != 'off');
 
+// configuration -> hosts (macro inheritance)
+define('MACRO_TYPE_INHERITED',	0x01);
+define('MACRO_TYPE_HOSTMACRO',	0x02);
+define('MACRO_TYPE_BOTH',		0x03);	// MACRO_TYPE_INHERITED | MACRO_TYPE_HOSTMACRO
+
 // if magic quotes on, then get rid of them
 if (get_magic_quotes_gpc()) {
 	function zbx_stripslashes($value) {
