@@ -69,16 +69,19 @@ jQuery(function($) {
 			ajaxUrl.setArgument('mode', screen.mode);
 			ajaxUrl.setArgument('timestamp', screen.timestampActual);
 			ajaxUrl.setArgument('flickerfreeScreenId', id);
-			ajaxUrl.setArgument('pageFile', screen.pageFile);
-			ajaxUrl.setArgument('screenid', screen.screenid);
-			ajaxUrl.setArgument('screenitemid', screen.screenitemid);
-			ajaxUrl.setArgument('groupid', screen.groupid);
-			ajaxUrl.setArgument('hostid', screen.hostid);
-			ajaxUrl.setArgument('profileIdx', empty(screen.profileIdx) ? null : screen.profileIdx);
 			ajaxUrl.setArgument('profileIdx2', empty(screen.profileIdx2) ? null : screen.profileIdx2);
-			ajaxUrl.setArgument('updateProfile', empty(screen.updateProfile) ? null : +screen.updateProfile);
-			ajaxUrl.setArgument('period', empty(screen.timeline.period) ? null : screen.timeline.period);
-			ajaxUrl.setArgument('stime', this.getCalculatedSTime(screen));
+
+			if (screen.resourcetype != 21) {
+				ajaxUrl.setArgument('pageFile', screen.pageFile);
+				ajaxUrl.setArgument('screenid', screen.screenid);
+				ajaxUrl.setArgument('screenitemid', screen.screenitemid);
+				ajaxUrl.setArgument('groupid', screen.groupid);
+				ajaxUrl.setArgument('hostid', screen.hostid);
+				ajaxUrl.setArgument('profileIdx', empty(screen.profileIdx) ? null : screen.profileIdx);
+				ajaxUrl.setArgument('updateProfile', empty(screen.updateProfile) ? null : +screen.updateProfile);
+				ajaxUrl.setArgument('period', empty(screen.timeline.period) ? null : screen.timeline.period);
+				ajaxUrl.setArgument('stime', this.getCalculatedSTime(screen));
+			}
 
 			// SCREEN_RESOURCE_GRAPH
 			// SCREEN_RESOURCE_SIMPLE_GRAPH
@@ -163,6 +166,12 @@ jQuery(function($) {
 				}
 			}
 
+			// SCREEN_RESOURCE_HTTPTEST_DETAILS
+			else if (screen.resourcetype == 21) {
+				ajaxUrl.setArgument('resourcetype', empty(screen.resourcetype) ? null : screen.resourcetype);
+				this.refreshHtml(id, ajaxUrl);
+			}
+
 			// others
 			else {
 				this.refreshHtml(id, ajaxUrl);
@@ -189,7 +198,7 @@ jQuery(function($) {
 			for (var id in this.screens) {
 				var screen = this.screens[id];
 
-				if (!empty(screen.id)) {
+				if (!empty(screen.id) && typeof screen.timeline !== 'undefined') {
 					screen.timeline.period = period;
 					screen.timeline.stime = stime;
 					screen.timeline.isNow = isNow;
@@ -556,7 +565,7 @@ jQuery(function($) {
 					}
 
 					// show loading indicator..
-					obj.append($('<div>', {'class': 'loading'})
+					obj.append($('<div>', {'class': 'preloader'})
 						.css({
 							width: '24px',
 							height: '24px',
@@ -566,15 +575,6 @@ jQuery(function($) {
 							left: item.position().left + Math.round(item.width() / 2) - 12
 						})
 					);
-					obj.find('.loading').activity({
-						segments: 12,
-						steps: 3,
-						opacity: 0.3,
-						width: 2,
-						space: 0,
-						length: 5,
-						color: '#0b0b0b'
-					});
 
 					timer.isShadowed = true;
 				}
@@ -591,7 +591,7 @@ jQuery(function($) {
 					return;
 				}
 
-				obj.find('.loading').remove();
+				obj.find('.preloader').remove();
 				obj.find('.shadow').remove();
 				obj.find(item.prop('nodeName')).fadeTo(0, 1);
 
@@ -621,10 +621,10 @@ jQuery(function($) {
 				}
 
 				// loading indicator
-				var loading = obj.find('.loading');
+				var preloader = obj.find('.preloader');
 
-				if (loading.length > 0) {
-					loading.css({
+				if (preloader.length > 0) {
+					preloader.css({
 						top: item.position().top + Math.round(item.height() / 2) - 12,
 						left: item.position().left + Math.round(item.width() / 2) - 12
 					});
