@@ -119,41 +119,56 @@ function getMenuPopupFavouriteData(label, data, favouriteObj, addParams) {
 				clickCallback: function() {
 					var obj = jQuery(this);
 
-					rm4favorites(favouriteObj, item.id);
+					sendAjaxData('zabbix.php?action=dashboard.favourite&operation=delete', {
+						data: {
+							object: favouriteObj,
+							'objectids[]': [item.id]
+						}
+					});
 
-					obj.closest('.menuPopup').fadeOut(100);
+					obj.closest('.action-menu').fadeOut(100);
 					obj.remove();
 				}
 			};
 		});
 	}
 
+	var add = {
+			label: t('Add'),
+			clickCallback: function() {
+				PopUp(addParams);
+
+				jQuery(this).closest('.action-menu').fadeOut(100);
+			}
+		},
+		remove = {
+			label: t('Remove')
+		},
+		remove_all = {
+			label: t('Remove all')
+		};
+
+	if (removeItems.length != 0) {
+		remove.items = removeItems;
+		remove_all.clickCallback = function() {
+			sendAjaxData('zabbix.php?action=dashboard.favourite&operation=delete', {
+				data: {
+					object: favouriteObj,
+					'objectids[]': [0]
+				}
+			});
+
+			jQuery(this).closest('.action-menu').fadeOut(100);
+		};
+	}
+	else {
+		remove.disabled = true;
+		remove_all.disabled = true;
+	}
+
 	return {
 		label: label,
-		items: [
-			{
-				label: t('Add'),
-				clickCallback: function() {
-					PopUp(addParams, 800, 450);
-
-					jQuery(this).closest('.menuPopup').fadeOut(100);
-				}
-			},
-			{
-				label: t('Remove'),
-				css: (removeItems.length == 0) ? 'ui-state-disabled' : '',
-				items: removeItems
-			},
-			{
-				label: t('Remove all'),
-				css: (removeItems.length == 0) ? 'ui-state-disabled' : '',
-				clickCallback: function() {
-					rm4favorites(favouriteObj, 0);
-
-					jQuery(this).closest('.menuPopup').fadeOut(100);
-				}
-			}
-		]
+		items: [add, remove, remove_all]
 	};
 }
 
@@ -226,40 +241,56 @@ function getMenuPopupHost(options) {
 
 	// go to section
 	if (options.hasGoTo) {
-		var gotos = [];
+		var gotos = [],
+			// inventory
+			host_inventory = {
+				label: t('Host inventory'),
+				url: new Curl('hostinventories.php?hostid=' + options.hostid).getUrl()
+			},
+			// latest
+			latest_data = {
+				label: t('Latest data'),
+				url: new Curl('latest.php?filter_set=1&hostids[]=' + options.hostid).getUrl()
+			},
+			// triggers
+			triggers = {
+				label: t('Triggers')
+			},
+			// graphs
+			graphs = {
+				label: t('Graphs')
+			},
+			// screens
+			screens = {
+				label: t('Host screens')
+			};
 
-		// inventory
-		gotos[gotos.length] = {
-			label: t('Host inventory'),
-			url: new Curl('hostinventories.php?hostid=' + options.hostid).getUrl()
-		};
+		if (!options.showTriggers) {
+			triggers.disabled = true;
+		}
+		else {
+			triggers.url = new Curl('tr_status.php?hostid=' + options.hostid).getUrl();
+		}
 
-		// latest
-		gotos[gotos.length] = {
-			label: t('Latest data'),
-			url: new Curl('latest.php?filter_set=1&hostids[]=' + options.hostid).getUrl()
-		};
+		if (!options.showGraphs) {
+			graphs.disabled = true;
+		}
+		else {
+			graphs.url = new Curl('charts.php?hostid=' + options.hostid).getUrl();
+		}
 
-		// triggers
-		gotos[gotos.length] = {
-			label: t('Triggers'),
-			css: options.showTriggers ? '' : 'ui-state-disabled',
-			url: new Curl('tr_status.php?hostid=' + options.hostid).getUrl()
-		};
+		if (!options.showScreens) {
+			screens.disabled = true;
+		}
+		else {
+			screens.url = new Curl('host_screen.php?hostid=' + options.hostid).getUrl();
+		}
 
-		// graphs
-		gotos[gotos.length] = {
-			label: t('Graphs'),
-			css: options.showGraphs ? '' : 'ui-state-disabled',
-			url: new Curl('charts.php?hostid=' + options.hostid).getUrl()
-		};
-
-		// screens
-		gotos[gotos.length] = {
-			label: t('Host screens'),
-			css: options.showScreens ?  '' : 'ui-state-disabled',
-			url: new Curl('host_screen.php?hostid=' + options.hostid).getUrl()
-		};
+		gotos[gotos.length] = host_inventory;
+		gotos[gotos.length] = latest_data;
+		gotos[gotos.length] = triggers;
+		gotos[gotos.length] = graphs;
+		gotos[gotos.length] = screens;
 
 		sections[sections.length] = {
 			label: t('Go to'),
@@ -343,52 +374,73 @@ function getMenuPopupMap(options) {
 
 		// trigger status
 		if (typeof options.gotos.triggerStatus !== 'undefined') {
-			var url = new Curl('tr_status.php?filter_set=1&show_maintenance=1');
-
-			jQuery.each(options.gotos.triggerStatus, function(name, value) {
-				url.setArgument(name, value);
-			});
-
-			gotos[gotos.length] = {
-				label: t('Triggers'),
-				css: options.gotos.showTriggers ? '' : 'ui-state-disabled',
-				url: url.getUrl()
+			var triggers = {
+				label: t('Triggers')
 			};
+
+			if (!options.gotos.showTriggers) {
+				triggers.disabled = true;
+			}
+			else {
+				var url = new Curl('tr_status.php?filter_set=1&show_maintenance=1');
+
+				jQuery.each(options.gotos.triggerStatus, function(name, value) {
+					url.setArgument(name, value);
+				});
+
+				triggers.url = url.getUrl();
+			}
+
+			gotos[gotos.length] = triggers;
 		}
 
 		// graphs
 		if (typeof options.gotos.graphs !== 'undefined') {
-			var url = new Curl('charts.php');
-
-			jQuery.each(options.gotos.graphs, function(name, value) {
-				url.setArgument(name, value);
-			});
-
-			gotos[gotos.length] = {
-				label: t('Graphs'),
-				css: options.gotos.showGraphs ? '' : 'ui-state-disabled',
-				url: url.getUrl()
+			var graphs = {
+				label: t('Graphs')
 			};
+
+			if (!options.gotos.showGraphs) {
+				graphs.disabled = true;
+			}
+			else {
+				var url = new Curl('charts.php');
+
+				jQuery.each(options.gotos.graphs, function(name, value) {
+					url.setArgument(name, value);
+				});
+
+				graphs.url = url.getUrl();
+			}
+
+			gotos[gotos.length] = graphs;
 		}
 
 		// screens
 		if (typeof options.gotos.screens !== 'undefined') {
-			var url = new Curl('host_screen.php');
-
-			jQuery.each(options.gotos.screens, function(name, value) {
-				url.setArgument(name, value);
-			});
-
-			gotos[gotos.length] = {
-				label: t('Host screens'),
-				css: options.gotos.showScreens ? '' : 'ui-state-disabled',
-				url: url.getUrl()
+			var screens = {
+				label: t('Host screens')
 			};
+
+			if (!options.gotos.showScreens) {
+				screens.disabled = true;
+			}
+			else {
+				var url = new Curl('host_screen.php');
+
+				jQuery.each(options.gotos.screens, function(name, value) {
+					url.setArgument(name, value);
+				});
+
+				screens.url = url.getUrl();
+			}
+
+			gotos[gotos.length] = screens;
 		}
 
 		// submap
 		if (typeof options.gotos.submap !== 'undefined') {
-			var url = new Curl('maps.php');
+			var url = new Curl('zabbix.php?action=map.view');
 
 			jQuery.each(options.gotos.submap, function(name, value) {
 				url.setArgument(name, value);
@@ -402,17 +454,24 @@ function getMenuPopupMap(options) {
 
 		// events
 		if (typeof options.gotos.events !== 'undefined') {
-			var url = new Curl('events.php?filter_set=1&source=0');
-
-			jQuery.each(options.gotos.events, function(name, value) {
-				url.setArgument(name, value);
-			});
-
-			gotos[gotos.length] = {
-				label: t('Events'),
-				css: options.gotos.showEvents ? '' : 'ui-state-disabled',
-				url: url.getUrl()
+			var events = {
+				label: t('Events')
 			};
+
+			if (!options.gotos.showEvents) {
+				events.disabled = true;
+			}
+			else {
+				var url = new Curl('events.php?filter_set=1&source=0');
+
+				jQuery.each(options.gotos.events, function(name, value) {
+					url.setArgument(name, value);
+				});
+
+				events.url = url.getUrl();
+			}
+
+			gotos[gotos.length] = events;
 		}
 
 		sections[sections.length] = {
@@ -466,9 +525,8 @@ function getMenuPopupRefresh(options) {
 			};
 
 	jQuery.each(intervals, function(value, label) {
-		items[items.length] = {
+		var item = {
 			label: label,
-			css: (value == options.currentRate) ? 'selected' : '',
 			data: {
 				value: value
 			},
@@ -476,30 +534,48 @@ function getMenuPopupRefresh(options) {
 				var obj = jQuery(this),
 					currentRate = obj.data('value');
 
-				sendAjaxData({
-					data: jQuery.extend({}, params, {
-						widgetName: options.widgetName,
-						widgetRefreshRate: currentRate
-					}),
-					dataType: 'script',
-					success: function(js) { js }
-				});
+				// it is a quick solution for slide refresh multiplier, should be replaced with slide.refresh or similar
+				if (options.multiplier) {
+					sendAjaxData('slides.php', {
+						data: jQuery.extend({}, params, {
+							widgetName: options.widgetName,
+							widgetRefreshRate: currentRate
+						}),
+						dataType: 'script',
+						success: function(js) { js }
+					});
+				}
+				else {
+					sendAjaxData('zabbix.php?action=dashboard.widget', {
+						data: jQuery.extend({}, params, {
+							widget: options.widgetName,
+							refreshrate: currentRate
+						}),
+						dataType: 'script',
+						success: function(js) { js }
+					});
+				}
 
-				jQuery('a', obj.closest('ul')).each(function() {
-					var a = jQuery(this),
-						li = a.parent();
+				jQuery('a').each(function() {
+					var link = jQuery(this);
 
-					if (a.data('value') == currentRate) {
-						li.addClass('selected');
+					if (link.data('value') == currentRate) {
+						link.addClass('selected');
 					}
 					else {
-						li.removeClass('selected');
+						link.removeClass('selected');
 					}
 				});
 
-				obj.closest('.menuPopup').fadeOut(100);
+				obj.closest('.action-menu').fadeOut(100);
 			}
 		};
+
+		if (value == options.currentRate) {
+			item.selected = true;
+		}
+
+		items[items.length] = item;
 	});
 
 	return [{
@@ -508,64 +584,6 @@ function getMenuPopupRefresh(options) {
 	}];
 }
 
-/**
- * Get menu popup service configuration section data.
- *
- * @param string options['serviceid']		service id
- * @param string options['name']			service name
- * @param bool   options['deletable']		service has dependencies and cannot be deleted
- *
- * @return array
- */
-function getMenuPopupServiceConfiguration(options) {
-	var items = [];
-
-	if (options.serviceid === null) {
-		// add
-		items[items.length] = {
-			label: t('Add child'),
-			url: new Curl('services.php?form=1&parentname=' + options.name).getUrl()
-		};
-	}
-	else {
-		// add
-		items[items.length] = {
-			label: t('Add child'),
-			url: new Curl('services.php?form=1&parentid=' + options.serviceid + '&parentname=' + options.name).getUrl()
-		};
-
-		// edit
-		items[items.length] = {
-			label: t('Edit'),
-			url: new Curl('services.php?form=1&serviceid=' + options.serviceid).getUrl()
-		};
-
-		// delete
-		if (options.deletable) {
-			items[items.length] = {
-				label: t('Delete'),
-				url: new Curl('services.php?delete=1&serviceid=' + options.serviceid).getUrl(),
-				clickCallback: function() {
-					jQuery(this).closest('.menuPopup').fadeOut(100);
-
-					return confirm(sprintf(t('Delete service "%1$s"?'), options.name));
-				}
-			};
-		}
-		else {
-			items[items.length] = {
-				label: t('Delete'),
-				css: 'ui-state-disabled'
-			};
-		}
-	}
-
-	return [{
-		label: sprintf(t('Service "%1$s"'), options.name),
-		items: items
-	}];
-}
-
 /**
  * Get menu popup trigger section data.
  *
@@ -577,8 +595,7 @@ function getMenuPopupServiceConfiguration(options) {
  * @param object options['items'][]['params']		item url parameters ("name" => "value")
  * @param object options['acknowledge']				link to acknowledge page (optional)
  * @param string options['acknowledge']['eventid']	event id
- * @param string options['acknowledge']['screenid']	screen id (optional)
- * @param string options['acknowledge']['backurl']	return url (optional)
+ * @param string options['acknowledge']['backurl']	return url
  * @param int    options['eventTime']				event page url navigation time parameter (optional)
  * @param object options['configuration']			link to trigger configuration page (optional)
  * @param string options['url']						trigger url link (optional)
@@ -596,19 +613,22 @@ function getMenuPopupTrigger(options) {
 		url.setArgument('nav_time', options.eventTime);
 	}
 
-	items[items.length] = {
-		label: t('Events'),
-		css: (typeof options.showEvents !== 'undefined' && options.showEvents) ? '' : 'ui-state-disabled',
-		url: url.getUrl()
+	var events = {
+		label: t('Events')
 	};
 
+	if (typeof options.showEvents !== 'undefined' && options.showEvents) {
+		events.url = url.getUrl();
+	}
+	else {
+		events.disabled = true;
+	}
+
+	items[items.length] = events;
+
 	// acknowledge
 	if (typeof options.acknowledge !== 'undefined' && objectSize(options.acknowledge) > 0) {
-		var url = new Curl('acknow.php');
-
-		jQuery.each(options.acknowledge, function(name, value) {
-			url.setArgument(name, value);
-		});
+		var url = new Curl('zabbix.php?action=acknowledge.edit&eventids[]=' + options.acknowledge.eventid + '&backurl=' + options.acknowledge.backurl);
 
 		items[items.length] = {
 			label: t('Acknowledge'),
@@ -684,15 +704,19 @@ function getMenuPopupTriggerLog(options) {
 			openWinCentered(
 				'tr_logform.php?sform=1&itemid=' + options.itemid,
 				'TriggerLog',
-				760,
-				540,
+				1000,
+				700,
 				'titlebar=no, resizable=yes, scrollbars=yes, dialog=no'
 			);
 
-			jQuery(this).closest('.menuPopup').fadeOut(100);
+			jQuery(this).closest('.action-menu').fadeOut(100);
 		}
 	};
 
+	var edit_trigger = {
+		label: t('Edit trigger')
+	};
+
 	// edit
 	if (options.triggers.length > 0) {
 		var triggers = [];
@@ -704,28 +728,24 @@ function getMenuPopupTriggerLog(options) {
 					openWinCentered(
 						'tr_logform.php?sform=1&itemid=' + options.itemid + '&triggerid=' + trigger.id,
 						'TriggerLog',
-						760,
-						540,
+						1000,
+						700,
 						'titlebar=no, resizable=yes, scrollbars=yes'
 					);
 
-					jQuery(this).closest('.menuPopup').fadeOut(100);
+					jQuery(this).closest('.action-menu').fadeOut(100);
 				}
 			};
 		});
 
-		items[items.length] = {
-			label: t('Edit trigger'),
-			items: triggers
-		};
+		edit_trigger.items = triggers;
 	}
 	else {
-		items[items.length] = {
-			label: t('Edit trigger'),
-			css: 'ui-state-disabled'
-		};
+		edit_trigger.disabled = true;
 	}
 
+	items[items.length] = edit_trigger;
+
 	return [{
 		label: sprintf(t('Item "%1$s"'), options.itemName),
 		items: items
@@ -762,7 +782,7 @@ function getMenuPopupTriggerMacro(options) {
 
 				expressionInput.val(expression.string);
 
-				jQuery(this).closest('.menuPopup').fadeOut(100);
+				jQuery(this).closest('.action-menu').fadeOut(100);
 			}
 		};
 	});
@@ -824,21 +844,21 @@ function getMenuPopupScriptData(scripts, hostId) {
 
 		if (objectSize(tree) > 0) {
 			jQuery.each(tree, function(name, data) {
-				items[items.length] = {
-					label: name,
-					items: (typeof data.items !== 'undefined' && objectSize(data.items) > 0)
-						? getMenuPopupScriptItems(data.items)
-						: [],
-					clickCallback: function(e) {
-						if (typeof data.params !== 'undefined' && typeof data.params.scriptId !== 'undefined') {
-							executeScript(data.params.hostId, data.params.scriptId, data.params.confirmation);
-						}
+				var item = {label: name};
 
+				if (typeof data.items !== 'undefined' && objectSize(data.items) > 0) {
+					item.items = getMenuPopupScriptItems(data.items);
+				}
+
+				if (typeof data.params !== 'undefined' && typeof data.params.scriptId !== 'undefined') {
+					item.clickCallback = function(e) {
+						executeScript(data.params.hostId, data.params.scriptId, data.params.confirmation);
 						cancelEvent(e);
+						jQuery(this).closest('.action-menu-top').fadeOut(100);
+					};
+				}
 
-						jQuery(this).closest('.menuPopup').fadeOut(100);
-					}
-				};
+				items[items.length] = item;
 			});
 		}
 
@@ -854,7 +874,7 @@ jQuery(function($) {
 	 * Menu popup.
 	 *
 	 * @param array  sections				menu sections
-	 * @param string sections[n]['label']	section title
+	 * @param string sections[n]['label']	(optional) section title
 	 * @param array  sections[n]['items']	section menu data (see createMenuItem() for available options)
 	 * @param object event					menu popup call event
 	 *
@@ -874,7 +894,7 @@ jQuery(function($) {
 			var display = menuPopup.css('display');
 
 			// hide all menu popups
-			jQuery('.menuPopup').css('display', 'none');
+			jQuery('.action-menu').css('display', 'none');
 
 			if (display === 'block') {
 				menuPopup.fadeOut(0);
@@ -892,32 +912,28 @@ jQuery(function($) {
 		else {
 			id = new Date().getTime();
 
-			menuPopup = $('<div>', {
+			menuPopup = $('<ul>', {
 				id: id,
-				'class': 'menuPopup'
+				'class': 'action-menu action-menu-top'
 			});
 
 			// create sections
 			if (sections.length > 0) {
 				$.each(sections, function(i, section) {
-					var sectionTitleBox = $('<div>', {
-							'class': 'title'
-						}),
-						sectionTitleText = $('<div>', {
-							'class': 'text',
-							text: section.label
-						}),
-						sectionMenu = $('<ul>', {
-							'class': 'menu'
-						});
-
-					if (section.items.length > 0) {
-						$.each(section.items, function(i, item) {
-							sectionMenu.append(createMenuItem(item));
-						});
+					if ((typeof section.label !== 'undefined') && (section.label.length > 0)) {
+						var h3 = $('<h3>').text(section.label);
+						var sectionItem = $('<li>').append(h3);
 					}
 
-					menuPopup.append(sectionTitleBox.append(sectionTitleText), sectionMenu);
+					// add section delimited for all sections except first one
+					if (i > 0) {
+						menuPopup.append($('<li>').append($('<div>')));
+					}
+					menuPopup.append(sectionItem);
+
+					$.each(section.items, function(i, item) {
+						menuPopup.append(createMenuItem(item));
+					});
 				});
 			}
 
@@ -926,9 +942,6 @@ jQuery(function($) {
 				return;
 			}
 
-			// build jQuery Menu
-			$('.menu', menuPopup).menu();
-
 			// set menu popup for map area
 			if (opener.prop('tagName') === 'AREA') {
 				$('.menuPopupContainer').remove();
@@ -953,7 +966,7 @@ jQuery(function($) {
 			}
 
 			// hide all menu popups
-			jQuery('.menuPopup').css('display', 'none');
+			jQuery('.action-menu').css('display', 'none');
 
 			// display
 			menuPopup
@@ -964,11 +977,6 @@ jQuery(function($) {
 
 					clearTimeout(window.menuPopupTimeoutHandler);
 				})
-				.mouseleave(function() {
-					menuPopup.data('is-active', false);
-
-					closeInactiveMenuPopup(menuPopup, 1000);
-				})
 				.position({
 					of: (opener.prop('tagName') === 'AREA') ? mapContainer : event,
 					my: 'left top',
@@ -976,7 +984,12 @@ jQuery(function($) {
 				});
 		}
 
-		closeInactiveMenuPopup(menuPopup, 2000);
+		$(document).click(function(e) {
+			if (!menuPopup.is(e.target) && menuPopup.has(e.target).length === 0) {
+				menuPopup.data('is-active', false);
+				menuPopup.fadeOut(0);
+			}
+		});
 	};
 
 	/**
@@ -996,11 +1009,12 @@ jQuery(function($) {
 			link = $('<a>');
 
 		if (typeof options.label !== 'undefined') {
-			link.html(options.label);
-		}
+			link.text(options.label);
 
-		if (typeof options.url !== 'undefined') {
-			link.attr('href', options.url);
+			if (typeof options.items !== 'undefined' && options.items.length > 0) {
+				// if submenu exists
+				link.append($('<span>', {'class': 'arrow-right'}));
+			}
 		}
 
 		if (typeof options.data !== 'undefined' && objectSize(options.data) > 0) {
@@ -1009,48 +1023,70 @@ jQuery(function($) {
 			});
 		}
 
-		if (typeof options.clickCallback !== 'undefined') {
-			link.click(options.clickCallback);
+		if (typeof options.disabled !== 'undefined' && options.disabled) {
+			link.addClass('action-menu-item-disabled');
+		}
+		else {
+			link.addClass('action-menu-item');
+
+			if (typeof options.url !== 'undefined') {
+				link.attr('href', options.url);
+			}
+
+			if (typeof options.clickCallback !== 'undefined') {
+				link.click(options.clickCallback);
+			}
 		}
 
-		if (typeof options.css !== 'undefined') {
-			item.addClass(options.css);
+		if (typeof options.selected !== 'undefined' && options.selected) {
+			link.addClass('selected');
 		}
 
 		item.append(link);
 
 		if (typeof options.items !== 'undefined' && options.items.length > 0) {
-			var menu = $('<ul>');
+			var menu = $('<ul>', {'class' : 'action-menu'});
 
 			$.each(options.items, function(i, item) {
 				menu.append(createMenuItem(item));
 			});
 
 			item.append(menu);
+
+			item.hover(function() {
+				var li = $(this),
+					pos = li.position(),
+					menu = li.closest('.action-menu');
+
+				for (var item = $('li:first-child', menu); item.length > 0; item = item.next()) {
+					if (item[0] == li[0]) {
+						var display = $('ul', item[0]).css('display');
+
+						if (display !== 'block') {
+							$('ul:first', item[0])
+								.css({
+									'top': pos.top - 6,
+									'left': pos.left + li.outerWidth() + 14,
+									'display': 'block'
+								});
+						}
+					}
+					else {
+						var display = $('ul', item[0]).css('display');
+
+						if (display === 'block') {
+							$('ul', item[0]).css({'display': 'none'});
+						}
+					}
+				}
+			});
+		}
+		else {
+			item.hover(function() {
+				$('li ul', $(this).closest('.action-menu')).css({'display': 'none'});
+			});
 		}
 
 		return item;
 	}
-
-	/**
-	 * Closing menu after delay.
-	 *
-	 * @param object menuPopup		menu popup
-	 * @param int    delay			delay to close menu popup
-	 */
-	function closeInactiveMenuPopup(menuPopup, delay) {
-		clearTimeout(window.menuPopupTimeoutHandler);
-
-		window.menuPopupTimeoutHandler = setTimeout(function() {
-			if (!menuPopup.data('is-active')) {
-				menuPopup.data('is-active', false);
-
-				$('.menu', menuPopup).each(function() {
-					$(this).menu('collapseAll', null, true);
-				});
-
-				menuPopup.fadeOut(0);
-			}
-		}, delay);
-	}
 });
