@@ -29,519 +29,636 @@ define('ZBX_PAGE_NO_MENU', 1);
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
-$metrics = array(
+$metrics = [
 	PARAM_TYPE_TIME => _('Time'),
 	PARAM_TYPE_COUNTS => _('Count')
-);
-$param1SecCount = array(
-	array(
-		'C' => _('Last of').' (T)',// caption
-		'T' => T_ZBX_INT, // type
-		'M' => $metrics // metrcis
-	),
-	array(
-		'C' => _('Time shift').' ', // caption
-		'T' => T_ZBX_INT // type
-	)
-);
-$param1Sec = array(
-	array(
-		'C' => _('Last of').' (T)', // caption
-		'T' => T_ZBX_INT // type
-	)
-);
-$param1Str = array(
-	array(
-		'C' => 'T', // caption
+];
+$param1SecCount = [
+	[
+		'C' => _('Last of').' (T)',	// caption
+		'T' => T_ZBX_INT,			// type
+		'M' => $metrics				// metrics
+	],
+	[
+		'C' => _('Time shift'),
+		'T' => T_ZBX_INT
+	]
+];
+$param1Sec = [
+	[
+		'C' => _('Last of').' (T)',
+		'T' => T_ZBX_INT
+	]
+];
+$param1Str = [
+	[
+		'C' => 'T',
 		'T' => T_ZBX_STR
-	)
-);
-$param2SecCount = array(
-	array(
-		'C' => 'V', // caption
+	]
+];
+$param2SecCount = [
+	[
+		'C' => 'V',
 		'T' => T_ZBX_STR
-	),
-	array(
-		'C' => _('Last of').' (T)', // caption
-		'T' => T_ZBX_INT, // type
-		'M' => $metrics // metrcis
-	)
-);
-$param3SecVal = array(
-	array(
-		'C' => _('Last of').' (T)', // caption
-		'T' => T_ZBX_INT, // type
-		'M' => $metrics // metrcis
-	),
-	array(
-		'C' => 'V', // caption
+	],
+	[
+		'C' => _('Last of').' (T)',
+		'T' => T_ZBX_INT,
+		'M' => $metrics
+	]
+];
+$param3SecVal = [
+	[
+		'C' => _('Last of').' (T)',
+		'T' => T_ZBX_INT,
+		'M' => $metrics
+	],
+	[
+		'C' => 'V',
 		'T' => T_ZBX_STR
-	),
-	array(
-		'C' => 'O', // caption
+	],
+	[
+		'C' => 'O',
 		'T' => T_ZBX_STR
-	),
-	array(
-		'C' => _('Time shift').' ', // caption
-		'T' => T_ZBX_INT // type
-	)
-);
-$paramSecIntCount = array(
-	array(
-		'C' => _('Last of').' (T)', // caption
-		'T' => T_ZBX_INT, // type
-		'M' => $metrics // metrics
-	),
-	array(
-		'C' => _('Mask'), // caption
+	],
+	[
+		'C' => _('Time shift'),
+		'T' => T_ZBX_INT
+	]
+];
+$param3SecPercent = [
+	[
+		'C' => _('Last of').' (T)',
+		'T' => T_ZBX_INT,
+		'M' => $metrics
+	],
+	[
+		'C' => _('Time shift'),
+		'T' => T_ZBX_INT
+	],
+	[
+		'C' => _('Percentage').' (P)',
+		'T' => T_ZBX_DBL
+	]
+];
+$paramSecIntCount = [
+	[
+		'C' => _('Last of').' (T)',
+		'T' => T_ZBX_INT,
+		'M' => $metrics
+	],
+	[
+		'C' => _('Mask'),
 		'T' => T_ZBX_STR
-	),
-	array(
-		'C' => _('Time shift').' ', // caption
-		'T' => T_ZBX_INT // type
-	)
-);
-$allowedTypesAny = array(
+	],
+	[
+		'C' => _('Time shift'),
+		'T' => T_ZBX_INT
+	]
+];
+$paramForecast = [
+	[
+		'C' => _('Last of').' (T)',
+		'T' => T_ZBX_INT,
+		'M' => $metrics
+	],
+	[
+		'C' => _('Time shift'),
+		'T' => T_ZBX_INT
+	],
+	[
+		'C' => _('Time').' (t)',
+		'T' => T_ZBX_INT
+	],
+	[
+		'C' => _('Fit'),
+		'T' => T_ZBX_STR
+	],
+	[
+		'C' => _('Mode'),
+		'T' => T_ZBX_STR
+	]
+];
+$paramTimeleft = [
+	[
+		'C' => _('Last of').' (T)',
+		'T' => T_ZBX_INT,
+		'M' => $metrics
+	],
+	[
+		'C' => _('Time shift'),
+		'T' => T_ZBX_INT
+	],
+	[
+		'C' => _('Threshold'),
+		'T' => T_ZBX_DBL
+	],
+	[
+		'C' => _('Fit'),
+		'T' => T_ZBX_STR
+	]
+];
+$allowedTypesAny = [
 	ITEM_VALUE_TYPE_FLOAT => 1,
 	ITEM_VALUE_TYPE_STR => 1,
 	ITEM_VALUE_TYPE_LOG => 1,
 	ITEM_VALUE_TYPE_UINT64 => 1,
 	ITEM_VALUE_TYPE_TEXT => 1
-);
-$allowedTypesNumeric = array(
+];
+$allowedTypesNumeric = [
 	ITEM_VALUE_TYPE_FLOAT => 1,
 	ITEM_VALUE_TYPE_UINT64 => 1
-);
-$allowedTypesStr = array(
+];
+$allowedTypesStr = [
 	ITEM_VALUE_TYPE_STR => 1,
 	ITEM_VALUE_TYPE_LOG => 1,
 	ITEM_VALUE_TYPE_TEXT => 1
-);
-$allowedTypesLog = array(
+];
+$allowedTypesLog = [
 	ITEM_VALUE_TYPE_LOG => 1
-);
-$allowedTypesInt = array(
+];
+$allowedTypesInt = [
 	ITEM_VALUE_TYPE_UINT64 => 1
-);
+];
 
-$functions = array(
-	'abschange[<]' => array(
+$functions = [
+	'abschange[<]' => [
 		'description' =>  _('Absolute difference between last and previous value is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'abschange[>]' => array(
+	],
+	'abschange[>]' => [
 		'description' =>  _('Absolute difference between last and previous value is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'abschange[=]' => array(
+	],
+	'abschange[=]' => [
 		'description' =>  _('Absolute difference between last and previous value is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'abschange[<>]' => array(
+	],
+	'abschange[<>]' => [
 		'description' =>  _('Absolute difference between last and previous value is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'avg[<]' => array(
+	],
+	'avg[<]' => [
 		'description' =>  _('Average value of a period T is < N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'avg[>]' => array(
+	],
+	'avg[>]' => [
 		'description' =>  _('Average value of a period T is > N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'avg[=]' => array(
+	],
+	'avg[=]' => [
 		'description' =>  _('Average value of a period T is = N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'avg[<>]' => array(
+	],
+	'avg[<>]' => [
 		'description' =>  _('Average value of a period T is NOT N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'delta[<]' => array(
+	],
+	'delta[<]' => [
 		'description' =>  _('Difference between MAX and MIN value of a period T is < N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'delta[>]' => array(
+	],
+	'delta[>]' => [
 		'description' =>  _('Difference between MAX and MIN value of a period T is > N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'delta[=]' => array(
+	],
+	'delta[=]' => [
 		'description' =>  _('Difference between MAX and MIN value of a period T is = N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'delta[<>]' => array(
+	],
+	'delta[<>]' => [
 		'description' =>  _('Difference between MAX and MIN value of a period T is NOT N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'change[<]' => array(
+	],
+	'change[<]' => [
 		'description' =>  _('Difference between last and previous value is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'change[>]' => array(
+	],
+	'change[>]' => [
 		'description' =>  _('Difference between last and previous value is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'change[=]' => array(
+	],
+	'change[=]' => [
 		'description' =>  _('Difference between last and previous value is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'change[<>]' => array(
+	],
+	'change[<>]' => [
 		'description' =>  _('Difference between last and previous value is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'count[<]' => array(
+	],
+	'count[<]' => [
 		'description' =>  _('Number of successfully retrieved values V (which fulfill operator O) for period T is < N'),
 		'params' => $param3SecVal,
 		'allowed_types' => $allowedTypesAny
-	),
-	'count[>]' => array(
+	],
+	'count[>]' => [
 		'description' =>  _('Number of successfully retrieved values V (which fulfill operator O) for period T is > N'),
 		'params' => $param3SecVal,
 		'allowed_types' => $allowedTypesAny
-	),
-	'count[=]' => array(
+	],
+	'count[=]' => [
 		'description' =>  _('Number of successfully retrieved values V (which fulfill operator O) for period T is = N'),
 		'params' => $param3SecVal,
 		'allowed_types' => $allowedTypesAny
-	),
-	'count[<>]' => array(
+	],
+	'count[<>]' => [
 		'description' =>  _('Number of successfully retrieved values V (which fulfill operator O) for period T is NOT N'),
 		'params' => $param3SecVal,
 		'allowed_types' => $allowedTypesAny
-	),
-	'diff[=]' => array(
+	],
+	'diff[=]' => [
 		'description' =>  _('Difference between last and preceding values, then N = 1, 0 - otherwise'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'diff[<>]' => array(
+	],
+	'diff[<>]' => [
 		'description' =>  _('Difference between last and preceding values, then N NOT 1, 0 - otherwise'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'last[<]' => array(
+	],
+	'last[<]' => [
 		'description' =>  _('Last (most recent) T value is < N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'last[>]' => array(
+	],
+	'last[>]' => [
 		'description' =>  _('Last (most recent) T value is > N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'last[=]' => array(
+	],
+	'last[=]' => [
 		'description' =>  _('Last (most recent) T value is = N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'last[<>]' => array(
+	],
+	'last[<>]' => [
 		'description' =>  _('Last (most recent) T value is NOT N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'max[<]' => array(
+	],
+	'max[<]' => [
 		'description' =>  _('Maximum value for period T is < N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'max[>]' => array(
+	],
+	'max[>]' => [
 		'description' =>  _('Maximum value for period T is > N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'max[=]' => array(
+	],
+	'max[=]' => [
 		'description' =>  _('Maximum value for period T is = N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'max[<>]' => array(
+	],
+	'max[<>]' => [
 		'description' =>  _('Maximum value for period T is NOT N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'min[<]' => array(
+	],
+	'min[<]' => [
 		'description' =>  _('Minimum value for period T is < N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-		),
-	'min[>]' => array(
+		],
+	'min[>]' => [
 		'description' =>  _('Minimum value for period T is > N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-		),
-	'min[=]' => array(
+		],
+	'min[=]' => [
 		'description' =>  _('Minimum value for period T is = N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-		),
-	'min[<>]' => array(
+		],
+	'min[<>]' => [
 		'description' =>  _('Minimum value for period T is NOT N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-		),
-	'prev[<]' => array(
+		],
+	'percentile[<]' => [
+		'description' =>  _('Percentile P of a period T is < N'),
+		'params' => $param3SecPercent,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'percentile[>]' => [
+		'description' =>  _('Percentile P of a period T is > N'),
+		'params' => $param3SecPercent,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'percentile[=]' => [
+		'description' =>  _('Percentile P of a period T is = N'),
+		'params' => $param3SecPercent,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'percentile[<>]' => [
+		'description' =>  _('Percentile P of a period T is NOT N'),
+		'params' => $param3SecPercent,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'prev[<]' => [
 		'description' =>  _('Previous value is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'prev[>]' => array(
+	],
+	'prev[>]' => [
 		'description' =>  _('Previous value is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'prev[=]' => array(
+	],
+	'prev[=]' => [
 		'description' =>  _('Previous value is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'prev[<>]' => array(
+	],
+	'prev[<>]' => [
 		'description' =>  _('Previous value is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'str[=]' => array(
+	],
+	'str[=]' => [
 		'description' =>  _('Find string V in last (most recent) value. N = 1 - if found, 0 - otherwise'),
 		'params' => $param2SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'str[<>]' => array(
+	],
+	'str[<>]' => [
 		'description' =>  _('Find string V in last (most recent) value. N NOT 1 - if found, 0 - otherwise'),
 		'params' => $param2SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'strlen[<]' => array(
+	],
+	'strlen[<]' => [
 		'description' =>  _('Length of last (most recent) T value in characters is < N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesStr
-	),
-	'strlen[>]' => array(
+	],
+	'strlen[>]' => [
 		'description' =>  _('Length of last (most recent) T value in characters is > N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesStr
-	),
-	'strlen[=]' => array(
+	],
+	'strlen[=]' => [
 		'description' =>  _('Length of last (most recent) T value in characters is = N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesStr
-	),
-	'strlen[<>]' => array(
+	],
+	'strlen[<>]' => [
 		'description' =>  _('Length of last (most recent) T value in characters is NOT N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesStr
-	),
-	'sum[<]' => array(
+	],
+	'sum[<]' => [
 		'description' =>  _('Sum of values of a period T is < N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'sum[>]' => array(
+	],
+	'sum[>]' => [
 		'description' =>  _('Sum of values of a period T is > N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'sum[=]' => array(
+	],
+	'sum[=]' => [
 		'description' =>  _('Sum of values of a period T is = N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'sum[<>]' => array(
+	],
+	'sum[<>]' => [
 		'description' =>  _('Sum of values of a period T is NOT N'),
 		'params' => $param1SecCount,
 		'allowed_types' => $allowedTypesNumeric
-	),
-	'date[<]' => array(
+	],
+	'date[<]' => [
 		'description' =>  _('Current date is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'date[>]' => array(
+	],
+	'date[>]' => [
 		'description' =>  _('Current date is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'date[=]' => array(
+	],
+	'date[=]' => [
 		'description' =>  _('Current date is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'date[<>]' => array(
+	],
+	'date[<>]' => [
 		'description' =>  _('Current date is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofweek[<]' => array(
+	],
+	'dayofweek[<]' => [
 		'description' =>  _('Day of week is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofweek[>]' => array(
+	],
+	'dayofweek[>]' => [
 		'description' =>  _('Day of week is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofweek[=]' => array(
+	],
+	'dayofweek[=]' => [
 		'description' =>  _('Day of week is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofweek[<>]' => array(
+	],
+	'dayofweek[<>]' => [
 		'description' =>  _('Day of week is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofmonth[<]' => array(
+	],
+	'dayofmonth[<]' => [
 		'description' =>  _('Day of month is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofmonth[>]' => array(
+	],
+	'dayofmonth[>]' => [
 		'description' =>  _('Day of month is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofmonth[=]' => array(
+	],
+	'dayofmonth[=]' => [
 		'description' =>  _('Day of month is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'dayofmonth[<>]' => array(
+	],
+	'dayofmonth[<>]' => [
 		'description' =>  _('Day of month is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'fuzzytime[=]' => array(
+	],
+	'fuzzytime[=]' => [
 		'description' =>  _('Timestamp not different from Zabbix server time for more than T seconds, then N = 1, 0 - otherwise'),
 		'params' => $param1Sec,
 		'allowed_types' => $allowedTypesAny
-	),
-	'fuzzytime[<>]' => array(
+	],
+	'fuzzytime[<>]' => [
 		'description' =>  _('Timestamp not different from Zabbix server time for more than T seconds, then N NOT 1, 0 - otherwise'),
 		'params' => $param1Sec,
 		'allowed_types' => $allowedTypesAny
-	),
-	'regexp[=]' => array(
+	],
+	'regexp[=]' => [
 		'description' =>  _('Regular expression V matching last value in period T, then N = 1, 0 - otherwise'),
 		'params' => $param2SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'regexp[<>]' => array(
+	],
+	'regexp[<>]' => [
 		'description' =>  _('Regular expression V matching last value in period T, then N NOT 1, 0 - otherwise'),
 		'params' => $param2SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'iregexp[=]' => array(
+	],
+	'iregexp[=]' => [
 		'description' =>  _('Regular expression V matching last value in period T, then N = 1, 0 - otherwise (non case-sensitive)'),
 		'params' => $param2SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'iregexp[<>]' => array(
+	],
+	'iregexp[<>]' => [
 		'description' =>  _('Regular expression V matching last value in period T, then N NOT 1, 0 - otherwise (non case-sensitive)'),
 		'params' => $param2SecCount,
 		'allowed_types' => $allowedTypesAny
-	),
-	'logeventid[=]' => array(
+	],
+	'logeventid[=]' => [
 		'description' =>  _('Event ID of last log entry matching regular expression T, then N = 1, 0 - otherwise'),
 		'params' => $param1Str,
 		'allowed_types' => $allowedTypesLog
-	),
-	'logeventid[<>]' => array(
+	],
+	'logeventid[<>]' => [
 		'description' =>  _('Event ID of last log entry matching regular expression T, then N NOT 1, 0 - otherwise'),
 		'params' => $param1Str,
 		'allowed_types' => $allowedTypesLog
-	),
-	'logseverity[<]' => array(
+	],
+	'logseverity[<]' => [
 		'description' =>  _('Log severity of the last log entry is < N'),
 		'allowed_types' => $allowedTypesLog
-	),
-	'logseverity[>]' => array(
+	],
+	'logseverity[>]' => [
 		'description' =>  _('Log severity of the last log entry is > N'),
 		'allowed_types' => $allowedTypesLog
-	),
-	'logseverity[=]' => array(
+	],
+	'logseverity[=]' => [
 		'description' =>  _('Log severity of the last log entry is = N'),
 		'allowed_types' => $allowedTypesLog
-	),
-	'logseverity[<>]' => array(
+	],
+	'logseverity[<>]' => [
 		'description' =>  _('Log severity of the last log entry is NOT N'),
 		'allowed_types' => $allowedTypesLog
-	),
-	'logsource[=]' => array(
+	],
+	'logsource[=]' => [
 		'description' =>  _('Log source of the last log entry matching parameter T, then N = 1, 0 - otherwise'),
 		'params' => $param1Str,
 		'allowed_types' => $allowedTypesLog
-	),
-	'logsource[<>]' => array(
+	],
+	'logsource[<>]' => [
 		'description' =>  _('Log source of the last log entry matching parameter T, then N NOT 1, 0 - otherwise'),
 		'params' => $param1Str,
 		'allowed_types' => $allowedTypesLog
-	),
-	'now[<]' => array(
+	],
+	'now[<]' => [
 		'description' =>  _('Number of seconds since the Epoch is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'now[>]' => array(
+	],
+	'now[>]' => [
 		'description' =>  _('Number of seconds since the Epoch is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'now[=]' => array(
+	],
+	'now[=]' => [
 		'description' =>  _('Number of seconds since the Epoch is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'now[<>]' => array(
+	],
+	'now[<>]' => [
 		'description' =>  _('Number of seconds since the Epoch is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'time[<]' => array(
+	],
+	'time[<]' => [
 		'description' =>  _('Current time is < N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'time[>]' => array(
+	],
+	'time[>]' => [
 		'description' =>  _('Current time is > N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'time[=]' => array(
+	],
+	'time[=]' => [
 		'description' =>  _('Current time is = N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'time[<>]' => array(
+	],
+	'time[<>]' => [
 		'description' =>  _('Current time is NOT N'),
 		'allowed_types' => $allowedTypesAny
-	),
-	'nodata[=]' => array(
+	],
+	'nodata[=]' => [
 		'description' =>  _('No data received during period of time T, then N = 1, 0 - otherwise'),
 		'params' => $param1Sec,
 		'allowed_types' => $allowedTypesAny
-	),
-	'nodata[<>]' => array(
+	],
+	'nodata[<>]' => [
 		'description' =>  _('No data received during period of time T, then N NOT 1, 0 - otherwise'),
 		'params' => $param1Sec,
 		'allowed_types' => $allowedTypesAny
-	),
-	'band[=]' => array(
+	],
+	'band[=]' => [
 		'description' =>  _('Bitwise AND of last (most recent) T value and mask is = N'),
 		'params' => $paramSecIntCount,
 		'allowed_types' => $allowedTypesInt
-	),
-	'band[<>]' => array(
+	],
+	'band[<>]' => [
 		'description' =>  _('Bitwise AND of last (most recent) T value and mask is NOT N'),
 		'params' => $paramSecIntCount,
 		'allowed_types' => $allowedTypesInt
-	)
-);
+	],
+	'forecast[<]' => [
+		'description' => _('Forecast for next t seconds based on period T is < N'),
+		'params' => $paramForecast,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'forecast[>]' => [
+		'description' => _('Forecast for next t seconds based on period T is > N'),
+		'params' => $paramForecast,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'forecast[=]' => [
+		'description' => _('Forecast for next t seconds based on period T is = N'),
+		'params' => $paramForecast,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'forecast[<>]' => [
+		'description' => _('Forecast for next t seconds based on period T is NOT N'),
+		'params' => $paramForecast,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'timeleft[<]' => [
+		'description' => _('Time to reach threshold estimated based on period T is < N'),
+		'params' => $paramTimeleft,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'timeleft[>]' => [
+		'description' => _('Time to reach threshold estimated based on period T is > N'),
+		'params' => $paramTimeleft,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'timeleft[=]' => [
+		'description' => _('Time to reach threshold estimated based on period T is = N'),
+		'params' => $paramTimeleft,
+		'allowed_types' => $allowedTypesNumeric
+	],
+	'timeleft[<>]' => [
+		'description' => _('Time to reach threshold estimated based on period T is NOT N'),
+		'params' => $paramTimeleft,
+		'allowed_types' => $allowedTypesNumeric
+	]
+];
 order_result($functions, 'description');
 
 //	VAR		TYPE	OPTIONAL FLAGS	VALIDATION	EXCEPTION
-$fields = array(
-	'dstfrm' =>				array(T_ZBX_STR, O_MAND, P_SYS, NOT_EMPTY,	null),
-	'dstfld1' =>			array(T_ZBX_STR, O_MAND, P_SYS, NOT_EMPTY,	null),
-	'expression' =>			array(T_ZBX_STR, O_OPT, null,	null,		null),
-	'itemid' =>				array(T_ZBX_INT, O_OPT, null,	null,		'isset({insert})'),
-	'parent_discoveryid' =>	array(T_ZBX_INT, O_OPT, null,	null,		null),
-	'expr_type'=>			array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({insert})'),
-	'params' =>				array(T_ZBX_STR, O_OPT, null,	0,			null),
-	'paramtype' =>			array(T_ZBX_INT, O_OPT, null,	IN(PARAM_TYPE_TIME.','.PARAM_TYPE_COUNTS), 'isset({insert})'),
-	'value' =>				array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({insert})'),
+$fields = [
+	'dstfrm' =>				[T_ZBX_STR, O_MAND, P_SYS, NOT_EMPTY,	null],
+	'dstfld1' =>			[T_ZBX_STR, O_MAND, P_SYS, NOT_EMPTY,	null],
+	'expression' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'itemid' =>				[T_ZBX_INT, O_OPT, null,	null,		'isset({insert})'],
+	'parent_discoveryid' =>	[T_ZBX_INT, O_OPT, null,	null,		null],
+	'expr_type'=>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({insert})'],
+	'params' =>				[T_ZBX_STR, O_OPT, null,	0,			null],
+	'paramtype' =>			[T_ZBX_INT, O_OPT, null,	IN(PARAM_TYPE_TIME.','.PARAM_TYPE_COUNTS), 'isset({insert})'],
+	'value' =>				[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({insert})'],
 	// action
-	'insert' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'cancel' =>				array(T_ZBX_STR, O_OPT, P_SYS,	null,		null)
-);
+	'insert' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null]
+];
 check_fields($fields);
 
 $dstfrm = getRequest('dstfrm', 0);
 $dstfld1 = getRequest('dstfld1', '');
 $itemId = getRequest('itemid', 0);
 $value = getRequest('value', 0);
-$params = getRequest('params', array());
+$params = getRequest('params', []);
 $paramType = getRequest('paramtype');
 $exprType = getRequest('expr_type', 'last[=]');
 
@@ -563,7 +680,7 @@ if (isset($_REQUEST['expression']) && $_REQUEST['dstfld1'] == 'expr_temp') {
 
 			// determine param type
 			$params = $functionMacroToken['data']['functionParams'];
-			$paramNumber = in_array($function, array('regexp', 'iregexp', 'str')) ? 1 : 0;
+			$paramNumber = in_array($function, ['regexp', 'iregexp', 'str']) ? 1 : 0;
 			if (isset($params[$paramNumber][0]) && $params[$paramNumber][0] == '#') {
 				$paramType = PARAM_TYPE_COUNTS;
 				$params[$paramNumber] = substr($params[$paramNumber], 1);
@@ -597,16 +714,16 @@ if (isset($_REQUEST['expression']) && $_REQUEST['dstfld1'] == 'expr_temp') {
 			$exprType = $function.'['.$operator.']';
 
 			// find the item
-			$item = API::Item()->get(array(
-				'output' => array('itemid', 'hostid', 'name', 'key_', 'value_type'),
-				'selectHosts' => array('name'),
+			$item = API::Item()->get([
+				'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type'],
+				'selectHosts' => ['name'],
 				'webitems' => true,
-				'filter' => array(
+				'filter' => [
 					'host' => $functionMacroToken['data']['host'],
 					'key_' => $functionMacroToken['data']['item'],
 					'flags' => null
-				)
-			));
+				]
+			]);
 			$item = reset($item);
 
 			if ($item) {
@@ -630,18 +747,18 @@ else {
 	}
 
 	// fetch item
-	$item = API::Item()->get(array(
-		'output' => array('itemid', 'hostid', 'name', 'key_', 'value_type'),
-		'selectHosts' => array('host', 'name'),
+	$item = API::Item()->get([
+		'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type'],
+		'selectHosts' => ['host', 'name'],
 		'itemids' => $itemId,
 		'webitems' => true,
-		'filter' => array('flags' => null)
-	));
+		'filter' => ['flags' => null]
+	]);
 	$item = reset($item);
 }
 
 if ($itemId) {
-	$items = CMacrosResolverHelper::resolveItemNames(array($item));
+	$items = CMacrosResolverHelper::resolveItemNames([$item]);
 	$item = $items[0];
 
 	$itemValueType = $item['value_type'];
@@ -665,7 +782,7 @@ elseif (is_null($paramType)) {
 /*
  * Display
  */
-$data = array(
+$data = [
 	'parent_discoveryid' => getRequest('parent_discoveryid'),
 	'dstfrm' => $dstfrm,
 	'dstfld1' => $dstfld1,
@@ -681,7 +798,7 @@ $data = array(
 	'expr_type' => $exprType,
 	'insert' => getRequest('insert'),
 	'cancel' => getRequest('cancel')
-);
+];
 
 // check if submitted function is usable with selected item
 foreach ($data['functions'] as $id => $f) {
@@ -710,16 +827,16 @@ if (isset($data['insert'])) {
 	try {
 		if ($data['description']) {
 			if ($data['paramtype'] == PARAM_TYPE_COUNTS) {
-				$paramNumber = in_array($function, array('regexp', 'iregexp', 'str')) ? 1 : 0;
+				$paramNumber = in_array($function, ['regexp', 'iregexp', 'str']) ? 1 : 0;
 				$data['params'][$paramNumber] = '#'.$data['params'][$paramNumber];
 			}
 
-			if ($data['paramtype'] == PARAM_TYPE_TIME && in_array($function, array('last', 'band', 'strlen'))) {
+			if ($data['paramtype'] == PARAM_TYPE_TIME && in_array($function, ['last', 'band', 'strlen'])) {
 				$data['params'][0] = '';
 			}
 
 			// quote function param
-			$quotedParams = array();
+			$quotedParams = [];
 			foreach ($data['params'] as $param) {
 				$quotedParams[] = quoteFunctionParam($param);
 			}
@@ -741,12 +858,12 @@ if (isset($data['insert'])) {
 
 				// validate trigger function
 				$triggerFunctionValidator = new CFunctionValidator();
-				$isValid = $triggerFunctionValidator->validate(array(
+				$isValid = $triggerFunctionValidator->validate([
 					'function' => $expressionData['function'],
 					'functionName' => $expressionData['functionName'],
 					'functionParamList' => $expressionData['functionParamList'],
 					'valueType' => $data['itemValueType']
-				));
+				]);
 				if (!$isValid) {
 					unset($data['insert']);
 					throw new Exception($triggerFunctionValidator->getError());
