@@ -21,32 +21,28 @@
 
 require_once dirname(__FILE__).'/include/config.inc.php';
 require_once dirname(__FILE__).'/include/hosts.inc.php';
-require_once dirname(__FILE__).'/include/reports.inc.php';
 
 $page['title'] = _('Availability report');
 $page['file'] = 'report2.php';
-$page['hist_arg'] = array('mode', 'groupid', 'hostid', 'tpl_triggerid');
-$page['scripts'] = array('class.calendar.js');
+$page['scripts'] = ['class.calendar.js'];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
-$fields = array(
-	'mode' =>				array(T_ZBX_INT,	O_OPT,	P_SYS,			IN('0,1'),	null),
-	'hostgroupid' =>		array(T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null),
-	'tpl_triggerid' =>		array(T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null),
-	'triggerid' =>			array(T_ZBX_INT,	O_OPT,	P_SYS|P_NZERO,	DB_ID,		null),
+$fields = [
+	'mode' =>				[T_ZBX_INT,	O_OPT,	P_SYS,			IN('0,1'),	null],
+	'hostgroupid' =>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
+	'tpl_triggerid' =>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
+	'triggerid' =>			[T_ZBX_INT,	O_OPT,	P_SYS|P_NZERO,	DB_ID,		null],
 	// filter
-	'filter_groupid'=>		array(T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null),
-	'filter_hostid' =>		array(T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null),
-	'filter_rst'=>			array(T_ZBX_STR,	O_OPT,	P_SYS,			null,		null),
-	'filter_set' =>			array(T_ZBX_STR,	O_OPT,	P_SYS,			null,		null),
-	'filter_timesince' =>	array(T_ZBX_STR,	O_OPT,	P_UNSET_EMPTY,	null,		null),
-	'filter_timetill' =>	array(T_ZBX_STR,	O_OPT,	P_UNSET_EMPTY,	null,		null),
-	// ajax
-	'filterState' =>		array(T_ZBX_INT,	O_OPT,	P_ACT,			null,		null)
-);
+	'filter_groupid'=>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
+	'filter_hostid' =>		[T_ZBX_INT,	O_OPT,	P_SYS,			DB_ID,		null],
+	'filter_rst'=>			[T_ZBX_STR,	O_OPT,	P_SYS,			null,		null],
+	'filter_set' =>			[T_ZBX_STR,	O_OPT,	P_SYS,			null,		null],
+	'filter_timesince' =>	[T_ZBX_STR,	O_OPT,	P_UNSET_EMPTY,	null,		null],
+	'filter_timetill' =>	[T_ZBX_STR,	O_OPT,	P_UNSET_EMPTY,	null,		null]
+];
 check_fields($fields);
 
 $availabilityReportMode = getRequest('mode', CProfile::get('web.avail_report.mode', AVAILABILITY_REPORT_BY_HOST));
@@ -56,43 +52,32 @@ CProfile::update('web.avail_report.mode', $availabilityReportMode, PROFILE_TYPE_
  * Permissions
  */
 if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
-	if (getRequest('hostgroupid') && !API::HostGroup()->isReadable(array($_REQUEST['hostgroupid']))
-			|| getRequest('filter_groupid') && !API::HostGroup()->isReadable(array($_REQUEST['filter_groupid']))
-			|| getRequest('filter_hostid') && !API::Host()->isReadable(array($_REQUEST['filter_hostid']))) {
+	if (getRequest('hostgroupid') && !API::HostGroup()->isReadable([$_REQUEST['hostgroupid']])
+			|| getRequest('filter_groupid') && !API::HostGroup()->isReadable([$_REQUEST['filter_groupid']])
+			|| getRequest('filter_hostid') && !API::Host()->isReadable([$_REQUEST['filter_hostid']])) {
 		access_deny();
 	}
 	if (getRequest('tpl_triggerid')) {
-		$trigger = API::Trigger()->get(array(
+		$trigger = API::Trigger()->get([
 			'triggerids' => $_REQUEST['tpl_triggerid'],
-			'output' => array('triggerid'),
-			'filter' => array('flags' => null)
-		));
+			'output' => ['triggerid'],
+			'filter' => ['flags' => null]
+		]);
 		if (!$trigger) {
 			access_deny();
 		}
 	}
 }
 else {
-	if (getRequest('filter_groupid') && !API::HostGroup()->isReadable(array($_REQUEST['filter_groupid']))
-			|| getRequest('filter_hostid') && !API::Host()->isReadable(array($_REQUEST['filter_hostid']))) {
+	if (getRequest('filter_groupid') && !API::HostGroup()->isReadable([$_REQUEST['filter_groupid']])
+			|| getRequest('filter_hostid') && !API::Host()->isReadable([$_REQUEST['filter_hostid']])) {
 		access_deny();
 	}
 }
-if (getRequest('triggerid') && !API::Trigger()->isReadable(array($_REQUEST['triggerid']))) {
+if (getRequest('triggerid') && !API::Trigger()->isReadable([$_REQUEST['triggerid']])) {
 	access_deny();
 }
 
-/*
- * Ajax
- */
-if (hasRequest('filterState')) {
-	CProfile::update('web.avail_report.filter.state', getRequest('filterState'), PROFILE_TYPE_INT);
-}
-if ($page['type'] == PAGE_TYPE_JS || $page['type'] == PAGE_TYPE_HTML_BLOCK) {
-	require_once dirname(__FILE__).'/include/page_footer.php';
-	exit;
-}
-
 /*
  * Filter
  */
@@ -151,16 +136,15 @@ $_REQUEST['filter_timetill'] = zbxDateToTime($_REQUEST['filter_timetill']
  * Header
  */
 $triggerData = isset($_REQUEST['triggerid'])
-	? API::Trigger()->get(array(
+	? API::Trigger()->get([
 		'triggerids' => $_REQUEST['triggerid'],
 		'output' => API_OUTPUT_EXTEND,
 		'selectHosts' => API_OUTPUT_EXTEND,
 		'expandDescription' => true
-	))
+	])
 	: null;
 
-$reportWidget = new CWidget();
-$reportWidget->addPageHeader(_('AVAILABILITY REPORT'));
+$reportWidget = (new CWidget())->setTitle(_('Availability report'));
 
 if ($triggerData) {
 	$triggerData = reset($triggerData);
@@ -169,58 +153,57 @@ if ($triggerData) {
 	$triggerData['hostid'] = $host['hostid'];
 	$triggerData['hostname'] = $host['name'];
 
-	$reportWidget->addHeader(array(
-		new CLink($triggerData['hostname'], '?filter_groupid='.$_REQUEST['filter_groupid']),
-		NAME_DELIMITER,
-		$triggerData['description']
-	), SPACE);
+	$reportWidget->setControls(
+		(new CList())
+			->addItem(new CLink($triggerData['hostname'], '?filter_groupid='.$_REQUEST['filter_groupid']))
+			->addItem($triggerData['description'])
+	);
 
-	$table = new CTableInfo(null, 'graph');
-	$table->addRow(new CImg('chart4.php?triggerid='.$_REQUEST['triggerid']));
+	$table = (new CTableInfo())
+		->addRow(new CImg('chart4.php?triggerid='.$_REQUEST['triggerid']));
 
-	$reportWidget->addItem(BR());
-	$reportWidget->addItem($table);
-	$reportWidget->show();
+	$reportWidget->addItem(BR())
+		->addItem($table)
+		->show();
 }
 elseif (isset($_REQUEST['filter_hostid'])) {
-	$modeComboBox = new CComboBox('mode', $availabilityReportMode, 'submit()');
-	$modeComboBox->addItem(AVAILABILITY_REPORT_BY_HOST, _('By host'));
-	$modeComboBox->addItem(AVAILABILITY_REPORT_BY_TEMPLATE, _('By trigger template'));
+	$controls = new CList();
+	$controls->addItem([_('Mode').SPACE, new CComboBox('mode', $availabilityReportMode, 'submit()', [
+		AVAILABILITY_REPORT_BY_HOST => _('By host'),
+		AVAILABILITY_REPORT_BY_TEMPLATE => _('By trigger template')
+	])]);
+	$headerForm = (new CForm('get'))->addItem($controls);
+	$reportWidget->setControls($headerForm);
 
-	$headerForm = new CForm('get');
-	$headerForm->addItem($modeComboBox);
-
-	$reportWidget->addHeader(_('Report'), array(_('Mode').SPACE, $headerForm));
-
-	$triggerOptions = array(
-		'output' => array('triggerid', 'description', 'expression', 'value'),
+	$triggerOptions = [
+		'output' => ['triggerid', 'description', 'expression', 'value'],
 		'expandDescription' => true,
 		'monitored' => true,
-		'selectHosts' => array('name'),
-		'filter' => array(),
+		'selectHosts' => ['name'],
+		'filter' => [],
 		'hostids' => null,
 		'limit' => $config['search_limit'] + 1
-	);
+	];
 
 	/*
 	 * Filter
 	 */
-	$filterForm = new CFormTable();
-	$filterForm->setTableClass('formtable old-filter');
-	$filterForm->setAttribute('name', 'zbx_filter');
-	$filterForm->setAttribute('id', 'zbx_filter');
-	$filterForm->addVar('config', $availabilityReportMode);
-	$filterForm->addVar('filter_timesince', date(TIMESTAMP_FORMAT, $_REQUEST['filter_timesince']));
-	$filterForm->addVar('filter_timetill', date(TIMESTAMP_FORMAT, $_REQUEST['filter_timetill']));
+	$filterForm = (new CFilter('web.avail_report.filter.state'))
+		->addVar('config', $availabilityReportMode)
+		->addVar('filter_timesince', date(TIMESTAMP_FORMAT, $_REQUEST['filter_timesince']))
+		->addVar('filter_timetill', date(TIMESTAMP_FORMAT, $_REQUEST['filter_timetill']));
+
+	$filterColumn1 = new CFormList();
+	$filterColumn2 = new CFormList();
 
 	// report by template
 	if ($availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) {
 		// trigger options
 		if (!empty($_REQUEST['filter_hostid'])) {
-			$hosts = API::Host()->get(array(
-				'output' => array('hostid'),
+			$hosts = API::Host()->get([
+				'output' => ['hostid'],
 				'templateids' => $_REQUEST['filter_hostid']
-			));
+			]);
 
 			$triggerOptions['hostids'] = zbx_objectValues($hosts, 'hostid');
 		}
@@ -235,36 +218,36 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 		$groupsComboBox = new CComboBox('filter_groupid', $_REQUEST['filter_groupid'], 'javascript: submit();');
 		$groupsComboBox->addItem(0, _('all'));
 
-		$groups = API::HostGroup()->get(array(
-			'output' => array('groupid', 'name'),
+		$groups = API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
 			'templated_hosts' => true,
 			'with_triggers' => true
-		));
+		]);
 		order_result($groups, 'name');
 
 		foreach ($groups as $group) {
 			$groupsComboBox->addItem($group['groupid'], $group['name']);
 		}
-		$filterForm->addRow(_('Template group'), $groupsComboBox);
+		$filterColumn1->addRow(_('Template group'), $groupsComboBox);
 
 		// filter template
 		$templateComboBox = new CComboBox('filter_hostid', $_REQUEST['filter_hostid'], 'javascript: submit();');
 		$templateComboBox->addItem(0, _('all'));
 
-		$templates = API::Template()->get(array(
-			'output' => array('templateid', 'name'),
+		$templates = API::Template()->get([
+			'output' => ['templateid', 'name'],
 			'groupids' => empty($_REQUEST['filter_groupid']) ? null : $_REQUEST['filter_groupid'],
 			'with_triggers' => true
-		));
+		]);
 		order_result($templates, 'name');
 
-		$templateIds = array();
+		$templateIds = [];
 		foreach ($templates as $template) {
 			$templateIds[$template['templateid']] = $template['templateid'];
 
 			$templateComboBox->addItem($template['templateid'], $template['name']);
 		}
-		$filterForm->addRow(_('Template'), $templateComboBox);
+		$filterColumn1->addRow(_('Template'), $templateComboBox);
 
 		// filter trigger
 		$triggerComboBox = new CComboBox('tpl_triggerid', getRequest('tpl_triggerid', 0), 'javascript: submit()');
@@ -297,18 +280,18 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			unset($triggerOptions['filter']['templateid']);
 		}
 
-		$filterForm->addRow(_('Template trigger'), $triggerComboBox);
+		$filterColumn1->addRow(_('Template trigger'), $triggerComboBox);
 
 		// filter host group
 		$hostGroupsComboBox = new CComboBox('hostgroupid', getRequest('hostgroupid', 0), 'javascript: submit()');
 		$hostGroupsComboBox->addItem(0, _('all'));
 
-		$hostGroups = API::HostGroup()->get(array(
-			'output' => array('groupid', 'name'),
+		$hostGroups = API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
 			'hostids' => $triggerOptions['hostids'],
 			'monitored_hosts' => true,
 			'preservekeys' => true
-		));
+		]);
 		order_result($hostGroups, 'name');
 
 		foreach ($hostGroups as $hostGroup) {
@@ -319,7 +302,7 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			unset($triggerOptions['groupids']);
 		}
 
-		$filterForm->addRow(_('Filter by host group'), $hostGroupsComboBox);
+		$filterColumn1->addRow(_('Filter by host group'), $hostGroupsComboBox);
 	}
 
 	// report by host
@@ -328,35 +311,35 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 		$groupsComboBox = new CComboBox('filter_groupid', $_REQUEST['filter_groupid'], 'javascript: submit();');
 		$groupsComboBox->addItem(0, _('all'));
 
-		$groups = API::HostGroup()->get(array(
-			'output' => array('groupid', 'name'),
+		$groups = API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
 			'monitored_hosts' => true,
 			'with_triggers' => true
-		));
+		]);
 		order_result($groups, 'name');
 
 		foreach ($groups as $group) {
 			$groupsComboBox->addItem($group['groupid'], $group['name']);
 		}
-		$filterForm->addRow(_('Host group'), $groupsComboBox);
+		$filterColumn1->addRow(_('Host group'), $groupsComboBox);
 
 		// filter host
 		$hostsComboBox = new CComboBox('filter_hostid', $_REQUEST['filter_hostid'], 'javascript: submit();');
 		$hostsComboBox->addItem(0, _('all'));
 
-		$hosts = API::Host()->get(array(
+		$hosts = API::Host()->get([
 			'groupids' => empty($_REQUEST['filter_groupid']) ? null : $_REQUEST['filter_groupid'],
-			'output' => array('hostid', 'name'),
+			'output' => ['hostid', 'name'],
 			'monitored_hosts' => true,
 			'with_triggers' => true
-		));
+		]);
 		order_result($hosts, 'name');
 		$hosts = zbx_toHash($hosts, 'hostid');
 
 		foreach ($hosts as $host) {
 			$hostsComboBox->addItem($host['hostid'], $host['name']);
 		}
-		$filterForm->addRow(_('Host'), $hostsComboBox);
+		$filterColumn1->addRow(_('Host'), $hostsComboBox);
 
 		// trigger options
 		if (!empty($_REQUEST['filter_groupid'])) {
@@ -368,38 +351,29 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 	}
 
 	// filter period
-	$timeSinceRow = createDateSelector('filter_timesince', $_REQUEST['filter_timesince'], 'filter_timetill');
-	array_unshift($timeSinceRow, _('From'));
-	$timeTillRow = createDateSelector('filter_timetill', $_REQUEST['filter_timetill'], 'filter_timesince');
-	array_unshift($timeTillRow, _('Till'));
+	$filterColumn2->addRow(_('From'), createDateSelector('filter_timesince', $_REQUEST['filter_timesince'], 'filter_timetill'));
+	$filterColumn2->addRow(_('To'), createDateSelector('filter_timetill', $_REQUEST['filter_timetill'], 'filter_timesince'));
 
-	$filterPeriodTable = new CTable(null, 'calendar');
-	$filterPeriodTable->addRow($timeSinceRow);
-	$filterPeriodTable->addRow($timeTillRow);
+	$filterForm->addColumn($filterColumn1);
+	$filterForm->addColumn($filterColumn2);
 
-	$filterForm->addRow(_('Period'), $filterPeriodTable);
-
-	// filter buttons
-	$filterForm->addItemToBottomRow(new CSubmit('filter_set',_('Filter')));
-	$filterForm->addItemToBottomRow(new CSubmit('filter_rst', _('Reset')));
-
-	$reportWidget->addFlicker($filterForm, CProfile::get('web.avail_report.filter.state', 0));
+	$reportWidget->addItem($filterForm);
 
 	/*
 	 * Triggers
 	 */
-	$triggerTable = new CTableInfo(_('No triggers found.'));
-	$triggerTable->setHeader(array(
-		($_REQUEST['filter_hostid'] == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) ? _('Host') : null,
-		_('Name'),
-		_('Problems'),
-		_('Ok'),
-		_('Graph')
-	));
+	$triggerTable = (new CTableInfo())
+		->setHeader([
+			($_REQUEST['filter_hostid'] == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE) ? _('Host') : null,
+			_('Name'),
+			_('Problems'),
+			_('Ok'),
+			_('Graph')
+		]);
 
 	$triggers = API::Trigger()->get($triggerOptions);
 
-	CArrayHelper::sort($triggers, array('host', 'description'));
+	CArrayHelper::sort($triggers, ['host', 'description']);
 
 	$paging = getPagingLine($triggers, ZBX_SORT_UP);
 
@@ -408,23 +382,22 @@ elseif (isset($_REQUEST['filter_hostid'])) {
 			getRequest('filter_timetill')
 		);
 
-		$triggerTable->addRow(array(
+		$triggerTable->addRow([
 			($_REQUEST['filter_hostid'] == 0 || $availabilityReportMode == AVAILABILITY_REPORT_BY_TEMPLATE)
 				? $trigger['hosts'][0]['name'] : null,
 			new CLink($trigger['description'], 'events.php?filter_set=1&source='.EVENT_SOURCE_TRIGGERS.
 				'&hostid='.$trigger['hosts'][0]['hostid'].
 				'&triggerid='.$trigger['triggerid']
 			),
-			new CSpan(sprintf('%.4f%%', $availability['true']), 'on'),
-			new CSpan(sprintf('%.4f%%', $availability['false']), 'off'),
+			$availability['true'] == 0 ? '' : (new CSpan(sprintf('%.4f%%', $availability['true'])))->addClass(ZBX_STYLE_RED),
+			$availability['false'] == 0 ? '' : (new CSpan(sprintf('%.4f%%', $availability['false'])))->addClass(ZBX_STYLE_GREEN),
 			new CLink(_('Show'), 'report2.php?filter_groupid='.$_REQUEST['filter_groupid'].
 				'&filter_hostid='.$_REQUEST['filter_hostid'].'&triggerid='.$trigger['triggerid'])
-		));
+		]);
 	}
 
-	$reportWidget->addItem(BR());
-	$reportWidget->addItem(array($paging, $triggerTable, $paging));
-	$reportWidget->show();
+	$reportWidget->addItem([$triggerTable, $paging])
+		->show();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
