@@ -23,8 +23,7 @@ require_once dirname(__FILE__).'/include/config.inc.php';
 
 $page['file'] = 'tr_status.php';
 $page['title'] = _('Status of triggers');
-$page['hist_arg'] = array('groupid', 'hostid');
-$page['scripts'] = array('class.cswitcher.js');
+$page['scripts'] = ['class.cswitcher.js'];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
 if ($page['type'] == PAGE_TYPE_HTML) {
@@ -34,72 +33,58 @@ if ($page['type'] == PAGE_TYPE_HTML) {
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
-$fields = array(
-	'groupid' =>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'hostid' =>				array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'fullscreen' =>			array(T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	null),
-	'btnSelect' =>			array(T_ZBX_STR, O_OPT, null,	null,		null),
+$fields = [
+	'groupid' =>			[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
+	'hostid' =>				[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
+	'fullscreen' =>			[T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	null],
+	'btnSelect' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
 	// filter
-	'filter_rst' =>			array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'filter_set' =>			array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'show_triggers' =>		array(T_ZBX_INT, O_OPT, null,	null,		null),
-	'show_events' =>		array(T_ZBX_INT, O_OPT, P_SYS,	null,		null),
-	'ack_status' =>			array(T_ZBX_INT, O_OPT, P_SYS,	null,		null),
-	'show_severity' =>		array(T_ZBX_INT, O_OPT, P_SYS,	null,		null),
-	'show_details' =>		array(T_ZBX_INT, O_OPT, null,	null,		null),
-	'show_maintenance' =>	array(T_ZBX_INT, O_OPT, null,	null,		null),
-	'status_change_days' =>	array(T_ZBX_INT, O_OPT, null,	BETWEEN(1, DAY_IN_YEAR * 2), null),
-	'status_change' =>		array(T_ZBX_INT, O_OPT, null,	null,		null),
-	'txt_select' =>			array(T_ZBX_STR, O_OPT, null,	null,		null),
-	'application' =>		array(T_ZBX_STR, O_OPT, null,	null,		null),
-	'inventory' =>			array(T_ZBX_STR, O_OPT, null,	null,		null),
-	// ajax
-	'filterState' =>		array(T_ZBX_INT, O_OPT, P_ACT,	null,		null),
+	'filter_rst' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_set' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'show_triggers' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
+	'show_events' =>		[T_ZBX_INT, O_OPT, P_SYS,	null,		null],
+	'ack_status' =>			[T_ZBX_INT, O_OPT, P_SYS,	null,		null],
+	'show_severity' =>		[T_ZBX_INT, O_OPT, P_SYS,	null,		null],
+	'show_details' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
+	'show_maintenance' =>	[T_ZBX_INT, O_OPT, null,	null,		null],
+	'status_change_days' =>	[T_ZBX_INT, O_OPT, null,	BETWEEN(1, DAY_IN_YEAR * 2), null],
+	'status_change' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
+	'txt_select' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'application' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
+	'inventory' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
 	// sort and sortorder
-	'sort' =>				array(T_ZBX_STR, O_OPT, P_SYS, IN('"description","lastchange","priority"'),	null),
-	'sortorder' =>			array(T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null)
-);
+	'sort' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"description","lastchange","priority"'),	null],
+	'sortorder' =>			[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
+];
 check_fields($fields);
 
 /*
  * Permissions
  */
-if (getRequest('groupid') && !API::HostGroup()->isReadable(array(getRequest('groupid')))) {
+if (getRequest('groupid') && !API::HostGroup()->isReadable([getRequest('groupid')])) {
 	access_deny();
 }
-if (getRequest('hostid') && !API::Host()->isReadable(array(getRequest('hostid')))) {
+if (getRequest('hostid') && !API::Host()->isReadable([getRequest('hostid')])) {
 	access_deny();
 }
 
-/*
- * Ajax
- */
-if (hasRequest('filterState')) {
-	CProfile::update('web.tr_status.filter.state', getRequest('filterState'), PROFILE_TYPE_INT);
-}
-
-if ($page['type'] == PAGE_TYPE_JS || $page['type'] == PAGE_TYPE_HTML_BLOCK) {
-	require_once dirname(__FILE__).'/include/page_footer.php';
-	exit;
-}
-
 /*
  * Filter
  */
 $config = select_config();
 
-$pageFilter = new CPageFilter(array(
-	'groups' => array(
+$pageFilter = new CPageFilter([
+	'groups' => [
 		'monitored_hosts' => true,
 		'with_monitored_triggers' => true
-	),
-	'hosts' => array(
+	],
+	'hosts' => [
 		'monitored_hosts' => true,
 		'with_monitored_triggers' => true
-	),
+	],
 	'hostid' => getRequest('hostid'),
 	'groupid' => getRequest('groupid')
-));
+]);
 $_REQUEST['groupid'] = $pageFilter->groupid;
 $_REQUEST['hostid'] = $pageFilter->hostid;
 
@@ -136,9 +121,9 @@ if (hasRequest('filter_set')) {
 	}
 
 	// update host inventory filter
-	$inventoryFields = array();
-	$inventoryValues = array();
-	foreach (getRequest('inventory', array()) as $field) {
+	$inventoryFields = [];
+	$inventoryValues = [];
+	foreach (getRequest('inventory', []) as $field) {
 		if ($field['value'] === '') {
 			continue;
 		}
@@ -188,16 +173,16 @@ if ($config['event_ack_enable'] == EVENT_ACK_DISABLED && $showEvents == EVENTS_O
 }
 
 // fetch filter from profiles
-$filter = array(
+$filter = [
 	'application' => CProfile::get('web.tr_status.filter.application', ''),
-	'inventory' => array()
-);
+	'inventory' => []
+];
 
-foreach (CProfile::getArray('web.tr_status.filter.inventory.field', array()) as $i => $field) {
-	$filter['inventory'][] = array(
+foreach (CProfile::getArray('web.tr_status.filter.inventory.field', []) as $i => $field) {
+	$filter['inventory'][] = [
 		'field' => $field,
 		'value' => CProfile::get('web.tr_status.filter.inventory.value', null, $i)
-	);
+	];
 }
 
 /*
@@ -212,24 +197,25 @@ CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR
 /*
  * Display
  */
-$triggerWidget = new CWidget();
+$triggerWidget = (new CWidget())->setTitle(_('Status of triggers'));
 
-$rightForm = new CForm('get');
-$rightForm->addItem(array(_('Group').SPACE, $pageFilter->getGroupsCB()));
-$rightForm->addItem(array(SPACE._('Host').SPACE, $pageFilter->getHostsCB()));
-$rightForm->addVar('fullscreen', $_REQUEST['fullscreen']);
+$rightForm = (new CForm('get'))
+	->addVar('fullscreen', $_REQUEST['fullscreen']);
 
-$triggerWidget->addPageHeader(
-	_('STATUS OF TRIGGERS').SPACE.'['.zbx_date2str(DATE_TIME_FORMAT_SECONDS).']',
-	get_icon('fullscreen', array('fullscreen' => $_REQUEST['fullscreen']))
-);
-$triggerWidget->addHeader(_('Triggers'), $rightForm);
-$triggerWidget->addHeaderRowNumber();
+$controls = new CList();
+$controls->addItem([_('Group').SPACE, $pageFilter->getGroupsCB()]);
+$controls->addItem([_('Host').SPACE, $pageFilter->getHostsCB()]);
+$controls->addItem(get_icon('fullscreen', ['fullscreen' => $_REQUEST['fullscreen']]));
+
+$rightForm->addItem($controls);
+
+$triggerWidget->setControls($rightForm);
 
 // filter
-$filterFormView = new CView('common.filter.trigger', array(
+$filterFormView = new CView('common.filter.trigger', [
 	'overview' => false,
-	'filter' => array(
+	'filter' => [
+		'filterid' => 'web.tr_status.filter.state',
 		'showTriggers' => $showTriggers,
 		'ackStatus' => $ackStatus,
 		'showEvents' => $showEvents,
@@ -244,68 +230,72 @@ $filterFormView = new CView('common.filter.trigger', array(
 		'hostId' => getRequest('hostid'),
 		'groupId' => getRequest('groupid'),
 		'fullScreen' => getRequest('fullscreen')
-	)
-));
-$filterForm = $filterFormView->render();
+	],
+	'config' => $config
+]);
 
-$triggerWidget->addFlicker($filterForm, CProfile::get('web.tr_status.filter.state', 0));
+$filterForm = $filterFormView->render();
+$triggerWidget->addItem($filterForm);
 
 /*
  * Form
  */
-if ($_REQUEST['fullscreen']) {
-	$triggerInfo = new CTriggersInfo($_REQUEST['groupid'], $_REQUEST['hostid']);
-	$triggerInfo->hideHeader();
-	$triggerInfo->show();
-}
-
-$triggerForm = new CForm('get', 'acknow.php');
-$triggerForm->setName('tr_status');
-$triggerForm->addVar('backurl', $page['file']);
+$triggerForm = (new CForm('get', 'zabbix.php'))
+	->setName('tr_status')
+	->addVar('backurl', $page['file'])
+	->addVar('acknowledge_type', ZBX_ACKNOWLEDGE_PROBLEM);
 
 /*
  * Table
  */
-$showEventColumn = ($config['event_ack_enable'] && $showEvents != EVENTS_OPTION_NOEVENT);
-
 $switcherName = 'trigger_switchers';
 
-$headerCheckBox = ($showEventColumn)
-	? new CCheckBox('all_events', false, "checkAll('".$triggerForm->GetName()."', 'all_events', 'events');")
-	: new CCheckBox('all_triggers', false, "checkAll('".$triggerForm->GetName()."', 'all_triggers', 'triggers');");
-
-if ($showEvents != EVENTS_OPTION_NOEVENT) {
-	$showHideAllDiv = new CDiv(SPACE, 'filterclosed');
-	$showHideAllDiv->setAttribute('id', $switcherName);
+if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
+	$showHideAllDiv = (new CColHeader(
+		(new CDiv())
+			->addClass(ZBX_STYLE_TREEVIEW)
+			->setId($switcherName)
+			->addItem((new CSpan())->addClass(ZBX_STYLE_ARROW_RIGHT))
+	))->addClass(ZBX_STYLE_CELL_WIDTH);
 }
 else {
 	$showHideAllDiv = null;
 }
 
-$triggerTable = new CTableInfo(_('No triggers found.'));
-$triggerTable->setHeader(array(
-	$showHideAllDiv,
-	$config['event_ack_enable'] ? $headerCheckBox : null,
-	make_sorting_header(_('Severity'), 'priority', $sortField, $sortOrder),
-	_('Status'),
-	_('Info'),
-	make_sorting_header(_('Last change'), 'lastchange', $sortField, $sortOrder),
-	_('Age'),
-	$showEventColumn ? _('Duration') : null,
-	$config['event_ack_enable'] ? _('Acknowledged') : null,
-	_('Host'),
-	make_sorting_header(_('Name'), 'description', $sortField, $sortOrder),
-	_('Description')
-));
+if ($config['event_ack_enable']) {
+	$headerCheckBox = (new CColHeader(
+		(new CCheckBox('all_eventids'))
+			->onClick("checkAll('".$triggerForm->GetName()."', 'all_eventids', 'eventids');")
+	))->addClass(ZBX_STYLE_CELL_WIDTH);
+}
+else {
+	$headerCheckBox = null;
+}
+
+$triggerTable = (new CTableInfo())
+	->setHeader([
+		$showHideAllDiv,
+		$headerCheckBox,
+		make_sorting_header(_('Severity'), 'priority', $sortField, $sortOrder),
+		_('Status'),
+		_('Info'),
+		make_sorting_header(_('Last change'), 'lastchange', $sortField, $sortOrder),
+		_('Age'),
+		($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) ? _('Duration') : null,
+		$config['event_ack_enable'] ? _('Ack') : null,
+		_('Host'),
+		make_sorting_header(_('Name'), 'description', $sortField, $sortOrder),
+		_('Description')
+	]);
 
 // get triggers
-$options = array(
-	'output' => array('triggerid', $sortField),
+$options = [
+	'output' => ['triggerid', $sortField],
 	'monitored' => true,
 	'skipDependent' => true,
 	'sortfield' => $sortField,
 	'limit' => $config['search_limit'] + 1
-);
+];
 
 if ($pageFilter->hostsSelected) {
 	if ($pageFilter->hostid > 0) {
@@ -316,42 +306,42 @@ if ($pageFilter->hostsSelected) {
 	}
 }
 else {
-	$options['hostids'] = array();
+	$options['hostids'] = [];
 }
 
 // inventory filter
 if ($filter['inventory']) {
-	$inventoryFilter = array();
+	$inventoryFilter = [];
 	foreach ($filter['inventory'] as $field) {
 		$inventoryFilter[$field['field']][] = $field['value'];
 	}
 
-	$hosts = API::Host()->get(array(
-		'output' => array('hostid'),
+	$hosts = API::Host()->get([
+		'output' => ['hostid'],
 		'hostids' => isset($options['hostids']) ? $options['hostids'] : null,
 		'searchInventory' => $inventoryFilter
-	));
+	]);
 	$options['hostids'] = zbx_objectValues($hosts, 'hostid');
 }
 
 // application filter
 if ($filter['application'] !== '') {
-	$applications = API::Application()->get(array(
-		'output' => array('applicationid'),
+	$applications = API::Application()->get([
+		'output' => ['applicationid'],
 		'hostids' => isset($options['hostids']) ? $options['hostids'] : null,
-		'search' => array('name' => $filter['application'])
-	));
+		'search' => ['name' => $filter['application']]
+	]);
 	$options['applicationids'] = zbx_objectValues($applications, 'applicationid');
 }
 
 if (!zbx_empty($txtSelect)) {
-	$options['search'] = array('description' => $txtSelect);
+	$options['search'] = ['description' => $txtSelect];
 }
 if ($showTriggers == TRIGGERS_OPTION_RECENT_PROBLEM) {
 	$options['only_true'] = 1;
 }
 elseif ($showTriggers == TRIGGERS_OPTION_IN_PROBLEM) {
-	$options['filter'] = array('value' => TRIGGER_VALUE_TRUE);
+	$options['filter'] = ['value' => TRIGGER_VALUE_TRUE];
 }
 if ($ackStatus == ZBX_ACK_STS_WITH_UNACK) {
 	$options['withUnacknowledgedEvents'] = 1;
@@ -373,25 +363,25 @@ $triggers = API::Trigger()->get($options);
 order_result($triggers, $sortField, $sortOrder);
 $paging = getPagingLine($triggers, $sortOrder);
 
-
-$triggers = API::Trigger()->get(array(
+$triggers = API::Trigger()->get([
 	'triggerids' => zbx_objectValues($triggers, 'triggerid'),
 	'output' => API_OUTPUT_EXTEND,
-	'selectHosts' => array(
-		'hostid',
-		'name',
-		'description',
-		'status',
-		'maintenanceid',
-		'maintenance_status',
-		'maintenance_type'
-	),
-	'selectItems' => array('itemid', 'hostid', 'name', 'key_', 'value_type'),
+	'selectHosts' => [
+		'hostid', 'name', 'description', 'status', 'maintenanceid', 'maintenance_status', 'maintenance_type'
+	],
+	'selectItems' => ['itemid', 'hostid', 'name', 'key_', 'value_type'],
 	'selectDependencies' => API_OUTPUT_EXTEND,
 	'selectLastEvent' => true,
 	'expandDescription' => true,
 	'preservekeys' => true
-));
+]);
+
+$triggers = CMacrosResolverHelper::resolveTriggerUrls($triggers);
+if ($showDetails) {
+	$triggers = CMacrosResolverHelper::resolveTriggerExpressions($triggers,
+		['html' => true, 'resolve_usermacros' => true, 'resolve_macros' => true]
+	);
+}
 
 order_result($triggers, $sortField, $sortOrder);
 
@@ -406,34 +396,34 @@ unset($trigger);
 $triggerIds = zbx_objectValues($triggers, 'triggerid');
 
 // get editable triggers
-$triggerEditable = API::Trigger()->get(array(
+$triggerEditable = API::Trigger()->get([
 	'triggerids' => $triggerIds,
-	'output' => array('triggerid'),
+	'output' => ['triggerid'],
 	'editable' => true,
 	'preservekeys' => true
-));
+]);
 
 // get events
 if ($config['event_ack_enable']) {
 	// get all unacknowledged events, if trigger has unacknowledged event => it has events
-	$eventCounts = API::Event()->get(array(
+	$eventCounts = API::Event()->get([
 		'source' => EVENT_SOURCE_TRIGGERS,
 		'object' => EVENT_OBJECT_TRIGGER,
 		'countOutput' => true,
 		'groupCount' => true,
 		'objectids' => $triggerIds,
-		'filter' => array(
+		'filter' => [
 			'acknowledged' => 0,
 			'value' => TRIGGER_VALUE_TRUE
-		)
-	));
+		]
+	]);
 	foreach ($eventCounts as $eventCount) {
 		$triggers[$eventCount['objectid']]['hasEvents'] = true;
 		$triggers[$eventCount['objectid']]['event_count'] = $eventCount['rowscount'];
 	}
 
 	// gather ids of triggers which don't have unack. events
-	$triggerIdsWithoutUnackEvents = array();
+	$triggerIdsWithoutUnackEvents = [];
 	foreach ($triggers as $tnum => $trigger) {
 		if (!isset($trigger['hasEvents'])) {
 			$triggerIdsWithoutUnackEvents[] = $trigger['triggerid'];
@@ -444,13 +434,13 @@ if ($config['event_ack_enable']) {
 	}
 	if (!empty($triggerIdsWithoutUnackEvents)) {
 		// for triggers without unack. events we try to select any event
-		$allEventCounts = API::Event()->get(array(
+		$allEventCounts = API::Event()->get([
 			'source' => EVENT_SOURCE_TRIGGERS,
 			'object' => EVENT_OBJECT_TRIGGER,
 			'countOutput' => true,
 			'groupCount' => true,
 			'objectids' => $triggerIdsWithoutUnackEvents
-		));
+		]);
 		$allEventCounts = zbx_toHash($allEventCounts, 'objectid');
 
 		foreach ($triggers as $tnum => $trigger) {
@@ -461,26 +451,26 @@ if ($config['event_ack_enable']) {
 	}
 }
 
-if ($showEvents != EVENTS_OPTION_NOEVENT) {
-	$options = array(
+if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
+	foreach ($triggers as &$trigger) {
+		$trigger['events'] = [];
+	}
+	unset($trigger);
+
+	$options = [
+		'output' => ['eventid', 'objectid', 'clock', 'value'],
 		'source' => EVENT_SOURCE_TRIGGERS,
 		'object' => EVENT_OBJECT_TRIGGER,
 		'objectids' => zbx_objectValues($triggers, 'triggerid'),
-		'output' => API_OUTPUT_EXTEND,
-		'select_acknowledges' => API_OUTPUT_COUNT,
 		'time_from' => time() - $config['event_expire'] * SEC_PER_DAY,
 		'time_till' => time(),
-		'sortfield' => array('clock', 'eventid'),
+		'sortfield' => ['clock', 'eventid'],
 		'sortorder' => ZBX_SORT_DOWN
-	);
+	];
 
-	switch ($showEvents) {
-		case EVENTS_OPTION_ALL:
-			break;
-		case EVENTS_OPTION_NOT_ACK:
-			$options['acknowledged'] = false;
-			$options['value'] = TRIGGER_VALUE_TRUE;
-			break;
+	if ($config['event_ack_enable']) {
+		$options['select_acknowledges'] = API_OUTPUT_COUNT;
+		$options['output'][] = 'acknowledged';
 	}
 	$events = API::Event()->get($options);
 
@@ -490,7 +480,7 @@ if ($showEvents != EVENTS_OPTION_NOEVENT) {
 }
 
 // get host ids
-$hostIds = array();
+$hostIds = [];
 foreach ($triggers as $tnum => $trigger) {
 	foreach ($trigger['hosts'] as $host) {
 		$hostIds[$host['hostid']] = $host['hostid'];
@@ -498,13 +488,13 @@ foreach ($triggers as $tnum => $trigger) {
 }
 
 // get hosts
-$hosts = API::Host()->get(array(
-	'output' => array('hostid', 'status'),
+$hosts = API::Host()->get([
+	'output' => ['hostid', 'status'],
 	'hostids' => $hostIds,
 	'preservekeys' => true,
 	'selectGraphs' => API_OUTPUT_COUNT,
 	'selectScreens' => API_OUTPUT_COUNT
-));
+]);
 
 // get host scripts
 $scriptsByHosts = API::Script()->getScriptsByHosts($hostIds);
@@ -515,11 +505,29 @@ $dbTriggerDependencies = DBselect(
 	' FROM trigger_depends'.
 	' WHERE '.dbConditionInt('triggerid_up', $triggerIds)
 );
-$triggerIdsDown = array();
+$triggerIdsDown = [];
 while ($row = DBfetch($dbTriggerDependencies)) {
 	$triggerIdsDown[$row['triggerid_up']][] = intval($row['triggerid_down']);
 }
 
+$maintenanceids = [];
+
+foreach ($triggers as $trigger) {
+	foreach ($trigger['hosts'] as $host) {
+		if ($host['maintenance_status'] == HOST_MAINTENANCE_STATUS_ON) {
+			$maintenanceids[$host['maintenanceid']] = true;
+		}
+	}
+}
+
+if ($maintenanceids) {
+	$maintenances = API::Maintenance()->get([
+		'maintenanceids' => array_keys($maintenanceids),
+		'output' => ['name', 'description'],
+		'preservekeys' => true
+	]);
+}
+
 foreach ($triggers as $trigger) {
 	/*
 	 * At this point "all" or one group is selected. And same goes for hosts. It is safe to pass 'groupid' and 'hostid'
@@ -528,38 +536,28 @@ foreach ($triggers as $trigger) {
 	$trigger['groupid'] = $pageFilter->groupid;
 	$trigger['hostid'] = $pageFilter->hostid;
 
-	$description = new CSpan($trigger['description'], 'link_menu');
-	$description->setMenuPopup(CMenuPopupHelper::getTrigger($trigger));
-
-	if ($showDetails) {
-		$description = array(
-			$description,
-			BR(),
-			new CDiv(explode_exp($trigger['expression'], true, true), 'trigger-expression')
-		);
-	}
+	$description = [];
 
 	if (!empty($trigger['dependencies'])) {
-		$dependenciesTable = new CTableInfo();
-		$dependenciesTable->setAttribute('style', 'width: 200px;');
-		$dependenciesTable->addRow(bold(_('Depends on').NAME_DELIMITER));
+		$dependenciesTable = (new CTable())
+			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
+			->addRow(_('Depends on').':');
 
 		foreach ($trigger['dependencies'] as $dependency) {
 			$dependenciesTable->addRow(' - '.CMacrosResolverHelper::resolveTriggerNameById($dependency['triggerid']));
 		}
 
-		$img = new CImg('images/general/arrow_down2.png', 'DEP_UP');
-		$img->setAttribute('style', 'vertical-align: middle; border: 0px;');
-		$img->setHint($dependenciesTable);
-
-		$description = array($img, SPACE, $description);
+		$description[] = (new CSpan())
+			->addClass(ZBX_STYLE_ICON_DEPEND_DOWN)
+			->addClass(ZBX_STYLE_CURSOR_POINTER)
+			->setHint($dependenciesTable);
 	}
 
 	$dependency = false;
-	$dependenciesTable = new CTableInfo();
-	$dependenciesTable->setAttribute('style', 'width: 200px;');
-	$dependenciesTable->addRow(bold(_('Dependent').NAME_DELIMITER));
-	if (!empty($triggerIdsDown[$trigger['triggerid']])) {
+	$dependenciesTable = (new CTable())
+		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
+		->addRow(_('Dependent').':');
+	if (array_key_exists($trigger['triggerid'], $triggerIdsDown) && $triggerIdsDown[$trigger['triggerid']]) {
 		$depTriggers = CMacrosResolverHelper::resolveTriggerNameByIds($triggerIdsDown[$trigger['triggerid']]);
 
 		foreach ($depTriggers as $depTrigger) {
@@ -569,66 +567,65 @@ foreach ($triggers as $trigger) {
 	}
 
 	if ($dependency) {
-		$img = new CImg('images/general/arrow_up2.png', 'DEP_UP');
-		$img->setAttribute('style', 'vertical-align: middle; border: 0px;');
-		$img->setHint($dependenciesTable);
-
-		$description = array($img, SPACE, $description);
+		$description[] = (new CSpan())
+			->addClass(ZBX_STYLE_ICON_DEPEND_UP)
+			->addClass(ZBX_STYLE_CURSOR_POINTER)
+			->setHint($dependenciesTable);
 	}
 	unset($img, $dependenciesTable, $dependency);
 
-	$triggerDescription = new CSpan($description, 'pointer');
+	$description[] = (new CSpan($trigger['description']))
+		->addClass(ZBX_STYLE_LINK_ACTION)
+		->setMenuPopup(CMenuPopupHelper::getTrigger($trigger));
+
+	if ($showDetails) {
+		$description[] = BR();
+		$description[] = $trigger['expression'];
+	}
 
 	// host js menu
-	$hostList = array();
-	foreach ($trigger['hosts'] as $triggerHost) {
+	$hostList = [];
+	foreach ($trigger['hosts'] as $host) {
 		// fetch scripts for the host js menu
-		$scripts = array();
-		if (isset($scriptsByHosts[$triggerHost['hostid']])) {
-			foreach ($scriptsByHosts[$triggerHost['hostid']] as $script) {
+		$scripts = [];
+		if (isset($scriptsByHosts[$host['hostid']])) {
+			foreach ($scriptsByHosts[$host['hostid']] as $script) {
 				$scripts[] = $script;
 			}
 		}
 
-		$hostName = new CSpan($triggerHost['name'], 'link_menu');
-		$hostName->setMenuPopup(CMenuPopupHelper::getHost($hosts[$triggerHost['hostid']], $scripts));
+		$host_name = (new CSpan($host['name']))
+			->addClass(ZBX_STYLE_LINK_ACTION)
+			->setMenuPopup(CMenuPopupHelper::getHost($hosts[$host['hostid']], $scripts));
 
 		// add maintenance icon with hint if host is in maintenance
-		if ($triggerHost['maintenance_status']) {
-			$maintenanceIcon = new CDiv(null, 'icon-maintenance-inline');
+		if ($host['maintenance_status'] == HOST_MAINTENANCE_STATUS_ON) {
+			$maintenance_icon = (new CSpan())
+				->addClass(ZBX_STYLE_ICON_MAINT)
+				->addClass(ZBX_STYLE_CURSOR_POINTER);
 
-			$maintenances = API::Maintenance()->get(array(
-				'maintenanceids' => $triggerHost['maintenanceid'],
-				'output' => API_OUTPUT_EXTEND,
-				'limit' => 1
-			));
+			if (array_key_exists($host['maintenanceid'], $maintenances)) {
+				$maintenance = $maintenances[$host['maintenanceid']];
 
-			if ($maintenance = reset($maintenances)) {
-				$hint = $maintenance['name'].' ['.($triggerHost['maintenance_type']
+				$hint = $maintenance['name'].' ['.($host['maintenance_type']
 					? _('Maintenance without data collection')
 					: _('Maintenance with data collection')).']';
 
-				if (isset($maintenance['description'])) {
-					// double quotes mandatory
+				if ($maintenance['description']) {
 					$hint .= "\n".$maintenance['description'];
 				}
 
-				$maintenanceIcon->setHint($hint);
-				$maintenanceIcon->addClass('pointer');
+				$maintenance_icon->setHint($hint);
 			}
 
-			$hostName->addItem($maintenanceIcon);
+			$host_name = (new CSpan([$host_name, $maintenance_icon]))->addClass(ZBX_STYLE_REL_CONTAINER);
 		}
 
-		$hostList[] = $hostName;
+		$hostList[] = $host_name;
 		$hostList[] = ', ';
 	}
 	array_pop($hostList);
 
-	// host
-	$hostColumn = new CCol($hostList);
-	$hostColumn->addStyle('white-space: normal;');
-
 	// status
 	$statusSpan = new CSpan(trigger_value2str($trigger['value']));
 
@@ -640,71 +637,57 @@ foreach ($triggers as $trigger) {
 		$config['event_ack_enable'] ? ($trigger['event_count'] == 0) : false
 	);
 
-	$lastChangeDate = zbx_date2str(DATE_TIME_FORMAT_SECONDS, $trigger['lastchange']);
-	$lastChange = empty($trigger['lastchange'])
-		? $lastChangeDate
-		: new CLink($lastChangeDate,
-			'events.php?filter_set=1&triggerid='.$trigger['triggerid'].'&source='.EVENT_SOURCE_TRIGGERS.
-				'&stime='.date(TIMESTAMP_FORMAT, $trigger['lastchange']).'&period='.ZBX_PERIOD_DEFAULT
-		);
-
+	// open or close
 	// acknowledge
 	if ($config['event_ack_enable']) {
 		if ($trigger['hasEvents']) {
+			$ack_checkbox = new CCheckBox('eventids['.$trigger['lastEvent']['eventid'].']',
+				$trigger['lastEvent']['eventid']
+			);
 			if ($trigger['event_count']) {
-				$ackColumn = new CCol(array(
-					new CLink(
-						_('Acknowledge'),
-						'acknow.php?'.
-							'triggers[]='.$trigger['triggerid'].
-							'&backurl='.$page['file'],
-						'on'
-					), ' ('.$trigger['event_count'].')'
-				));
+				$ackColumn = [
+					(new CLink(_('No'),
+						'zabbix.php?action=acknowledge.edit'.
+							'&acknowledge_type='.ZBX_ACKNOWLEDGE_PROBLEM.
+							'&eventids[]='.$trigger['lastEvent']['eventid'].
+							'&backurl='.$page['file']
+					))
+						->addClass(ZBX_STYLE_LINK_ALT)
+						->addClass(ZBX_STYLE_RED),
+					CViewHelper::showNum($trigger['event_count'])
+				];
 			}
 			else {
-				$ackColumn = new CCol(
-					new CLink(
-						_('Acknowledged'),
-						'acknow.php?'.
-							'eventid='.$trigger['lastEvent']['eventid'].
-							'&triggerid='.$trigger['lastEvent']['objectid'].
-							'&backurl='.$page['file'],
-						'off'
-				));
+				$ackColumn = (new CLink(_('Yes'),
+					'zabbix.php?action=acknowledge.edit'.
+						'&acknowledge_type='.ZBX_ACKNOWLEDGE_PROBLEM.
+						'&eventids[]='.$trigger['lastEvent']['eventid'].
+						'&backurl='.$page['file']
+				))
+					->addClass(ZBX_STYLE_LINK_ALT)
+					->addClass(ZBX_STYLE_GREEN);
 			}
 		}
 		else {
-			$ackColumn = new CCol(_('No events'), 'unknown');
+			$ack_checkbox = '';
+			$ackColumn = (new CCol(_('No events')))->addClass(ZBX_STYLE_GREY);
 		}
 	}
 	else {
+		$ack_checkbox = null;
 		$ackColumn = null;
 	}
 
-	// open or close
-	if ($showEvents != EVENTS_OPTION_NOEVENT && !empty($trigger['events'])) {
-		$openOrCloseDiv = new CDiv(SPACE, 'filterclosed');
-		$openOrCloseDiv->setAttribute('data-switcherid', $trigger['triggerid']);
-	}
-	elseif ($showEvents == EVENTS_OPTION_NOEVENT) {
-		$openOrCloseDiv = null;
+	if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
+		$openOrCloseDiv = $trigger['events']
+			? (new CDiv())
+				->addClass(ZBX_STYLE_TREEVIEW)
+				->setAttribute('data-switcherid', $trigger['triggerid'])
+				->addItem((new CSpan())->addClass(ZBX_STYLE_ARROW_RIGHT))
+			: '';
 	}
 	else {
-		$openOrCloseDiv = SPACE;
-	}
-
-	// severity
-	$severityColumn = getSeverityCell($trigger['priority'], null, !$trigger['value']);
-	if ($showEventColumn) {
-		$severityColumn->setColSpan(2);
-	}
-
-	// unknown triggers
-	$unknown = SPACE;
-	if ($trigger['state'] == TRIGGER_STATE_UNKNOWN) {
-		$unknown = new CDiv(SPACE, 'status_icon iconunknown');
-		$unknown->setHint($trigger['error'], 'on');
+		$openOrCloseDiv = null;
 	}
 
 	// comments
@@ -713,77 +696,66 @@ foreach ($triggers as $trigger) {
 	}
 	else {
 		$comments = zbx_empty($trigger['comments'])
-			? new CSpan('-')
+			? new CSpan('')
 			: new CLink(_('Show'), 'tr_comments.php?triggerid='.$trigger['triggerid']);
 	}
 
-	$triggerTable->addRow(array(
+	$triggerTable->addRow([
 		$openOrCloseDiv,
-		$config['event_ack_enable'] ?
-			($showEventColumn ? null : new CCheckBox('triggers['.$trigger['triggerid'].']', 'no', null, $trigger['triggerid'])) : null,
-		$severityColumn,
+		$ack_checkbox,
+		getSeverityCell($trigger['priority'], $config, null, !$trigger['value']),
 		$statusSpan,
-		$unknown,
-		$lastChange,
-		empty($trigger['lastchange']) ? '-' : zbx_date2age($trigger['lastchange']),
-		$showEventColumn ? SPACE : null,
+		($trigger['state'] == TRIGGER_STATE_UNKNOWN) ? makeUnknownIcon($trigger['error']) : '',
+		($trigger['lastchange'] == 0)
+			? _('Never')
+			: new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $trigger['lastchange']),
+				'events.php?filter_set=1&triggerid='.$trigger['triggerid'].'&source='.EVENT_SOURCE_TRIGGERS.
+					'&stime='.date(TIMESTAMP_FORMAT, $trigger['lastchange']).'&period='.ZBX_PERIOD_DEFAULT
+			),
+		($trigger['lastchange'] == 0) ? '' : zbx_date2age($trigger['lastchange']),
+		($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) ? '' : null,
 		$ackColumn,
-		$hostColumn,
-		$triggerDescription,
+		$hostList,
+		$description,
 		$comments
-	), 'even_row');
+	]);
 
-	if ($showEvents != EVENTS_OPTION_NOEVENT && !empty($trigger['events'])) {
-		$i = 1;
-		foreach ($trigger['events'] as $enum => $event) {
-			$i++;
+	if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
+		$next_event_clock = time();
+
+		foreach (array_slice($trigger['events'], 0, $config['event_show_max']) as $enum => $event) {
+			if ($showEvents == EVENTS_OPTION_NOT_ACK) {
+				if ($event['acknowledged'] || $event['value'] != TRIGGER_VALUE_TRUE) {
+					continue;
+				}
+			}
 
 			$eventStatusSpan = new CSpan(trigger_value2str($event['value']));
 
 			// add colors and blinking to span depending on configuration and trigger parameters
-			addTriggerValueStyle(
-				$eventStatusSpan,
-				$event['value'],
-				$event['clock'],
-				$event['acknowledged']
-			);
-
-			$statusSpan = new CCol($eventStatusSpan);
-			$statusSpan->setColSpan(2);
-
-			$ack = getEventAckState($event, true);
-
-			$ackCheckBox = ($event['acknowledged'] == 0 && $event['value'] == TRIGGER_VALUE_TRUE)
-				? new CCheckBox('events['.$event['eventid'].']', 'no', null, $event['eventid'])
-				: SPACE;
+			addTriggerValueStyle($eventStatusSpan, $event['value'], $event['clock'],
+				$config['event_ack_enable'] && $event['acknowledged']);
 
 			$clock = new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
 				'tr_events.php?triggerid='.$trigger['triggerid'].'&eventid='.$event['eventid']);
 
-			$nextClock = isset($trigger['events'][$enum - 1]) ? $trigger['events'][$enum - 1]['clock'] : time();
-
-			$emptyColumn = new CCol(SPACE);
-			$emptyColumn->setColSpan(3);
-			$ackCheckBoxColumn = new CCol($ackCheckBox);
-			$ackCheckBoxColumn->setColSpan(2);
-
-			$row = new CRow(array(
-				SPACE,
-				$config['event_ack_enable'] ? $ackCheckBoxColumn : null,
-				$statusSpan,
-				$clock,
-				zbx_date2age($event['clock']),
-				zbx_date2age($nextClock, $event['clock']),
-				($config['event_ack_enable']) ? $ack : null,
-				$emptyColumn
-			), 'odd_row');
-			$row->setAttribute('data-parentid', $trigger['triggerid']);
-			$row->addStyle('display: none;');
-			$triggerTable->addRow($row);
-
-			if ($i > $config['event_show_max']) {
-				break;
+			if ($enum != 0) {
+				$next_event_clock = $trigger['events'][$enum - 1]['clock'];
 			}
+
+			$triggerTable->addRow(
+				(new CRow([
+					(new CCol())->setColSpan($config['event_ack_enable'] ? 3 : 2),
+					(new CCol($eventStatusSpan))->setColSpan(2),
+					$clock,
+					zbx_date2age($event['clock']),
+					zbx_date2age($next_event_clock, $event['clock']),
+					$config['event_ack_enable'] ? getEventAckState($event, $page['file']) : null,
+					(new CCol())->setColSpan(3)
+				]))
+					->setAttribute('data-parentid', $trigger['triggerid'])
+					->addStyle('display: none;')
+			);
 		}
 	}
 }
@@ -793,22 +765,13 @@ foreach ($triggers as $trigger) {
  */
 $footer = null;
 if ($config['event_ack_enable']) {
-	$goComboBox = new CComboBox('action');
-	$goComboBox->addItem('trigger.bulkacknowledge', _('Bulk acknowledge'));
-
-	$goButton = new CSubmit('goButton', _('Go').' (0)');
-	$goButton->setAttribute('id', 'goButton');
-
-	$showEventColumn
-		? zbx_add_post_js('chkbxRange.pageGoName = "events";')
-		: zbx_add_post_js('chkbxRange.pageGoName = "triggers";');
-
-	$footer = get_table_header(array($goComboBox, $goButton));
+	$footer = new CActionButtonList('action', 'eventids', [
+		'acknowledge.edit' => ['name' => _('Bulk acknowledge')]
+	]);
 }
 
-$triggerForm->addItem(array($paging, $triggerTable, $paging, $footer));
-$triggerWidget->addItem($triggerForm);
-$triggerWidget->show();
+$triggerForm->addItem([$triggerTable, $paging, $footer]);
+$triggerWidget->addItem($triggerForm)->show();
 
 zbx_add_post_js('jqBlink.blink();');
 zbx_add_post_js('var switcher = new CSwitcher(\''.$switcherName.'\');');
