@@ -26,66 +26,76 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of triggers');
 $page['file'] = 'triggers.php';
-$page['hist_arg'] = array('hostid', 'groupid');
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
-$fields = array(
-	'groupid' =>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'hostid' =>				array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'triggerid' =>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'),
-	'copy_type' =>			array(T_ZBX_INT, O_OPT, P_SYS,	IN(array(COPY_TYPE_TO_HOST, COPY_TYPE_TO_TEMPLATE, COPY_TYPE_TO_HOST_GROUP)), 'isset({copy})'),
-	'copy_mode' =>			array(T_ZBX_INT, O_OPT, P_SYS,	IN('0'),	null),
-	'type' =>				array(T_ZBX_INT, O_OPT, null,	IN('0,1'),	null),
-	'description' =>		array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Name')),
-	'expression' =>			array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Expression')),
-	'priority' =>			array(T_ZBX_INT, O_OPT, null,	IN('0,1,2,3,4,5'), 'isset({add}) || isset({update})'),
-	'comments' =>			array(T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'),
-	'url' =>				array(T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'),
-	'status' =>				array(T_ZBX_STR, O_OPT, null,	null,		null),
-	'input_method' =>		array(T_ZBX_INT, O_OPT, null,	NOT_EMPTY,	'isset({toggle_input_method})'),
-	'expr_temp' =>			array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({add_expression}) || isset({and_expression}) || isset({or_expression}) || isset({replace_expression}))', _('Expression')),
-	'expr_target_single' => array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({and_expression}) || isset({or_expression}) || isset({replace_expression}))'),
-	'dependencies' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID,		null),
-	'new_dependency' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID.'{}>0', 'isset({add_dependency})'),
-	'g_triggerid' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID,		null),
-	'copy_targetid' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID,		null),
-	'copy_groupid' =>		array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({copy}) && (isset({copy_type}) && {copy_type} == 0)'),
-	'showdisabled' =>		array(T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	null),
-	'visible' =>			array(T_ZBX_STR, O_OPT, null,	null,		null),
+$fields = [
+	'groupid' =>			[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
+	'hostid' =>				[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null],
+	'triggerid' =>			[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'],
+	'copy_type' =>			[T_ZBX_INT, O_OPT, P_SYS,	IN([COPY_TYPE_TO_HOST, COPY_TYPE_TO_TEMPLATE, COPY_TYPE_TO_HOST_GROUP]), 'isset({copy})'],
+	'copy_mode' =>			[T_ZBX_INT, O_OPT, P_SYS,	IN('0'),	null],
+	'type' =>				[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
+	'description' =>		[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Name')],
+	'expression' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Expression')],
+	'priority' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1,2,3,4,5'), 'isset({add}) || isset({update})'],
+	'comments' =>			[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
+	'url' =>				[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
+	'status' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'input_method' =>		[T_ZBX_INT, O_OPT, null,	NOT_EMPTY,	'isset({toggle_input_method})'],
+	'expr_temp' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({add_expression}) || isset({and_expression}) || isset({or_expression}) || isset({replace_expression}))', _('Expression')],
+	'expr_target_single' => [T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({and_expression}) || isset({or_expression}) || isset({replace_expression}))'],
+	'dependencies' =>		[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
+	'new_dependency' =>		[T_ZBX_INT, O_OPT, null,	DB_ID.'{}>0', 'isset({add_dependency})'],
+	'g_triggerid' =>		[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
+	'copy_targetid' =>		[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
+	'copy_groupid' =>		[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({copy}) && (isset({copy_type}) && {copy_type} == 0)'],
+	'visible' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	// filter
+	'filter_set' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_rst' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_priority' =>	[T_ZBX_INT, O_OPT, null,
+		IN([
+			-1, TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_INFORMATION, TRIGGER_SEVERITY_WARNING,
+			TRIGGER_SEVERITY_AVERAGE, TRIGGER_SEVERITY_HIGH, TRIGGER_SEVERITY_DISASTER
+		]), null
+	],
+	'filter_state' =>		[T_ZBX_INT, O_OPT, null,	IN([-1, TRIGGER_STATE_NORMAL, TRIGGER_STATE_UNKNOWN]), null],
+	'filter_status' =>		[T_ZBX_INT, O_OPT, null,
+		IN([-1, TRIGGER_STATUS_ENABLED, TRIGGER_STATUS_DISABLED]), null
+	],
 	// actions
-	'action' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,
+	'action' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 								IN('"trigger.masscopyto","trigger.massdelete","trigger.massdisable",'.
 									'"trigger.massenable","trigger.massupdate","trigger.massupdateform"'
 								),
 								null
-							),
-	'toggle_input_method' =>array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'add_expression' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'and_expression' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'or_expression' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'replace_expression' =>	array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'remove_expression' =>	array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'test_expression' =>	array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'add_dependency' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'group_enable' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'group_disable' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'group_delete' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'copy' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'clone' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'add' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'update' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'massupdate' =>			array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'delete' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'cancel' =>				array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'form' =>				array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'form_refresh' =>		array(T_ZBX_INT, O_OPT, null,	null,		null),
+							],
+	'toggle_input_method' =>[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'add_expression' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'and_expression' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'or_expression' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'replace_expression' =>	[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'remove_expression' =>	[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'test_expression' =>	[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'add_dependency' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'group_enable' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'group_disable' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'group_delete' =>		[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'copy' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'clone' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'add' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'update' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'massupdate' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'delete' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
+	'cancel' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'form' =>				[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'form_refresh' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
 	// sort and sortorder
-	'sort' =>				array(T_ZBX_STR, O_OPT, P_SYS, IN('"description","priority","status"'),		null),
-	'sortorder' =>			array(T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null)
-);
-$_REQUEST['showdisabled'] = getRequest('showdisabled', CProfile::get('web.triggers.showdisabled', 1));
+	'sort' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"description","priority","status"'),		null],
+	'sortorder' =>			[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
+];
 
 check_fields($fields);
 
@@ -96,12 +106,12 @@ $_REQUEST['type'] = isset($_REQUEST['type']) ? TRIGGER_MULT_EVENT_ENABLED : TRIG
 $triggerId = getRequest('triggerid');
 
 if ($triggerId !== null) {
-	$trigger = API::Trigger()->get(array(
-		'triggerids' => array($triggerId),
-		'output' => array('triggerid'),
-		'filter' => array('flags' => ZBX_FLAG_DISCOVERY_NORMAL),
+	$trigger = API::Trigger()->get([
+		'output' => ['triggerid'],
+		'triggerids' => [$triggerId],
+		'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
 		'editable' => true
-	));
+	]);
 
 	if (!$trigger) {
 		access_deny();
@@ -109,17 +119,25 @@ if ($triggerId !== null) {
 }
 
 // Validate permissions to a group of triggers for mass enable/disable actions.
-$triggerIds = getRequest('g_triggerid', array());
+$triggerIds = getRequest('g_triggerid', []);
 $triggerIds = zbx_toArray($triggerIds);
 
 if ($triggerIds && !API::Trigger()->isWritable($triggerIds)) {
 	access_deny();
 }
 
-// Validate permissions to host.
-if (getRequest('hostid') && !API::Host()->isWritable(array($_REQUEST['hostid']))) {
+// Validate permissions to group.
+$groupId = getRequest('groupid');
+if ($groupId && !API::HostGroup()->isWritable([$groupId])) {
 	access_deny();
 }
+
+// Validate permissions to host.
+$hostId = getRequest('hostid');
+if ($hostId && !API::Host()->isWritable([$hostId])) {
+	access_deny();
+}
+
 /*
  * Actions
  */
@@ -146,37 +164,40 @@ elseif (isset($_REQUEST['clone']) && isset($_REQUEST['triggerid'])) {
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('add') || hasRequest('update')) {
-	$trigger = array(
+	$trigger = [
 		'expression' => getRequest('expression'),
 		'description' => getRequest('description'),
-		'priority' => getRequest('priority'),
-		'status' => getRequest('status'),
-		'type' => getRequest('type'),
-		'comments' => getRequest('comments'),
 		'url' => getRequest('url'),
-		'dependencies' => zbx_toObject(getRequest('dependencies', array()), 'triggerid')
-	);
+		'status' => getRequest('status'),
+		'priority' => getRequest('priority'),
+		'comments' => getRequest('comments'),
+		'type' => getRequest('type'),
+		'dependencies' => zbx_toObject(getRequest('dependencies', []), 'triggerid')
+	];
 
 	if (hasRequest('update')) {
 		// update only changed fields
-		$oldTrigger = API::Trigger()->get(array(
-			'triggerids' => getRequest('triggerid'),
-			'output' => API_OUTPUT_EXTEND,
-			'selectDependencies' => array('triggerid')
-		));
-		if (!$oldTrigger) {
+
+		$old_triggers = API::Trigger()->get([
+			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'type'],
+			'selectDependencies' => ['triggerid'],
+			'triggerids' => getRequest('triggerid')
+		]);
+		if (!$old_triggers) {
 			access_deny();
 		}
 
-		$oldTrigger = reset($oldTrigger);
-		$oldTrigger['dependencies'] = zbx_toHash(zbx_objectValues($oldTrigger['dependencies'], 'triggerid'));
+		$old_triggers = CMacrosResolverHelper::resolveTriggerExpressions($old_triggers);
+
+		$old_trigger = reset($old_triggers);
+		$old_trigger['dependencies'] = zbx_toHash(zbx_objectValues($old_trigger['dependencies'], 'triggerid'));
 
 		$newDependencies = $trigger['dependencies'];
-		$oldDependencies = $oldTrigger['dependencies'];
-		unset($trigger['dependencies']);
-		unset($oldTrigger['dependencies']);
+		$oldDependencies = $old_trigger['dependencies'];
 
-		$triggerToUpdate = array_diff_assoc($trigger, $oldTrigger);
+		unset($trigger['dependencies'], $old_trigger['dependencies']);
+
+		$triggerToUpdate = array_diff_assoc($trigger, $old_trigger);
 		$triggerToUpdate['triggerid'] = getRequest('triggerid');
 
 		// dependencies
@@ -194,11 +215,14 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		if ($updateDepencencies) {
 			$triggerToUpdate['dependencies'] = $newDependencies;
 		}
+
 		$result = API::Trigger()->update($triggerToUpdate);
+
 		show_messages($result, _('Trigger updated'), _('Cannot update trigger'));
 	}
 	else {
 		$result = API::Trigger()->create($trigger);
+
 		show_messages($result, _('Trigger added'), _('Cannot add trigger'));
 	}
 
@@ -210,7 +234,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 elseif (isset($_REQUEST['delete']) && isset($_REQUEST['triggerid'])) {
 	DBstart();
 
-	$result = API::Trigger()->delete(array(getRequest('triggerid')));
+	$result = API::Trigger()->delete([getRequest('triggerid')]);
 	$result = DBend($result);
 
 	if ($result) {
@@ -221,7 +245,7 @@ elseif (isset($_REQUEST['delete']) && isset($_REQUEST['triggerid'])) {
 }
 elseif (isset($_REQUEST['add_dependency']) && isset($_REQUEST['new_dependency'])) {
 	if (!isset($_REQUEST['dependencies'])) {
-		$_REQUEST['dependencies'] = array();
+		$_REQUEST['dependencies'] = [];
 	}
 	foreach ($_REQUEST['new_dependency'] as $triggerid) {
 		if (!uint_in_array($triggerid, $_REQUEST['dependencies'])) {
@@ -229,53 +253,54 @@ elseif (isset($_REQUEST['add_dependency']) && isset($_REQUEST['new_dependency'])
 		}
 	}
 }
-elseif (hasRequest('action') && getRequest('action') == 'trigger.massupdate' && hasRequest('massupdate') && hasRequest('g_triggerid')) {
-	$visible = getRequest('visible', array());
+elseif (hasRequest('action') && getRequest('action') === 'trigger.massupdate'
+		&& hasRequest('massupdate') && hasRequest('g_triggerid')) {
+	$result = true;
+	$visible = getRequest('visible', []);
 
-	// update triggers
-	$triggersToUpdate = array();
-	foreach (getRequest('g_triggerid') as $triggerid) {
-		$trigger = array('triggerid' => $triggerid);
+	if ($visible) {
+		$triggersToUpdate = [];
 
-		if (isset($visible['priority'])) {
-			$trigger['priority'] = getRequest('priority');
-		}
-		if (isset($visible['dependencies'])) {
-			$trigger['dependencies'] = zbx_toObject(getRequest('dependencies', array()), 'triggerid');
+		foreach (getRequest('g_triggerid') as $triggerid) {
+			$trigger = ['triggerid' => $triggerid];
+
+			if (isset($visible['priority'])) {
+				$trigger['priority'] = getRequest('priority');
+			}
+			if (isset($visible['dependencies'])) {
+				$trigger['dependencies'] = zbx_toObject(getRequest('dependencies', []), 'triggerid');
+			}
+
+			$triggersToUpdate[] = $trigger;
 		}
 
-		$triggersToUpdate[] = $trigger;
+		$result = (bool) API::Trigger()->update($triggersToUpdate);
 	}
 
-	DBstart();
-
-	$result = API::Trigger()->update($triggersToUpdate);
-	$result = DBend($result);
-
 	if ($result) {
 		unset($_REQUEST['form'], $_REQUEST['g_triggerid']);
 		uncheckTableRows(getRequest('hostid'));
 	}
 	show_messages($result, _('Trigger updated'), _('Cannot update trigger'));
 }
-elseif (hasRequest('action') && str_in_array(getRequest('action'), array('trigger.massenable', 'trigger.massdisable')) && hasRequest('g_triggerid')) {
+elseif (hasRequest('action') && str_in_array(getRequest('action'), ['trigger.massenable', 'trigger.massdisable']) && hasRequest('g_triggerid')) {
 	$enable = (getRequest('action') == 'trigger.massenable');
 	$status = $enable ? TRIGGER_STATUS_ENABLED : TRIGGER_STATUS_DISABLED;
-	$update = array();
+	$update = [];
 
 	// get requested triggers with permission check
-	$dbTriggers = API::Trigger()->get(array(
-		'output' => array('triggerid', 'status'),
+	$dbTriggers = API::Trigger()->get([
+		'output' => ['triggerid', 'status'],
 		'triggerids' => getRequest('g_triggerid'),
 		'editable' => true
-	));
+	]);
 
 	if ($dbTriggers) {
 		foreach ($dbTriggers as $dbTrigger) {
-			$update[] = array(
+			$update[] = [
 				'triggerid' => $dbTrigger['triggerid'],
 				'status' => $status
-			);
+			];
 		}
 
 		$result = API::Trigger()->update($update);
@@ -306,7 +331,7 @@ elseif (hasRequest('action') && getRequest('action') == 'trigger.masscopyto' &&
 		}
 		// host groups
 		else {
-			$hosts_ids = array();
+			$hosts_ids = [];
 			$group_ids = getRequest('copy_targetid');
 
 			$db_hosts = DBselect(
@@ -373,44 +398,75 @@ else {
 	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
 	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
-	$data = array(
-		'showdisabled' => getRequest('showdisabled', 1),
-		'parent_discoveryid' => null,
-		'triggers' => array(),
+	if (hasRequest('filter_set')) {
+		CProfile::update('web.triggers.filter_priority', getRequest('filter_priority', -1), PROFILE_TYPE_INT);
+		CProfile::update('web.triggers.filter_state', getRequest('filter_state', -1), PROFILE_TYPE_INT);
+		CProfile::update('web.triggers.filter_status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
+	}
+	elseif (hasRequest('filter_rst')) {
+		CProfile::delete('web.triggers.filter_priority');
+		CProfile::delete('web.triggers.filter_state');
+		CProfile::delete('web.triggers.filter_status');
+	}
+
+	$config = select_config();
+
+	$data = [
+		'filter_priority' => CProfile::get('web.triggers.filter_priority', -1),
+		'filter_state' => CProfile::get('web.triggers.filter_state', -1),
+		'filter_status' => CProfile::get('web.triggers.filter_status', -1),
+		'triggers' => [],
 		'sort' => $sortField,
-		'sortorder' => $sortOrder
-	);
+		'sortorder' => $sortOrder,
+		'config' => $config
+	];
 
-	CProfile::update('web.triggers.showdisabled', $data['showdisabled'], PROFILE_TYPE_INT);
-
-	$data['pageFilter'] = new CPageFilter(array(
-		'groups' => array('with_hosts_and_templates' => true, 'editable' => true),
-		'hosts' => array('templated_hosts' => true, 'editable' => true),
-		'triggers' => array('editable' => true),
+	$data['pageFilter'] = new CPageFilter([
+		'groups' => ['with_hosts_and_templates' => true, 'editable' => true],
+		'hosts' => ['templated_hosts' => true, 'editable' => true],
+		'triggers' => ['editable' => true],
 		'groupid' => getRequest('groupid'),
 		'hostid' => getRequest('hostid')
-	));
+	]);
 	$data['groupid'] = $data['pageFilter']->groupid;
 	$data['hostid'] = $data['pageFilter']->hostid;
 
 	// get triggers
 	if ($data['pageFilter']->hostsSelected) {
-		$options = array(
+		$options = [
 			'editable' => true,
 			'sortfield' => $sortField,
 			'limit' => $config['search_limit'] + 1
-		);
+		];
 
 		if ($sortField === 'status') {
-			$options['output'] = array('triggerid', 'status', 'state');
+			$options['output'] = ['triggerid', 'status', 'state'];
 		}
 		else {
-			$options['output'] = array('triggerid', $sortField);
+			$options['output'] = ['triggerid', $sortField];
 		}
 
-		if (empty($data['showdisabled'])) {
-			$options['filter']['status'] = TRIGGER_STATUS_ENABLED;
+		if ($data['filter_priority'] != -1) {
+			$options['filter']['priority'] = $data['filter_priority'];
 		}
+
+		switch ($data['filter_state']) {
+			case TRIGGER_STATE_NORMAL:
+				$options['filter']['state'] = TRIGGER_STATE_NORMAL;
+				$options['filter']['status'] = TRIGGER_STATUS_ENABLED;
+				break;
+
+			case TRIGGER_STATE_UNKNOWN:
+				$options['filter']['state'] = TRIGGER_STATE_UNKNOWN;
+				$options['filter']['status'] = TRIGGER_STATUS_ENABLED;
+				break;
+
+			default:
+				if ($data['filter_status'] != -1) {
+					$options['filter']['status'] = $data['filter_status'];
+				}
+		}
+
 		if ($data['pageFilter']->hostid > 0) {
 			$options['hostids'] = $data['pageFilter']->hostid;
 		}
@@ -433,15 +489,13 @@ else {
 	// paging
 	$data['paging'] = getPagingLine($data['triggers'], $sortOrder);
 
-	$data['triggers'] = API::Trigger()->get(array(
-		'triggerids' => zbx_objectValues($data['triggers'], 'triggerid'),
-		'output' => API_OUTPUT_EXTEND,
-		'selectHosts' => API_OUTPUT_EXTEND,
-		'selectItems' => array('itemid', 'hostid', 'key_', 'type', 'flags', 'status'),
-		'selectFunctions' => API_OUTPUT_EXTEND,
-		'selectDependencies' => array('triggerid', 'description'),
-		'selectDiscoveryRule' => API_OUTPUT_EXTEND
-	));
+	$data['triggers'] = API::Trigger()->get([
+		'output' => ['triggerid', 'expression', 'description', 'status', 'priority', 'error', 'templateid', 'state'],
+		'selectHosts' => ['hostid', 'host', 'name'],
+		'selectDependencies' => ['triggerid', 'description'],
+		'selectDiscoveryRule' => ['itemid', 'name'],
+		'triggerids' => zbx_objectValues($data['triggers'], 'triggerid')
+	]);
 
 	// sort for displaying full results
 	if ($sortField === 'status') {
@@ -451,37 +505,31 @@ else {
 		order_result($data['triggers'], $sortField, $sortOrder);
 	}
 
-	$dependencyIds = array();
+	$depTriggerIds = [];
 	foreach ($data['triggers'] as $trigger) {
 		foreach ($trigger['dependencies'] as $depTrigger) {
-			$dependencyIds[$depTrigger['triggerid']] = $depTrigger['triggerid'];
+			$depTriggerIds[$depTrigger['triggerid']] = true;
 		}
 	}
 
-	$dependencyTriggers = array();
-	if ($dependencyIds) {
-		$dependencyTriggers = API::Trigger()->get(array(
-			'triggerids' => $dependencyIds,
-			'output' => array('triggerid', 'flags', 'description', 'status'),
-			'selectHosts' => array('hostid', 'name'),
+	$dependencyTriggers = [];
+	if ($depTriggerIds) {
+		$dependencyTriggers = API::Trigger()->get([
+			'output' => ['triggerid', 'description', 'status', 'flags'],
+			'selectHosts' => ['hostid', 'name'],
+			'triggerids' => array_keys($depTriggerIds),
 			'preservekeys' => true
-		));
+		]);
 
-		// sort dependencies
 		foreach ($data['triggers'] as &$trigger) {
-			if (count($trigger['dependencies']) > 1) {
-				order_result($trigger['dependencies'], 'description', ZBX_SORT_UP);
-			}
+			order_result($trigger['dependencies'], 'description', ZBX_SORT_UP);
 		}
 		unset($trigger);
 
-		// sort dependency trigger hosts
-		foreach ($dependencyTriggers as &$trigger) {
-			if (count($dependencyTriggers[$trigger['triggerid']]['hosts']) > 1) {
-				order_result($dependencyTriggers[$trigger['triggerid']]['hosts'], 'name', ZBX_SORT_UP);
-			}
+		foreach ($dependencyTriggers as &$dependencyTrigger) {
+			order_result($dependencyTrigger['hosts'], 'name', ZBX_SORT_UP);
 		}
-		unset($trigger);
+		unset($dependencyTrigger);
 	}
 
 	$data['dependencyTriggers'] = $dependencyTriggers;
@@ -489,12 +537,12 @@ else {
 	// get real hosts
 	$data['realHosts'] = getParentHostsByTriggers($data['triggers']);
 
-	// determine, show or not column of errors
-	if ($data['hostid'] > 0) {
-		$data['showInfoColumn'] = (bool) API::Host()->get(array(
-			'hostids' => $data['hostid'],
-			'output' => array('status')
-		));
+	// do not show 'Info' column, if it is a template
+	if ($data['hostid']) {
+		$data['showInfoColumn'] = (bool) API::Host()->get([
+			'output' => [],
+			'hostids' => $data['hostid']
+		]);
 	}
 	else {
 		$data['showInfoColumn'] = true;
