@@ -69,6 +69,7 @@ struct	_DC_TRIGGER;
 #define ZBX_DB_SERVER	1
 #define ZBX_DB_PROXY	2
 
+#define TRIGGER_URL_LEN			255
 #define TRIGGER_DESCRIPTION_LEN		255
 #define TRIGGER_EXPRESSION_LEN		2048
 #define TRIGGER_EXPRESSION_LEN_MAX	(TRIGGER_EXPRESSION_LEN + 1)
@@ -84,7 +85,8 @@ struct	_DC_TRIGGER;
 #define HOST_HOST_LEN			MAX_ZBX_HOSTNAME_LEN
 #define HOST_HOST_LEN_MAX		(HOST_HOST_LEN + 1)
 #define HOST_NAME_LEN			128
-#define HOST_ERROR_LEN			128
+#define HOST_ERROR_LEN			2048
+#define HOST_ERROR_LEN_MAX		(HOST_ERROR_LEN + 1)
 #define HOST_IPMI_USERNAME_LEN		16
 #define HOST_IPMI_USERNAME_LEN_MAX	(HOST_IPMI_USERNAME_LEN + 1)
 #define HOST_IPMI_PASSWORD_LEN		20
@@ -101,6 +103,7 @@ struct	_DC_TRIGGER;
 
 #define ITEM_NAME_LEN			255
 #define ITEM_KEY_LEN			255
+#define ITEM_UNITS_LEN			255
 #define ITEM_SNMP_COMMUNITY_LEN		64
 #define ITEM_SNMP_COMMUNITY_LEN_MAX	(ITEM_SNMP_COMMUNITY_LEN + 1)
 #define ITEM_SNMP_OID_LEN		255
@@ -304,28 +307,23 @@ typedef struct
 {
 	zbx_uint64_t		mediatypeid;
 	zbx_media_type_t	type;
-	char	*description;
-	char	*smtp_server;
-	char	*smtp_helo;
-	char	*smtp_email;
-	char	*exec_path;
-	char	*gsm_modem;
-	char	*username;
-	char	*passwd;
+	char			*description;
+	char			*smtp_server;
+	char			*smtp_helo;
+	char			*smtp_email;
+	char			*exec_path;
+	char			*exec_params;
+	char			*gsm_modem;
+	char			*username;
+	char			*passwd;
+	unsigned short		smtp_port;
+	unsigned char		smtp_security;
+	unsigned char		smtp_verify_peer;
+	unsigned char		smtp_verify_host;
+	unsigned char		smtp_authentication;
 }
 DB_MEDIATYPE;
 
-typedef struct
-{
-	zbx_uint64_t	actionid;
-	char		*shortdata;
-	char		*longdata;
-	int		esc_period;
-	unsigned char	eventsource;
-	unsigned char	recovery_msg;
-}
-DB_ACTION;
-
 typedef struct
 {
 	zbx_uint64_t	conditionid;
@@ -485,8 +483,6 @@ char	*DBdyn_escape_string(const char *src);
 char	*DBdyn_escape_string_len(const char *src, size_t max_src_len);
 char	*DBdyn_escape_like_pattern(const char *src);
 
-void    DBget_item_from_db(DB_ITEM *item, DB_ROW row);
-
 zbx_uint64_t	DBadd_host(char *server, int port, int status, int useip, char *ip, int disable_until, int available);
 int	DBhost_exists(char *server);
 int	DBadd_templates_to_host(int hostid,int host_templateid);
@@ -539,6 +535,9 @@ zbx_uint64_t	DBadd_interface(zbx_uint64_t hostid, unsigned char type,
 const char	*DBget_inventory_field(unsigned char inventory_link);
 unsigned short	DBget_inventory_field_len(unsigned char inventory_link);
 
+void	DBset_host_inventory(zbx_uint64_t hostid, int inventory_mode);
+void	DBadd_host_inventory(zbx_uint64_t hostid, int inventory_mode);
+
 int	DBtxn_status(void);
 int	DBtxn_ongoing(void);
 
@@ -599,4 +598,48 @@ void	zbx_db_insert_clean(zbx_db_insert_t *self);
 void	zbx_db_insert_autoincrement(zbx_db_insert_t *self, const char *field_name);
 int	zbx_db_get_database_type(void);
 
+/* agent (ZABBIX, SNMP, IPMI, JMX) availability data */
+typedef struct
+{
+	/* flags specifying which fields are set, see ZBX_FLAGS_AGENT_STATUS_* defines */
+	unsigned char	flags;
+
+	/* agent availability fields */
+	unsigned char	available;
+	char		*error;
+	int		errors_from;
+	int		disable_until;
+}
+zbx_agent_availability_t;
+
+#define ZBX_FLAGS_AGENT_STATUS_NONE		0x00000000
+#define ZBX_FLAGS_AGENT_STATUS_AVAILABLE	0x00000001
+#define ZBX_FLAGS_AGENT_STATUS_ERROR		0x00000002
+#define ZBX_FLAGS_AGENT_STATUS_ERRORS_FROM	0x00000004
+#define ZBX_FLAGS_AGENT_STATUS_DISABLE_UNTIL	0x00000008
+
+#define ZBX_FLAGS_AGENT_STATUS		(ZBX_FLAGS_AGENT_STATUS_AVAILABLE |	\
+					ZBX_FLAGS_AGENT_STATUS_ERROR |		\
+					ZBX_FLAGS_AGENT_STATUS_ERRORS_FROM |	\
+					ZBX_FLAGS_AGENT_STATUS_DISABLE_UNTIL)
+
+#define ZBX_AGENT_ZABBIX	(INTERFACE_TYPE_AGENT - 1)
+#define ZBX_AGENT_SNMP		(INTERFACE_TYPE_SNMP - 1)
+#define ZBX_AGENT_IPMI		(INTERFACE_TYPE_IPMI - 1)
+#define ZBX_AGENT_JMX		(INTERFACE_TYPE_JMX - 1)
+#define ZBX_AGENT_UNKNOWN 	255
+#define ZBX_AGENT_MAX		INTERFACE_TYPE_COUNT
+
+typedef struct
+{
+	zbx_uint64_t			hostid;
+
+	zbx_agent_availability_t	agents[ZBX_AGENT_MAX];
+}
+zbx_host_availability_t;
+
+
+int	zbx_sql_add_host_availability(char **sql, size_t *sql_alloc, size_t *sql_offset,
+		const zbx_host_availability_t *ha);
+
 #endif
