@@ -27,8 +27,6 @@
 #define ZBX_SYNC_PARTIAL	0
 #define	ZBX_SYNC_FULL		1
 
-#define ZBX_SYNC_MAX	1000
-
 #define	ZBX_NO_POLLER			255
 #define	ZBX_POLLER_TYPE_NORMAL		0
 #define	ZBX_POLLER_TYPE_UNREACHABLE	1
@@ -42,13 +40,17 @@
 #define MAX_POLLER_ITEMS	128	/* MAX(MAX_JAVA_ITEMS, MAX_SNMP_ITEMS) */
 #define MAX_PINGER_ITEMS	128
 
+#define ZBX_TRIGGER_DEPENDENCY_LEVELS_MAX	32
+
+#define ZBX_SNMPTRAP_LOGGING_ENABLED	1
+
 extern char	*CONFIG_FILE;
 extern int	CONFIG_TIMEOUT;
 
 extern zbx_uint64_t	CONFIG_CONF_CACHE_SIZE;
 extern zbx_uint64_t	CONFIG_HISTORY_CACHE_SIZE;
+extern zbx_uint64_t	CONFIG_HISTORY_INDEX_CACHE_SIZE;
 extern zbx_uint64_t	CONFIG_TRENDS_CACHE_SIZE;
-extern zbx_uint64_t	CONFIG_TEXT_CACHE_SIZE;
 
 extern int	CONFIG_POLLER_FORKS;
 extern int	CONFIG_UNREACHABLE_POLLER_FORKS;
@@ -103,6 +105,18 @@ typedef struct
 	int		jmx_disable_until;
 	char		inventory_mode;
 	unsigned char	status;
+	unsigned char	tls_connect;
+	unsigned char	tls_accept;
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	char		tls_issuer[HOST_TLS_ISSUER_LEN_MAX];
+	char		tls_subject[HOST_TLS_SUBJECT_LEN_MAX];
+	char		tls_psk_identity[HOST_TLS_PSK_IDENTITY_LEN_MAX];
+	char		tls_psk[HOST_TLS_PSK_LEN_MAX];
+#endif
+	char		error[HOST_ERROR_LEN_MAX];
+	char		snmp_error[HOST_ERROR_LEN_MAX];
+	char		ipmi_error[HOST_ERROR_LEN_MAX];
+	char		jmx_error[HOST_ERROR_LEN_MAX];
 }
 DC_HOST;
 
@@ -198,9 +212,33 @@ typedef struct
 	char		port_orig[INTERFACE_PORT_LEN_MAX];
 	char		*addr;
 	unsigned short	port;
+	unsigned char	tls_connect;				/* how to connect: ZBX_TCP_SEC_UNENCRYPTED, */
+								/* ZBX_TCP_SEC_TLS_PSK or ZBX_TCP_SEC_TLS_CERT */
+#if HOST_TLS_ISSUER_LEN_MAX > HOST_TLS_PSK_IDENTITY_LEN_MAX
+	char		tls_arg1[HOST_TLS_ISSUER_LEN_MAX];	/* for passing 'tls_issuer' or 'tls_psk_identity' */
+								/* depending on value of 'tls_connect' */
+#else
+	char		tls_arg1[HOST_TLS_PSK_IDENTITY_LEN_MAX];
+#endif
+#if HOST_TLS_SUBJECT_LEN_MAX > HOST_TLS_PSK_LEN_MAX
+	char		tls_arg2[HOST_TLS_SUBJECT_LEN_MAX];	/* for passing 'tls_subject' or 'tls_psk' */
+								/* depending on value of 'tls_connect' */
+#else
+	char		tls_arg2[HOST_TLS_PSK_LEN_MAX];
+#endif
 }
 DC_PROXY;
 
+typedef struct
+{
+	zbx_uint64_t		actionid;
+	char			*formula;
+	unsigned char		eventsource;
+	unsigned char		evaltype;
+	zbx_vector_ptr_t	conditions;
+}
+zbx_action_eval_t;
+
 typedef struct
 {
 	const char	*host;
@@ -232,6 +270,30 @@ typedef struct
 }
 zbx_config_hk_t;
 
+/* global configuration data (loaded from config table) */
+typedef struct
+{
+	/* the fields set by zbx_config_get() function, see ZBX_CONFIG_FLAGS_ defines */
+	zbx_uint64_t	flags;
+
+	char		**severity_name;
+	zbx_uint64_t	discovery_groupid;
+	int		default_inventory_mode;
+	int		refresh_unsupported;
+	unsigned char	snmptrap_logging;
+
+	/* housekeeping related configuration data */
+	zbx_config_hk_t	hk;
+}
+zbx_config_t;
+
+#define ZBX_CONFIG_FLAGS_SEVERITY_NAME			0x00000001
+#define ZBX_CONFIG_FLAGS_DISCOVERY_GROUPID		0x00000002
+#define ZBX_CONFIG_FLAGS_DEFAULT_INVENTORY_MODE		0x00000004
+#define ZBX_CONFIG_FLAGS_REFRESH_UNSUPPORTED		0x00000008
+#define ZBX_CONFIG_FLAGS_SNMPTRAP_LOGGING		0x00000010
+#define ZBX_CONFIG_FLAGS_HOUSEKEEPER			0x00000020
+
 typedef struct
 {
 	zbx_uint64_t		itemid;
@@ -250,23 +312,13 @@ typedef struct
 }
 zbx_queue_item_t;
 
-typedef struct
-{
-	zbx_uint64_t	hostid;
-	int		errors_from;
-	int		disable_until;
-	unsigned char	type;
-	unsigned char	available;
-}
-zbx_host_availability_t;
-
 int	is_item_processed_by_server(unsigned char type, const char *key);
 int	in_maintenance_without_data_collection(unsigned char maintenance_status, unsigned char maintenance_type,
 		unsigned char type);
-void	dc_add_history(zbx_uint64_t itemid, unsigned char value_type, unsigned char flags, AGENT_RESULT *value,
+void	dc_add_history(zbx_uint64_t itemid, unsigned char value_type, unsigned char item_flags, AGENT_RESULT *result,
 		const zbx_timespec_t *ts, unsigned char state, const char *error);
 void	dc_flush_history();
-int	DCsync_history(int sync_type);
+int	DCsync_history(int sync_type, int *sync_num);
 void	init_database_cache();
 void	free_database_cache();
 
@@ -288,18 +340,18 @@ void	DCflush_nextchecks();
 #define ZBX_STATS_TREND_USED		12
 #define ZBX_STATS_TREND_FREE		13
 #define ZBX_STATS_TREND_PFREE		14
-#define ZBX_STATS_TEXT_TOTAL		15
-#define ZBX_STATS_TEXT_USED		16
-#define ZBX_STATS_TEXT_FREE		17
-#define ZBX_STATS_TEXT_PFREE		18
+#define ZBX_STATS_HISTORY_INDEX_TOTAL	15
+#define ZBX_STATS_HISTORY_INDEX_USED	16
+#define ZBX_STATS_HISTORY_INDEX_FREE	17
+#define ZBX_STATS_HISTORY_INDEX_PFREE	18
 void	*DCget_stats(int request);
 
 zbx_uint64_t	DCget_nextid(const char *table_name, int num);
 
 void	DCsync_configuration(void);
-void	init_configuration_cache();
-void	free_configuration_cache();
-void	DCload_config();
+void	init_configuration_cache(void);
+void	free_configuration_cache(void);
+void	DCload_config(void);
 
 void	DCconfig_get_triggers_by_triggerids(DC_TRIGGER *triggers, const zbx_uint64_t *triggerids, int *errcode,
 		size_t num);
@@ -312,7 +364,7 @@ void	DCconfig_get_functions_by_functionids(DC_FUNCTION *functions,
 		zbx_uint64_t *functionids, int *errcodes, size_t num);
 void	DCconfig_clean_functions(DC_FUNCTION *functions, int *errcodes, size_t num);
 void	DCconfig_clean_triggers(DC_TRIGGER *triggers, int *errcodes, size_t num);
-void	DCconfig_lock_triggers_by_itemids(zbx_uint64_t *itemids, int itemids_num, zbx_vector_uint64_t *triggerids);
+int	DCconfig_lock_triggers_by_history_items(zbx_vector_ptr_t *history_items, zbx_vector_uint64_t *triggerids);
 void	DCconfig_unlock_triggers(const zbx_vector_uint64_t *triggerids);
 void	DCconfig_unlock_all_triggers();
 void	DCconfig_get_triggers_by_itemids(zbx_hashset_t *trigger_info, zbx_vector_ptr_t *trigger_order,
@@ -328,19 +380,13 @@ int	DCconfig_get_poller_items(unsigned char poller_type, DC_ITEM *items);
 int	DCconfig_get_snmp_interfaceids_by_addr(const char *addr, zbx_uint64_t **interfaceids);
 size_t	DCconfig_get_snmp_items_by_interfaceid(zbx_uint64_t interfaceid, DC_ITEM **items);
 
-#define	CONFIG_REFRESH_UNSUPPORTED	1
-#define	CONFIG_DISCOVERY_GROUPID	2
-#define	CONFIG_SNMPTRAP_LOGGING		3
-
 #define ZBX_HK_OPTION_DISABLED		0
 #define ZBX_HK_OPTION_ENABLED		1
 
-void	*DCconfig_get_config_data(void *data, int type);
-void	DCconfig_get_config_hk(zbx_config_hk_t *data);
-int	DCget_trigger_severity_name(unsigned char priority, char **replace_to);
-
 void	DCrequeue_items(zbx_uint64_t *itemids, unsigned char *states, int *lastclocks, zbx_uint64_t *lastlogsizes,
 		int *mtimes, int *errcodes, size_t num);
+void	DCpoller_requeue_items(zbx_uint64_t *itemids, unsigned char *states, int *lastclocks, zbx_uint64_t *lastlogsizes,
+		int *mtimes, int *errcodes, size_t num, unsigned char poller_type, int *nextcheck);
 int	DCconfig_activate_host(DC_ITEM *item);
 int	DCconfig_deactivate_host(DC_ITEM *item, int now);
 
@@ -358,17 +404,17 @@ void	DCconfig_set_maintenance(const zbx_uint64_t *hostids, int hostids_num, int
 void	*DCconfig_get_stats(int request);
 
 int	DCconfig_get_proxypoller_hosts(DC_PROXY *proxies, int max_hosts);
-int	DCconfig_get_proxypoller_nextcheck();
+int	DCconfig_get_proxypoller_nextcheck(void);
 void	DCrequeue_proxy(zbx_uint64_t hostid, unsigned char update_nextcheck);
 void	DCconfig_set_proxy_timediff(zbx_uint64_t hostid, const zbx_timespec_t *timediff);
 
 void	DCget_user_macro(zbx_uint64_t *hostids, int host_num, const char *macro, char **replace_to);
 
-int	DChost_activate(zbx_host_availability_t *in, zbx_host_availability_t *out);
+int	DChost_activate(zbx_uint64_t hostid, unsigned char agent_type, const zbx_timespec_t *ts,
+		zbx_agent_availability_t *in, zbx_agent_availability_t *out);
 
-int	DChost_deactivate(const zbx_timespec_t *ts, zbx_host_availability_t *in, zbx_host_availability_t *out);
-
-void	DChost_update_availability(const zbx_host_availability_t *availability, int availability_num);
+int	DChost_deactivate(zbx_uint64_t hostid, unsigned char agent, const zbx_timespec_t *ts,
+		zbx_agent_availability_t *in, zbx_agent_availability_t *out, const char *error);
 
 void	DCget_delta_items(zbx_hashset_t *items, const zbx_vector_uint64_t *ids);
 void	DCset_delta_items(zbx_hashset_t *items);
@@ -376,17 +422,80 @@ void	DCset_delta_items(zbx_hashset_t *items);
 void	DCfree_item_queue(zbx_vector_ptr_t *queue);
 int	DCget_item_queue(zbx_vector_ptr_t *queue, int from, int to);
 
-int	DCget_item_count();
-int	DCget_item_unsupported_count();
-int	DCget_trigger_count();
-double	DCget_required_performance();
-int	DCget_host_count();
-
-void	DCget_functions_hostids(zbx_vector_uint64_t *hostids, const zbx_vector_uint64_t *functionids);
+int	DCget_item_count(zbx_uint64_t hostid);
+int	DCget_item_unsupported_count(zbx_uint64_t hostid);
+int	DCget_trigger_count(void);
+double	DCget_required_performance(void);
+int	DCget_host_count(void);
 
 void	DCget_expressions_by_names(zbx_vector_ptr_t *expressions, const char * const *names, int names_num);
 void	DCget_expressions_by_name(zbx_vector_ptr_t *expressions, const char *name);
 
 int	DCget_data_expected_from(zbx_uint64_t itemid, int *seconds);
 
+/* a set of identifiers assigned to another identifier */
+typedef struct
+{
+	zbx_uint64_t		id;
+	zbx_vector_uint64_t	ids;
+}
+zbx_idset_t;
+
+/* local user macro cache support */
+void	zbx_umc_init(zbx_hashset_t *cache);
+void	zbx_umc_destroy(zbx_hashset_t *cache);
+void	zbx_umc_add_expression(zbx_hashset_t *cache, zbx_uint64_t objectid, const char *expression);
+void	zbx_umc_add_hostids(zbx_hashset_t *cache, zbx_uint64_t objectid, const zbx_uint64_t *hostids, int hostids_num);
+void	zbx_umc_resolve(zbx_hashset_t *cache);
+const char	*zbx_umc_get_macro_value(zbx_hashset_t *cache, zbx_uint64_t objectid, const char *macro);
+
+void	DCget_bulk_hostids_by_functionids(zbx_vector_ptr_t *functionids, zbx_vector_ptr_t *hostids);
+void	DCget_hostids_by_functionids(zbx_vector_uint64_t *functionids, zbx_vector_uint64_t *hostids);
+void	zbx_idset_free(zbx_idset_t *idset);
+
+/* global configuration support */
+void	zbx_config_get(zbx_config_t *cfg, zbx_uint64_t flags);
+void	zbx_config_clean(zbx_config_t *cfg);
+
+/* flags to specify which host interfaces have enabled items */
+#define ZBX_FLAG_INTERFACE_NONE		0x00
+#define ZBX_FLAG_INTERFACE_ZABBIX	(0x01 << (INTERFACE_TYPE_AGENT - 1))
+#define ZBX_FLAG_INTERFACE_SNMP		(0x01 << (INTERFACE_TYPE_SNMP - 1))
+#define ZBX_FLAG_INTERFACE_IPMI		(0x01 << (INTERFACE_TYPE_IPMI - 1))
+#define ZBX_FLAG_INTERFACE_JMX		(0x01 << (INTERFACE_TYPE_JMX - 1))
+#define ZBX_FLAG_INTERFACE_UNKNOWN	0x80
+
+int	DCset_hosts_availability(zbx_vector_ptr_t *availabilities);
+
+int	DCreset_hosts_availability(zbx_vector_ptr_t *hosts);
+void	DCupdate_hosts_availability();
+
+void	zbx_dc_get_actions_eval(zbx_vector_ptr_t *actions);
+void	zbx_action_eval_free(zbx_action_eval_t *action);
+
+int	DCget_hosts_availability(zbx_vector_ptr_t *hosts, int *ts);
+
+void	zbx_host_availability_init(zbx_host_availability_t *availability, zbx_uint64_t hostid);
+void	zbx_host_availability_clean(zbx_host_availability_t *availability);
+void	zbx_host_availability_free(zbx_host_availability_t *availability);
+int	zbx_host_availability_is_set(const zbx_host_availability_t *ha);
+
+void	zbx_set_availability_diff_ts(int ts);
+
+#define ZBX_HC_ITEM_STATUS_NORMAL	0
+#define ZBX_HC_ITEM_STATUS_BUSY		1
+#define ZBX_HC_ITEM_STATUS_QUEUED	2
+
+struct zbx_hc_data_t;
+
+typedef struct
+{
+	zbx_uint64_t		itemid;
+	unsigned char		status;
+
+	struct zbx_hc_data_t	*tail;
+	struct zbx_hc_data_t	*head;
+}
+zbx_hc_item_t;
+
 #endif
