@@ -24,11 +24,11 @@
 #define ZBX_STR(str)	ZBX_STR2(str)
 
 #define APPLICATION_NAME	"Zabbix Agent"
-#define ZABBIX_REVDATE		"20 April 2016"
-#define ZABBIX_VERSION_MAJOR	2
-#define ZABBIX_VERSION_MINOR	4
-#define ZABBIX_VERSION_PATCH	8
-#define ZABBIX_VERSION_REVISION	59539
+#define ZABBIX_REVDATE		"26 February 2016"
+#define ZABBIX_VERSION_MAJOR	3
+#define ZABBIX_VERSION_MINOR	0
+#define ZABBIX_VERSION_PATCH	1
+#define ZABBIX_VERSION_REVISION	58734
 #define ZABBIX_VERSION_RC	""
 #define ZABBIX_VERSION		ZBX_STR(ZABBIX_VERSION_MAJOR) "." ZBX_STR(ZABBIX_VERSION_MINOR) "." \
 				ZBX_STR(ZABBIX_VERSION_PATCH) ZABBIX_VERSION_RC
