@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2016 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,24 +9,27 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
 
 static char	data_static[ZBX_MAX_B64_LEN];
 
-/* Get DATA from <tag>DATA</tag> */
-int xml_get_data_dyn(const char *xml, const char *tag, char **data)
+/******************************************************************************
+ *                                                                            *
+ * Purpose: get DATA from <tag>DATA</tag>                                     *
+ *                                                                            *
+ ******************************************************************************/
+int	xml_get_data_dyn(const char *xml, const char *tag, char **data)
 {
-	int	len;
+	size_t	len, sz;
 	char	*start, *end;
-	size_t	sz;
 
 	sz = sizeof(data_static);
 
@@ -44,7 +47,7 @@ int xml_get_data_dyn(const char *xml, const char *tag, char **data)
 	start += len;
 	len = end - start;
 
-	if (len > (int)sz - 1)
+	if (len > sz - 1)
 		*data = zbx_malloc(*data, len + 1);
 	else
 		*data = data_static;
