@@ -213,6 +213,8 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 	int		ret = ZBX_DB_OK, last_txn_error, last_txn_level;
 #if defined(HAVE_IBM_DB2)
 	char		*connect = NULL;
+#elif defined(HAVE_MYSQL)
+	my_bool		mysql_reconnect = 1;
 #elif defined(HAVE_ORACLE)
 	char		*connect = NULL;
 	sword		err = OCI_SUCCESS;
@@ -322,6 +324,15 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 		ret = ZBX_DB_FAIL;
 	}
 
+	/* The RECONNECT option setting is placed here, AFTER the connection	*/
+	/* is made, due to a bug in MySQL versions prior to 5.1.6 where it	*/
+	/* reset the options value to the default, regardless of what it was	*/
+	/* set to prior to the connection. MySQL allows changing connection	*/
+	/* options on an open connection, so setting it here is safe.		*/
+
+	if (0 != mysql_options(conn, MYSQL_OPT_RECONNECT, &mysql_reconnect))
+		zabbix_log(LOG_LEVEL_WARNING, "Cannot set MySQL reconnect option.");
+
 	if (ZBX_DB_OK == ret && 0 != mysql_select_db(conn, dbname))
 	{
 		zabbix_errlog(ERR_Z3001, dbname, mysql_errno(conn), mysql_error(conn));
@@ -540,7 +551,7 @@ out:
 #if defined(HAVE_SQLITE3)
 void	zbx_create_sqlite3_mutex(void)
 {
-	if (ZBX_MUTEX_ERROR == zbx_mutex_create_force(&sqlite_access, ZBX_MUTEX_SQLITE3))
+	if (FAIL == zbx_mutex_create_force(&sqlite_access, ZBX_MUTEX_SQLITE3))
 	{
 		zbx_error("cannot create mutex for SQLite3");
 		exit(EXIT_FAILURE);
@@ -1085,15 +1096,12 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 					zabbix_log(LOG_LEVEL_DEBUG, "cannot retrieve result set");
 					break;
 				}
+				else
+					ret += (int)mysql_affected_rows(conn);
 
-				ret += (int)mysql_affected_rows(conn);
-
-				/* more results? 0 = yes (keep looping), -1 = no, >0 = error */
+				/* more results? -1 = no, >0 = error, 0 = yes (keep looping) */
 				if (0 < (status = mysql_next_result(conn)))
-				{
 					zabbix_errlog(ERR_Z3005, mysql_errno(conn), mysql_error(conn), sql);
-					ret = (SUCCEED == is_recoverable_mysql_error() ? ZBX_DB_DOWN : ZBX_DB_FAIL);
-				}
 			}
 			while (0 == status);
 		}
