@@ -30,7 +30,7 @@
 
 #ifndef HAVE_SQLITE3
 
-extern unsigned char daemon_type;
+extern unsigned char program_type;
 
 static int	DBpatch_2030000(void)
 {
@@ -378,7 +378,7 @@ static int	DBpatch_2030037(void)
 				NULL
 			};
 
-	if (ZBX_DAEMON_TYPE_SERVER == daemon_type)
+	if (ZBX_PROGRAM_TYPE_SERVER == program_type)
 		return SUCCEED;
 
 	return DBcreate_table(&table);
@@ -386,7 +386,7 @@ static int	DBpatch_2030037(void)
 
 static int	DBpatch_2030038(void)
 {
-	if (ZBX_DAEMON_TYPE_SERVER == daemon_type)
+	if (ZBX_PROGRAM_TYPE_SERVER == program_type)
 		return SUCCEED;
 
 	if (ZBX_DB_OK <= DBexecute(
@@ -430,7 +430,7 @@ static int	DBpatch_2030040(void)
 
 static int	DBpatch_2030041(void)
 {
-	if (ZBX_DAEMON_TYPE_SERVER == daemon_type)
+	if (ZBX_PROGRAM_TYPE_SERVER == program_type)
 		return SUCCEED;
 
 	if (ZBX_DB_OK <= DBexecute(
@@ -444,7 +444,7 @@ static int	DBpatch_2030041(void)
 
 static int	DBpatch_2030042(void)
 {
-	if (ZBX_DAEMON_TYPE_SERVER == daemon_type)
+	if (ZBX_PROGRAM_TYPE_SERVER == program_type)
 		return SUCCEED;
 
 	return DBdrop_table("ids_tmp");
@@ -716,7 +716,7 @@ static int	DBpatch_2030067(void)
 
 static int	DBpatch_2030068(void)
 {
-	if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY))
+	if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY))
 	{
 		/* "name" is empty on proxy side, because it is not synchronized between server and proxy */
 		/* in 2.2, and should therefore be filled with unique values to create a unique index.    */
@@ -906,7 +906,6 @@ static int	DBpatch_2030094(void)
 	while (SUCCEED == ret && NULL != (row = DBfetch(result)))
 	{
 		expr_offset = 0;
-		zbx_strcpy_alloc(&expr, &expr_alloc, &expr_offset, "");
 
 		for (p = row[1]; '\0' != *p; p++)
 		{
