@@ -37,7 +37,7 @@ static int	__get_function_parameter_uint31(zbx_uint64_t hostid, const char *para
 	if (NULL == (parameter = get_param_dyn(parameters, Nparam)))
 		goto out;
 
-	if (SUCCEED == substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL,
+	if (SUCCEED == substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL, NULL, NULL,
 			&parameter, MACRO_TYPE_COMMON, NULL, 0))
 	{
 		if (1 == defaults_on_empty && '\0' == *parameter)
@@ -93,7 +93,7 @@ static int	get_function_parameter_uint64(zbx_uint64_t hostid, const char *parame
 	if (NULL == (parameter = get_param_dyn(parameters, Nparam)))
 		goto out;
 
-	if (SUCCEED == substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL,
+	if (SUCCEED == substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL, NULL, NULL,
 			&parameter, MACRO_TYPE_COMMON, NULL, 0))
 	{
 		if (SUCCEED == is_uint64(parameter, value))
@@ -113,6 +113,59 @@ out:
 	return ret;
 }
 
+static int	get_function_parameter_float(zbx_uint64_t hostid, const char *parameters, int Nparam, double *value)
+{
+	const char	*__function_name = "get_function_parameter_float";
+	char		*parameter;
+	int		ret = FAIL;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() parameters:'%s' Nparam:%d", __function_name, parameters, Nparam);
+
+	if (NULL == (parameter = get_param_dyn(parameters, Nparam)))
+		goto out;
+
+	if (SUCCEED == substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL, NULL, NULL,
+			&parameter, MACRO_TYPE_COMMON, NULL, 0))
+	{
+		int		digits;
+		const char	*p = parameter, *dot;
+
+		while (0 != isdigit(*p))
+			p++;
+
+		digits = p - parameter;
+
+		if ('.' == *p)
+		{
+			dot = p++;
+
+			while (0 != isdigit(*p))
+				p++;
+
+			if (4 < p - dot - 1)	/* limit to 4 digits after the decimal point */
+				goto clean;
+
+			digits += p - dot - 1;
+		}
+
+		if ('\0' != *p || 0 == digits)
+			goto clean;
+
+		*value = atof(parameter);
+
+		ret = SUCCEED;
+	}
+
+	if (SUCCEED == ret)
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() value:" ZBX_FS_DBL, __function_name, *value);
+clean:
+	zbx_free(parameter);
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+
+	return ret;
+}
+
 static int	get_function_parameter_str(zbx_uint64_t hostid, const char *parameters, int Nparam, char **value)
 {
 	const char	*__function_name = "get_function_parameter_str";
@@ -123,7 +176,7 @@ static int	get_function_parameter_str(zbx_uint64_t hostid, const char *parameter
 	if (NULL == (*value = get_param_dyn(parameters, Nparam)))
 		goto out;
 
-	ret = substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL,
+	ret = substitute_simple_macros(NULL, NULL, NULL, NULL, &hostid, NULL, NULL, NULL, NULL,
 			value, MACRO_TYPE_COMMON, NULL, 0);
 
 	if (SUCCEED == ret)
@@ -150,7 +203,7 @@ out:
  *                                                                            *
  ******************************************************************************/
 static int	evaluate_LOGEVENTID(char *value, DC_ITEM *item, const char *function, const char *parameters,
-		time_t now)
+		time_t now, char **error)
 {
 	const char		*__function_name = "evaluate_LOGEVENTID";
 
@@ -174,8 +227,16 @@ static int	evaluate_LOGEVENTID(char *value, DC_ITEM *item, const char *function,
 		goto out;
 
 	if ('@' == *arg1)
+	{
 		DCget_expressions_by_name(&regexps, arg1 + 1);
 
+		if (0 == regexps.values_num)
+		{
+			*error = zbx_dsprintf(*error, "global regular expression \"%s\" does not exist", arg1 + 1);
+			goto out;
+		}
+	}
+
 	if (SUCCEED == zbx_vc_get_value(item->itemid, item->value_type, &ts, &vc_value))
 	{
 		char	logeventid[16];
@@ -191,9 +252,9 @@ static int	evaluate_LOGEVENTID(char *value, DC_ITEM *item, const char *function,
 	}
 	else
 		zabbix_log(LOG_LEVEL_DEBUG, "result for LOGEVENTID is empty");
-
-	zbx_free(arg1);
 out:
+	zbx_free(arg1);
+
 	zbx_regexp_clean_expressions(&regexps);
 	zbx_vector_ptr_destroy(&regexps);
 
@@ -995,6 +1056,117 @@ out:
 	return ret;
 }
 
+static int	__history_record_float_compare(const zbx_history_record_t *d1, const zbx_history_record_t *d2)
+{
+	ZBX_RETURN_IF_NOT_EQUAL(d1->value.dbl, d2->value.dbl);
+
+	return 0;
+}
+
+static int	__history_record_uint64_compare(const zbx_history_record_t *d1, const zbx_history_record_t *d2)
+{
+	ZBX_RETURN_IF_NOT_EQUAL(d1->value.ui64, d2->value.ui64);
+
+	return 0;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: evaluate_PERCENTILE                                              *
+ *                                                                            *
+ * Purpose: evaluate function 'percentile' for the item                       *
+ *                                                                            *
+ * Parameters: item       - [IN] item (performance metric)                    *
+ *             parameters - [IN] seconds/values, time shift (optional),       *
+ *                               percentage                                   *
+ *                                                                            *
+ * Return value: SUCCEED - evaluated successfully, result is stored in        *
+ *                         'value'                                            *
+ *               FAIL    - failed to evaluate function                        *
+ *                                                                            *
+ ******************************************************************************/
+static int	evaluate_PERCENTILE(char *value, DC_ITEM *item, const char *function, const char *parameters,
+		time_t now, char **error)
+{
+	const char			*__function_name = "evaluate_PERCENTILE";
+
+	int				nparams, arg1, time_shift, flag, ret = FAIL, seconds = 0, nvalues = 0;
+	double				percentage;
+	zbx_vector_history_record_t	values;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+
+	zbx_history_record_vector_create(&values);
+
+	if (ITEM_VALUE_TYPE_FLOAT != item->value_type && ITEM_VALUE_TYPE_UINT64 != item->value_type)
+	{
+		*error = zbx_strdup(*error, "invalid value type");
+		goto out;
+	}
+
+	if (3 != (nparams = num_param(parameters)))
+	{
+		*error = zbx_strdup(*error, 3 < nparams ? "too many parameters" : "invalid number of parameters");
+		goto out;
+	}
+
+	if (SUCCEED != get_function_parameter_uint31(item->host.hostid, parameters, 1, &arg1, &flag) || 0 == arg1)
+	{
+		*error = zbx_strdup(*error, "invalid first parameter");
+		goto out;
+	}
+
+	if (ZBX_FLAG_SEC == flag)
+		seconds = arg1;
+	else
+		nvalues = arg1;
+
+	if (SUCCEED != get_function_parameter_uint31_default(item->host.hostid, parameters, 2, &time_shift, &flag, 0,
+			ZBX_FLAG_SEC) || ZBX_FLAG_SEC != flag)
+	{
+		*error = zbx_strdup(*error, "invalid second parameter");
+		goto out;
+	}
+
+	now -= time_shift;
+
+	if (SUCCEED != get_function_parameter_float(item->host.hostid, parameters, 3, &percentage) || percentage > 100)
+	{
+		*error = zbx_strdup(*error, "invalid third parameter");
+		goto out;
+	}
+
+	if (FAIL == zbx_vc_get_value_range(item->itemid, item->value_type, &values, seconds, nvalues, now))
+		goto out;
+
+	if (0 < values.values_num)
+	{
+		int	index;
+
+		if (ITEM_VALUE_TYPE_FLOAT == item->value_type)
+			zbx_vector_history_record_sort(&values, (zbx_compare_func_t)__history_record_float_compare);
+		else
+			zbx_vector_history_record_sort(&values, (zbx_compare_func_t)__history_record_uint64_compare);
+
+		if (0 == percentage)
+			index = 1;
+		else
+			index = (int)ceil(values.values_num * (percentage / 100));
+
+		zbx_vc_history_value2str(value, MAX_BUFFER_LEN, &values.values[index - 1].value, item->value_type);
+
+		ret = SUCCEED;
+	}
+	else
+		*error = zbx_strdup(*error, "not enough data");
+out:
+	zbx_history_record_vector_destroy(&values, item->value_type);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+
+	return ret;
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: evaluate_DELTA                                                   *
@@ -1119,15 +1291,13 @@ static int	evaluate_NODATA(char *value, DC_ITEM *item, const char *function, con
 
 	if (1 < num_param(parameters))
 	{
-		if (NULL != error)
-			*error = zbx_strdup(*error, "too many parameters");
+		*error = zbx_strdup(*error, "too many parameters");
 		goto out;
 	}
 
 	if (SUCCEED != get_function_parameter_uint31(item->host.hostid, parameters, 1, &arg1, &flag))
 	{
-		if (NULL != error)
-			*error = zbx_strdup(*error, "invalid first parameter");
+		*error = zbx_strdup(*error, "invalid first parameter");
 		goto out;
 	}
 
@@ -1136,8 +1306,8 @@ static int	evaluate_NODATA(char *value, DC_ITEM *item, const char *function, con
 
 	now = (int)time(NULL);
 
-	if (SUCCEED == zbx_vc_get_value_range(item->itemid, item->value_type, &values, 0, 1, now) &&
-			1 == values.values_num && values.values[0].timestamp.sec + arg1 > now)
+	if (SUCCEED == zbx_vc_get_value_range(item->itemid, item->value_type, &values, arg1, 1, now) &&
+			1 == values.values_num)
 	{
 		zbx_strlcpy(value, "0", MAX_BUFFER_LEN);
 	}
@@ -1147,21 +1317,14 @@ static int	evaluate_NODATA(char *value, DC_ITEM *item, const char *function, con
 
 		if (SUCCEED != DCget_data_expected_from(item->itemid, &seconds))
 		{
-			if (NULL != error)
-			{
-				*error = zbx_strdup(*error, "item does not exist, is disabled"
-						" or belongs to a disabled host");
-			}
+			*error = zbx_strdup(*error, "item does not exist, is disabled or belongs to a disabled host");
 			goto out;
 		}
 
 		if (seconds + arg1 > now)
 		{
-			if (NULL != error)
-			{
-				*error = zbx_strdup(*error, "item does not have enough data"
-						" after server start or item creation");
-			}
+			*error = zbx_strdup(*error,
+					"item does not have enough data after server start or item creation");
 			goto out;
 		}
 
@@ -1202,7 +1365,9 @@ static int	evaluate_ABSCHANGE(char *value, DC_ITEM *item, const char *function,
 
 	if (SUCCEED != zbx_vc_get_value_range(item->itemid, item->value_type, &values, 0, 2, now) ||
 			2 > values.values_num)
+	{
 		goto out;
+	}
 
 	switch (item->value_type)
 	{
@@ -1274,7 +1439,9 @@ static int	evaluate_CHANGE(char *value, DC_ITEM *item, const char *function, con
 
 	if (SUCCEED != zbx_vc_get_value_range(item->itemid, item->value_type, &values, 0, 2, now) ||
 			2 > values.values_num)
+	{
 		goto out;
+	}
 
 	switch (item->value_type)
 	{
@@ -1420,7 +1587,8 @@ static int	evaluate_STR_one(int func, zbx_vector_ptr_t *regexps, const char *val
 	return FAIL;
 }
 
-static int	evaluate_STR(char *value, DC_ITEM *item, const char *function, const char *parameters, time_t now)
+static int	evaluate_STR(char *value, DC_ITEM *item, const char *function, const char *parameters, time_t now,
+		char **error)
 {
 	const char			*__function_name = "evaluate_STR";
 	char				*arg1 = NULL;
@@ -1469,8 +1637,16 @@ static int	evaluate_STR(char *value, DC_ITEM *item, const char *function, const
 	}
 
 	if ((ZBX_FUNC_REGEXP == func || ZBX_FUNC_IREGEXP == func) && '@' == *arg1)
+	{
 		DCget_expressions_by_name(&regexps, arg1 + 1);
 
+		if (0 == regexps.values_num)
+		{
+			*error = zbx_dsprintf(*error, "global regular expression \"%s\" does not exist", arg1 + 1);
+			goto out;
+		}
+	}
+
 	if (ZBX_FLAG_SEC == flag)
 		seconds = arg2;
 	else
@@ -1690,6 +1866,268 @@ clean:
 	return ret;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Function: evaluate_FORECAST                                                *
+ *                                                                            *
+ * Purpose: evaluate function 'forecast' for the item                         *
+ *                                                                            *
+ * Parameters: item - item (performance metric)                               *
+ *             parameters - number of seconds/values and time shift (optional)*
+ *                                                                            *
+ * Return value: SUCCEED - evaluated successfully, result is stored in 'value'*
+ *               FAIL - failed to evaluate function                           *
+ *                                                                            *
+ ******************************************************************************/
+static int	evaluate_FORECAST(char *value, DC_ITEM *item, const char *function, const char *parameters, time_t now,
+		char **error)
+{
+	const char			*__function_name = "evaluate_FORECAST";
+	char				*fit_str = NULL, *mode_str = NULL;
+	double				*t = NULL, *x = NULL;
+	int				nparams, time, time_flag, arg1, flag, i, ret = FAIL, seconds = 0, nvalues = 0,
+					time_shift, time_shift_flag;
+	unsigned			k = 0;
+	zbx_vector_history_record_t	values;
+	zbx_timespec_t			zero_time;
+	zbx_fit_t			fit;
+	zbx_mode_t			mode;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+
+	zbx_history_record_vector_create(&values);
+
+	if (ITEM_VALUE_TYPE_FLOAT != item->value_type && ITEM_VALUE_TYPE_UINT64 != item->value_type)
+		goto out;
+
+	if (5 < (nparams = num_param(parameters)))
+		goto out;
+
+	if (3 > nparams)
+		goto out;
+
+	if (SUCCEED != get_function_parameter_uint31(item->host.hostid, parameters, 1, &arg1, &flag) || 0 == arg1)
+		goto out;
+
+	if (SUCCEED != get_function_parameter_uint31_default(item->host.hostid, parameters, 2, &time_shift,
+			&time_shift_flag, 0, ZBX_FLAG_SEC) || ZBX_FLAG_SEC != time_shift_flag)
+		goto out;
+
+	if (SUCCEED != get_function_parameter_uint31(item->host.hostid, parameters, 3, &time, &time_flag) ||
+			ZBX_FLAG_SEC != time_flag)
+		goto out;
+
+	if (4 <= nparams)
+	{
+		if (SUCCEED != get_function_parameter_str(item->host.hostid, parameters, 4, &fit_str) ||
+				SUCCEED != zbx_fit_code(fit_str, &fit, &k, error))
+			goto out;
+	}
+	else
+	{
+		fit = FIT_LINEAR;
+	}
+
+	if (5 == nparams)
+	{
+		if (SUCCEED != get_function_parameter_str(item->host.hostid, parameters, 5, &mode_str) ||
+				SUCCEED != zbx_mode_code(mode_str, &mode, error))
+			goto out;
+	}
+	else
+	{
+		mode = MODE_VALUE;
+	}
+
+	if (ZBX_FLAG_SEC == flag)
+		seconds = arg1;
+	else
+		nvalues = arg1;
+
+	if (FAIL == zbx_vc_get_value_range(item->itemid, item->value_type, &values, seconds, nvalues, now - time_shift))
+	{
+		*error = zbx_strdup(*error, "unable to get values from value cache");
+		goto out;
+	}
+
+	if (0 < values.values_num)
+	{
+		t = zbx_malloc(t, values.values_num * sizeof(double));
+		x = zbx_malloc(x, values.values_num * sizeof(double));
+
+		zero_time.sec = values.values[values.values_num - 1].timestamp.sec;
+		zero_time.ns = values.values[values.values_num - 1].timestamp.ns;
+
+		if (ITEM_VALUE_TYPE_FLOAT == item->value_type)
+		{
+			for (i = 0; i < values.values_num; i++)
+			{
+				t[i] = values.values[i].timestamp.sec - zero_time.sec + 1.0e-9 *
+						(values.values[i].timestamp.ns - zero_time.ns + 1);
+				x[i] = values.values[i].value.dbl;
+			}
+		}
+		else
+		{
+			for (i = 0; i < values.values_num; i++)
+			{
+				t[i] = values.values[i].timestamp.sec - zero_time.sec + 1.0e-9 *
+						(values.values[i].timestamp.ns - zero_time.ns + 1);
+				x[i] = values.values[i].value.ui64;
+			}
+		}
+
+		zbx_snprintf(value, MAX_BUFFER_LEN, ZBX_FS_DBL, zbx_forecast(t, x, values.values_num,
+				now - zero_time.sec - 1.0e-9 * (zero_time.ns + 1), time, fit, k, mode));
+	}
+	else
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "no data available");
+		zbx_snprintf(value, MAX_BUFFER_LEN, ZBX_FS_DBL, ZBX_MATH_ERROR);
+	}
+
+	ret = SUCCEED;
+out:
+	zbx_history_record_vector_destroy(&values, item->value_type);
+
+	zbx_free(fit_str);
+	zbx_free(mode_str);
+
+	zbx_free(t);
+	zbx_free(x);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+
+	return ret;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: evaluate_TIMELEFT                                                *
+ *                                                                            *
+ * Purpose: evaluate function 'timeleft' for the item                         *
+ *                                                                            *
+ * Parameters: item - item (performance metric)                               *
+ *             parameters - number of seconds/values and time shift (optional)*
+ *                                                                            *
+ * Return value: SUCCEED - evaluated successfully, result is stored in 'value'*
+ *               FAIL - failed to evaluate function                           *
+ *                                                                            *
+ ******************************************************************************/
+static int	evaluate_TIMELEFT(char *value, DC_ITEM *item, const char *function, const char *parameters, time_t now,
+		char **error)
+{
+	const char			*__function_name = "evaluate_TIMELEFT";
+	char				*fit_str = NULL;
+	double				*t = NULL, *x = NULL, threshold;
+	int				nparams, arg1, flag, i, ret = FAIL, seconds = 0, nvalues = 0, time_shift,
+					time_shift_flag;
+	unsigned			k = 0;
+	zbx_vector_history_record_t	values;
+	zbx_timespec_t			zero_time;
+	zbx_fit_t			fit;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+
+	zbx_history_record_vector_create(&values);
+
+	if (ITEM_VALUE_TYPE_FLOAT != item->value_type && ITEM_VALUE_TYPE_UINT64 != item->value_type)
+		goto out;
+
+	if (4 < (nparams = num_param(parameters)))
+		goto out;
+
+	if (3 > nparams)
+		goto out;
+
+	if (SUCCEED != get_function_parameter_uint31(item->host.hostid, parameters, 1, &arg1, &flag) || 0 == arg1)
+		goto out;
+
+	if (SUCCEED != get_function_parameter_uint31_default(item->host.hostid, parameters, 2, &time_shift,
+			&time_shift_flag, 0, ZBX_FLAG_SEC) || ZBX_FLAG_SEC != time_shift_flag)
+		goto out;
+
+	if (SUCCEED != get_function_parameter_float(item->host.hostid, parameters, 3, &threshold))
+		goto out;
+
+	if (4 == nparams)
+	{
+		if (SUCCEED != get_function_parameter_str(item->host.hostid, parameters, 4, &fit_str) ||
+				SUCCEED != zbx_fit_code(fit_str, &fit, &k, error))
+			goto out;
+	}
+	else
+	{
+		fit = FIT_LINEAR;
+	}
+
+	if ((FIT_EXPONENTIAL == fit || FIT_POWER == fit) && 0.0 >= threshold)
+	{
+		*error = zbx_strdup(*error, "exponential and power functions are always positive");
+		goto out;
+	}
+
+	if (ZBX_FLAG_SEC == flag)
+		seconds = arg1;
+	else
+		nvalues = arg1;
+
+	if (FAIL == zbx_vc_get_value_range(item->itemid, item->value_type, &values, seconds, nvalues, now - time_shift))
+	{
+		*error = zbx_strdup(*error, "unable to get values from value cache");
+		goto out;
+	}
+
+	if (0 < values.values_num)
+	{
+		t = zbx_malloc(t, values.values_num * sizeof(double));
+		x = zbx_malloc(x, values.values_num * sizeof(double));
+
+		zero_time.sec = values.values[values.values_num - 1].timestamp.sec;
+		zero_time.ns = values.values[values.values_num - 1].timestamp.ns;
+
+		if (ITEM_VALUE_TYPE_FLOAT == item->value_type)
+		{
+			for (i = 0; i < values.values_num; i++)
+			{
+				t[i] = values.values[i].timestamp.sec - zero_time.sec + 1.0e-9 *
+						(values.values[i].timestamp.ns - zero_time.ns + 1);
+				x[i] = values.values[i].value.dbl;
+			}
+		}
+		else
+		{
+			for (i = 0; i < values.values_num; i++)
+			{
+				t[i] = values.values[i].timestamp.sec - zero_time.sec + 1.0e-9 *
+						(values.values[i].timestamp.ns - zero_time.ns + 1);
+				x[i] = values.values[i].value.ui64;
+			}
+		}
+
+		zbx_snprintf(value, MAX_BUFFER_LEN, ZBX_FS_DBL, zbx_timeleft(t, x, values.values_num,
+				now - zero_time.sec - 1.0e-9 * (zero_time.ns + 1), threshold, fit, k));
+	}
+	else
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "no data available");
+		zbx_snprintf(value, MAX_BUFFER_LEN, ZBX_FS_DBL, ZBX_MATH_ERROR);
+	}
+
+	ret = SUCCEED;
+out:
+	zbx_history_record_vector_destroy(&values, item->value_type);
+
+	zbx_free(fit_str);
+
+	zbx_free(t);
+	zbx_free(x);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+
+	return ret;
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: evaluate_function                                                *
@@ -1741,6 +2179,10 @@ int	evaluate_function(char *value, DC_ITEM *item, const char *function, const ch
 	{
 		ret = evaluate_SUM(value, item, function, parameter, now);
 	}
+	else if (0 == strcmp(function, "percentile"))
+	{
+		ret = evaluate_PERCENTILE(value, item, function, parameter, now, error);
+	}
 	else if (0 == strcmp(function, "count"))
 	{
 		ret = evaluate_COUNT(value, item, function, parameter, now);
@@ -1791,7 +2233,7 @@ int	evaluate_function(char *value, DC_ITEM *item, const char *function, const ch
 	}
 	else if (0 == strcmp(function, "str") || 0 == strcmp(function, "regexp") || 0 == strcmp(function, "iregexp"))
 	{
-		ret = evaluate_STR(value, item, function, parameter, now);
+		ret = evaluate_STR(value, item, function, parameter, now, error);
 	}
 	else if (0 == strcmp(function, "strlen"))
 	{
@@ -1808,7 +2250,7 @@ int	evaluate_function(char *value, DC_ITEM *item, const char *function, const ch
 	}
 	else if (0 == strcmp(function, "logeventid"))
 	{
-		ret = evaluate_LOGEVENTID(value, item, function, parameter, now);
+		ret = evaluate_LOGEVENTID(value, item, function, parameter, now, error);
 	}
 	else if (0 == strcmp(function, "logseverity"))
 	{
@@ -1822,6 +2264,14 @@ int	evaluate_function(char *value, DC_ITEM *item, const char *function, const ch
 	{
 		ret = evaluate_BAND(value, item, function, parameter, now);
 	}
+	else if (0 == strcmp(function, "forecast"))
+	{
+		ret = evaluate_FORECAST(value, item, function, parameter, now, error);
+	}
+	else if (0 == strcmp(function, "timeleft"))
+	{
+		ret = evaluate_TIMELEFT(value, item, function, parameter, now, error);
+	}
 	else
 	{
 		*error = zbx_strdup(*error, "function is not supported");
@@ -2224,7 +2674,7 @@ int	evaluate_macro_function(char *value, const char *host, const char *key, cons
 		{
 			zbx_format_value(value, MAX_BUFFER_LEN, item.valuemapid, item.units, item.value_type);
 		}
-		else if (SUCCEED == str_in_list("abschange,avg,change,delta,max,min,sum", function, ','))
+		else if (SUCCEED == str_in_list("abschange,avg,change,delta,max,min,percentile,sum,forecast", function, ','))
 		{
 			switch (item.value_type)
 			{
@@ -2236,6 +2686,10 @@ int	evaluate_macro_function(char *value, const char *host, const char *key, cons
 					;
 			}
 		}
+		else if (SUCCEED == str_in_list("timeleft", function, ','))
+		{
+			add_value_suffix(value, MAX_BUFFER_LEN, "s", ITEM_VALUE_TYPE_FLOAT);
+		}
 	}
 
 	zbx_free(error);
