@@ -66,14 +66,25 @@ static int	check_procargs(struct procentry64 *procentry, const char *proccomm)
 
 int	PROC_MEM(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
-	char			*param, *procname, *proccomm;
+#define ZBX_VSIZE	0
+#define ZBX_RSS		1
+#define ZBX_PMEM	2
+#define ZBX_SIZE	3
+#define ZBX_DSIZE	4
+#define ZBX_TSIZE	5
+#define ZBX_SDSIZE	6
+#define ZBX_DRSS	7
+#define ZBX_TRSS	8
+
+	char			*param, *procname, *proccomm, *mem_type = NULL;
 	struct passwd		*usrinfo;
 	struct procentry64	procentry;
 	pid_t			pid = 0;
-	int			do_task;
-	zbx_uint64_t		memsize = 0, proccount = 0, value;
+	int			do_task, mem_type_code, proccount = 0, invalid_user = 0;
+	zbx_uint64_t		mem_size = 0, byte_value = 0;
+	double			pct_size = 0.0, pct_value = 0.0;
 
-	if (4 < request->nparam)
+	if (5 < request->nparam)
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Too many parameters."));
 		return SYSINFO_RET_FAIL;
@@ -85,10 +96,7 @@ int	PROC_MEM(AGENT_REQUEST *request, AGENT_RESULT *result)
 	if (NULL != param && '\0' != *param)
 	{
 		if (NULL == (usrinfo = getpwnam(param)))
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain user information."));
-			return SYSINFO_RET_FAIL;
-		}
+			invalid_user = 1;
 	}
 	else
 		usrinfo = NULL;
@@ -110,6 +118,52 @@ int	PROC_MEM(AGENT_REQUEST *request, AGENT_RESULT *result)
 	}
 
 	proccomm = get_rparam(request, 3);
+	mem_type = get_rparam(request, 4);
+
+	if (NULL == mem_type || '\0' == *mem_type || 0 == strcmp(mem_type, "vsize"))
+	{
+		mem_type_code = ZBX_VSIZE;		/* virtual memory size */
+	}
+	else if (0 == strcmp(mem_type, "rss"))
+	{
+		mem_type_code = ZBX_RSS;		/* resident set size */
+	}
+	else if (0 == strcmp(mem_type, "pmem"))
+	{
+		mem_type_code = ZBX_PMEM;		/* percentage of real memory used by process */
+	}
+	else if (0 == strcmp(mem_type, "size"))
+	{
+		mem_type_code = ZBX_SIZE;		/* size of process (code + data) */
+	}
+	else if (0 == strcmp(mem_type, "dsize"))
+	{
+		mem_type_code = ZBX_DSIZE;		/* data size */
+	}
+	else if (0 == strcmp(mem_type, "tsize"))
+	{
+		mem_type_code = ZBX_TSIZE;		/* text size */
+	}
+	else if (0 == strcmp(mem_type, "sdsize"))
+	{
+		mem_type_code = ZBX_SDSIZE;		/* data size from shared library */
+	}
+	else if (0 == strcmp(mem_type, "drss"))
+	{
+		mem_type_code = ZBX_DRSS;		/* data resident set size */
+	}
+	else if (0 == strcmp(mem_type, "trss"))
+	{
+		mem_type_code = ZBX_TRSS;		/* text resident set size */
+	}
+	else
+	{
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid fifth parameter."));
+		return SYSINFO_RET_FAIL;
+	}
+
+	if (1 == invalid_user)	/* handle 0 for non-existent user after all parameters have been parsed and validated */
+		goto out;
 
 	while (0 < getprocs64(&procentry, (int)sizeof(struct procentry64), NULL, 0, &pid, 1))
 	{
@@ -122,30 +176,99 @@ int	PROC_MEM(AGENT_REQUEST *request, AGENT_RESULT *result)
 		if (NULL != proccomm && '\0' != *proccomm && SUCCEED != check_procargs(&procentry, proccomm))
 			continue;
 
-		value = procentry.pi_size;
-		value <<= 12;	/* number of pages to bytes */
-
-		if (0 == proccount++)
+		switch (mem_type_code)
 		{
-			memsize = value;
+			case ZBX_VSIZE:
+				/* historically default proc.mem[] on AIX */
+				byte_value = (zbx_uint64_t)procentry.pi_size << 12;	/* number of pages to bytes */
+				break;
+			case ZBX_RSS:
+				/* try to be compatible with "ps -o rssize" */
+				byte_value = ((zbx_uint64_t)procentry.pi_drss << procentry.pi_data_l2psize) +
+						((zbx_uint64_t)procentry.pi_trss << procentry.pi_text_l2psize);
+				break;
+			case ZBX_PMEM:
+				/* try to be compatible with "ps -o pmem" */
+				pct_value = procentry.pi_prm;
+				break;
+			case ZBX_SIZE:
+				/* try to be compatible with "ps gvw" SIZE column */
+				byte_value = (zbx_uint64_t)procentry.pi_dvm << procentry.pi_data_l2psize;
+				break;
+			case ZBX_DSIZE:
+				byte_value = procentry.pi_dsize;
+				break;
+			case ZBX_TSIZE:
+				/* try to be compatible with "ps gvw" TSIZ column */
+				byte_value = procentry.pi_tsize;
+				break;
+			case ZBX_SDSIZE:
+				byte_value = procentry.pi_sdsize;
+				break;
+			case ZBX_DRSS:
+				byte_value = (zbx_uint64_t)procentry.pi_drss << procentry.pi_data_l2psize;
+				break;
+			case ZBX_TRSS:
+				byte_value = (zbx_uint64_t)procentry.pi_trss << procentry.pi_text_l2psize;
+				break;
+		}
+
+		if (ZBX_PMEM != mem_type_code)
+		{
+			if (0 != proccount++)
+			{
+				if (ZBX_DO_MAX == do_task)
+					mem_size = MAX(mem_size, byte_value);
+				else if (ZBX_DO_MIN == do_task)
+					mem_size = MIN(mem_size, byte_value);
+				else
+					mem_size += byte_value;
+			}
+			else
+				mem_size = byte_value;
 		}
 		else
 		{
-			if (ZBX_DO_MAX == do_task)
-				memsize = MAX(memsize, value);
-			else if (ZBX_DO_MIN == do_task)
-				memsize = MIN(memsize, value);
+			if (0 != proccount++)
+			{
+				if (ZBX_DO_MAX == do_task)
+					pct_size = MAX(pct_size, pct_value);
+				else if (ZBX_DO_MIN == do_task)
+					pct_size = MIN(pct_size, pct_value);
+				else
+					pct_size += pct_value;
+			}
 			else
-				memsize += value;
+				pct_size = pct_value;
 		}
         }
-
-	if (ZBX_DO_AVG == do_task)
-		SET_DBL_RESULT(result, 0 == proccount ? 0 : memsize / (double)proccount);
+out:
+	if (ZBX_PMEM != mem_type_code)
+	{
+		if (ZBX_DO_AVG == do_task)
+			SET_DBL_RESULT(result, 0 == proccount ? 0.0 : (double)mem_size / (double)proccount);
+		else
+			SET_UI64_RESULT(result, mem_size);
+	}
 	else
-		SET_UI64_RESULT(result, memsize);
+	{
+		if (ZBX_DO_AVG == do_task)
+			SET_DBL_RESULT(result, 0 == proccount ? 0.0 : pct_size / (double)proccount);
+		else
+			SET_DBL_RESULT(result, pct_size);
+	}
 
 	return SYSINFO_RET_OK;
+
+#undef ZBX_SIZE
+#undef ZBX_RSS
+#undef ZBX_VSIZE
+#undef ZBX_PMEM
+#undef ZBX_TSIZE
+#undef ZBX_DSIZE
+#undef ZBX_SDSIZE
+#undef ZBX_DRSS
+#undef ZBX_TRSS
 }
 
 int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
@@ -154,8 +277,7 @@ int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
 	struct passwd		*usrinfo;
 	struct procentry64	procentry;
 	pid_t			pid = 0;
-	int			zbx_proc_stat;
-	zbx_uint64_t		proccount = 0;
+	int			proccount = 0, invalid_user = 0, zbx_proc_stat;
 
 	if (4 < request->nparam)
 	{
@@ -169,10 +291,7 @@ int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
 	if (NULL != param && '\0' != *param)
 	{
 		if (NULL == (usrinfo = getpwnam(param)))
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain user information."));
-			return SYSINFO_RET_FAIL;
-		}
+			invalid_user = 1;
 	}
 	else
 		usrinfo = NULL;
@@ -195,6 +314,9 @@ int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 	proccomm = get_rparam(request, 3);
 
+	if (1 == invalid_user)	/* handle 0 for non-existent user after all parameters have been parsed and validated */
+		goto out;
+
 	while (0 < getprocs64(&procentry, (int)sizeof(struct procentry64), NULL, 0, &pid, 1))
 	{
 		if (NULL != procname && '\0' != *procname && 0 != strcmp(procname, procentry.pi_comm))
@@ -211,7 +333,7 @@ int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
 
 		proccount++;
         }
-
+out:
 	SET_UI64_RESULT(result, proccount);
 
 	return SYSINFO_RET_OK;
