@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2016 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,46 +21,6 @@
 #include "sysinfo.h"
 #include "stats.h"
 #include "log.h"
-#include "zbxalgo.h"
-#include "zbxjson.h"
-
-int	SYSTEM_CPU_DISCOVERY(AGENT_REQUEST *request, AGENT_RESULT *result)
-{
-	int			i;
-	zbx_vector_uint64_t	cpus;
-	struct zbx_json		json;
-
-	zbx_vector_uint64_create(&cpus);
-
-	if (SUCCEED != get_cpu_statuses(&cpus))
-	{
-		SET_MSG_RESULT(result, zbx_strdup(NULL, "Collector is not started."));
-		zbx_vector_uint64_destroy(&cpus);
-		return SYSINFO_RET_FAIL;
-	}
-
-	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
-	zbx_json_addarray(&json, ZBX_PROTO_TAG_DATA);
-
-	for (i = 0; i < cpus.values_num; i++)
-	{
-		zbx_json_addobject(&json, NULL);
-
-		zbx_json_adduint64(&json, "{#CPU.NUMBER}", i);
-		zbx_json_addstring(&json, "{#CPU.STATUS}", (SYSINFO_RET_OK == cpus.values[i] ?
-				"online" : "offline"), ZBX_JSON_TYPE_STRING);
-
-		zbx_json_close(&json);
-	}
-
-	zbx_json_close(&json);
-	SET_STR_RESULT(result, zbx_strdup(result->str, json.buffer));
-
-	zbx_json_free(&json);
-	zbx_vector_uint64_destroy(&cpus);
-
-	return SYSINFO_RET_OK;
-}
 
 int	SYSTEM_CPU_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
@@ -111,14 +71,12 @@ int	SYSTEM_CPU_UTIL(AGENT_REQUEST *request, AGENT_RESULT *result)
 	tmp = get_rparam(request, 0);
 
 	if (NULL == tmp || '\0' == *tmp || 0 == strcmp(tmp, "all"))
-		cpu_num = 0;
+		cpu_num = ZBX_CPUNUM_ALL;
 	else if (SUCCEED != is_uint31_1(tmp, &cpu_num))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid first parameter."));
 		return SYSINFO_RET_FAIL;
 	}
-	else
-		cpu_num++;
 
 	tmp = get_rparam(request, 1);
 
