@@ -223,7 +223,7 @@ static int	check_trigger_condition(const DB_EVENT *event, DB_CONDITION *conditio
 	{
 		tmp_str = zbx_strdup(tmp_str, event->trigger.description);
 
-		substitute_simple_macros(NULL, event, NULL, NULL, NULL, NULL, NULL,
+		substitute_simple_macros(NULL, event, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 				&tmp_str, MACRO_TYPE_TRIGGER_DESCRIPTION, NULL, 0);
 
 		switch (condition->operator)
@@ -269,12 +269,12 @@ static int	check_trigger_condition(const DB_EVENT *event, DB_CONDITION *conditio
 	}
 	else if (CONDITION_TYPE_TRIGGER_VALUE == condition->conditiontype)
 	{
-		condition_value = atoi(condition->value);
+		int	condition_value_i = atoi(condition->value);
 
 		switch (condition->operator)
 		{
 			case CONDITION_OPERATOR_EQUAL:
-				if (event->value == condition_value)
+				if (event->value == condition_value_i)
 					ret = SUCCEED;
 				break;
 			default:
@@ -286,11 +286,11 @@ static int	check_trigger_condition(const DB_EVENT *event, DB_CONDITION *conditio
 		switch (condition->operator)
 		{
 			case CONDITION_OPERATOR_IN:
-				if (SUCCEED == check_time_period(condition->value, (time_t)event->clock))
+				if (SUCCEED == check_time_period(condition->value, (time_t)0))
 					ret = SUCCEED;
 				break;
 			case CONDITION_OPERATOR_NOT_IN:
-				if (FAIL == check_time_period(condition->value, (time_t)event->clock))
+				if (FAIL == check_time_period(condition->value, (time_t)0))
 					ret = SUCCEED;
 				break;
 			default:
@@ -446,8 +446,7 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 	DB_RESULT	result;
 	DB_ROW		row;
 	zbx_uint64_t	condition_value;
-	int		tmp_int, now;
-	int		ret = FAIL;
+	int		tmp_int, ret = FAIL;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
@@ -524,12 +523,12 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 	}
 	else if (CONDITION_TYPE_DOBJECT == condition->conditiontype)
 	{
-		condition_value = atoi(condition->value);
+		int	condition_value_i = atoi(condition->value);
 
 		switch (condition->operator)
 		{
 			case CONDITION_OPERATOR_EQUAL:
-				if (event->object == condition_value)
+				if (event->object == condition_value_i)
 					ret = SUCCEED;
 				break;
 			default:
@@ -665,7 +664,7 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 	{
 		if (EVENT_OBJECT_DSERVICE == event->object)
 		{
-			condition_value = atoi(condition->value);
+			int	condition_value_i = atoi(condition->value);
 
 			result = DBselect(
 					"select type"
@@ -680,11 +679,11 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 				switch (condition->operator)
 				{
 					case CONDITION_OPERATOR_EQUAL:
-						if (condition_value == tmp_int)
+						if (condition_value_i == tmp_int)
 							ret = SUCCEED;
 						break;
 					case CONDITION_OPERATOR_NOT_EQUAL:
-						if (condition_value != tmp_int)
+						if (condition_value_i != tmp_int)
 							ret = SUCCEED;
 						break;
 					default:
@@ -696,16 +695,16 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 	}
 	else if (CONDITION_TYPE_DSTATUS == condition->conditiontype)
 	{
-		condition_value = atoi(condition->value);
+		int	condition_value_i = atoi(condition->value);
 
 		switch (condition->operator)
 		{
 			case CONDITION_OPERATOR_EQUAL:
-				if (condition_value == event->value)
+				if (condition_value_i == event->value)
 					ret = SUCCEED;
 				break;
 			case CONDITION_OPERATOR_NOT_EQUAL:
-				if (condition_value != event->value)
+				if (condition_value_i != event->value)
 					ret = SUCCEED;
 				break;
 			default:
@@ -714,7 +713,7 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 	}
 	else if (CONDITION_TYPE_DUPTIME == condition->conditiontype)
 	{
-		condition_value = atoi(condition->value);
+		int	condition_value_i = atoi(condition->value);
 
 		if (EVENT_OBJECT_DHOST == event->object)
 		{
@@ -735,17 +734,19 @@ static int	check_discovery_condition(const DB_EVENT *event, DB_CONDITION *condit
 
 		if (NULL != (row = DBfetch(result)))
 		{
-			tmp_int = DOBJECT_STATUS_UP == atoi(row[0]) ? atoi(row[1]) : atoi(row[2]);
+			int	now;
+
 			now = time(NULL);
+			tmp_int = DOBJECT_STATUS_UP == atoi(row[0]) ? atoi(row[1]) : atoi(row[2]);
 
 			switch (condition->operator)
 			{
 				case CONDITION_OPERATOR_LESS_EQUAL:
-					if (0 != tmp_int && (now - tmp_int) <= condition_value)
+					if (0 != tmp_int && (now - tmp_int) <= condition_value_i)
 						ret = SUCCEED;
 					break;
 				case CONDITION_OPERATOR_MORE_EQUAL:
-					if (0 != tmp_int && (now - tmp_int) >= condition_value)
+					if (0 != tmp_int && (now - tmp_int) >= condition_value_i)
 						ret = SUCCEED;
 					break;
 				default:
@@ -1289,45 +1290,31 @@ int	check_action_condition(const DB_EVENT *event, DB_CONDITION *condition)
  * Author: Alexei Vladishev                                                   *
  *                                                                            *
  ******************************************************************************/
-static int	check_action_conditions(const DB_EVENT *event, zbx_uint64_t actionid, unsigned char evaltype,
-		const char *formula)
+static int	check_action_conditions(const DB_EVENT *event, zbx_action_eval_t *action)
 {
 	const char	*__function_name = "check_action_conditions";
 
-	DB_RESULT	result;
-	DB_ROW		row;
-	DB_CONDITION	condition;
-	int		condition_result, ret = SUCCEED, id_len;
+	DB_CONDITION	*condition;
+	int		condition_result, ret = SUCCEED, id_len, i;
 	unsigned char	old_type = 0xff;
 	char		*expression = NULL, tmp[ZBX_MAX_UINT64_LEN + 2], *ptr, error[256];
 	double		eval_result;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() actionid:" ZBX_FS_UI64, __function_name, actionid);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() actionid:" ZBX_FS_UI64, __function_name, action->actionid);
 
-	if (evaltype == CONDITION_EVAL_TYPE_EXPRESSION)
-		expression = zbx_strdup(expression, formula);
+	if (action->evaltype == CONDITION_EVAL_TYPE_EXPRESSION)
+		expression = zbx_strdup(expression, action->formula);
 
-	result = DBselect(
-			"select conditionid,conditiontype,operator,value"
-			" from conditions"
-			" where actionid=" ZBX_FS_UI64
-			" order by conditiontype",
-			actionid);
-
-	while (NULL != (row = DBfetch(result)))
+	for (i = 0; i < action->conditions.values_num; i++)
 	{
-		ZBX_STR2UINT64(condition.conditionid, row[0]);
-		condition.actionid = actionid;
-		condition.conditiontype = (unsigned char)atoi(row[1]);
-		condition.operator = (unsigned char)atoi(row[2]);
-		condition.value = row[3];
+		condition = (DB_CONDITION *)action->conditions.values[i];
 
-		condition_result = check_action_condition(event, &condition);
+		condition_result = check_action_condition(event, condition);
 
-		switch (evaltype)
+		switch (action->evaltype)
 		{
 			case CONDITION_EVAL_TYPE_AND_OR:
-				if (old_type == condition.conditiontype)
+				if (old_type == condition->conditiontype)
 				{
 					if (SUCCEED == condition_result)
 						ret = SUCCEED;
@@ -1338,7 +1325,7 @@ static int	check_action_conditions(const DB_EVENT *event, zbx_uint64_t actionid,
 						goto clean;
 
 					ret = condition_result;
-					old_type = condition.conditiontype;
+					old_type = condition->conditiontype;
 				}
 
 				break;
@@ -1360,7 +1347,7 @@ static int	check_action_conditions(const DB_EVENT *event, zbx_uint64_t actionid,
 
 				break;
 			case CONDITION_EVAL_TYPE_EXPRESSION:
-				zbx_snprintf(tmp, sizeof(tmp), "{" ZBX_FS_UI64 "}", condition.conditionid);
+				zbx_snprintf(tmp, sizeof(tmp), "{" ZBX_FS_UI64 "}", condition->conditionid);
 				id_len = strlen(tmp);
 
 				for (ptr = expression; NULL != (ptr = strstr(ptr, tmp)); ptr += id_len)
@@ -1376,7 +1363,7 @@ static int	check_action_conditions(const DB_EVENT *event, zbx_uint64_t actionid,
 		}
 	}
 
-	if (evaltype == CONDITION_EVAL_TYPE_EXPRESSION)
+	if (action->evaltype == CONDITION_EVAL_TYPE_EXPRESSION)
 	{
 		if (SUCCEED == evaluate(&eval_result, expression, error, sizeof(error)))
 			ret = (SUCCEED != zbx_double_compare(eval_result, 0) ? SUCCEED : FAIL);
@@ -1384,7 +1371,6 @@ static int	check_action_conditions(const DB_EVENT *event, zbx_uint64_t actionid,
 		zbx_free(expression);
 	}
 clean:
-	DBfree_result(result);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
 
@@ -1408,7 +1394,6 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 
 	DB_RESULT		result;
 	DB_ROW			row;
-	unsigned char		operationtype;
 	zbx_uint64_t		groupid, templateid;
 	zbx_vector_uint64_t	lnk_templateids, del_templateids,
 				new_groupids, del_groupids;
@@ -1421,18 +1406,23 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 	zbx_vector_uint64_create(&del_groupids);
 
 	result = DBselect(
-			"select o.operationtype,g.groupid,t.templateid"
+			"select o.operationtype,g.groupid,t.templateid,oi.inventory_mode"
 			" from operations o"
 				" left join opgroup g on g.operationid=o.operationid"
 				" left join optemplate t on t.operationid=o.operationid"
+				" left join opinventory oi on oi.operationid=o.operationid"
 			" where o.actionid=" ZBX_FS_UI64,
 			actionid);
 
 	while (NULL != (row = DBfetch(result)))
 	{
+		int		inventory_mode;
+		unsigned char	operationtype;
+
 		operationtype = (unsigned char)atoi(row[0]);
 		ZBX_DBROW2UINT64(groupid, row[1]);
 		ZBX_DBROW2UINT64(templateid, row[2]);
+		inventory_mode = (SUCCEED == DBis_null(row[3]) ? 0 : atoi(row[3]));
 
 		switch (operationtype)
 		{
@@ -1464,6 +1454,9 @@ static void	execute_operations(const DB_EVENT *event, zbx_uint64_t actionid)
 				if (0 != templateid)
 					zbx_vector_uint64_append(&del_templateids, templateid);
 				break;
+			case OPERATION_TYPE_HOST_INVENTORY:
+				op_host_inventory_mode(event, inventory_mode);
+				break;
 			default:
 				;
 		}
@@ -1556,42 +1549,41 @@ void	process_actions(const DB_EVENT *events, size_t events_num)
 	DB_RESULT			result;
 	DB_ROW				row;
 	zbx_uint64_t			actionid;
-	unsigned char			evaltype;
 	size_t				i;
 	zbx_vector_uint64_t		rec_actionids;	/* actionids of possible recovery events */
 	zbx_vector_uint64_pair_t	rec_mapping;	/* which action is possibly recovered by which event */
 	const DB_EVENT			*event;
 	zbx_db_insert_t			db_insert;
-	int				escalations_num = 0;
+	int				escalations_num = 0, j;
+	zbx_vector_ptr_t		actions;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() events_num:" ZBX_FS_SIZE_T, __function_name, (zbx_fs_size_t)events_num);
 
 	zbx_vector_uint64_create(&rec_actionids);
 	zbx_vector_uint64_pair_create(&rec_mapping);
 
+	zbx_vector_ptr_create(&actions);
+	zbx_dc_get_actions_eval(&actions);
+
 	for (i = 0; i < events_num; i++)
 	{
 		event = &events[i];
 
-		result = DBselect("select actionid,evaltype,formula"
-				" from actions"
-				" where status=%d"
-					" and eventsource=%d",
-				ACTION_STATUS_ACTIVE, event->source);
-
-		while (NULL != (row = DBfetch(result)))
+		for (j = 0; j < actions.values_num; j++)
 		{
-			ZBX_STR2UINT64(actionid, row[0]);
-			ZBX_STR2UCHAR(evaltype, row[1]);
+			zbx_action_eval_t	*action = (zbx_action_eval_t *)actions.values[j];
 
-			if (SUCCEED == check_action_conditions(event, actionid, evaltype, row[2]))
+			if (action->eventsource != event->source)
+				continue;
+
+			if (SUCCEED == check_action_conditions(event, action))
 			{
-				escalation_add_values(&db_insert, escalations_num++, actionid, event, 0);
+				escalation_add_values(&db_insert, escalations_num++, action->actionid, event, 0);
 
 				if (EVENT_SOURCE_DISCOVERY == event->source ||
 						EVENT_SOURCE_AUTO_REGISTRATION == event->source)
 				{
-					execute_operations(event, actionid);
+					execute_operations(event, action->actionid);
 				}
 			}
 			else if (EVENT_SOURCE_TRIGGERS == event->source || EVENT_SOURCE_INTERNAL == event->source)
@@ -1601,17 +1593,19 @@ void	process_actions(const DB_EVENT *events, size_t events_num)
 
 				zbx_uint64_pair_t	pair;
 
-				pair.first = actionid;
+				pair.first = action->actionid;
 				pair.second = (zbx_uint64_t)i;
 
 				zbx_vector_uint64_pair_append(&rec_mapping, pair);
 
-				zbx_vector_uint64_append(&rec_actionids, actionid);
+				zbx_vector_uint64_append(&rec_actionids, action->actionid);
 			}
 		}
-		DBfree_result(result);
 	}
 
+	zbx_vector_ptr_clear_ext(&actions, (zbx_clean_func_t)zbx_action_eval_free);
+	zbx_vector_ptr_destroy(&actions);
+
 	if (0 != rec_actionids.values_num)
 	{
 		char		*sql = NULL;
@@ -1638,12 +1632,12 @@ void	process_actions(const DB_EVENT *events, size_t events_num)
 			ZBX_DBROW2UINT64(triggerid, row[1]);
 			ZBX_DBROW2UINT64(itemid, row[2]);
 
-			for (i = 0; i < rec_mapping.values_num; i++)
+			for (j = 0; j < rec_mapping.values_num; j++)
 			{
-				if (actionid != rec_mapping.values[i].first)
+				if (actionid != rec_mapping.values[j].first)
 					continue;
 
-				event = &events[(int)rec_mapping.values[i].second];
+				event = &events[(int)rec_mapping.values[j].second];
 
 				/* only add recovery if it matches event */
 				switch (event->source)
