@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2016 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "common.h"
 #include "db.h"
 
-int	check_action_condition(DB_EVENT *event, DB_CONDITION *condition);
-void    process_actions(DB_EVENT *event);
+int	check_action_condition(const DB_EVENT *event, DB_CONDITION *condition);
+void	process_actions(const DB_EVENT *events, size_t events_num);
 
 #endif
