@@ -28,9 +28,17 @@
 #include "../operations.h"
 #include "../actions.h"
 #include "../scripts.h"
+#include "../../libs/zbxcrypto/tls.h"
+#include "comms.h"
+
+extern int	CONFIG_ESCALATOR_FORKS;
 
 #define CONFIG_ESCALATOR_FREQUENCY	3
 
+#define ZBX_ESCALATION_SOURCE_DEFAULT	0
+#define ZBX_ESCALATION_SOURCE_ITEM	1
+#define ZBX_ESCALATION_SOURCE_TRIGGER	2
+
 typedef struct
 {
 	zbx_uint64_t	userid;
@@ -41,7 +49,20 @@ typedef struct
 }
 ZBX_USER_MSG;
 
-extern unsigned char	process_type, daemon_type;
+typedef struct
+{
+	zbx_uint64_t	actionid;
+	char		*shortdata;
+	char		*longdata;
+	char		*r_shortdata;
+	char		*r_longdata;
+	int		esc_period;
+	unsigned char	eventsource;
+	unsigned char	recovery_msg;
+}
+DB_ACTION;
+
+extern unsigned char	process_type, program_type;
 extern int		server_num, process_num;
 
 /******************************************************************************
@@ -298,9 +319,9 @@ static void	add_object_msg(zbx_uint64_t actionid, zbx_uint64_t operationid, zbx_
 		subject_dyn = zbx_strdup(NULL, subject);
 		message_dyn = zbx_strdup(NULL, message);
 
-		substitute_simple_macros(&actionid, event, NULL, &userid, NULL, NULL, NULL,
+		substitute_simple_macros(&actionid, event, NULL, &userid, NULL, NULL, NULL, NULL, NULL,
 				&subject_dyn, MACRO_TYPE_MESSAGE_NORMAL, NULL, 0);
-		substitute_simple_macros(&actionid, event, NULL, &userid, NULL, NULL, NULL,
+		substitute_simple_macros(&actionid, event, NULL, &userid, NULL, NULL, NULL, NULL, NULL,
 				&message_dyn, MACRO_TYPE_MESSAGE_NORMAL, NULL, 0);
 
 		add_user_msg(userid, mediatypeid, user_msg, subject_dyn, message_dyn);
@@ -339,23 +360,34 @@ static void	add_command_alert(zbx_db_insert_t *db_insert, int alerts_num, const
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
 
+#ifdef HAVE_OPENIPMI
+#	define ZBX_IPMI_FIELDS_NUM	4	/* number of selected IPMI-related fields in functions */
+						/* get_dynamic_hostid() and execute_commands() */
+#else
+#	define ZBX_IPMI_FIELDS_NUM	0
+#endif
+
 static int	get_dynamic_hostid(DB_EVENT *event, DC_HOST *host, char *error, size_t max_error_len)
 {
 	const char	*__function_name = "get_dynamic_hostid";
 	DB_RESULT	result;
 	DB_ROW		row;
-	char		sql[512];
+	char		sql[512];	/* do not forget to adjust size if SQLs change */
 	size_t		offset;
 	int		ret = SUCCEED;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	offset = zbx_snprintf(sql, sizeof(sql), "select distinct h.hostid,h.host");
+	offset = zbx_snprintf(sql, sizeof(sql), "select distinct h.hostid,h.host,h.tls_connect");
 #ifdef HAVE_OPENIPMI
 	offset += zbx_snprintf(sql + offset, sizeof(sql) - offset,
+			/* do not forget to update ZBX_IPMI_FIELDS_NUM if number of selected IPMI fields changes */
 			",h.ipmi_authtype,h.ipmi_privilege,h.ipmi_username,h.ipmi_password");
 #endif
-
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	offset += zbx_snprintf(sql + offset, sizeof(sql) - offset,
+			",h.tls_issuer,h.tls_subject,h.tls_psk_identity,h.tls_psk");
+#endif
 	switch (event->source)
 	{
 		case EVENT_SOURCE_TRIGGERS:
@@ -427,10 +459,17 @@ static int	get_dynamic_hostid(DB_EVENT *event, DC_HOST *host, char *error, size_
 		ZBX_STR2UINT64(host->hostid, row[0]);
 		strscpy(host->host, row[1]);
 #ifdef HAVE_OPENIPMI
-		host->ipmi_authtype = (signed char)atoi(row[2]);
-		host->ipmi_privilege = (unsigned char)atoi(row[3]);
-		strscpy(host->ipmi_username, row[4]);
-		strscpy(host->ipmi_password, row[5]);
+		host->ipmi_authtype = (signed char)atoi(row[3]);
+		host->ipmi_privilege = (unsigned char)atoi(row[4]);
+		strscpy(host->ipmi_username, row[5]);
+		strscpy(host->ipmi_password, row[6]);
+#endif
+		host->tls_connect = (unsigned char)atoi(row[2]);
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+		strscpy(host->tls_issuer, row[3 + ZBX_IPMI_FIELDS_NUM]);
+		strscpy(host->tls_subject, row[4 + ZBX_IPMI_FIELDS_NUM]);
+		strscpy(host->tls_psk_identity, row[5 + ZBX_IPMI_FIELDS_NUM]);
+		strscpy(host->tls_psk, row[6 + ZBX_IPMI_FIELDS_NUM]);
 #endif
 	}
 	DBfree_result(result);
@@ -455,25 +494,31 @@ static int	get_dynamic_hostid(DB_EVENT *event, DC_HOST *host, char *error, size_
 
 static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_t operationid, int esc_step)
 {
-	const char	*__function_name = "execute_commands";
-	DB_RESULT	result;
-	DB_ROW		row;
-	zbx_db_insert_t	db_insert;
-	int		alerts_num = 0;
-	char		*buffer = NULL;
-	size_t		buffer_alloc = ZBX_KIBIBYTE, buffer_offset = 0;
+	const char		*__function_name = "execute_commands";
+	DB_RESULT		result;
+	DB_ROW			row;
+	zbx_db_insert_t		db_insert;
+	int			alerts_num = 0;
+	char			*buffer = NULL;
+	size_t			buffer_alloc = 2 * ZBX_KIBIBYTE, buffer_offset = 0;
+	zbx_vector_uint64_t	executed_on_hosts;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
 	buffer = zbx_malloc(buffer, buffer_alloc);
 
 	zbx_strcpy_alloc(&buffer, &buffer_alloc, &buffer_offset,
+			/* the 1st 'select' works if remote command target is "Host group" */
 			"select distinct h.hostid,h.host,o.type,o.scriptid,o.execute_on,o.port"
-				",o.authtype,o.username,o.password,o.publickey,o.privatekey,o.command");
+				",o.authtype,o.username,o.password,o.publickey,o.privatekey,o.command,h.tls_connect"
 #ifdef HAVE_OPENIPMI
-	zbx_strcpy_alloc(&buffer, &buffer_alloc, &buffer_offset,
-			",h.ipmi_authtype,h.ipmi_privilege,h.ipmi_username,h.ipmi_password");
+			/* do not forget to update ZBX_IPMI_FIELDS_NUM if number of selected IPMI fields changes */
+			",h.ipmi_authtype,h.ipmi_privilege,h.ipmi_username,h.ipmi_password"
 #endif
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+			",h.tls_issuer,h.tls_subject,h.tls_psk_identity,h.tls_psk"
+#endif
+			);
 	zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset,
 			" from opcommand o,opcommand_grp og,hosts_groups hg,hosts h"
 			" where o.operationid=og.operationid"
@@ -481,14 +526,19 @@ static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_
 				" and hg.hostid=h.hostid"
 				" and o.operationid=" ZBX_FS_UI64
 				" and h.status=%d"
-			" union "
-			"select distinct h.hostid,h.host,o.type,o.scriptid,o.execute_on,o.port"
-				",o.authtype,o.username,o.password,o.publickey,o.privatekey,o.command",
+			" union ",
 			operationid, HOST_STATUS_MONITORED);
-#ifdef HAVE_OPENIPMI
 	zbx_strcpy_alloc(&buffer, &buffer_alloc, &buffer_offset,
-			",h.ipmi_authtype,h.ipmi_privilege,h.ipmi_username,h.ipmi_password");
+			/* the 2nd 'select' works if remote command target is "Host" */
+			"select distinct h.hostid,h.host,o.type,o.scriptid,o.execute_on,o.port"
+				",o.authtype,o.username,o.password,o.publickey,o.privatekey,o.command,h.tls_connect"
+#ifdef HAVE_OPENIPMI
+			",h.ipmi_authtype,h.ipmi_privilege,h.ipmi_username,h.ipmi_password"
 #endif
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+			",h.tls_issuer,h.tls_subject,h.tls_psk_identity,h.tls_psk"
+#endif
+			);
 	zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset,
 			" from opcommand o,opcommand_hst oh,hosts h"
 			" where o.operationid=oh.operationid"
@@ -496,11 +546,17 @@ static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_
 				" and o.operationid=" ZBX_FS_UI64
 				" and h.status=%d"
 			" union "
+			/* the 3rd 'select' works if remote command target is "Current host" */
 			"select distinct 0,null,o.type,o.scriptid,o.execute_on,o.port"
-				",o.authtype,o.username,o.password,o.publickey,o.privatekey,o.command",
-			operationid, HOST_STATUS_MONITORED);
+				",o.authtype,o.username,o.password,o.publickey,o.privatekey,o.command,%d",
+			operationid, HOST_STATUS_MONITORED, ZBX_TCP_SEC_UNENCRYPTED);
 #ifdef HAVE_OPENIPMI
-	zbx_strcpy_alloc(&buffer, &buffer_alloc, &buffer_offset, ",0,2,null,null");
+	zbx_strcpy_alloc(&buffer, &buffer_alloc, &buffer_offset,
+				",0,2,null,null");
+#endif
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	zbx_strcpy_alloc(&buffer, &buffer_alloc, &buffer_offset,
+				",null,null,null,null");
 #endif
 	zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset,
 			" from opcommand o,opcommand_hst oh"
@@ -512,6 +568,7 @@ static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_
 	result = DBselect("%s", buffer);
 
 	zbx_free(buffer);
+	zbx_vector_uint64_create(&executed_on_hosts);
 
 	while (NULL != (row = DBfetch(result)))
 	{
@@ -530,7 +587,7 @@ static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_
 		if (ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT != script.type)
 		{
 			script.command = zbx_strdup(script.command, row[11]);
-			substitute_simple_macros(&actionid, event, NULL, NULL, NULL, NULL, NULL,
+			substitute_simple_macros(&actionid, event, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 					&script.command, MACRO_TYPE_MESSAGE_NORMAL, NULL, 0);
 		}
 
@@ -543,16 +600,38 @@ static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_
 
 			if (0 != host.hostid)
 			{
+				if (FAIL != zbx_vector_uint64_search(&executed_on_hosts, host.hostid,
+						ZBX_DEFAULT_UINT64_COMPARE_FUNC))
+				{
+					goto skip;
+				}
+
+				zbx_vector_uint64_append(&executed_on_hosts, host.hostid);
 				strscpy(host.host, row[1]);
+				host.tls_connect = (unsigned char)atoi(row[12]);
 #ifdef HAVE_OPENIPMI
-				host.ipmi_authtype = (signed char)atoi(row[12]);
-				host.ipmi_privilege = (unsigned char)atoi(row[13]);
-				strscpy(host.ipmi_username, row[14]);
-				strscpy(host.ipmi_password, row[15]);
+				host.ipmi_authtype = (signed char)atoi(row[13]);
+				host.ipmi_privilege = (unsigned char)atoi(row[14]);
+				strscpy(host.ipmi_username, row[15]);
+				strscpy(host.ipmi_password, row[16]);
+#endif
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+				strscpy(host.tls_issuer, row[13 + ZBX_IPMI_FIELDS_NUM]);
+				strscpy(host.tls_subject, row[14 + ZBX_IPMI_FIELDS_NUM]);
+				strscpy(host.tls_psk_identity, row[15 + ZBX_IPMI_FIELDS_NUM]);
+				strscpy(host.tls_psk, row[16 + ZBX_IPMI_FIELDS_NUM]);
 #endif
 			}
-			else
-				rc = get_dynamic_hostid(event, &host, error, sizeof(error));
+			else if (SUCCEED == (rc = get_dynamic_hostid(event, &host, error, sizeof(error))))
+			{
+				if (FAIL != zbx_vector_uint64_search(&executed_on_hosts, host.hostid,
+						ZBX_DEFAULT_UINT64_COMPARE_FUNC))
+				{
+					goto skip;
+				}
+
+				zbx_vector_uint64_append(&executed_on_hosts, host.hostid);
+			}
 		}
 
 		if (SUCCEED == rc)
@@ -581,10 +660,11 @@ static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_
 
 		add_command_alert(&db_insert, alerts_num++, &host, event->eventid, actionid, esc_step, script.command,
 				status, error);
-
+skip:
 		zbx_script_clean(&script);
 	}
 	DBfree_result(result);
+	zbx_vector_uint64_destroy(&executed_on_hosts);
 
 	if (0 < alerts_num)
 	{
@@ -596,6 +676,8 @@ static void	execute_commands(DB_EVENT *event, zbx_uint64_t actionid, zbx_uint64_
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
 
+#undef ZBX_IPMI_FIELDS_NUM
+
 static void	add_message_alert(DB_ESCALATION *escalation, DB_EVENT *event, DB_EVENT *r_event, DB_ACTION *action,
 		zbx_uint64_t userid, zbx_uint64_t mediatypeid, const char *subject, const char *message)
 {
@@ -651,7 +733,7 @@ static void	add_message_alert(DB_ESCALATION *escalation, DB_EVENT *event, DB_EVE
 			continue;
 		}
 
-		if (FAIL == check_time_period(row[3], (time_t)NULL))
+		if (FAIL == check_time_period(row[3], (time_t)0))
 		{
 			zabbix_log(LOG_LEVEL_DEBUG, "will not send message (period)");
 			continue;
@@ -995,13 +1077,13 @@ static void	process_recovery_msg(DB_ESCALATION *escalation, DB_EVENT *event, DB_
 					break;
 			}
 
-			subject_dyn = zbx_strdup(NULL, action->shortdata);
-			message_dyn = zbx_strdup(NULL, action->longdata);
+			subject_dyn = zbx_strdup(NULL, action->r_shortdata);
+			message_dyn = zbx_strdup(NULL, action->r_longdata);
 
-			substitute_simple_macros(&action->actionid, event, r_event, &userid, NULL, NULL, NULL,
-					&subject_dyn, MACRO_TYPE_MESSAGE_RECOVERY, NULL, 0);
-			substitute_simple_macros(&action->actionid, event, r_event, &userid, NULL, NULL, NULL,
-					&message_dyn, MACRO_TYPE_MESSAGE_RECOVERY, NULL, 0);
+			substitute_simple_macros(&action->actionid, event, r_event, &userid, NULL, NULL, NULL, NULL,
+					NULL, &subject_dyn, MACRO_TYPE_MESSAGE_RECOVERY, NULL, 0);
+			substitute_simple_macros(&action->actionid, event, r_event, &userid, NULL, NULL, NULL, NULL,
+					NULL, &message_dyn, MACRO_TYPE_MESSAGE_RECOVERY, NULL, 0);
 
 			escalation->esc_step = 0;
 
@@ -1135,14 +1217,17 @@ static void	free_event_info(DB_EVENT *event)
  *             source    - [IN] the esclation event source                    *
  *             error     - [OUT] message in case escalation is cancelled      *
  *                                                                            *
+ * Return value: FAIL if dependent trigger is in PROBLEM state                *
+ *               SUCCEED otherwise                                            *
+ *                                                                            *
  ******************************************************************************/
-static void	check_escalation_trigger(zbx_uint64_t triggerid, unsigned char source, char **error)
+static int	check_escalation_trigger(zbx_uint64_t triggerid, unsigned char source, char **error)
 {
 	DC_TRIGGER		trigger;
-	int			i, errcode, *errcodes = NULL;
 	zbx_vector_uint64_t	functionids, itemids;
 	DC_ITEM			*items = NULL;
 	DC_FUNCTION		*functions = NULL;
+	int			i, errcode, *errcodes = NULL, ret = SUCCEED;
 
 	/* trigger disabled or deleted? */
 	DCconfig_get_triggers_by_triggerids(&trigger, &triggerid, &errcode, 1);
@@ -1212,8 +1297,12 @@ static void	check_escalation_trigger(zbx_uint64_t triggerid, unsigned char sourc
 
 	zbx_vector_uint64_destroy(&itemids);
 	zbx_vector_uint64_destroy(&functionids);
+
+	ret = DCconfig_check_trigger_dependencies(trigger.triggerid);
 out:
 	DCconfig_clean_triggers(&trigger, &errcode, 1);
+
+	return ret;
 }
 
 /******************************************************************************
@@ -1224,21 +1313,21 @@ out:
  *          hosts, and actions are still present and were not disabled)       *
  *                                                                            *
  * Parameters: escalation - [IN] escalation data                              *
- *             action     - [OUT] action data (optional)                      *
+ *             action     - [OUT] action data                                 *
  *             error      - [OUT] message in case escalation is cancelled     *
  *                                                                            *
- * Comments: If 'action' is not NULL, it gathers information about it. If     *
- *           information could not be gathered, its 'actionid' is set to 0.   *
+ * Comments: 'action' is filled with information about action. If information *
+ *           could not be gathered, 'action->actionid' is set to 0.           *
  *                                                                            *
  ******************************************************************************/
-static void	check_escalation(const DB_ESCALATION *escalation, DB_ACTION *action, char **error)
+static int	check_escalation(const DB_ESCALATION *escalation, DB_ACTION *action, char **error)
 {
 	const char	*__function_name = "check_escalation";
 	DB_RESULT	result;
 	DB_ROW		row;
 	unsigned char	source = 0xff, object = 0xff;
 	DC_ITEM		item;
-	int		errcode;
+	int		errcode, ret = SUCCEED;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() escalationid:" ZBX_FS_UI64 " status:%s",
 			__function_name, escalation->escalationid, zbx_escalation_status_string(escalation->status));
@@ -1257,7 +1346,7 @@ static void	check_escalation(const DB_ESCALATION *escalation, DB_ACTION *action,
 	DBfree_result(result);
 
 	if (NULL == *error && EVENT_OBJECT_TRIGGER == object)
-		check_escalation_trigger(escalation->triggerid, source, error);
+		ret = check_escalation_trigger(escalation->triggerid, source, error);
 
 	if (NULL == *error && EVENT_SOURCE_INTERNAL == source)
 	{
@@ -1283,95 +1372,81 @@ static void	check_escalation(const DB_ESCALATION *escalation, DB_ACTION *action,
 		}
 	}
 
-	switch (escalation->status)
-	{
-		case ESCALATION_STATUS_ACTIVE:
-			result = DBselect(
-					"select actionid,name,status%s"
-					" from actions"
-					" where actionid=" ZBX_FS_UI64,
-					NULL == action ? "" : ",eventsource,esc_period"
-						",def_shortdata,def_longdata,recovery_msg",
-					escalation->actionid);
-			break;
-		case ESCALATION_STATUS_RECOVERY:
-		case ESCALATION_STATUS_SLEEP:
-			result = DBselect(
-					"select actionid,name,status%s"
-					" from actions"
-					" where actionid=" ZBX_FS_UI64,
-					NULL == action ? "" : ",eventsource,esc_period"
-						",r_shortdata,r_longdata,recovery_msg",
-					escalation->actionid);
-			break;
-		default:
-			THIS_SHOULD_NEVER_HAPPEN;
-			return;
-	}
+	result = DBselect(
+			"select actionid,name,status,eventsource,esc_period,def_shortdata,def_longdata,r_shortdata,"
+				"r_longdata,recovery_msg"
+			" from actions"
+			" where actionid=" ZBX_FS_UI64,
+			escalation->actionid);
 
 	if (NULL != (row = DBfetch(result)))
 	{
 		if (ACTION_STATUS_ACTIVE != atoi(row[2]))
 			*error = zbx_dsprintf(*error, "action '%s' disabled.", row[1]);
 
-		if (NULL != action)
-		{
-			memset(action, 0, sizeof(*action));
-			ZBX_STR2UINT64(action->actionid, row[0]);
-			action->eventsource = atoi(row[3]);
-			action->esc_period = atoi(row[4]);
-			action->shortdata = zbx_strdup(NULL, row[5]);
-			action->longdata = zbx_strdup(NULL, row[6]);
-			action->recovery_msg = atoi(row[7]);
-		}
+		ZBX_STR2UINT64(action->actionid, row[0]);
+		action->eventsource = atoi(row[3]);
+		action->esc_period = atoi(row[4]);
+		action->shortdata = zbx_strdup(NULL, row[5]);
+		action->longdata = zbx_strdup(NULL, row[6]);
+		action->r_shortdata = zbx_strdup(NULL, row[7]);
+		action->r_longdata = zbx_strdup(NULL, row[8]);
+		action->recovery_msg = atoi(row[9]);
 	}
 	else
 	{
-		if (NULL != action)
-			action->actionid = 0;
+		action->actionid = 0;
 
 		*error = zbx_dsprintf(*error, "action id:" ZBX_FS_UI64 " deleted", escalation->actionid);
 	}
+
 	DBfree_result(result);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() error: %s", __function_name, ZBX_NULL2STR(*error));
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s error: '%s'", __function_name, zbx_result_string(ret),
+			ZBX_NULL2STR(*error));
+
+	return ret;
 }
 
-static void	execute_escalation(DB_ESCALATION *escalation)
+static void	execute_escalation(DB_ESCALATION *escalation, DB_ACTION *action, const char *error)
 {
 	const char	*__function_name = "execute_escalation";
-	DB_ACTION	action;
 	DB_EVENT	event, r_event;
-	char		*error = NULL;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() escalationid:" ZBX_FS_UI64 " status:%s",
 			__function_name, escalation->escalationid, zbx_escalation_status_string(escalation->status));
 
-	check_escalation(escalation, &action, &error);
-
-	if (0 != action.actionid)
+	if (0 != action->actionid)
 	{
-		if (NULL != error)
-		{
-			action.longdata = zbx_dsprintf(action.longdata, "NOTE: Escalation cancelled: %s\n%s",
-					error, action.longdata);
-		}
-
 		switch (escalation->status)
 		{
 			case ESCALATION_STATUS_ACTIVE:
+				if (NULL != error)
+				{
+					action->longdata = zbx_dsprintf(action->longdata,
+							"NOTE: Escalation cancelled: %s\n%s",
+							error, action->longdata);
+				}
+
 				if (SUCCEED == get_event_info(escalation->eventid, &event))
 				{
-					execute_operations(escalation, &event, &action);
+					execute_operations(escalation, &event, action);
 					free_event_info(&event);
 				}
 				break;
 			case ESCALATION_STATUS_RECOVERY:
+				if (NULL != error)
+				{
+					action->r_longdata = zbx_dsprintf(action->r_longdata,
+							"NOTE: Escalation cancelled: %s\n%s",
+							error, action->r_longdata);
+				}
+
 				if (SUCCEED == get_event_info(escalation->eventid, &event))
 				{
 					if (SUCCEED == get_event_info(escalation->r_eventid, &r_event))
 					{
-						process_recovery_msg(escalation, &event, &r_event, &action);
+						process_recovery_msg(escalation, &event, &r_event, action);
 						free_event_info(&r_event);
 					}
 					free_event_info(&event);
@@ -1380,30 +1455,37 @@ static void	execute_escalation(DB_ESCALATION *escalation)
 			default:
 				break;
 		}
-
-		zbx_free(action.longdata);
-		zbx_free(action.shortdata);
 	}
 
 	if (NULL != error)
 	{
 		escalation->status = ESCALATION_STATUS_COMPLETED;
 		zabbix_log(LOG_LEVEL_WARNING, "escalation cancelled: %s", error);
-		zbx_free(error);
 	}
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
 
-static int	process_escalations(int now, int *nextcheck)
+static void	free_db_action(DB_ACTION *action)
+{
+	if (0 != action->actionid)
+	{
+		zbx_free(action->shortdata);
+		zbx_free(action->longdata);
+		zbx_free(action->r_shortdata);
+		zbx_free(action->r_longdata);
+	}
+}
+
+static int	process_escalations(int now, int *nextcheck, unsigned int escalation_source)
 {
 	const char		*__function_name = "process_escalations";
 	DB_RESULT		result;
 	DB_ROW			row;
 	DB_ESCALATION		escalation, last_escalation;
 	zbx_vector_uint64_t	escalationids;
-	char			*sql = NULL;
-	size_t			sql_alloc = ZBX_KIBIBYTE, sql_offset;
+	char			*sql = NULL, *filter = NULL;
+	size_t			sql_alloc = ZBX_KIBIBYTE, sql_offset, filter_alloc = 0, filter_offset = 0;
 	int			res;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
@@ -1411,14 +1493,49 @@ static int	process_escalations(int now, int *nextcheck)
 	zbx_vector_uint64_create(&escalationids);
 	sql = zbx_malloc(sql, sql_alloc);
 
+	switch (escalation_source)
+	{
+		case ZBX_ESCALATION_SOURCE_TRIGGER:
+			zbx_strcpy_alloc(&filter, &filter_alloc, &filter_offset, "triggerid is not null");
+			if (1 < CONFIG_ESCALATOR_FORKS)
+			{
+				zbx_snprintf_alloc(&filter, &filter_alloc, &filter_offset,
+						" and " ZBX_SQL_MOD(triggerid, %d) "=%d",
+						CONFIG_ESCALATOR_FORKS, process_num - 1);
+			}
+			break;
+		case ZBX_ESCALATION_SOURCE_ITEM:
+			zbx_strcpy_alloc(&filter, &filter_alloc, &filter_offset, "itemid is not null");
+			if (1 < CONFIG_ESCALATOR_FORKS)
+			{
+				zbx_snprintf_alloc(&filter, &filter_alloc, &filter_offset,
+						" and " ZBX_SQL_MOD(itemid, %d) "=%d",
+						CONFIG_ESCALATOR_FORKS, process_num - 1);
+			}
+			break;
+		case ZBX_ESCALATION_SOURCE_DEFAULT:
+			zbx_strcpy_alloc(&filter, &filter_alloc, &filter_offset,
+					"triggerid is null and itemid is null");
+			if (1 < CONFIG_ESCALATOR_FORKS)
+			{
+				zbx_snprintf_alloc(&filter, &filter_alloc, &filter_offset,
+						" and " ZBX_SQL_MOD(escalationid, %d) "=%d",
+						CONFIG_ESCALATOR_FORKS, process_num - 1);
+			}
+			break;
+	}
+
 	result = DBselect(
 			"select escalationid,actionid,triggerid,eventid,r_eventid,nextcheck,esc_step,status,itemid"
 			" from escalations"
-			" order by actionid,triggerid,itemid,escalationid");
+			" where %s"
+			" order by actionid,triggerid,itemid,escalationid",
+			filter);
 
 	*nextcheck = now + CONFIG_ESCALATOR_FREQUENCY;
 	memset(&escalation, 0, sizeof(escalation));
 
+	/* we read escalations into @last_escalation variable and process @escalation, which was previously read */
 	do
 	{
 		unsigned char	esc_superseded = 0;
@@ -1452,6 +1569,7 @@ static int	process_escalations(int now, int *nextcheck)
 			goto next;
 		}
 
+		/* handle superseded escalations (multiple problem events) */
 		if (0 != last_escalation.escalationid)
 		{
 			esc_superseded = (escalation.actionid == last_escalation.actionid &&
@@ -1476,20 +1594,36 @@ static int	process_escalations(int now, int *nextcheck)
 
 		if (escalation.nextcheck <= now || 0 != escalation.r_eventid)
 		{
+			DB_ACTION	action;
+			char		*error = NULL;
+
 			DBbegin();
 
 			sql_offset = 0;
 
+			if (SUCCEED != check_escalation(&escalation, &action, &error))
+			{
+				/* Dependable trigger in PROBLEM state. If escalation is cancelled we process */
+				/* it normally in order to send notification about the error otherwise we     */
+				/* skip the escalation until dependable trigger changes value to OK.          */
+				if (NULL == error)
+				{
+					DBrollback();
+					free_db_action(&action);
+					goto next;
+				}
+			}
+
 			if (ESCALATION_STATUS_ACTIVE == escalation.status)
 			{
 				if (escalation.nextcheck <= now)
-					execute_escalation(&escalation);
+					execute_escalation(&escalation, &action, error);
 
-				/* execute recovery */
+				/* execute recovery if the same record has it */
 				if (ESCALATION_STATUS_COMPLETED != escalation.status && 0 != escalation.r_eventid)
 				{
 					escalation.status = ESCALATION_STATUS_RECOVERY;
-					execute_escalation(&escalation);
+					execute_escalation(&escalation, &action, error);
 				}
 				else if (0 != esc_superseded)
 				{
@@ -1518,14 +1652,9 @@ static int	process_escalations(int now, int *nextcheck)
 			}
 			else	/* ESCALATION_STATUS_SLEEP == escalation.status */
 			{
-				char	*error = NULL;
-
-				check_escalation(&escalation, NULL, &error);
-
 				if (NULL != error)
 				{
 					zabbix_log(LOG_LEVEL_WARNING, "escalation cancelled: %s", error);
-					zbx_free(error);
 
 					zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
 							"delete from escalations where escalationid=" ZBX_FS_UI64,
@@ -1541,6 +1670,9 @@ static int	process_escalations(int now, int *nextcheck)
 				}
 			}
 
+			free_db_action(&action);
+			zbx_free(error);
+
 			DBexecute("%s", sql);
 
 			DBcommit();
@@ -1556,6 +1688,7 @@ next:
 
 	DBfree_result(result);
 
+	zbx_free(filter);
 	zbx_free(sql);
 
 	res = escalationids.values_num;		/* performance metric */
@@ -1602,27 +1735,36 @@ ZBX_THREAD_ENTRY(escalator_thread, args)
 	server_num = ((zbx_thread_args_t *)args)->server_num;
 	process_num = ((zbx_thread_args_t *)args)->process_num;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_daemon_type_string(daemon_type),
+	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_program_type_string(program_type),
 			server_num, get_process_type_string(process_type), process_num);
 
 #define STAT_INTERVAL	5	/* if a process is busy and does not sleep then update status not faster than */
 				/* once in STAT_INTERVAL seconds */
 
-	zbx_setproctitle("%s [connecting to the database]", get_process_type_string(process_type));
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	zbx_tls_init_child();
+#endif
+	zbx_setproctitle("%s #%d [connecting to the database]", get_process_type_string(process_type), process_num);
 	last_stat_time = time(NULL);
 
 	DBconnect(ZBX_DB_CONNECT_NORMAL);
 
 	for (;;)
 	{
+		zbx_handle_log();
+
 		if (0 != sleeptime)
 		{
-			zbx_setproctitle("%s [processed %d escalations in " ZBX_FS_DBL " sec, processing escalations]",
-					get_process_type_string(process_type), old_escalations_count, old_total_sec);
+			zbx_setproctitle("%s #%d [processed %d escalations in " ZBX_FS_DBL
+					" sec, processing escalations]", get_process_type_string(process_type),
+					process_num, old_escalations_count, old_total_sec);
 		}
 
 		sec = zbx_time();
-		escalations_count += process_escalations(time(NULL), &nextcheck);
+		escalations_count += process_escalations(time(NULL), &nextcheck, ZBX_ESCALATION_SOURCE_TRIGGER);
+		escalations_count += process_escalations(time(NULL), &nextcheck, ZBX_ESCALATION_SOURCE_ITEM);
+		escalations_count += process_escalations(time(NULL), &nextcheck, ZBX_ESCALATION_SOURCE_DEFAULT);
+
 		total_sec += zbx_time() - sec;
 
 		sleeptime = calculate_sleeptime(nextcheck, CONFIG_ESCALATOR_FREQUENCY);
@@ -1633,15 +1775,15 @@ ZBX_THREAD_ENTRY(escalator_thread, args)
 		{
 			if (0 == sleeptime)
 			{
-				zbx_setproctitle("%s [processed %d escalations in " ZBX_FS_DBL
+				zbx_setproctitle("%s #%d [processed %d escalations in " ZBX_FS_DBL
 						" sec, processing escalations]", get_process_type_string(process_type),
-						escalations_count, total_sec);
+						process_num, escalations_count, total_sec);
 			}
 			else
 			{
-				zbx_setproctitle("%s [processed %d escalations in " ZBX_FS_DBL " sec, idle %d sec]",
-						get_process_type_string(process_type), escalations_count, total_sec,
-						sleeptime);
+				zbx_setproctitle("%s #%d [processed %d escalations in " ZBX_FS_DBL " sec, idle %d sec]",
+						get_process_type_string(process_type), process_num, escalations_count,
+						total_sec, sleeptime);
 
 				old_escalations_count = escalations_count;
 				old_total_sec = total_sec;
