@@ -20,9 +20,16 @@
 #include "common.h"
 #include "comms.h"
 #include "log.h"
+#include "../../libs/zbxcrypto/tls_tcp_active.h"
 
 #include "checks_agent.h"
 
+extern volatile sig_atomic_t	zbx_timed_out;
+
+#if !(defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL))
+extern unsigned char	program_type;
+#endif
+
 /******************************************************************************
  *                                                                            *
  * Function: get_value_agent                                                  *
@@ -46,15 +53,46 @@
 int	get_value_agent(DC_ITEM *item, AGENT_RESULT *result)
 {
 	const char	*__function_name = "get_value_agent";
-	zbx_sock_t	s;
-	char		*buffer = NULL;
+	zbx_socket_t	s;
+	char		*buffer = NULL, *tls_arg1, *tls_arg2;
 	int		ret = SUCCEED;
 	ssize_t		received_len;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host:'%s' addr:'%s' key:'%s'",
-			__function_name, item->host.host, item->interface.addr, item->key);
+	if (SUCCEED == zabbix_check_log_level(LOG_LEVEL_DEBUG))
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "In %s() host:'%s' addr:'%s' key:'%s' conn:'%s'", __function_name,
+				item->host.host, item->interface.addr, item->key,
+				zbx_tls_connection_type_name(item->host.tls_connect));
+	}
 
-	if (SUCCEED == (ret = zbx_tcp_connect(&s, CONFIG_SOURCE_IP, item->interface.addr, item->interface.port, 0)))
+	if (ZBX_TCP_SEC_UNENCRYPTED == item->host.tls_connect)
+	{
+		tls_arg1 = NULL;
+		tls_arg2 = NULL;
+	}
+	else
+	{
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+		if (ZBX_TCP_SEC_TLS_CERT == item->host.tls_connect)
+		{
+			tls_arg1 = item->host.tls_issuer;
+			tls_arg2 = item->host.tls_subject;
+		}
+		else	/* ZBX_TCP_SEC_TLS_PSK */
+		{
+			tls_arg1 = item->host.tls_psk_identity;
+			tls_arg2 = item->host.tls_psk;
+		}
+#else
+		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "A TLS connection is configured to be used with agent but"
+				" support for TLS was not compiled into %s.", get_program_type_string(program_type)));
+		ret = NETWORK_ERROR;
+		goto out;
+#endif
+	}
+
+	if (SUCCEED == (ret = zbx_tcp_connect(&s, CONFIG_SOURCE_IP, item->interface.addr, item->interface.port, 0,
+			item->host.tls_connect, tls_arg1, tls_arg2)))
 	{
 		buffer = zbx_dsprintf(buffer, "%s\n", item->key);
 		zabbix_log(LOG_LEVEL_DEBUG, "Sending [%s]", buffer);
@@ -64,8 +102,10 @@ int	get_value_agent(DC_ITEM *item, AGENT_RESULT *result)
 			ret = NETWORK_ERROR;
 		else if (FAIL != (received_len = zbx_tcp_recv_ext(&s, ZBX_TCP_READ_UNTIL_CLOSE, 0)))
 			ret = SUCCEED;
-		else
+		else if (1 == zbx_timed_out)
 			ret = TIMEOUT_ERROR;
+		else
+			ret = NETWORK_ERROR;
 
 		zbx_free(buffer);
 	}
@@ -105,8 +145,11 @@ int	get_value_agent(DC_ITEM *item, AGENT_RESULT *result)
 			ret = NOTSUPPORTED;
 	}
 	else
-		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Get value from agent failed: %s", zbx_tcp_strerror()));
+		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Get value from agent failed: %s", zbx_socket_strerror()));
 
+#if !(defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL))
+out:
+#endif
 	zbx_tcp_close(&s);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
