@@ -29,21 +29,23 @@
 #include "zbxjson.h"
 #include "log.h"
 #include "proxy.h"
+#include "../../libs/zbxcrypto/tls.h"
 
-extern unsigned char	process_type, daemon_type;
+extern unsigned char	process_type, program_type;
 extern int		server_num, process_num;
 
-static int	connect_to_proxy(DC_PROXY *proxy, zbx_sock_t *sock, int timeout)
+static int	connect_to_proxy(DC_PROXY *proxy, zbx_socket_t *sock, int timeout)
 {
 	const char	*__function_name = "connect_to_proxy";
 	int		ret;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() address:%s port:%hu timeout:%d", __function_name, proxy->addr,
-			proxy->port, timeout);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() address:%s port:%hu timeout:%d conn:%u", __function_name, proxy->addr,
+			proxy->port, timeout, (unsigned int)proxy->tls_connect);
 
-	if (FAIL == (ret = zbx_tcp_connect(sock, CONFIG_SOURCE_IP, proxy->addr, proxy->port, timeout)))
+	if (FAIL == (ret = zbx_tcp_connect(sock, CONFIG_SOURCE_IP, proxy->addr, proxy->port, timeout,
+			proxy->tls_connect, proxy->tls_arg1, proxy->tls_arg2)))
 	{
-		zabbix_log(LOG_LEVEL_ERR, "cannot connect to proxy \"%s\": %s", proxy->host, zbx_tcp_strerror());
+		zabbix_log(LOG_LEVEL_ERR, "cannot connect to proxy \"%s\": %s", proxy->host, zbx_socket_strerror());
 		ret = NETWORK_ERROR;
 	}
 
@@ -52,7 +54,7 @@ static int	connect_to_proxy(DC_PROXY *proxy, zbx_sock_t *sock, int timeout)
 	return ret;
 }
 
-static int	send_data_to_proxy(DC_PROXY *proxy, zbx_sock_t *sock, const char *data)
+static int	send_data_to_proxy(DC_PROXY *proxy, zbx_socket_t *sock, const char *data)
 {
 	const char	*__function_name = "send_data_to_proxy";
 	int		ret;
@@ -61,7 +63,7 @@ static int	send_data_to_proxy(DC_PROXY *proxy, zbx_sock_t *sock, const char *dat
 
 	if (FAIL == (ret = zbx_tcp_send(sock, data)))
 	{
-		zabbix_log(LOG_LEVEL_ERR, "cannot send data to proxy \"%s\": %s", proxy->host, zbx_tcp_strerror());
+		zabbix_log(LOG_LEVEL_ERR, "cannot send data to proxy \"%s\": %s", proxy->host, zbx_socket_strerror());
 
 		ret = NETWORK_ERROR;
 	}
@@ -71,7 +73,7 @@ static int	send_data_to_proxy(DC_PROXY *proxy, zbx_sock_t *sock, const char *dat
 	return ret;
 }
 
-static int	recv_data_from_proxy(DC_PROXY *proxy, zbx_sock_t *sock)
+static int	recv_data_from_proxy(DC_PROXY *proxy, zbx_socket_t *sock)
 {
 	const char	*__function_name = "recv_data_from_proxy";
 	int		ret;
@@ -79,16 +81,19 @@ static int	recv_data_from_proxy(DC_PROXY *proxy, zbx_sock_t *sock)
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
 	if (FAIL == (ret = zbx_tcp_recv(sock)))
-		zabbix_log(LOG_LEVEL_ERR, "cannot obtain data from proxy \"%s\": %s", proxy->host, zbx_tcp_strerror());
+	{
+		zabbix_log(LOG_LEVEL_ERR, "cannot obtain data from proxy \"%s\": %s", proxy->host,
+				zbx_socket_strerror());
+	}
 	else
-		zabbix_log(LOG_LEVEL_DEBUG, "obtained data from proxy \"%s\": %s", proxy->host, sock->buffer);
+		zabbix_log(LOG_LEVEL_DEBUG, "obtained data from proxy \"%s\": [%s]", proxy->host, sock->buffer);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
 
 	return ret;
 }
 
-static void	disconnect_proxy(zbx_sock_t *sock)
+static void	disconnect_proxy(zbx_socket_t *sock)
 {
 	const char	*__function_name = "disconnect_proxy";
 
@@ -118,7 +123,7 @@ static void	disconnect_proxy(zbx_sock_t *sock)
 static int	get_data_from_proxy(DC_PROXY *proxy, const char *request, char **data, zbx_timespec_t *ts)
 {
 	const char	*__function_name = "get_data_from_proxy";
-	zbx_sock_t	s;
+	zbx_socket_t	s;
 	struct zbx_json	j;
 	int		ret = FAIL;
 
@@ -171,7 +176,7 @@ static int	process_proxy(void)
 	int			num, i, ret;
 	struct zbx_json		j;
 	struct zbx_json_parse	jp, jp_data;
-	zbx_sock_t		s;
+	zbx_socket_t		s;
 	char			*answer = NULL, *port = NULL;
 	time_t			now;
 	unsigned char		update_nextcheck;
@@ -198,7 +203,8 @@ static int	process_proxy(void)
 		proxy.addr = proxy.addr_orig;
 
 		port = zbx_strdup(port, proxy.port_orig);
-		substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, NULL, NULL, &port, MACRO_TYPE_COMMON, NULL, 0);
+		substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+				&port, MACRO_TYPE_COMMON, NULL, 0);
 		if (FAIL == is_ushort(port, &proxy.port))
 		{
 			zabbix_log(LOG_LEVEL_ERR, "invalid proxy \"%s\" port: \"%s\"", proxy.host, port);
@@ -217,28 +223,27 @@ static int	process_proxy(void)
 
 			if (SUCCEED != (ret = get_proxyconfig_data(proxy.hostid, &j, &error)))
 			{
-				zabbix_log(LOG_LEVEL_ERR, "cannot collect proxy configuration: %s", error);
+				zabbix_log(LOG_LEVEL_ERR, "cannot collect configuration data for proxy \"%s\": %s",
+						proxy.host, error);
 				zbx_free(error);
 
 				goto network_error;
 			}
 
-			zabbix_log(LOG_LEVEL_WARNING, "sending configuration data to proxy \"%s\", datalen "
-					ZBX_FS_SIZE_T, proxy.host, (zbx_fs_size_t)j.buffer_size);
-
 			if (SUCCEED == (ret = connect_to_proxy(&proxy, &s, CONFIG_TRAPPER_TIMEOUT)))
 			{
+				zabbix_log(LOG_LEVEL_WARNING, "sending configuration data to proxy \"%s\" at \"%s\","
+						" datalen " ZBX_FS_SIZE_T,
+						proxy.host, s.peer, (zbx_fs_size_t)j.buffer_size);
+
 				if (SUCCEED == (ret = send_data_to_proxy(&proxy, &s, j.buffer)))
 				{
-					char	*info = NULL, *error = NULL;
-
-					if (SUCCEED != (ret = zbx_recv_response(&s, &info, 0, &error)))
+					if (SUCCEED != (ret = zbx_recv_response(&s, 0, &error)))
 					{
-						zabbix_log(LOG_LEVEL_WARNING, "cannot send proxy configuration, "
-								"error:\"%s\", info:\"%s\"", ZBX_NULL2EMPTY_STR(error),
-								ZBX_NULL2EMPTY_STR(info));
+						zabbix_log(LOG_LEVEL_WARNING, "cannot send configuration data to proxy"
+								" \"%s\" at \"%s\": %s", proxy.host, s.peer, error);
 					}
-					zbx_free(info);
+
 					zbx_free(error);
 				}
 
@@ -254,6 +259,15 @@ static int	process_proxy(void)
 			if (SUCCEED == get_data_from_proxy(&proxy,
 					ZBX_PROTO_VALUE_HOST_AVAILABILITY, &answer, NULL))
 			{
+				if ('\0' == *answer)
+				{
+					zabbix_log(LOG_LEVEL_WARNING, "proxy \"%s\" at \"%s\" returned no host"
+							" availability data: check allowed connection types and"
+							" access rights", proxy.host, proxy.addr);
+					zbx_free(answer);
+					goto network_error;
+				}
+
 				if (SUCCEED == zbx_json_open(answer, &jp))
 					process_host_availability(&jp);
 
@@ -265,6 +279,15 @@ retry_history:
 			if (SUCCEED == get_data_from_proxy(&proxy,
 					ZBX_PROTO_VALUE_HISTORY_DATA, &answer, &ts))
 			{
+				if ('\0' == *answer)
+				{
+					zabbix_log(LOG_LEVEL_WARNING, "proxy \"%s\" at \"%s\" returned no history"
+							" data: check allowed connection types and access rights",
+							proxy.host, proxy.addr);
+					zbx_free(answer);
+					goto network_error;
+				}
+
 				if (SUCCEED == zbx_json_open(answer, &jp))
 				{
 					process_hist_data(NULL, &jp, proxy.hostid, &ts, NULL);
@@ -278,6 +301,7 @@ retry_history:
 						}
 					}
 				}
+
 				zbx_free(answer);
 			}
 			else
@@ -286,6 +310,15 @@ retry_dhistory:
 			if (SUCCEED == get_data_from_proxy(&proxy,
 					ZBX_PROTO_VALUE_DISCOVERY_DATA, &answer, NULL))
 			{
+				if ('\0' == *answer)
+				{
+					zabbix_log(LOG_LEVEL_WARNING, "proxy \"%s\" at \"%s\" returned no discovery"
+							" data: check allowed connection types and access rights",
+							proxy.host, proxy.addr);
+					zbx_free(answer);
+					goto network_error;
+				}
+
 				if (SUCCEED == zbx_json_open(answer, &jp))
 				{
 					process_dhis_data(&jp);
@@ -299,6 +332,7 @@ retry_dhistory:
 						}
 					}
 				}
+
 				zbx_free(answer);
 			}
 			else
@@ -307,6 +341,15 @@ retry_autoreg_host:
 			if (SUCCEED == get_data_from_proxy(&proxy,
 					ZBX_PROTO_VALUE_AUTO_REGISTRATION_DATA, &answer, NULL))
 			{
+				if ('\0' == *answer)
+				{
+					zabbix_log(LOG_LEVEL_WARNING, "proxy \"%s\" at \"%s\" returned no auto"
+							" registration data: check allowed connection types and"
+							" access rights", proxy.host, proxy.addr);
+					zbx_free(answer);
+					goto network_error;
+				}
+
 				if (SUCCEED == zbx_json_open(answer, &jp))
 				{
 					process_areg_data(&jp, proxy.hostid);
@@ -320,6 +363,7 @@ retry_autoreg_host:
 						}
 					}
 				}
+
 				zbx_free(answer);
 			}
 			else
@@ -352,12 +396,15 @@ ZBX_THREAD_ENTRY(proxypoller_thread, args)
 	server_num = ((zbx_thread_args_t *)args)->server_num;
 	process_num = ((zbx_thread_args_t *)args)->process_num;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_daemon_type_string(daemon_type),
+	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_program_type_string(program_type),
 			server_num, get_process_type_string(process_type), process_num);
 
 #define STAT_INTERVAL	5	/* if a process is busy and does not sleep then update status not faster than */
 				/* once in STAT_INTERVAL seconds */
 
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	zbx_tls_init_child();
+#endif
 	zbx_setproctitle("%s #%d [connecting to the database]", get_process_type_string(process_type), process_num);
 	last_stat_time = time(NULL);
 
@@ -365,6 +412,8 @@ ZBX_THREAD_ENTRY(proxypoller_thread, args)
 
 	for (;;)
 	{
+		zbx_handle_log();
+
 		if (0 != sleeptime)
 		{
 			zbx_setproctitle("%s #%d [exchanged data with %d proxies in " ZBX_FS_DBL " sec,"
