@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2016 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -53,7 +53,8 @@ static int	zbx_execute_script_on_agent(DC_HOST *host, const char *command, char
 	}
 
 	port = zbx_strdup(port, item.interface.port_orig);
-	substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, &port, MACRO_TYPE_COMMON, NULL, 0);
+	substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
+			&port, MACRO_TYPE_COMMON, NULL, 0);
 
 	if (SUCCEED != (ret = is_ushort(port, &item.interface.port)))
 	{
@@ -68,7 +69,7 @@ static int	zbx_execute_script_on_agent(DC_HOST *host, const char *command, char
 
 	init_result(&agent_result);
 
-	alarm(CONFIG_TIMEOUT);
+	zbx_alarm_on(CONFIG_TIMEOUT);
 
 	if (SUCCEED != (ret = get_value_agent(&item, &agent_result)))
 	{
@@ -79,7 +80,7 @@ static int	zbx_execute_script_on_agent(DC_HOST *host, const char *command, char
 	else if (NULL != result && ISSET_TEXT(&agent_result))
 		*result = zbx_strdup(*result, agent_result.text);
 
-	alarm(0);
+	zbx_alarm_off();
 
 	free_result(&agent_result);
 
@@ -113,7 +114,8 @@ static int	zbx_execute_ipmi_command(DC_HOST *host, const char *command, char *er
 	}
 
 	port = zbx_strdup(port, item.interface.port_orig);
-	substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, &port, MACRO_TYPE_COMMON, NULL, 0);
+	substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL, &port,
+			MACRO_TYPE_COMMON, NULL, 0);
 
 	if (SUCCEED != (ret = is_ushort(port, &item.interface.port)))
 	{
@@ -175,7 +177,7 @@ static int	zbx_execute_script_on_terminal(DC_HOST *host, zbx_script_t *script, c
 			break;
 	}
 
-	substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL,
+	substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
 			&script->port, MACRO_TYPE_COMMON, NULL, 0);
 
 	if ('\0' != *script->port && SUCCEED != (ret = is_ushort(script->port, NULL)))
@@ -203,7 +205,7 @@ static int	zbx_execute_script_on_terminal(DC_HOST *host, zbx_script_t *script, c
 
 	init_result(&agent_result);
 
-	alarm(CONFIG_TIMEOUT);
+	zbx_alarm_on(CONFIG_TIMEOUT);
 
 	if (SUCCEED != (ret = function(&item, &agent_result)))
 	{
@@ -214,7 +216,7 @@ static int	zbx_execute_script_on_terminal(DC_HOST *host, zbx_script_t *script, c
 	else if (NULL != result && ISSET_TEXT(&agent_result))
 		*result = zbx_strdup(*result, agent_result.text);
 
-	alarm(0);
+	zbx_alarm_off();
 
 	free_result(&agent_result);
 
@@ -308,12 +310,9 @@ void	zbx_script_clean(zbx_script_t *script)
  *                                                                            *
  * Purpose: executing user scripts or remote commands                         *
  *                                                                            *
- * Parameters:                                                                *
- *                                                                            *
  * Return value:  SUCCEED - processed successfully                            *
  *                FAIL - an error occurred                                    *
- *                                                                            *
- * Author: Alexander Vladishev                                                *
+ *                TIMEOUT_ERROR - a timeout occurred                          *
  *                                                                            *
  * Comments: !!! always call 'zbx_script_clean' function after                *
  *           'zbx_execute_script' to clear allocated memory                   *
@@ -362,9 +361,9 @@ int	zbx_execute_script(DC_HOST *host, zbx_script_t *script, char **result, char
 			break;
 		case ZBX_SCRIPT_TYPE_SSH:
 #ifdef HAVE_SSH2
-			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL,
+			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
 					&script->publickey, MACRO_TYPE_COMMON, NULL, 0);
-			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL,
+			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
 					&script->privatekey, MACRO_TYPE_COMMON, NULL, 0);
 			/* break; is not missing here */
 #else
@@ -372,9 +371,9 @@ int	zbx_execute_script(DC_HOST *host, zbx_script_t *script, char **result, char
 			break;
 #endif
 		case ZBX_SCRIPT_TYPE_TELNET:
-			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL,
+			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
 					&script->username, MACRO_TYPE_COMMON, NULL, 0);
-			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL,
+			substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
 					&script->password, MACRO_TYPE_COMMON, NULL, 0);
 
 			ret = zbx_execute_script_on_terminal(host, script, result, error, max_error_len);
@@ -389,7 +388,7 @@ int	zbx_execute_script(DC_HOST *host, zbx_script_t *script, char **result, char
 
 			if (SUCCEED == check_script_permissions(groupid, host->hostid, error, max_error_len))
 			{
-				substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, host, NULL,
+				substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, host, NULL, NULL, NULL,
 						&script->command, MACRO_TYPE_SCRIPT, NULL, 0);
 
 				ret = zbx_execute_script(host, script, result, error, max_error_len);	/* recursion */
@@ -399,7 +398,7 @@ int	zbx_execute_script(DC_HOST *host, zbx_script_t *script, char **result, char
 			zbx_snprintf(error, max_error_len, "Invalid command type [%d]", (int)script->type);
 	}
 
-	if (FAIL == ret && NULL != result)
+	if (SUCCEED != ret && NULL != result)
 		*result = zbx_strdup(*result, "");
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
