@@ -35,7 +35,7 @@
 
 #define ZBX_TRIGGERS_MAX	1000
 
-extern unsigned char	process_type, daemon_type;
+extern unsigned char	process_type, program_type;
 extern int		server_num, process_num;
 
 /******************************************************************************
@@ -767,7 +767,7 @@ ZBX_THREAD_ENTRY(timer_thread, args)
 	server_num = ((zbx_thread_args_t *)args)->server_num;
 	process_num = ((zbx_thread_args_t *)args)->process_num;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_daemon_type_string(daemon_type),
+	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_program_type_string(program_type),
 			server_num, get_process_type_string(process_type), process_num);
 
 #define STAT_INTERVAL	5	/* if a process is busy and does not sleep then update status not faster than */
@@ -839,6 +839,8 @@ ZBX_THREAD_ENTRY(timer_thread, args)
 
 		zbx_sleep_loop(sleeptime);
 
+		zbx_handle_log();
+
 		if (0 != sleeptime)
 		{
 			if (1 != process_num)
