@@ -25,7 +25,7 @@
 
 extern int	CONFIG_TIMEOUT;
 
-void	send_proxyconfig(zbx_sock_t *sock, struct zbx_json_parse *jp);
-void	recv_proxyconfig(zbx_sock_t *sock, struct zbx_json_parse *jp);
+void	send_proxyconfig(zbx_socket_t *sock, struct zbx_json_parse *jp);
+void	recv_proxyconfig(zbx_socket_t *sock, struct zbx_json_parse *jp);
 
 #endif
