@@ -36,9 +36,12 @@
 #include "proxyhosts.h"
 
 #include "daemon.h"
+#include "../../libs/zbxcrypto/tls.h"
+#include "../../libs/zbxcrypto/tls_tcp_active.h"
 
-extern unsigned char	process_type, daemon_type;
+extern unsigned char	process_type, program_type;
 extern int		server_num, process_num;
+extern size_t		(*find_psk_in_cache)(const unsigned char *, unsigned char *, size_t);
 
 /******************************************************************************
  *                                                                            *
@@ -47,7 +50,7 @@ extern int		server_num, process_num;
  * Purpose: processes the received values from active agents and senders      *
  *                                                                            *
  ******************************************************************************/
-static void	recv_agenthistory(zbx_sock_t *sock, struct zbx_json_parse *jp, zbx_timespec_t *ts)
+static void	recv_agenthistory(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_timespec_t *ts)
 {
 	const char	*__function_name = "recv_agenthistory";
 	char		*info = NULL;
@@ -71,7 +74,7 @@ static void	recv_agenthistory(zbx_sock_t *sock, struct zbx_json_parse *jp, zbx_t
  * Purpose: processes the received values from active proxies                 *
  *                                                                            *
  ******************************************************************************/
-static void	recv_proxyhistory(zbx_sock_t *sock, struct zbx_json_parse *jp, zbx_timespec_t *ts)
+static void	recv_proxyhistory(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_timespec_t *ts)
 {
 	const char	*__function_name = "recv_proxyhistory";
 	zbx_uint64_t	proxy_hostid;
@@ -80,10 +83,10 @@ static void	recv_proxyhistory(zbx_sock_t *sock, struct zbx_json_parse *jp, zbx_t
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (SUCCEED != (ret = get_active_proxy_id(jp, &proxy_hostid, host, &error)))
+	if (SUCCEED != (ret = get_active_proxy_id(jp, &proxy_hostid, host, sock, &error)))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "history data from active proxy on \"%s\" failed: %s",
-				get_ip_by_socket(sock), error);
+		zabbix_log(LOG_LEVEL_WARNING, "cannot parse history data from active proxy at \"%s\": %s",
+				sock->peer, error);
 		goto out;
 	}
 
@@ -104,20 +107,24 @@ out:
  *                                                                            *
  * Purpose: send history data to a Zabbix server                              *
  *                                                                            *
- * Author: Alexander Vladishev                                                *
- *                                                                            *
  ******************************************************************************/
-static void	send_proxyhistory(zbx_sock_t *sock, zbx_timespec_t *ts)
+static void	send_proxyhistory(zbx_socket_t *sock, zbx_timespec_t *ts)
 {
 	const char	*__function_name = "send_proxyhistory";
 
 	struct zbx_json	j;
 	zbx_uint64_t	lastid;
-	int		records;
-	char		*info = NULL, *error = NULL;
+	int		records, ret = FAIL;
+	char		*error = NULL;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
+	if (SUCCEED != check_access_passive_proxy(sock, ZBX_DO_NOT_SEND_RESPONSE, "history data request"))
+	{
+		/* do not send any reply to server in this case as the server expects history data */
+		goto out1;
+	}
+
 	zbx_json_init(&j, ZBX_JSON_STAT_BUF_LEN);
 
 	zbx_json_addarray(&j, ZBX_PROTO_TAG_DATA);
@@ -131,24 +138,24 @@ static void	send_proxyhistory(zbx_sock_t *sock, zbx_timespec_t *ts)
 
 	if (SUCCEED != zbx_tcp_send_to(sock, j.buffer, CONFIG_TIMEOUT))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "error while sending history data to server: %s", zbx_tcp_strerror());
+		error = zbx_strdup(error, zbx_socket_strerror());
 		goto out;
 	}
 
-	if (SUCCEED != zbx_recv_response(sock, &info, CONFIG_TIMEOUT, &error))
-	{
-		zabbix_log(LOG_LEVEL_WARNING, "sending history data to server: error:\"%s\", info:\"%s\"",
-				ZBX_NULL2EMPTY_STR(error), ZBX_NULL2EMPTY_STR(info));
+	if (SUCCEED != zbx_recv_response(sock, CONFIG_TIMEOUT, &error))
 		goto out;
-	}
 
 	if (0 != records)
 		proxy_set_hist_lastid(lastid);
-out:
-	zbx_json_free(&j);
-	zbx_free(info);
-	zbx_free(error);
 
+	ret = SUCCEED;
+out:
+	if (SUCCEED != ret)
+		zabbix_log(LOG_LEVEL_WARNING, "cannot send history data to server at \"%s\": %s", sock->peer, error);
+
+	zbx_json_free(&j);
+	zbx_free(error);
+out1:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
 
@@ -164,7 +171,7 @@ out:
  * Author: Alexander Vladishev                                                *
  *                                                                            *
  ******************************************************************************/
-static void	recv_proxy_heartbeat(zbx_sock_t *sock, struct zbx_json_parse *jp)
+static void	recv_proxy_heartbeat(zbx_socket_t *sock, struct zbx_json_parse *jp)
 {
 	const char	*__function_name = "recv_proxy_heartbeat";
 
@@ -174,10 +181,10 @@ static void	recv_proxy_heartbeat(zbx_sock_t *sock, struct zbx_json_parse *jp)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (SUCCEED != (ret = get_active_proxy_id(jp, &proxy_hostid, host, &error)))
+	if (SUCCEED != (ret = get_active_proxy_id(jp, &proxy_hostid, host, sock, &error)))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "heartbeat from active proxy on \"%s\" failed: %s",
-				get_ip_by_socket(sock), error);
+		zabbix_log(LOG_LEVEL_WARNING, "cannot parse heartbeat from active proxy at \"%s\": %s",
+				sock->peer, error);
 		goto out;
 	}
 
@@ -332,7 +339,7 @@ static int	zbx_session_validate(const char *sessionid, int access_level)
  *                FAIL - an error occurred                                    *
  *                                                                            *
  ******************************************************************************/
-static int	recv_getqueue(zbx_sock_t *sock, struct zbx_json_parse *jp)
+static int	recv_getqueue(zbx_socket_t *sock, struct zbx_json_parse *jp)
 {
 	const char		*__function_name = "recv_getqueue";
 	int			ret = FAIL, request_type = -1, now, i;
@@ -462,16 +469,19 @@ out:
 	return ret;
 }
 
-static void	active_passive_misconfig(zbx_sock_t *sock)
+static void	active_passive_misconfig(zbx_socket_t *sock)
 {
-	const char	*msg = "misconfiguration error: the proxy is running in the active mode but server sends "
-			"requests to it as to proxy in passive mode";
+	char   *msg = NULL;
+
+	msg = zbx_dsprintf(msg, "misconfiguration error: the proxy is running in the active mode but server at \"%s\""
+			" sends requests to it as to proxy in passive mode", sock->peer);
 
 	zabbix_log(LOG_LEVEL_WARNING, "%s", msg);
 	zbx_send_response(sock, FAIL, msg, CONFIG_TIMEOUT);
+	zbx_free(msg);
 }
 
-static int	process_trap(zbx_sock_t	*sock, char *s, zbx_timespec_t *ts)
+static int	process_trap(zbx_socket_t *sock, char *s, zbx_timespec_t *ts)
 {
 	int	ret = SUCCEED;
 
@@ -488,7 +498,7 @@ static int	process_trap(zbx_sock_t	*sock, char *s, zbx_timespec_t *ts)
 		{
 			zbx_send_response(sock, FAIL, zbx_json_strerror(), CONFIG_TIMEOUT);
 			zabbix_log(LOG_LEVEL_WARNING, "received invalid JSON object from %s: %s",
-					get_ip_by_socket(sock), zbx_json_strerror());
+					sock->peer, zbx_json_strerror());
 			return FAIL;
 		}
 
@@ -496,18 +506,18 @@ static int	process_trap(zbx_sock_t	*sock, char *s, zbx_timespec_t *ts)
 		{
 			if (0 == strcmp(value, ZBX_PROTO_VALUE_PROXY_CONFIG))
 			{
-				if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
+				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 				{
 					send_proxyconfig(sock, &jp);
 				}
-				else if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY_PASSIVE))
+				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
 				{
-					zabbix_log(LOG_LEVEL_WARNING, "received configuration data from server,"
-							" datalen " ZBX_FS_SIZE_T,
-							(zbx_fs_size_t)(jp.end - jp.start + 1));
+					zabbix_log(LOG_LEVEL_WARNING, "received configuration data from server"
+							" at \"%s\", datalen " ZBX_FS_SIZE_T,
+							sock->peer, (zbx_fs_size_t)(jp.end - jp.start + 1));
 					recv_proxyconfig(sock, &jp);
 				}
-				else if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY_ACTIVE))
+				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_ACTIVE))
 				{
 					/* This is a misconfiguration: the proxy is configured in active mode */
 					/* but server sends requests to it as to a proxy in passive mode. To  */
@@ -524,28 +534,28 @@ static int	process_trap(zbx_sock_t	*sock, char *s, zbx_timespec_t *ts)
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_HISTORY_DATA))
 			{
-				if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
+				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					recv_proxyhistory(sock, &jp, ts);
-				else if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY_PASSIVE))
+				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
 					send_proxyhistory(sock, ts);
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_DISCOVERY_DATA))
 			{
-				if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
+				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					recv_discovery_data(sock, &jp);
-				else if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY_PASSIVE))
+				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
 					send_discovery_data(sock);
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_AUTO_REGISTRATION_DATA))
 			{
-				if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
+				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					recv_areg_data(sock, &jp);
-				else if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY_PASSIVE))
+				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
 					send_areg_data(sock);
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_PROXY_HEARTBEAT))
 			{
-				if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
+				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					recv_proxy_heartbeat(sock, &jp);
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_GET_ACTIVE_CHECKS))
@@ -554,9 +564,9 @@ static int	process_trap(zbx_sock_t	*sock, char *s, zbx_timespec_t *ts)
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_HOST_AVAILABILITY))
 			{
-				if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
+				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					recv_host_availability(sock, &jp);
-				else if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY_PASSIVE))
+				else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE))
 					send_host_availability(sock);
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_COMMAND))
@@ -565,7 +575,7 @@ static int	process_trap(zbx_sock_t	*sock, char *s, zbx_timespec_t *ts)
 			}
 			else if (0 == strcmp(value, ZBX_PROTO_VALUE_GET_QUEUE))
 			{
-				if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
+				if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
 					ret = recv_getqueue(sock, &jp);
 			}
 			else
@@ -628,16 +638,16 @@ static int	process_trap(zbx_sock_t	*sock, char *s, zbx_timespec_t *ts)
 
 		process_mass_data(sock, 0, &av, 1, NULL);
 
-		alarm(CONFIG_TIMEOUT);
+		zbx_alarm_on(CONFIG_TIMEOUT);
 		if (SUCCEED != zbx_tcp_send_raw(sock, "OK"))
 			zabbix_log(LOG_LEVEL_WARNING, "Error sending result back");
-		alarm(0);
+		zbx_alarm_off();
 	}
 
 	return ret;
 }
 
-static void	process_trapper_child(zbx_sock_t *sock, zbx_timespec_t *ts)
+static void	process_trapper_child(zbx_socket_t *sock, zbx_timespec_t *ts)
 {
 	if (SUCCEED != zbx_tcp_recv_to(sock, CONFIG_TRAPPER_TIMEOUT))
 		return;
@@ -648,29 +658,38 @@ static void	process_trapper_child(zbx_sock_t *sock, zbx_timespec_t *ts)
 ZBX_THREAD_ENTRY(trapper_thread, args)
 {
 	double		sec = 0.0;
-	zbx_sock_t	s;
+	zbx_socket_t	s;
 
 	process_type = ((zbx_thread_args_t *)args)->process_type;
 	server_num = ((zbx_thread_args_t *)args)->server_num;
 	process_num = ((zbx_thread_args_t *)args)->process_num;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_daemon_type_string(daemon_type),
+	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_program_type_string(program_type),
 			server_num, get_process_type_string(process_type), process_num);
 
-	memcpy(&s, (zbx_sock_t *)((zbx_thread_args_t *)args)->args, sizeof(zbx_sock_t));
+	memcpy(&s, (zbx_socket_t *)((zbx_thread_args_t *)args)->args, sizeof(zbx_socket_t));
 
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	zbx_tls_init_child();
+	find_psk_in_cache = DCget_psk_by_identity;
+#endif
 	zbx_setproctitle("%s #%d [connecting to the database]", get_process_type_string(process_type), process_num);
 
 	DBconnect(ZBX_DB_CONNECT_NORMAL);
 
 	for (;;)
 	{
+		zbx_handle_log();
+
 		zbx_setproctitle("%s #%d [processed data in " ZBX_FS_DBL " sec, waiting for connection]",
 				get_process_type_string(process_type), process_num, sec);
 
 		update_selfmon_counter(ZBX_PROCESS_STATE_IDLE);
 
-		if (SUCCEED == zbx_tcp_accept(&s))
+		/* Trapper has to accept all types of connections it can accept with the specified configuration. */
+		/* Only after receiving data it is known who has sent them and one can decide to accept or discard */
+		/* the data. */
+		if (SUCCEED == zbx_tcp_accept(&s, ZBX_TCP_SEC_TLS_CERT | ZBX_TCP_SEC_TLS_PSK | ZBX_TCP_SEC_UNENCRYPTED))
 		{
 			zbx_timespec_t	ts;
 
@@ -688,10 +707,10 @@ ZBX_THREAD_ENTRY(trapper_thread, args)
 
 			zbx_tcp_unaccept(&s);
 		}
-		else if (EINTR != zbx_sock_last_error())
+		else if (EINTR != zbx_socket_last_error())
 		{
 			zabbix_log(LOG_LEVEL_WARNING, "failed to accept an incoming connection: %s",
-					zbx_tcp_strerror());
+					zbx_socket_strerror());
 		}
 	}
 }
