@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2015 Zabbix SIA
+** Copyright (C) 2001-2016 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,6 +31,15 @@
 
 #define ZBX_VMWARE_STATE_UPDATING	0x100
 #define ZBX_VMWARE_STATE_UPDATING_PERF	0x200
+#define ZBX_VMWARE_STATE_REMOVING	0x400
+
+#define ZBX_VMWARE_STATE_BUSY		(ZBX_VMWARE_STATE_UPDATING | ZBX_VMWARE_STATE_UPDATING_PERF \
+							| ZBX_VMWARE_STATE_REMOVING)
+
+/* the vmware performance counter state */
+#define ZBX_VMWARE_COUNTER_NEW		0x00
+#define ZBX_VMWARE_COUNTER_READY	0x01
+#define ZBX_VMWARE_COUNTER_UPDATING	0x10
 
 /* performance counter data */
 typedef struct
@@ -42,6 +51,9 @@ typedef struct
 	/*    pair->first  - instance               */
 	/*    pair->second - value                  */
 	zbx_vector_ptr_pair_t	values;
+
+	/* the counter state, see ZBX_VMAWRE_COUNTER_* defines */
+	unsigned char		state;
 }
 zbx_vmware_perf_counter_t;
 
